﻿/********************************************************
*                                                        *
*   © Microsoft. All rights reserved.                    *
*                                                        *
*********************************************************/
(function () {
'use strict';      
WinJS.Namespace.define("WeatherAppJS.Controls.Clusters",
{
ClusterController: WinJS.Class.define(function ClusterController_constructor(element, config) {
/// <summary>
/// Cluster Controller - Controls data fetching and rendering tasks of a cluster. 
/// </summary>
if (!this._validateConfig(config)) {
WeatherAppJS.Controls.Utilities.interfaceException(this._name, "invalid cluster configuration");
}
WeatherAppJS.Controls.Utilities.writeProfilerMark(this._name, "constructor", "StartTM");
this._element = element ? element : document.createElement("div");
WinJS.Utilities.addClass(this._element, "weatherClusterController");
this._config = config;
this._initializeModel(config);
this._initializeView(config);
this._hideTitleIfReq();
this._id = this._config.id ? this._config.id : "";
WeatherAppJS.Controls.Utilities.writeProfilerMark(this._name, "constructor", "StopTM");
}, {
_name: "WeatherAppJS.Controls.Clusters.ClusterController",
_id: "WeatherAppJS.Controls.Clusters.ClusterController",
_element: null,
_config: null,
_modelInstance: null,
_viewHelper: null,
_renderClusterPromise: null,
// flag to track, we have refreshed the cluster atleast once.
_isFirstRefresh: true,
_isDisposed: false,
// make sure all mandatory configurations are specified
// INPUT: configuration object
// OUTPUT: returns true if the input config is valid
_validateConfig: function _validateConfig(config) {
if (!config || !config.model || !config.model.modelControl || !config.view) {
return false;
}
return true;
},
// creates instance of data model
// INPUT: configuration object for the model
_initializeModel: function _initializeModel(config) {
var modelConfig = config.model;
this._modelInstance = PlatformJS.Utilities.createObject(modelConfig.modelControl, modelConfig.modelControlConfig);
if (!this._modelInstance) {
WeatherAppJS.Controls.Utilities.interfaceException(this._name, "unable to create model instance");
}
},
_cleanUpModel: function _cleanUpModel() {
if (this._modelInstance && typeof this._modelInstance.cleanUp === "function") {
return this._modelInstance.cleanUp();
}
},
// creates instance of view helper that would take care of view initialization 
// INPUT: configuration object for the view helper
_initializeView: function _initializeView(config) {
// TODO: check we can use new element instead of this._element. 
// It might help reducing count of document reflow while rendering.
config.retryCallback = this._retryCallback.bind(this);
this._viewHelper = PlatformJS.Utilities.createObject("WeatherAppJS.Controls.Clusters.ClusterControllerViewHelper", this._element, config);
if (!this._viewHelper) {
WeatherAppJS.Controls.Utilities.interfaceException(this._name, "unable to create view instance");
}
},
_cleanUpView: function _cleanUpView() {
if (this._viewHelper && typeof this._viewHelper.cleanUp === "function") {
return this._viewHelper.cleanUp();
}
},
_hideTitleIfReq: function _hideTitleIfReq() {
var that = this;
if (!that._config.hideTitleUntilRender) {
return;
}
var element = that._element;
var clusterNode = element.parentNode;
var titleNode = clusterNode ? clusterNode.querySelector(".platformClusterHeader") : null;
if (!titleNode) {
return;
}
titleNode.style.visibility = "hidden";
that._isTitleHidden = true;
},
_showTitleIfReq: function _showTitleIfReq() {
var that = this;
if (!that._isTitleHidden) {
return;
}
var element = that._element;
var clusterNode = element.parentNode;
var titleNode = clusterNode ? clusterNode.querySelector(".platformClusterHeader") : null;
if (!titleNode) {
return;
}
titleNode.style.visibility = "";
that._isTitleHidden = false;
},
// function to be used to recover from errors Eg: refresh button click in error module)
_retryCallback: function _retryCallback() {
this._refreshCluster();
},
// list of tasks to be done post render
_onClusterRenderComplete: function _onClusterRenderComplete() {
// schedule cluster refresh task
//this._registerForClusterRefresh(this._clusterResponse);
},
// function to get data from the model
// INPUT: parameters object to be passed to data model while making the data fetch call
// OUTPUT: returns promise for the data fetch task
_fetchData: function _fetchData(params) {
if (this._modelInstance && typeof this._modelInstance.getData === "function") {
return this._modelInstance.getData(params);
}
WeatherAppJS.Controls.Utilities.interfaceException(this._name, "data model interface violation");
return WinJS.Promise.wrapError(null);
},
// INPUT: current response object
// OUTPUT: returns promise for the render task
_renderView: function _renderView(response) {
if (this._viewHelper && typeof this._viewHelper.render === "function") {
return this._viewHelper.render(response);
}
WeatherAppJS.Controls.Utilities.interfaceException(this._name, "view helper interface violation");
return WinJS.Promise.wrapError({message: "invalid view helper"});
},
_renderCluster: function _renderCluster() {
var that = this;
CommonJS.Progress.showProgress(CommonJS.Progress.headerProgressType);
WeatherAppJS.Controls.Utilities.writeProfilerMark(that._name, "_renderCluster", "StartTM");
var p = new WinJS.Promise(function ClusterController_renderClusterInit(complete, error) {
WeatherAppJS.Controls.Utilities.writeProfilerMark(that._name, "_fetchData", "StartTM");
that._renderClusterPromise = that._fetchData().then(function ClusterController_fetchDataCompleteHandler(response) {
WeatherAppJS.Controls.Utilities.writeProfilerMark(that._name, "_fetchData", "StopTM");
WeatherAppJS.Controls.Utilities.writeProfilerMark(that._name, "_renderView", "StartTM");
return that._renderView(response);
}, function ClusterController_fetchDataErrorHandler(errorResponse) {
if (WeatherAppJS.Controls.Utilities.isCancellationError(errorResponse)) {
// dont break the promise chain here when the render is canceled. Propagate the cancellation error here.
// We catch it at the end (renderViewErrorHandler)
return WinJS.Promise.wrapError(errorResponse);
}
// send the error response to view control to decide on rendering error.
return that._renderView(errorResponse);
}, function ClusterController_fetchDataProgressHandler(response) {
return that._renderView(response);
}).then(function ClusterController_renderViewCompleteHandler() {
WeatherAppJS.Controls.Utilities.writeProfilerMark(that._name, "_renderView", "StopTM");
that._showTitleIfReq();
that._onClusterRenderComplete();
WeatherAppJS.Controls.Utilities.writeProfilerMark(that._name, "_renderCluster", "StopTM");
complete(true);
CommonJS.Progress.hideProgress(CommonJS.Progress.headerProgressType);
}, function ClusterController_renderViewErrorHandler(errorMessage) {
that._removeCluster();
CommonJS.Progress.hideProgress(CommonJS.Progress.headerProgressType);
if (WeatherAppJS.Controls.Utilities.isCancellationError(errorMessage)) {
return;
}
complete(false); // just for completeness sake... this will never happen... 
WeatherAppJS.Controls.Utilities.interfaceException(that._name, "something has gone wrong while rendering");
});
}, function () {
CommonJS.Progress.hideProgress(CommonJS.Progress.headerProgressType);
WeatherAppJS.Controls.Utilities.cancelPromise(that._renderClusterPromise);
});
return p;
},
_refreshCluster: function _refreshCluster() {
this._isFirstRefresh = false;
return this._renderCluster();
},
render: function render() {
if (this._isDisposed) {
return;
}
return this._renderCluster();
},
_removeCluster: function _removeCluster() {
var that = this;
var clusterKey = that._config.clusterKey;
if (clusterKey && that._config.currentIPage && that._config.currentIPage._data) {
that.dispose();
var clusters = that._config.currentIPage._data;
for (var i = 0; i < clusters.length; i++) {
if (clusters.getItem(i).data.clusterKey === clusterKey) {
clusters.splice(i, 1);
break;
}
}
}
return;
},
dispose: function dispose() {
this.cleanUp();
},
cleanUp: function cleanUp() {
var that = this;
if (that._isDisposed) {
return;
}
that._isDisposed = true;
var name = that._name;
WeatherAppJS.Controls.Utilities.writeProfilerMark(name, "cleanUp", "StartTM");
WeatherAppJS.Controls.Utilities.cancelPromise(this._renderClusterPromise);
that._element.innerHTML = "";
this._cleanUpModel();
this._cleanUpView();
//WeatherAppJS.Controls.Utilities.setObjectPropertiesToNull(this);
WeatherAppJS.Controls.Utilities.writeProfilerMark(name, "cleanUp", "StopTM");
}
}),
/// <summary>
/// Cluster View Helper - Helps cluster controller to control views (css/html work). 
/// </summary>
ClusterControllerViewHelper: WinJS.Class.define(function ClusterControllerViewHelper_constructor(element, config) {
if (!this._validateConfig(config) || !element) {
WeatherAppJS.Controls.Utilities.interfaceException(this._name, "invalid cluster view helper configuration");
}
this._element = element ? element : document.createElement("div");
this._config = config;
}, {
_name: "WeatherAppJS.Controls.Clusters.ClusterControllerViewHelper",
_element: null,
_config: null,
_viewPortElement: null,
// flag to track, we have rendered the view atleast once.
_isFirstRendering: true,
// instance of view control used for the cluster
_viewControl: null,
_loadViewPromise: null,
// make sure all mandatory configurations are specified
// INPUT: configuration object
// OUTPUT: returns true if the input config is valid
_validateConfig: function _validateConfig(config) {
// Note: retry can be done by event handling.
// but it is fine as view helper is also part of the cluster control
if (!config || !config.retryCallback) {
return false;
}
return true;
},
// checks whether the given response can be consumed
// INPUT: new response object
// OUTPUT: returns true if the response can be consumed
_validateResponseAsync: function _validateResponseAsync(response) {
return WinJS.Promise.wrap(response);
},
// clean up the given view control
// INPUT: view control object which needs to be destoryed
_cleanupViewControl: function _cleanupViewControl(control) {
if (control && typeof control.cleanUp === "function") {
control.cleanUp();
}
},
// start the rendering task using the given given view control and data
// INPUT: view control object and data object
// OUTPUT: returns promise for the rendering task
_renderViewControl: function _renderViewControl(control, data) {
if (control && typeof control.render === "function") {
return control.render(data);
}
return WinJS.Promise.wrapError({ message: "invalid view control" });
},
// Creats a view control object
// INPUT: type => to identify the required view control, element => control element
// OUTPUT: returns promise for the creating view control task
_createViewControlAsync: function _createViewControlAsync(config, element) {
return WeatherAppJS.Controls.Utilities.createViewControlAsync(config, element);
},
// Loads the view fragment
// Note: when the new response requires deifferent view control to render, 
//       it destorys the current view and loads a new view
// INPUT: new response object
// OUTPUT: returns promise for the loading view task
_loadView: function _loadView(response) {
this._cleanupViewControl(this._viewControl);
this._element.innerHTML = "";
this._viewPortElement = WeatherAppJS.Controls.Utilities.createDivWithClassName("weatherClusterViewBlock");
this._element.appendChild(this._viewPortElement);
this._loadViewPromise = this._createViewControlAsync(this._config, this._viewPortElement);
return this._loadViewPromise;
},
_onRenderViewControlComplete: function _onRenderViewControlComplete() {
// as of now, just a place holder
},
// List of tasks to be done when cluster render task is failed
// INPUT: error message/error response
_onRenderViewControlsError: function _onRenderViewControlsError(errorResponse) {
this._element.innerHTML = "";
},
// creates error module to show when cluster render failed.
// INPUT: error message/error response
// OUTPUT: promise for the loading error module task
_loadErrorModule: function _loadErrorModule(errorResponse) {
var errorModule = CommonJS.Error.getErrorModuleItem(WinJS.Utilities.markSupportedForProcessing(this._config.retryCallback));
if (errorModule) {
errorModule.errorContainerClassName += " weatherErrorItem";
errorModule.message = PlatformJS.Services.resourceLoader.getString("CouldNotLoadDataMsg");
// hide the refresh button in case of offline.
if (!WeatherAppJS.Networking.isInternetAvailable) {
errorModule.errorContainerClassName += " weatherErrorItemOffline"; 
}
this._viewPortElement = WeatherAppJS.Controls.Utilities.createDivWithClassName("weatherClusterErrorViewBlock");
this._element.appendChild(this._viewPortElement);
return WeatherAppJS.Controls.Utilities.loadModule(errorModule.moduleInfo, errorModule, this._viewPortElement, null, null).then(function ClusterViewHelper_loadErrorModuleComplete(fragment) {
return WinJS.Resources.processAll(fragment);
});
}
return WinJS.Promise.wrapError(errorResponse);
},
render: function render(response) {
var that = this;
return that._validateResponseAsync(response).then(function ClusterControllerViewHelper_validateResponseComplete() {
return that._loadView(response);
}).then(function ClusterControllerViewHelper_loadViewCompleteHandler(viewControl) {
that._viewControl = viewControl;
return that._renderViewControl(viewControl, response);
}).then(function ClusterControllerViewHelper_renderViewControlsCompleteHandler () {
that._isFirstRendering = false;
that._onRenderViewControlComplete();
return WinJS.Promise.wrap(response);
// Do not handle error scenario and show error cluster; instead the error is propagated further
//}, function ClusterControllerViewHelper_renderErrorHandler(errorMessage) {
//    if (WeatherAppJS.Controls.Utilities.isCancellationError(errorMessage)) {
//        return WinJS.Promise.wrapError(errorMessage);
//    }
//    if (that._isFirstRendering) {
//        that._onRenderViewControlsError(errorMessage);
//        return that._loadErrorModule(errorMessage);
//    }
//    return WinJS.Promise.wrap(errorMessage);
});
},
cleanUp: function cleanUp() {
this._cleanupViewControl(this._viewControl);
WeatherAppJS.Controls.Utilities.setObjectPropertiesToNull(this);
}
})
});
})();