﻿/********************************************************
*                                                        *
*   © Microsoft. All rights reserved.                    *
*                                                        *
*********************************************************/
'use strict';
WinJS.Namespace.define("WeatherAppJS.Core", {
WeatherBasePage: WinJS.Class.define(function (state) {
this._state = state || {};
var that = this;
WeatherAppJS.SettingsManager.addEventListener('settingchanged', this.onSettingsChangeBase, false);
}, {
_promisesStack: [],
_timersStack: [],
_timeoutsStack: [],
_state: null,
_lastUpdatedTime: null,
isImmersive: false,
displayLastUpdatedTime: function displayLastUpdatedTime() {
if (this._lastUpdatedTime) {
var timeStampDiv = document.querySelector(".lastUpdatedTimeStamp");
if (timeStampDiv) {
var ts = new Date(this._lastUpdatedTime);
var formatTimeStamp = PlatformJS.Services.resourceLoader.getString("LastUpdatedDataTime").format(WeatherAppJS.Utilities.Formatting.getFormattedLastUpdateTime(ts.getFullYear(), ts.getMonth(), ts.getDate(), ts.getHours(), ts.getMinutes(), ts.getSeconds()));
timeStampDiv.innerText = formatTimeStamp;
timeStampDiv.style.display = "block";
}
}
},
/// <summary>
/// hide lastupdated time stamp whem zoom in /out on worldweather maps
/// </summary>
hideLastUpdatedTime: function hideLastUpdatedTime() {
var timeStampdiv = document.querySelector(".lastUpdatedTimeStamp");
if (timeStampdiv) {
timeStampdiv.style.display = "none";
}
},
setPlatformTheme: function setPlatformTheme(theme) {
CommonJS.setTheme(theme, true);
},
/// <summary>
/// This function cleans up all the timers registered by the page
/// </summary>
cleanPeriodicTimersStack: function cleanPeriodicTimersStack() {
while (this._timersStack.length > 0) {
var timer = this._timersStack.pop();
if (timer) {
clearInterval(timer);
timer = null;
}
}
},
/// <summary>
/// This function cleans up all the timers registered by the page
/// </summary>
cleanTimeoutsStack: function cleanTimeoutsStack() {
while (this._timeoutsStack.length > 0) {
var timer = this._timeoutsStack.pop();
if (timer) {
clearTimeout(timer);
timer = null;
}
}
},        
/// <summary>
/// method that handles the 'settingschange' event
/// </summary>
onSettingsChangeBase: function onSettingsChangeBase(e) {
var currentPage = PlatformJS.Navigation.currentIPage;
if (currentPage.onSettingsChange && currentPage.onSettingsChange instanceof Function) {
currentPage.onSettingsChange(e);
}
},
/// <summary>
/// method that handles page refresh
/// </summary>
onPageRefresh: function onPageRefresh(e) {
},
/// <summary>
/// This function is invoked on navigate away.
/// We clean up our promises, timers, registered callbacks which are common across all pages.
/// The page can choose to have a _disposePage method to clean up page-specific objects.
/// </summary>
disposePage: function disposePage() {
WeatherAppJS.Utilities.Common.cleanupPromiseStack(this._promisesStack);
this.cleanPeriodicTimersStack();
this.cleanTimeoutsStack();
WeatherAppJS.SettingsManager.removeEventListener('settingchanged', this.onSettingsChangeBase);
if (this._disposePage) {
this._disposePage();
}
},
/// <summary>
/// This function is invoked by the UX Framework when the page is being navigated away
/// </summary>
/// <param name="reason">
/// The reason why the page is being navigated away: "backNavigation" or "newNavigation"
/// </param>
onNavigateAway: function onNavigateAway(reason) {
this.disposePage();
},
/// <summary>
/// This function when a layoutchange event is triggered (example: switch to snap mode).
/// We call page dispose for all pages since we have a seperate snap page implementation.
/// </summary>
onLayoutChange: function onLayoutChange(layoutState) {
var that = this;
var viewState = Windows.UI.ViewManagement.ApplicationViewState;
if (layoutState === viewState.snapped) {
that.disposePage();
}
},
/// <summary>
/// This function is invoked by the UX Framework when the data binding is complete
/// </summary>
onBindingComplete: function onBindingComplete() {
},
/// <summary>
/// This function is invoked by the UX Framework when the page is being navigated away and when the app is suspending.
/// Return any data that you need to resurrect your UX when the page is recreated
/// Keep this state object as small as possible
/// </summary>
/// <returns>
/// This function must return a JSON object that can be serializable. Do not pass a Managed object
/// </returns>
getPageState: function getPageState() {
},
/// <summary>
/// call pageRefresh when page refresh button on bottom edgy is clicked
/// </summary>
onRefreshClick: function onRefreshClick(e) {
CommonJS.dismissAllEdgies();
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NumRefreshClicked");
Platform.Telemetry.FlightRecorder.logUserAction(Platform.Telemetry.LogLevel.normal, Platform.Telemetry.ActionContext.bottomEdgy, "Refresh", Platform.Telemetry.UserActionOperation.click, Platform.Telemetry.UserActionMethod.unknown, 0);
var geoLoc = WeatherAppJS.GeocodeCache.getLocation(this._locID);
if (geoLoc) {
Platform.Telemetry.FlightRecorder.logSearch(Platform.Telemetry.LogLevel.normal, geoLoc.getFullDisplayName(), "Bing", Platform.Telemetry.SearchMethod.view, false, -1, false, false, false, WeatherAppJS.Instrumentation.FormCodes.Default, geoLoc.latitude + ' - ' + geoLoc.longitude);
}
if (WeatherAppJS.Networking.internetAvailable) {
this.onPageRefresh(true);
}
else {
WeatherAppJS.Error.showMessageBar(PlatformJS.Services.resourceLoader.getString("NoInternetConnectionError"), false);
}
},
/// <summary>
/// method to call when change-to-Fahrenheit button on bottom edgy is clicked
/// </summary>
onChangeUnitToFClick: function onChangeUnitToFClick(e) {
CommonJS.dismissAllEdgies();
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NumBottomEdgyUnitChanged");
WeatherAppJS.SettingsManager.setDisplayUnitAsync('F', Platform.Telemetry.ActionContext.bottomEdgy).then();
},
/// <summary>
/// method to call when change-to-Celsius button on bottom edgy is clicked
/// </summary>
onChangeUnitToCClick: function onChangeUnitToCClick(e) {
CommonJS.dismissAllEdgies();
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NumBottomEdgyUnitChanged");
WeatherAppJS.SettingsManager.setDisplayUnitAsync('C', Platform.Telemetry.ActionContext.bottomEdgy).then();
},
/// <summary>
/// method to call when Change Home button on bottom edgy is clicked
/// </summary>
onChangeHomeClick: function onChangeHomeClick(event) {
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NumChangeHomeClick");
Platform.Telemetry.FlightRecorder.logUserAction(Platform.Telemetry.LogLevel.normal, Platform.Telemetry.ActionContext.bottomEdgy, "Change Home", Platform.Telemetry.UserActionOperation.click, Platform.Telemetry.UserActionMethod.unknown, 0);
var changeHomeFlyout = document.getElementById("changeHomeFlyout");
var changeHomeContainer = document.createElement("div");
WinJS.Utilities.addClass(changeHomeContainer, "changeHomeContainer");
var flyoutControl = changeHomeFlyout.winControl;
WeatherAppJS.Utilities.UI.dismissPlatformFlyout();
function changeHomeFlyoutClick(locationName) {
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NumUseExistingCity");
var eventAttr = { currentHome: WeatherAppJS.SettingsManager.getDefaultLocation().getFullDisplayName(), pickedHome: locationName.getFullDisplayName() };
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, "Change Home Flyout", "Fav/Recent Location", Platform.Telemetry.UserActionOperation.select, Platform.Telemetry.UserActionMethod.unknown, 0, JSON.stringify(eventAttr));
flyoutControl.hide();
CommonJS.dismissAllEdgies();
WeatherAppJS.SettingsManager.setDefaultLocationAsync(locationName, false).then(null,
function (e) {
if (e.errorcode === WeatherAppJS.ErrorCodes.SettingsManager.FAVORITES_MAX_LIMIT_REACHED) {
WeatherAppJS.Error.showMessageBar(PlatformJS.Services.resourceLoader.getString("MaxFavoritesLimitReached"), false);
}
else if (e.errorcode === WeatherAppJS.ErrorCodes.SettingsManager.LOCATION_PINNED_TO_START) {
WeatherAppJS.Error.showMessageBar(PlatformJS.Services.resourceLoader.getString("LocationPinnedToStart"), false);
}
});
}
function addNewLocClick () {
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NumChangeHomeAdd");
var eventAttr = { currentHome: WeatherAppJS.SettingsManager.getDefaultLocation().getFullDisplayName() };
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, "Change Home Flyout", "Add Location", Platform.Telemetry.UserActionOperation.click, Platform.Telemetry.UserActionMethod.unknown, 0, JSON.stringify(eventAttr));
flyoutControl.hide();
if (WeatherAppJS.SettingsManager.isFavoritesMaxLimitReached()) {
WeatherAppJS.Error.showMessageBar(PlatformJS.Services.resourceLoader.getString("MaxFavoritesLimitReached"), false);
}
else if (WeatherAppJS.Networking.internetAvailable === false) {
WeatherAppJS.Error.showMessageBar(PlatformJS.Services.resourceLoader.getString("NoInternetConnectionError"), false);
}
else {
WeatherAppJS.Utilities.UI.addNewHomeLoc();
}
}
var currentSelectedIndex = -1;
function changeHomeFlyout_onKeydown(e) {
var Key = WinJS.Utilities.Key;
var isUpKey = false;
switch (e.keyCode) {
case Key.upArrow:
isUpKey = true;
break;
case Key.downArrow:
break;
case Key.enter:
case Key.space:
// Prevent the default behaviour of "Space" and "Enter" keys from scrolling the items
e.preventDefault();
return;
default:
return;
}
e.preventDefault();
e.stopImmediatePropagation();
var itemList = e.currentTarget.querySelectorAll(".changeHomeSectionDiv");
var curIndex = currentSelectedIndex;
// user has used tab and arrow keys interchangeably
if (document.activeElement !== itemList[curIndex]) {
for (var i = 0; i < itemList.length; i++) {
if (document.activeElement === itemList[i]) {
curIndex = i;
break;
}
}
}
if (itemList.length === 0) {
return;
}
if (curIndex === -1) {
if (isUpKey) {
curIndex = itemList.length - 1;
}
else {
curIndex = 0;
}
}
else {
if (isUpKey) {
curIndex -= 1;
if (curIndex < 0) {
curIndex = itemList.length - 1;
}
}
else {
curIndex += 1;
if (curIndex >= itemList.length) {
curIndex = 0;
}
}
}
itemList[curIndex].focus();
currentSelectedIndex = curIndex;
};
function buildChangeHomeFlyout() {
var listLength = 0;
changeHomeFlyout.innerHTML = "";
changeHomeFlyout.onkeydown = changeHomeFlyout_onKeydown;
CommonJS.loadModule({ fragmentPath: "/html/templates.html", templateId: "changeHomeFlyoutTemplate" }, {}).then(
function (template) {
var resourceLoader = PlatformJS.Services.resourceLoader;
var section = template.querySelector(".changeHomeSection");
var sectionDiv = template.querySelector(".changeHomeSectionDiv");
function addToSection(locationObj, parentSection, /*@optional*/isHomeLoc) {
if (!locationObj) {
return;
}
var structure = sectionDiv.cloneNode(true);
structure.onkeyup = WeatherAppJS.Utilities.TabIndexManager.HandleKeyboardInvocation;
var displayName = WeatherAppJS.GeocodeCache.getFullDisplayName(locationObj.getId());
structure.querySelector(".changeHomeSectionText").innerText = displayName;
if (isHomeLoc) {
WinJS.Utilities.addClass(structure.querySelector(".changeHomeSectionIcon"), "changeHomeDefaultLocIcon");
var label = resourceLoader.getString("CurrentHomeNarratorString");
structure.querySelector(".changeHomeSectionText").setAttribute("aria-hidden", "true");
structure.setAttribute("aria-label", label.format(displayName));
}
(function (loc) {
structure.addEventListener("click",
function (e) {
changeHomeFlyoutClick(loc);
});
})(locationObj);
parentSection.appendChild(structure);
}
WeatherAppJS.SettingsManager.getFavoritesAsync().then(
function (favorites) {
var favSection = section.cloneNode(true);
favSection.querySelector(".changeHomeSectionText").innerText = resourceLoader.getString("Favorites");
addToSection(WeatherAppJS.SettingsManager.getDefaultLocation(), favSection, true);
if (favorites && favorites.length > 0) {
listLength += favorites.length;
for (var i = 0; i < favorites.length; i++) {
addToSection(favorites[i], favSection);
}
}
changeHomeContainer.appendChild(favSection);
});
WeatherAppJS.SettingsManager.getRecentLocationsAsync().then(
function (recentLocs) {
if (recentLocs && recentLocs.length > 0) {
listLength += recentLocs.length;
var recSection = section.cloneNode(true);
recSection.querySelector(".changeHomeSectionText").innerText = resourceLoader.getString("RecentSearches");
for (var i = 0; i < recentLocs.length; i++) {
addToSection(recentLocs[i], recSection);
}
changeHomeContainer.appendChild(recSection);
}
});
var addNewSection = sectionDiv.cloneNode(true);
addNewSection.tabIndex = "0";
addNewSection.setAttribute("role", "button");
addNewSection.onkeyup = WeatherAppJS.Utilities.TabIndexManager.HandleKeyboardInvocation;
addNewSection.querySelector(".changeHomeSectionText").innerText = resourceLoader.getString("AddNewLocationString");
WinJS.Utilities.addClass(addNewSection, "changeHomeAddNew");
addNewSection.addEventListener("click", addNewLocClick);
changeHomeFlyout.appendChild(changeHomeContainer);
changeHomeFlyout.appendChild(addNewSection);
});
return (listLength !== 0);
}
function changeHomeFlyoutCleanup() {
CommonJS.dismissAllEdgies();
changeHomeFlyout.innerHTML = "";
flyoutControl.removeEventListener("afterhide", changeHomeFlyoutCleanup);
}
if (buildChangeHomeFlyout()) {
flyoutControl.addEventListener("afterhide", changeHomeFlyoutCleanup);
var changeHomeButton = document.getElementById('changeHomeButton');
flyoutControl.show(changeHomeButton, "top");
changeHomeFlyout.focus();
}
else {
// No favorites + No recent searches scenario, launch add location flyout based on internet avalability
if (WeatherAppJS.Networking.internetAvailable === false) {
WeatherAppJS.Error.showMessageBar(PlatformJS.Services.resourceLoader.getString("NoInternetConnectionError"), false);
}
else {
WeatherAppJS.Utilities.UI.addNewHomeLoc();
}
}
},
/// <summary>
/// method to call when Current Location button on bottom edgy is clicked
/// </summary>
onCurrentLocationClick: function onCurrentLocationClick(event, currentPageLocation) {
var that = this;
CommonJS.dismissAllEdgies();
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NumCurrentLocationBottomEdgyClicked");
if (WeatherAppJS.Networking.internetAvailable) {
CommonJS.Progress.showProgress(CommonJS.Progress.headerProgressType);
var detectAndFetchLocationPromise = WeatherAppJS.LocationTracking.detectAndFetchCurrentLocationDataAsync();
that._promisesStack.push(detectAndFetchLocationPromise);
detectAndFetchLocationPromise.then(
function (locationObj) {
CommonJS.Progress.hideProgress(CommonJS.Progress.headerProgressType);
var eventAttr = { detectedLocation: locationObj.getFullDisplayName(), accuracy: locationObj.locDetectAccuracy };
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, Platform.Telemetry.ActionContext.bottomEdgy, "Detect Location", Platform.Telemetry.UserActionOperation.click, Platform.Telemetry.UserActionMethod.unknown, 0, JSON.stringify(eventAttr));
if (locationObj.locDetectAccuracy > PlatformJS.Services.appConfig.getInt32("DetectedLocationAccuracyThreshold")) {
WeatherAppJS.Utilities.UI.showAddLocationFlyout(locationObj);
} else {
if (locationObj && locationObj.getId() !== currentPageLocation) {
if (locationObj.getId() === WeatherAppJS.SettingsManager.getDefaultLocationId()) {
PlatformJS.Navigation.navigateToChannel('Home');
}
else {
PlatformJS.Navigation.navigateToChannel('Home', { locID: locationObj.getId(), geocodeLocation: locationObj });
}
}
}
},
function (err) {
CommonJS.Progress.hideProgress(CommonJS.Progress.headerProgressType);
if (err.errorcode === WeatherAppJS.ErrorCodes.LocationTracking.LOCATION_TRACKING_DISABLED) {
WeatherAppJS.Error.showMessageBar(PlatformJS.Services.resourceLoader.getString("LocationDisabledError"), false);
}
else if (err.errorcode === WeatherAppJS.ErrorCodes.LocationTracking.LOCATION_DETECTION_FAILED) {
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NumCurrentLocationBottomEdgyFailed");
Platform.Telemetry.FlightRecorder.logContentError(Platform.Telemetry.LogLevel.high, "", "Current Location detection failed", err.errorcode, "", "");
WeatherAppJS.Error.showMessageBar(PlatformJS.Services.resourceLoader.getString("CurrentLocationFetchErrorMsg"), false);
}
else if (err.errorcode === WeatherAppJS.ErrorCodes.LocationTracking.NO_WEATHER_DATA) {
Platform.Telemetry.FlightRecorder.logContentError(Platform.Telemetry.LogLevel.high, "", "No weather data available when location detected", err.errorcode, "", "");
WeatherAppJS.Error.showMessageBar(PlatformJS.Services.resourceLoader.getString("CurrentLocDataNotAvailable"), false);
}
});
}
else {
WeatherAppJS.Error.showMessageBar(PlatformJS.Services.resourceLoader.getString("NoInternetConnectionError"), false);
}
},
})
});
WinJS.Namespace.define("WeatherAppJS.Core", {
WeatherPanoramaBasePage: WinJS.Class.derive(WeatherAppJS.Core.WeatherBasePage,
function (state) {
WeatherAppJS.Core.WeatherBasePage.call(this, state);
this._panoramaControl = document.getElementById("mainPanorama").winControl;
if (state && state.panoramaState) {
this._panoramaControl.panoramaState = state.panoramaState;
}
var that = this;
that._data = new PlatformJS.Utilities.Binding.List();
that._clusterDefs = [];
}, {
_panoControl: null,
_panoState: null,
_data: null,
/// Handle to cluster and module definitions
_clusterDefs: null,
/// <summary>
/// This function is invoked by the device has been roated
/// </summary>
/// <param name="event">
/// orientation change event
/// </param>
onOrientationChange: function onOrientationChange(event) {
msWriteProfilerMark("WeatherApp:OrientationChange:s");
if (this._panoramaControl && this._panoramaControl.onOrientationChange) {
this._panoramaControl.onOrientationChange(event);
}
msWriteProfilerMark("WeatherApp:OrientationChange:e");
},
/// <summary>
/// This function is invoked when 'make everything on your screen bigger' setting is changed.
/// </summary>
/// <param name="event">
/// dpi change event
/// </param>
/// <returns>
/// Nothing
/// </returns>
onDpiChange: function onDpiChange(event) {
var panoControl = this._panoramaControl;
if (panoControl && panoControl.onDpiChange) {
panoControl.onDpiChange(event);
}
},
/// <summary>
/// dispose event handler for the page.
/// panorama event handlers need to be disposed off explicitly.
/// </summary>
dispose: function dispose() {
this._clusterDefs = null;
this._data = null;
if (this._panoramaControl && this._panoramaControl.dispose) {
this._panoramaControl.dispose();
this._panoramaControl = null;
}
if (this.disposePage) {
this.disposePage();
}
},
displayLastUpdatedTime: function displayLastUpdatedTime() {
if (this._lastUpdatedTime && this._panoramaControl) {
var ts = new Date(this._lastUpdatedTime);
var formatTimeStamp = PlatformJS.Services.resourceLoader.getString("LastUpdatedDataTime").format(WeatherAppJS.Utilities.Formatting.getFormattedLastUpdateTime(ts.getFullYear(), ts.getMonth(), ts.getDate(), ts.getHours(), ts.getMinutes(), ts.getSeconds()));
this._panoramaControl.watermark = formatTimeStamp;
}
},
pushIntoClusterDefs: function pushIntoClusterDefs(clusterDef) {
if (!this._clusterDefs) {
this._clusterDefs = [];
}
// Provide clusterKey and currentIPage to contentOptions to make them available to ClusterControl
// Used to remove cluster for a pano
if (clusterDef && clusterDef.clusterContent && clusterDef.clusterContent.contentOptions) {
clusterDef.clusterContent.contentOptions.clusterKey = clusterDef.clusterKey;
clusterDef.clusterContent.contentOptions.currentIPage = this;
}
this._clusterDefs.push(clusterDef);
},
})
});