﻿/********************************************************
*                                                        *
*   © Microsoft. All rights reserved.                    *
*                                                        *
*********************************************************/
(function () {
'use strict';
var notifications = Windows.UI.Notifications;
WinJS.Namespace.define("WeatherAppJS.App", {
_isInitialized: false,
_isPullNotificationEnabled: false,
// The tileID can be a maximum of 64 chars length
// if it's more than that then all tile related operations fail to work.
// so, before using tileID, we trim it by 64chars. There is very less chance 
// of getting a duplicate tileID for two different locations.
_maxTileIDLength: 64,
/// <summary>
/// Alerts are currently only available for US. Check whether the location is in any of US states.
/// </summary>
isBadgeApplicableForLocation: function isBadgeApplicableForLocation(locationObj) {
return WeatherAppJS.GeocodeCache.isAlertsSupportedLocation(locationObj.getId());
},
getTileIDForLocID: function getTileIDForLocID(locID) {
// The WinRT API for Secondary Tile throws an exception if there is a comma or a space in the tile ID.
// Replacing it with a hyphen.
var tileID;
tileID = locID.replace(/\,/g, "-");
tileID = tileID.replace(/ /g, "-");
tileID = tileID.replace(/\//g, "-");
return tileID.substring(0, this._maxTileIDLength);
},
getGeoLocFromTileArguments: function getGeoLocFromTileArguments(args) {
var geocodeCache = WeatherAppJS.GeocodeCache;
var geoLoc;
try {
var loc = JSON.parse(args);
geoLoc = WeatherAppJS.GeocodeLocation.createGeocodeLocation(loc);
// Set the tile market
if (loc.tileMarket) {
geoLoc.tileMarket = loc.tileMarket;
}
} catch (err) {
// For backward compatibility with already pinned tiles, assume tile argument as locID. 
geoLoc = geocodeCache.getLocation(args);
if (!geoLoc) {
geoLoc = WeatherAppJS.GeocodeLocation.createGeocodeLocation({ fullName: args, fetchByLatLong: false });
}
}
if (geoLoc && geoLoc.getId()) {
// Add it to geocode cache if it's not already there.
if (!geocodeCache.getLocation(geoLoc.getId())) {
geocodeCache.addLocation(geoLoc);
}
}
return geoLoc;
},
/// <summary>
/// The function returns the city name for the input location, if available. Otherwise, returns the full name.
/// </summary>
getShortNameForLocation: function getShortNameForLocation(locationObj) {
return (locationObj.city && locationObj.city !== "") ? locationObj.city : locationObj.fullName;
},
/// <summary>
/// The function returns the full display name for the input location. Otherwise, returns the full name.
/// </summary>
getDisplayNameForLocation: function getDisplayNameForLocation(locationObj) {
return locationObj.getFullDisplayName();
},
isLocationPinned: function isLocationPinned(locID) {
var isPinned;
var tileID = this.getTileIDForLocID(locID);
try {
isPinned = Windows.UI.StartScreen.SecondaryTile.exists(tileID);
} catch (e) {
isPinned = false;
}
return isPinned;
},
pinLocationAsync: function pinLocationAsync(element, locID) {
//
// Verify if we are still under the maximum pinned locations limit.
//
var that = this;
return new WinJS.Promise(
function (c, e) {
Windows.UI.StartScreen.SecondaryTile.findAllAsync().then(
function (tiles) {
if (tiles) {
var MAX_PINNED_LOCATIONS = PlatformJS.Services.appConfig.getInt32("MaxRecentLocationsLimit");
if (tiles.size < MAX_PINNED_LOCATIONS) {
//
// We are under the limit. Go ahead and pin the location.
//
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NumLocationsPinned");
var eventAttr = { location: WeatherAppJS.GeocodeCache.getLocation(locID).getFullDisplayName()};
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, Platform.Telemetry.ActionContext.bottomEdgy, "Pin to Mogo", Platform.Telemetry.UserActionOperation.click, Platform.Telemetry.UserActionMethod.unknown, 0, JSON.stringify(eventAttr));
var tileID = that.getTileIDForLocID(locID);
var locationObj = WeatherAppJS.GeocodeCache.getLocation(locID);
locationObj.tileMarket = Platform.Globalization.Marketization.getMarketLocation(); // Persist market info with the tile
var geocodeInfo = JSON.stringify(locationObj);
var uriLogo = new Windows.Foundation.Uri("ms-appx:///images/logo.png");
var uriWideLogo = new Windows.Foundation.Uri("ms-appx:///images/widelogo.png");
var uriSmallLogo = new Windows.Foundation.Uri("ms-appx:///images/smalllogo.png");
var tile = new Windows.UI.StartScreen.SecondaryTile(tileID,
that.getShortNameForLocation(locationObj),
that.getDisplayNameForLocation(locationObj),
geocodeInfo,
Windows.UI.StartScreen.TileOptions.showNameOnLogo + Windows.UI.StartScreen.TileOptions.showNameOnWideLogo + Windows.UI.StartScreen.TileOptions.copyOnDeployment,
uriLogo,
uriWideLogo);
tile.smallLogo = uriSmallLogo;
var selectionRect = element.getBoundingClientRect();
tile.requestCreateForSelectionAsync({ x: selectionRect.left, y: selectionRect.top, width: selectionRect.width, height: selectionRect.height }).then(
function (isCreated) {
if (isCreated) {
// The tile was successfully created. Start periodic updates.
that.startPeriodicUpdateForSecondaryTile(locationObj, tileID);
}
c();
},
function (err) {
e(err);
});
} else {
e({ errorcode: WeatherAppJS.ErrorCodes.SettingsManager.PINNED_MAX_LIMIT_REACHED, errormessage: 'Max ' + MAX_PINNED_LOCATIONS + ' pinned locations limit reached.' });
}
}
},
function (err) {
e(err);
});
});
},
unpinLocationAsync: function unpinLocationAsync(element, locID) {
var that = this;
return new WinJS.Promise(
function (c, e) {
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NumLocationsUnpinned");
var eventAttr = { location: WeatherAppJS.GeocodeCache.getLocation(locID).getFullDisplayName()};
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, Platform.Telemetry.ActionContext.bottomEdgy, "Unpin from Mogo", Platform.Telemetry.UserActionOperation.click, Platform.Telemetry.UserActionMethod.unknown, 0, JSON.stringify(eventAttr));
var tileID = that.getTileIDForLocID(locID);
var tile = Windows.UI.StartScreen.SecondaryTile(tileID);
var selectionRect = element.getBoundingClientRect();
tile.requestDeleteForSelectionAsync({ x: selectionRect.left, y: selectionRect.top, width: selectionRect.width, height: selectionRect.height }).then(
function (isDeleted) {
c();
},
function (err) {
e(err);
});
});
},
getNotificationServerUriForLocation: function getNotificationServerUriForLocation(appserverhost, appId, locationObj, mkt, region, pullNotification, scenario, isTianQiLocation) {
var serverUri;
var locationName;
var configName;
var middletierhost = PlatformJS.Services.appConfig.getString("MiddleTierHostTile");
var trimLatLong = WeatherAppJS.Utilities.Common.trimLatLong;
if (isTianQiLocation) {
configName = "ServerUriForTianQi" + scenario;
serverUri = pullNotification.getString(configName);
locationName = WeatherAppJS.Utilities.TianQi.getEncodedLocationName(locationObj.city);
serverUri = serverUri.replace("\{city\}", locationName);
} else {
if (locationObj.isSkiLocation) { 
configName = "ServerUriForSki" + scenario;
serverUri = pullNotification.getString(configName);
serverUri = serverUri.replace("\{id\}", locationObj.id);
}
else if (locationObj.fetchByLatLong) {  
configName = "ServerUriFor" + scenario + "ByLatLong";
serverUri = pullNotification.getString(configName);
locationName = this.getShortNameForLocation(locationObj);
} else {
configName = "ServerUriFor" + scenario + "ByLocationName";
serverUri = pullNotification.getString(configName);
locationName = locationObj.fullName;
}
serverUri = serverUri.replace("\{unit\}", WeatherAppJS.SettingsManager.getDisplayUnit());
serverUri = serverUri.replace("\{mkt\}", mkt);
serverUri = serverUri.replace("\{region\}", region);
serverUri = serverUri.replace("\{lang\}", Platform.Globalization.Marketization.getQualifiedLanguageString());
}
serverUri = serverUri.replace("\{appserverhost\}", appserverhost);
serverUri = serverUri.replace("\{middletierhosttile\}", middletierhost);
serverUri = serverUri.replace("\{latitude\}", trimLatLong(locationObj.latitude));
serverUri = serverUri.replace("\{longitude\}", trimLatLong(locationObj.longitude));
serverUri = serverUri.replace("\{appid\}", appId);
serverUri = serverUri.replace("\{locationName\}", locationName);
return (isTianQiLocation) ? serverUri : encodeURI(serverUri);
},
getRecurrence: function getRecurrence(pullNotification, scenario, isTianQiLocation) {
var configName;
if (isTianQiLocation) {
configName = "UpdateRecurrenceForTianQi" + scenario;
} else {
configName = "UpdateRecurrenceFor" + scenario;
}
var updateRecurrence = pullNotification.getString(configName);
var recurrence = notifications.PeriodicUpdateRecurrence[updateRecurrence];
return recurrence;
},
startPeriodicUpdateForAppTile: function startPeriodicUpdateForAppTile() {
if (this._isPullNotificationEnabled) {
try {
var defaultLocation = WeatherAppJS.SettingsManager.getDefaultLocation();
if (defaultLocation) {
var appConfig = PlatformJS.Services.appConfig;
var appserverhost = appConfig.getString("AppServerHost");
var appId = appConfig.getString("AppID");
var mkt = appConfig.getString("AppServerMarket");
var region = appConfig.getString("AppServerRegion");
var pullNotification = appConfig.getDictionary("PullNotification");
var isTianQiLocation = Platform.Utilities.Globalization.isFeatureEnabled("UseTianQi") && WeatherAppJS.GeocodeCache.isTianQiSupportedLocation(defaultLocation.getId());
var serverUriForTile = this.getNotificationServerUriForLocation(appserverhost, appId, defaultLocation, mkt, region, pullNotification, "Tile", isTianQiLocation);
var recurrenceForTile = this.getRecurrence(pullNotification, "Tile", isTianQiLocation);
notifications.TileUpdateManager.createTileUpdaterForApplication().startPeriodicUpdate(new Windows.Foundation.Uri(serverUriForTile), recurrenceForTile);
// Ensure we disable the cycling after we setup the default app tile.
notifications.TileUpdateManager.createTileUpdaterForApplication().enableNotificationQueue(false);
if (this.isBadgeApplicableForLocation(defaultLocation)) {
var serverUriForBadge = this.getNotificationServerUriForLocation(appserverhost, appId, defaultLocation, mkt, region, pullNotification, "Badge", isTianQiLocation);
var recurrenceForBadge = this.getRecurrence(pullNotification, "Badge", isTianQiLocation);
notifications.BadgeUpdateManager.createBadgeUpdaterForApplication().startPeriodicUpdate(new Windows.Foundation.Uri(serverUriForBadge), recurrenceForBadge);
} else {
this.clearBadgeUpdates();
}
}
} catch (ex) {
}
}
},
startPeriodicUpdateForSecondaryTile: function startPeriodicUpdateForSecondaryTile(locationObj, tileID) {
if (this._isPullNotificationEnabled) {
try {
var appConfig = PlatformJS.Services.appConfig;
var appserverhost = appConfig.getString("AppServerHost");
var appId = appConfig.getString("AppID");
var mkt = appConfig.getString("AppServerMarket");
var region = appConfig.getString("AppServerRegion");
var pullNotification = appConfig.getDictionary("PullNotification");
var isTianQiLocation = Platform.Utilities.Globalization.isFeatureEnabled("UseTianQi") && locationObj.isTianQiSupportedLocation();
var serverUriForTile = this.getNotificationServerUriForLocation(appserverhost, appId, locationObj, mkt, region, pullNotification, "PinnedTile", isTianQiLocation);
var recurrenceForTile = this.getRecurrence(pullNotification, "Tile", isTianQiLocation);
notifications.TileUpdateManager.createTileUpdaterForSecondaryTile(tileID).startPeriodicUpdate(new Windows.Foundation.Uri(serverUriForTile), recurrenceForTile);
if (this.isBadgeApplicableForLocation(locationObj)) {
var serverUriForBadge = this.getNotificationServerUriForLocation(appserverhost, appId, locationObj, mkt, region, pullNotification, "Badge", isTianQiLocation);
var recurrenceForBadge = this.getRecurrence(pullNotification, "Badge", isTianQiLocation);
notifications.BadgeUpdateManager.createBadgeUpdaterForSecondaryTile(tileID).startPeriodicUpdate(new Windows.Foundation.Uri(serverUriForBadge), recurrenceForBadge);
} else {
this.clearBadgeUpdates(tileID);
}
} catch (ex) {
}
}
},
/// <summary>
/// This function updates the app tile according to default location and unit
/// </summary>
updateAppTile: function updateAppTile() {
this.startPeriodicUpdateForAppTile();
},
/// <summary>
/// This function updates all pinned tiles according to their location and unit
/// </summary>
updateAllTiles: function updateAllTiles() {
this.updateAppTile();
var that = this;
Windows.UI.StartScreen.SecondaryTile.findAllAsync().then(function (tiles) {
if (tiles) {
tiles.forEach(function (tile) {
if (that.isTileSupportedInCurrentMarket(tile.arguments)) {
var geoLoc = that.getGeoLocFromTileArguments(tile.arguments);
that.startPeriodicUpdateForSecondaryTile(geoLoc, tile.tileId);
} else {
// Clear the tile notifications if it is not supported in current market
that.clearTileUpdates(tile.tileId);
that.clearBadgeUpdates(tile.tileId);
}
});
}
});
},
/// <summary>
/// This function clears periodic notification for all tiles (primarily in blocked market scenario)
/// </summary>
clearAllTiles: function clearAllTiles() {
// clear app tile
this.clearTileUpdates();
this.clearBadgeUpdates();
var that = this;
Windows.UI.StartScreen.SecondaryTile.findAllAsync().then(function (tiles) {
if (tiles) {
tiles.forEach(function (tile) {
that.clearTileUpdates(tile.tileId);
that.clearBadgeUpdates(tile.tileId);
});
}
});
},
/// <summary>
/// This function clears periodic notifications for the provided tile
/// </summary>
/// <param name="tileID">
/// The tile id for the secondary tile. if empty, then default tile is to be cleared.
/// </param>
clearTileUpdates: function clearTileUpdates(/*@optional*/tileID) {
var tileUpdater = null;
// clear secondary tile.
if (tileID) {
tileUpdater = notifications.TileUpdateManager.createTileUpdaterForSecondaryTile(tileID);
} else {
// clear default tile.
tileUpdater = notifications.TileUpdateManager.createTileUpdaterForApplication();
}
if (tileUpdater) {
tileUpdater.clear();
tileUpdater.stopPeriodicUpdate();
}
},
/// <summary>
/// This function clears periodic badge notifications for the provided tile
/// </summary>
/// <param name="tileID">
/// The tile id for the secondary tile. if empty, then default tile is to be cleared.
/// </param>
clearBadgeUpdates: function clearBadgeUpdates(/*@optional*/tileID) {
var badgeUpdater = null;
// clear secondary tile.
if (tileID) {
badgeUpdater = notifications.BadgeUpdateManager.createBadgeUpdaterForSecondaryTile(tileID);
} else {
// clear default tile.
badgeUpdater = notifications.BadgeUpdateManager.createBadgeUpdaterForApplication();
}
if (badgeUpdater) {
badgeUpdater.clear();
badgeUpdater.stopPeriodicUpdate();
}
},
/// <summary>
/// This function checks if the tile is supported in the current market or not.
/// </summary>
isTileSupportedInCurrentMarket: function isTileSupportedInCurrentMarket(tileArguments) {
var geoLoc = this.getGeoLocFromTileArguments(tileArguments);
var marketLocation = Platform.Globalization.Marketization.getMarketLocation();
var useTianQi = Platform.Utilities.Globalization.isFeatureEnabled("UseTianQi");
if (geoLoc) {
// Check if tileMarket is not same as the current market
if (geoLoc.tileMarket !== marketLocation) {
// For TianQi, disable all tianQisupported locations from other markets
if (useTianQi && geoLoc.isTianQiSupportedLocation() === true) {
return false;
}
// For other markets,  disable all tianQi supported locations pinned in TianQi market
if (!useTianQi && geoLoc.isTianQiSupportedLocation() === true && geoLoc.tileMarket === "CN") {
return false;
}
}
return true;
}
},
/// <summary>
/// This function is invoked when the default location is changed
/// </summary>
onDefaultLocationUpdate: function onDefaultLocationUpdate() {
// We should clear the tile before starting new updates to make sure that previous tile data is overwritten.
// If the new tile doesn't have any caption, tile won't get updates and if didnt clear the previous tile data
// it'd keep showing the old data.
WeatherAppJS.App.clearTileUpdates();
WeatherAppJS.App.updateAppTile();
},
/// <summary>
/// This function is invoked when the app settings are changed
/// </summary>
onSettingsChange: function onSettingsChange(e) {
if (e.detail.setting === "unitChange") {
WeatherAppJS.App.updateAllTiles();
}
},
/// <summary>
/// Reinitialize app tile manager and clear out the cached config values
/// </summary>
reinitialize: function reinitialize() {
this._isInitialized = false;
this.initAppTileManager();
},
/// <summary>
/// This function initializes app tile manager
/// </summary>
initAppTileManager: function initAppTileManager() {
if (!this._isInitialized) {
var appConfig = PlatformJS.Services.appConfig;
var settingsManager = WeatherAppJS.SettingsManager;
/// Start listening on the default location change event
settingsManager.addEventListener('defaultlocationupdated', this.onDefaultLocationUpdate, false);
/// Start listening on the settings change event
settingsManager.addEventListener('settingchanged', this.onSettingsChange, false);
var pullNotification = appConfig.getDictionary("PullNotification");
this._isPullNotificationEnabled = pullNotification.getBool("Enabled");
if (!this._isPullNotificationEnabled) {
this.clearAllTiles();
}
if (settingsManager.isFRE() === false) {
/// Update app and pinned tiles in case the user changed the default location
/// on another machine or the setting was updated via roaming profile.
this.updateAllTiles();
/// Detect if we are launching in a blocked market.
var that = this;
var qsForPing = new PlatformJS.DataService.QueryService("AppServerPing");
var urlParams = PlatformJS.Collections.createStringDictionary();
urlParams.insert("appserverhost", appConfig.getString("AppServerHost"));
urlParams.insert("mkt", appConfig.getString("AppServerMarket"));
urlParams.insert("region", appConfig.getString("AppServerRegion"));
var options = new Platform.DataServices.QueryServiceOptions();
options.useCompatDataModel = true;
options.bypassCache = true;
options.enableAutoRefresh = false;
qsForPing.downloadDataAsync(urlParams, null, null, options).then(
function (response) {
WeatherAppJS.LocationsManager.isMarketBlockedByIP = false;
},
function (e) {
if (PlatformJS.Utilities.isPromiseCanceled(e)) {
return;
}
if (WeatherAppJS.Error.isMarketBlockedError(e)) {
WeatherAppJS.LocationsManager.isMarketBlockedByIP = true;
that.clearAllTiles();
}
});
}
this._isInitialized = true;
}
},
});
})();