﻿/********************************************************
*                                                        *
*   © Microsoft. All rights reserved.                    *
*                                                        *
*********************************************************/
(function () {
'use strict';
WinJS.Namespace.define("WeatherAppJS", {
GeocodeLocation: WinJS.Class.define(
function (locationData) {
if (this._isValidLocation(locationData)) {
// Required fields
this.fullName = locationData.fullName;
this.latitude = locationData.latitude;
this.longitude = locationData.longitude;
this.id = locationData.id ? locationData.id : null;
this.locType = locationData.locType ? locationData.locType : WeatherAppJS.GeocodeLocation.locationType.cityLocation;
// Optional fields
var optionalFields = ["city", "country", "state", "type", "name", "zipcode", "matchCode", "confidence", "source", "fetchByLatLong", "score", "isoCode"];
// Set optional fields from location object
for (var propertyName in locationData) {
if (optionalFields.indexOf(propertyName) !== -1 && !(locationData[propertyName] instanceof Function)) {
this[propertyName] = locationData[propertyName];
}
}
// If isoCode is not already set, fallback to setting ISO country code by matching the country name.
if (!this.isoCode && this.country) {
var countryCode = WeatherAppJS.GeocodeLocation.getCountryRegionCode(this.country);
if (countryCode) {
this.isoCode = countryCode;
}
}
} else {
throw "Invalid location data"; 
}
},
{
_isValidLocation: function _isValidLocation(locationObj) {
// Validate latitude, longitude and location name
var isValid = false;
if (locationObj && locationObj.fullName) {
if (locationObj.fetchByLatLong === false) {
/// Fetched by name, no need to test latitude and longitude info.
isValid = true;
} else {
isValid = (locationObj.latitude && locationObj.latitude <= 90 && locationObj.latitude >= -90
&& locationObj.longitude && locationObj.longitude <= 180 && locationObj.longitude >= -180);
}
if (locationObj.locType === WeatherAppJS.GeocodeLocation.locationType.skiLocation && !locationObj.id) {
isValid = false;
}
}
return isValid;
},
isfetchByName: function isfetchByName() {
return (this.fetchByLatLong) ? false : true;
},
isfetchByLatLong: function isfetchByLatLong() {
return (this.fetchByLatLong) ? true : false;
},
setFetchByLatLong: function setFetchByLatLong(value) {
this.fetchByLatLong = (value) ? true : false;
},
getId: function getId() {
return (this.id) ? this.id : this.fullName;
},
getFullName: function getFullName() {
return this.fullName;
},
getLocType: function getLocType() {
return this.locType;
},
getLocTypeString: function getLocTypeString() {
if (this.locType === WeatherAppJS.GeocodeLocation.locationType.skiLocation) {
return "ski";
} else {
return "location";
}
},
/// <summary>
/// Check if the location is supported by the TianQi answer. If isoCode is not valid returns undefined.
/// </summary>
isTianQiSupportedLocation: function isTianQiSupportedLocation() {
var tianQiSupportedIsoCodes = ["CN", "MO", "HK", "TW"];
if (this.isoCode) {
return (tianQiSupportedIsoCodes.indexOf(this.isoCode) !== -1 );
}
},
/// <summary>
/// Checks if we support weather alerts for this location. If isoCode is not valid returns undefined.
/// </summary>
isAlertsSupportedLocation: function isAlertsSupportedLocation() {
/// Currently we only support weather alerts in US and its dependent territories.
var alertSupportedIsoCodes = ["US", "AS", "GU", "MP", "PR", "VI", "UM"];
if (this.isoCode) {
return (alertSupportedIsoCodes.indexOf(this.isoCode) !== -1);
}
},
/// <summary>
/// Checks if the location is local to user current market by matching the isoCode. If isoCode field is not valid fallback to country field.
/// If both isoCode and country fields are not valid return undefined.
/// </summary>
isCurrentMarketLocation: function isCurrentMarketLocation() {
if (this.isoCode) {
return (this.isoCode === Platform.Globalization.Marketization.getMarketLocation());
} else if (this.country) {
return WeatherAppJS.Utilities.Common.isCurrentRegion(this.country);
}
},
/// <summary>
/// Check if the location is a Ski Resort location
/// </summary>
isSkiLocation: {
get: function get() {
return this.locType === WeatherAppJS.GeocodeLocation.locationType.skiLocation;
}
},
getFullDisplayName: function getFullDisplayName(bypassCountryCheck, ignoreNameFormatReversal) {
var loc = this;
// Check if city or country field is missing for this location.
if (!loc.city || !loc.country) {
// We cannot format the location name, return fullname as it is
return loc.fullName;
} else {
var isReversed = (ignoreNameFormatReversal) ? false : PlatformJS.Services.appConfig.getBool("LocationNameFormatReversed");
var shortFormat = PlatformJS.Services.resourceLoader.getString("LocationNameShortFormat");
var longFormat = PlatformJS.Services.resourceLoader.getString("LocationNameLongFormat");
// Check if state field is missing, then use short format. OR if state and city/country names are same For eg: "Delhi, Delhi, India", "Kathmandu, Nepal, Nepal"
if (!loc.state || loc.state === loc.city || loc.state === loc.country) {
return (isReversed) ? shortFormat.format(loc.country, loc.city) : shortFormat.format(loc.city, loc.country);
}
// Check if the market location is same as user's country
if (!bypassCountryCheck && loc.isCurrentMarketLocation()) {
return (isReversed) ? shortFormat.format(loc.state, loc.city) : shortFormat.format(loc.city, loc.state);
}
return (isReversed) ? longFormat.format(loc.country, loc.state, loc.city) : longFormat.format(loc.city, loc.state, loc.country);
}
},
getShortDisplayName: function getShortDisplayName() {
var loc = this;
// Check if city or country field is missing for this location.
if (!loc.city || !loc.country) {
// We cannot format the location name, return fullname as it is
return loc.fullName;
} else {
var isReversed = PlatformJS.Services.appConfig.getBool("LocationNameFormatReversed");
var shortFormat = PlatformJS.Services.resourceLoader.getString("LocationNameShortFormat");
if (!loc.state || loc.state === loc.city || loc.state === loc.country) {
// No state field, return city and country.
// OR If state is same as city/country, return city and country even if country is same as the user region
return (isReversed) ? shortFormat.format(loc.country, loc.city) : shortFormat.format(loc.city, loc.country);
}
if (loc.isCurrentMarketLocation()) {
return (isReversed) ? shortFormat.format(loc.state, loc.city) : shortFormat.format(loc.city, loc.state);
} else {
return (isReversed) ? shortFormat.format(loc.country, loc.city) : shortFormat.format(loc.city, loc.country);
}
}
},
getTwoLineDisplayName: function getTwoLineDisplayName() {
var loc = this;
// Check if city or country field is missing for this location.
if (!loc.city || !loc.country) {
// We cannot format the location name, return fullname as it is
return [loc.fullName, ""];
} else {
var isReversed = PlatformJS.Services.appConfig.getBool("LocationNameFormatReversed");
var shortFormat = PlatformJS.Services.resourceLoader.getString("LocationNameShortFormat");
if (!loc.state || loc.state === loc.city || loc.state === loc.country) {
return (isReversed) ? [loc.country, loc.city] : [loc.city, loc.country];
}
return (isReversed) ? [shortFormat.format(loc.country, loc.state), loc.city] : [loc.city, shortFormat.format(loc.state, loc.country)];
}
},
getThreeLineDisplayName: function getThreeLineDisplayName() {
var loc = this;
// Check if city or country field is missing for this location.
if (!loc.city || !loc.country) {
// We cannot format the location name, return fullname as it is
return [loc.fullName, "", ""];
} else {
var isReversed = PlatformJS.Services.appConfig.getBool("LocationNameFormatReversed");
if (!loc.state || loc.state === loc.city || loc.state === loc.country) {
return (isReversed) ? [loc.country, "", loc.city] : [loc.city, "", loc.country];
}
return (isReversed) ? [loc.country, loc.state, loc.city] : [loc.city, loc.state, loc.country];
}
},
// <summary>
// Return the name which should be used to query VE for this location.
// </summary>
getQueryName: function getQueryName() {
var loc = this;
if (!loc.city || !loc.country) {
return loc.fullName;
} else {
return loc.city + (loc.state ? (", " + loc.state) : "") + ", " + loc.country;
}
}
},
{
_countryCodesList: null,
locationType: {
cityLocation: 1,
skiLocation: 2,
},
locationTypeString: {
cityLocationString: "location",
skiLocationString: "ski",
},
getLocTypeFromString: function getLocTypeFromString(entityTypeName) {
if (entityTypeName && (entityTypeName.toLowerCase() === this.locationTypeString.skiLocationString)) {
return WeatherAppJS.GeocodeLocation.locationType.skiLocation;
} else {
return WeatherAppJS.GeocodeLocation.locationType.cityLocation;
}
},
createGeocodeLocation: function createGeocodeLocation(locationData) {
try {
return new WeatherAppJS.GeocodeLocation(locationData);
} catch (exp) {
return null;
}
},
/// <summary>
/// Try to find the country region code given the country name by looking up a local database. Returns the isoCode when the countryRegion matches, else returns undefined.
/// Due to async loading of the country codes file, this function may return undefined when the app is still initializing.
/// </summary>
getCountryRegionCode: function getCountryRegionCode(countryRegion) {
if (countryRegion && WeatherAppJS.GeocodeLocation._countryCodesList) {
return WeatherAppJS.GeocodeLocation._countryCodesList[countryRegion];
}
},
init: function init() {
// Initialize country region codes dictionary.
var countryRegionCodesFile = "resources/countryRegionCodes.json"; // Appconfig has not been init'ed till now, therefore can't move filename to config.
WinJS.xhr({ url: countryRegionCodesFile }).then(
function (request) {
try {
WeatherAppJS.GeocodeLocation._countryCodesList = JSON.parse(request.responseText);
} catch (ex) {
WeatherAppJS.GeocodeLocation._countryCodesList = {};
}
},
function (ex) {
WeatherAppJS.GeocodeLocation._countryCodesList = {};
});
},
})
});
WeatherAppJS.GeocodeLocation.init();
})();