﻿/********************************************************
*                                                        *
*   © Microsoft. All rights reserved.                    *
*                                                        *
*********************************************************/
(function () {
"use strict";
WinJS.Namespace.define("WeatherAppJS.Utilities.UI.MapsFlyout",
{
// placeholder for image elements to clear off the events on them
imageHolder: null,
isShown: false,
thumbnailUrl: "",
loadMapsFlyout: function loadMapsFlyout(mapdata, mapCategory, sourceTile) {
var resourceLoader = PlatformJS.Services.resourceLoader;
if (!mapdata || !mapdata.mapList || !mapdata.mapList[0] || !mapdata.mapList[0].ImageUrl) {
return;
}
WeatherAppJS.Utilities.UI.MapsFlyout.imageHolder = document.createElement('div');
var flyoutAnchor = document.getElementById("mapsFlyoutAnchor");
var flyoutNode = flyoutAnchor.querySelector("#mapsFlyout");
var p = WinJS.Promise.wrap(true);
if (!flyoutNode) {
p = CommonJS.loadModule({ fragmentPath: "/html/mapsTemplate.html", templateId: "mapsFlyoutTemplate" }, {}, null, null, null).then(function (template) {
flyoutNode = template.querySelector("#mapsFlyout");
flyoutAnchor.appendChild(template);
});
}
p.then(function () {
var weatherFormatting = WeatherAppJS.Utilities.Formatting;
var shortDateTimeFormat = PlatformJS.Services.resourceLoader.getString('ShortDateTimeFormatString');
var that = WeatherAppJS.Utilities.UI.MapsFlyout;
var snapTime = weatherFormatting.getFormattedMapSnapTime(mapdata.mapList[0].SnapTime, shortDateTimeFormat);
var ariaText = resourceLoader.getString("MapFlyoutAriaText").format(mapdata.type, mapCategory, snapTime);
flyoutNode.setAttribute("aria-label", ariaText);
WeatherAppJS.Utilities.UI.ShowFlyoutBackground(false);
WeatherAppJS.Utilities.Instrumentation.incrementInt32("MapsTileTapped");
msWriteProfilerMark("WeatherApp:MapsFlyoutLoad:s");
var mapsHeaderDiv = flyoutNode.querySelector("#mapsFlyoutHeaderDiv");
mapsHeaderDiv.innerText = '';
var mapsHeader = document.createElement("h2");
mapsHeader.innerText = mapdata.type;
WinJS.Utilities.addClass(mapsHeader, "mapsFlyoutHeader");
mapsHeaderDiv.appendChild(mapsHeader);
var mapsColorCodeTextDiv = flyoutNode.querySelector("#mapsFlyoutColorCodeTextDiv");
var mapsColorCodeImageDiv = flyoutNode.querySelector("#mapsFlyoutColorCodeImageDiv");
var mapsColorCodeImage = mapsColorCodeImageDiv.querySelector("#colorCodeImg");
if (mapdata.type === resourceLoader.getString("DopplerRadar")) {
mapsColorCodeTextDiv.style.display = "block";
mapsColorCodeImageDiv.style.display = "block";
mapsColorCodeImage.src = "/images/mapsFlyout/dopplerlegend.png";
}
else if (mapdata.type === resourceLoader.getString("Precipitation")) {
mapsColorCodeTextDiv.style.display = "block";
mapsColorCodeImageDiv.style.display = "block";
mapsColorCodeImage.src = "/images/mapsFlyout/preciplegend.png";
}
else {
mapsColorCodeTextDiv.style.display = "none";
mapsColorCodeImageDiv.style.display = "none";
}
var attributionElem = flyoutNode.querySelector("#mapsFlyoutAttribution");
var weatherDotCom = resourceLoader.getString("Weather.com");
if (mapdata.attributionUrl && mapdata.ProviderName === weatherDotCom) {
var attributionLinkElem = document.createElement("a");
attributionLinkElem.setAttribute("href", mapdata.attributionUrl);
attributionLinkElem.innerText = WeatherAppJS.Utilities.Common.getWeatherDotComBaseUrl();
attributionElem.innerHTML = resourceLoader.getString("WeatherDotComMapsAttribution").format(attributionLinkElem.outerHTML);
attributionElem.style.display = "block";
} else {
attributionElem.style.display = "none";
}
that.thumbnailUrl = mapdata.thumbnailUrl;
document.getElementById("mapsFlyoutError").style.display = "none";
var mapsFlyout = flyoutNode.winControl;
mapsFlyout.addEventListener("beforehide", function mapsFlyout_beforeHide(event) {
// if the user immediately dismisses the flyout, while the image is loading.
// then we need hide the progress indicator
var progress = CommonJS.Progress;
progress.hideProgress(progress.centerProgressType);
});
mapsFlyout.addEventListener("afterhide", function mapsFlyout_afterHide(event) {
that.resetFlyout();
sourceTile.focus();
});
var anchor = document.getElementById('mapsFlyoutAnchor');
if (mapsFlyout && mapsFlyout.hidden) {
var viewId = mapdata.type + '_' + mapCategory;
that.addMapView(viewId, mapdata.mapList);
mapsFlyout.show(anchor);
if (mapdata.isAnimated) {
WinJS.UI.Animation.showPopup(flyoutNode).done(function () {
that.initMapsAnimation();
changeWamView(viewId);
});
} else {
wamStaticMapDisplay(viewId);
WinJS.UI.Animation.showPopup(flyoutNode);
}
}
that.isShown = true;
msWriteProfilerMark("WeatherApp:MapsFlyoutLoad:e");
}, function (err) {
return;
});
},
/// <summary>
/// Creates a new view and adds to it the list of map images.
/// </summary>
/// <param name="view">
/// A unique string representing the maptype. It should also be unique across locations.
/// </param>
/// <param name="mapList">
/// A list containing ImageUrls and SnapTimes.
/// </param>
addMapView: function addMapView(view, mapList) {
var maxviews = 0;
var wt_indx = 0;
var twt = "";
var numOfImages = 0;
var snapTime = PlatformJS.Services.resourceLoader.getString("SnapTime");
var weatherFormatting = WeatherAppJS.Utilities.Formatting;
var shortDateTimeFormat = PlatformJS.Services.resourceLoader.getString('ShortDateTimeFormatString');
if (mapList && mapList.length > 0) {
wamviews[view] = {};
wamviews[view].button = wamconsts.buttonString;
wamviews[view].imgs = [];
wt_indx = 0;
numOfImages = mapList.length;
for (var tempmap in mapList) {
wamviews[view].imgs[wt_indx] = {};
wamviews[view].imgs[wt_indx].url = mapList[tempmap].ImageUrl;
wamviews[view].imgs[wt_indx].status = wamconsts.NOTLOADED;
var mapSnapTime = weatherFormatting.getFormattedMapSnapTime(mapList[tempmap].SnapTime, shortDateTimeFormat);
twt = snapTime.format(mapSnapTime);
wamviews[view].imgs[wt_indx].timestamp = twt;
wt_indx++;
}
wamviews[view].imagesToLoad = wt_indx; /*counts how many images need preload*/
wamviews[view].imgs[0].status = wamconsts.LOADED;/* do not want to loaded first image */
wamviews[view].maximgs = wt_indx;
wamobj.viewid = view;
}
},
/// <summary>
/// Initialize the maps animation control. Needs to be init'ed for a page.
/// </summary>
initMapsAnimation: function initMapsAnimation() {
// Check if slider has been init'ed on this page
if (jQuery) {
if ($('#slider-range-min > div').size() === 0) {
$(document).ready(function () {
if ((typeof (wamDocumentReady) !== "undefined")) {
wamDocumentReady();
resetWamParams();
}
});
}
}
},
resetFlyout: function resetFlyout() {
var that = WeatherAppJS.Utilities.UI.MapsFlyout;
if (!that.isShown) {
return;
}
WeatherAppJS.Utilities.UI.ShowFlyoutBackground(false);
/// Sometimes, flyout reset function is being called after the page navigation has already happened.
/// We need to test the element objects (mapsFlyoutImageb0) before setting properties on them.
var mapsImg = document.getElementById("mapsFlyoutImageb0");
if (mapsImg) {
mapsImg.src = wamconsts.clearImgUrl;
}
clearInterval(wam_timer_key);
var mapsSlider = document.getElementById('mapsFlyoutSliderDiv');
if (mapsSlider) {
mapsSlider.style.display = '';
}
resetWamParams();
destroySlider();
// An async ChangeWamView() call may sometimes get executed after the "beforehide" event handler.
// So, before clearing out the image elements, reset the progress bar explicitly. 
CommonJS.Progress.resetProgress(CommonJS.Progress.centerProgressType);
that.imageHolder.innerHTML = "";
that.isShown = false;
},
/// <summary>
/// Onclick handler for play/pause button.
/// </summary>
mapsPlayHandler: function mapsPlayHandler() {
wamPlay();
}
});
/* Consts for WAM */
var wamconsts = {
PLAY: 0,   /*slider state constant*/
PAUSE: 1,   /*slider state constant*/
STOP: 2,   /*slider state constant*/
LOADED: 0,   /*image loaded*/
NOTLOADED: 1,   /*image not loaded yet*/
ERROR: 2,   /*image can not be loaded, missing URL*/
numImgThread: 20,   /* Number of images to load in paralled*/
buttonString: "mapsFlyoutImageb",
clearImgUrl: "/images/mapsFlyout/a_clear.gif",
img_tick: 512, /* image will tick after img_tick milliseconds */
timer_tick: 1 << 5, /* power of 2 */
slider_inc: 1 << 5, /*power comes from timer_tick*/
};
/* Hold parameters for WAM*/
var wamparams = {
imgNumThreshold: 20, /*drop half of the images if images are more than this threshold*/
};
/* State management across tabs*/
var wamobj = {
viewid: null, /*index of currently viewed tab*/
error: false, /*indicates whether error occured*/
};
/* currently doppler, clouds, satellite and temperature*/
var wamviews = [];
/*handle to the timer. can clear and set timer using this handle*/
var wam_timer_key;
var waminit;
/* For the weather maps */
/* *********************************************************************
* animation controls implementation  - use jquery slider
* - wrapper over jquery slider
* - write code to animate slider handle
* - synchronize images with slider
* - support pause, play, stop and other user interventions
* - support multiple tabs
*
*   Code flow:
*   1. use timer to call "slider handle" incrementing function after regular interval
*   2. check "slider handle" position at every change
*   3. Display image depending on slider location
*   4. When "slider handle" reaches the end of sliding bar, change slider status to STOP
*   5. Reset slider position when user sends PLAY on stopped slider
*   6. STOP position merely indicates the END of sliding bar.
*
*   Preloading:
*   Parallel dispatch for image load at page load - one time process
*   - Load half of the images for each tab
*   Parallel dispatch for image load at each tab change - one time process
*   - Load all the images for respective tab and then start play
*   * */
/* wam --> weather maps animation */
function wamDisplayError() {
/* verify the thumbnail url before loading it. */
var img = document.createElement('img');
WeatherAppJS.Utilities.UI.MapsFlyout.imageHolder.appendChild(img);
$(img).attr('src',WeatherAppJS.Utilities.UI.MapsFlyout.thumbnailUrl);
$(img).load(function () {
document.getElementById(wamviews[wamobj.viewid].button + "0").src = WeatherAppJS.Utilities.UI.MapsFlyout.thumbnailUrl;
});
$(img).error(function () {
document.getElementById(wamviews[wamobj.viewid].button + "0").style.display = "none";
});
document.getElementById(wamviews[wamobj.viewid].button + "ts").innerText = "";
document.getElementById("mapsFlyoutError").style.display = "";
}
/* display image at index  */
function wamDisplay(index) {
if (index === wamviews[wamobj.viewid].maximgs) {
return;
}
if (wamobj.img_index !== index) {
var bid = wamviews[wamobj.viewid].button;
var ts = "#" + bid + index;
// bug 165062 crash fix : test if imgs[index] is a valid reference before accessing the 'url' property
if (wamviews[wamobj.viewid] && wamviews[wamobj.viewid].imgs && wamviews[wamobj.viewid].imgs[index]) {
document.getElementById(wamviews[wamobj.viewid].button + "0").src = wamviews[wamobj.viewid].imgs[index].url;
document.getElementById(wamviews[wamobj.viewid].button + "ts").innerText = wamviews[wamobj.viewid].imgs[index].timestamp;
document.getElementById(wamviews[wamobj.viewid].button + "0").style.display = "block";
wamobj.img_index = index;
}
}
}
/* Display static maps */
function wamStaticMapDisplay(viewId) {
document.getElementById('mapsFlyoutSliderDiv').style.display = 'none';
wamobj.viewid = viewId;
wamobj.img_index = -1;
var img = document.createElement('img');
WeatherAppJS.Utilities.UI.MapsFlyout.imageHolder.appendChild(img);
if ($) {
$(img).load(function () {
});
$(img).error(function () {
wamDisplayError();
return;
});
}
// bug 165062 crash fix : test if the image is available before accessing the 'url' property
if (wamviews[viewId] && wamviews[viewId].imgs && wamviews[viewId].imgs.length > 0) {
$(img).attr('src', wamviews[viewId].imgs[0].url);
}
wamDisplay(0);
}
/* called at every tab switching  */
function resetWamParams() {
wamobj.img_index = 0;
wamobj.slider_state = wamconsts.PAUSE;
wamobj.error = false;
/* do not reset view id */
/* all calculation in terms of shift, operations to avoid any floating point calculations */
wamparams.slider_max = (wamviews[wamobj.viewid].maximgs - 1) * (wamconsts.img_tick);
/*Change slider state*/
resetSlider();
}
function cleanOldViewState(viewId) {
/* do any state cleaning here */
endImgLoadingEffect(viewId);
}
/* helps switch tab switching by reseting desired variables */
function changeWamView(viewId) {
/* clear timer immediately*/
clearInterval(wam_timer_key);
wamobj.viewid = viewId;
wamobj.img_index = -1;
startImgLoadingEffect(viewId);
showFirstImage(); /* display first image on changing views for the subsequent view */
resetWamParams();
wamPlay();
}
function showFirstImage() {
var index = 0;
if (wamobj.img_index !== index) {
var bid = wamviews[wamobj.viewid].button;
var ts = "#" + bid + index;
if (wamviews[wamobj.viewid] && wamviews[wamobj.viewid].imgs && wamviews[wamobj.viewid].imgs[index]) {
/* verify the thumbnail url before loading it. */
var img = document.createElement('img');
WeatherAppJS.Utilities.UI.MapsFlyout.imageHolder.appendChild(img);
$(img).attr('src', wamviews[wamobj.viewid].imgs[index].url);
$(img).load(function () {
document.getElementById(wamviews[wamobj.viewid].button + "0").src = wamviews[wamobj.viewid].imgs[index].url;
document.getElementById(wamviews[wamobj.viewid].button + "ts").innerText = wamviews[wamobj.viewid].imgs[index].timestamp;
document.getElementById(wamviews[wamobj.viewid].button + "0").style.display = "block";
wamobj.img_index = index;
});
$(img).error(function () {
endImgLoadingEffect(wamobj.viewid);
wamDisplayError();
});
}
}
}
/* function called at every timer tick. provides control to slider*/
/* Resolution of timer changes with number of images and hence tabs, called at every tab change*/
function resetSlider() {
$("#slider-range-min").slider("option", "max", wamparams.slider_max);
$("#slider-range-min").trigger("onsliderinit");
if (wamviews[wamobj.viewid].imagesToLoad !== 0) {
$("#slider-range-min").slider("disable");
}
else {
$("#slider-range-min").slider("enable");
}
/* change button to play, */
wamSetBtState("play");
}
function destroySlider() {
var sliderDiv = $("#slider-range-min");
if (sliderDiv) {
sliderDiv.slider("destroy");
sliderDiv.unbind();
}
}
function triggerSlider() {
$("#slider-range-min").trigger("ontimertick");
}
/* Add the slider to DOM and initialize it.*/
function wamDocumentReady() {
/*jquery based slider */
$("#slider-range-min").slider({
range: "min",
value: 0,
min: 0,
max: wamparams.slider_max,
animate: false,
disabled: true,
start: function start(event, ui) {
clearInterval(wam_timer_key);
/* once user holds the slider, stop goes to pause*/
if (getSliderState() === wamconsts.STOP) {
setSliderState(wamconsts.PAUSE);
}
},
/* called as soon as user slided the slider handle*/
slide: function slide(event, ui) {
var index = Math.floor(ui.value / wamconsts.img_tick);
clearInterval(wam_timer_key);
wamDisplay(index);
},
/* called at every slider value change change done programmatically */
change: function change(event, ui) {
var index = Math.floor(ui.value / wamconsts.img_tick);
wamDisplay(index);
if (index === (wamviews[wamobj.viewid].maximgs - 1)) {
/* we have reached the end of animation */
clearInterval(wam_timer_key);
setSliderState(wamconsts.STOP);
}
},
/* called when user stops sliding */
stop: function stop(event, ui) {
var tmp_index = wamconsts.slider_inc * Math.round(ui.value / wamconsts.slider_inc);
var index = 0;
ui.value = tmp_index;
index = Math.floor(ui.value / wamconsts.img_tick);
$(this).slider('value', ui.value);
wamDisplay(index);
if (getSliderState() === wamconsts.PLAY) {
/* after sliding stops, play only the playing slider */
wam_timer_key = setInterval(triggerSlider, wamconsts.timer_tick);
}
}
});
$("#slider-range-min").bind('ontimertick', function (e) {
$("#slider-range-min").each(function (e, ui) {
var tmp = $(this).slider('value') + wamconsts.slider_inc;
/* prevent value overflow using saturation*/
if (tmp > wamparams.slider_max) {
tmp = wamparams.slider_max;
}
$(this).slider('value', tmp);
});
});
/* called by reset slider*/
$("#slider-range-min").bind('onsliderinit', function (e) {
$("#slider-range-min").each(function (e, ui) {
$(this).slider('value', 0);
});
});
}
/* Set play/pause button state */
function wamSetBtState(state) {
var button = "#wam_bt";
$(button).attr("aria-label", PlatformJS.Services.resourceLoader.getString(state));
$(button).removeClass("loading play pause");
$(button).addClass(state);
}
/* Play/pause the animation */
function wamPlay() {
var button = "#wam_bt";
if ($(button).hasClass("play")) {
if (wamviews[wamobj.viewid].imagesToLoad > 0) {
wamSetBtState("loading");
preLoadAnimImages(wamobj.viewid);
}
else {
wamSetBtState("pause");
sliderCmdPlay();
}
}
else if ($(button).hasClass("pause")) {
wamSetBtState("play");
sliderCmdPause();
}
else if ($(button).hasClass("loading")) {
abortLoadImage(wamobj.viewid);
wamSetBtState("play");
$(button).attr("alt", PlatformJS.Services.resourceLoader.getString("loading"));
}
}
/* Callback when the image loads successfully */
function wamImgLoad(source, viewId, imageId) {
wamviews[viewId].imgs[imageId].status = wamconsts.LOADED;
wamviews[viewId].imagesToLoad -= 1;
if (wamviews[viewId].imagesToLoad === 0) {
endImgLoadingEffect(viewId);
if (viewId === wamobj.viewid) {
/* If error in loading a single image, show error */
if (wamobj.error) {
wamDisplayError();
return;
}
wamSetBtState("pause");
sliderCmdPlay();
}
/* start timer play */
}
}
/* Callback when their is an error while fetching the image */
function wamImgError(source, viewId, imageId) {
wamobj.error = true;
wamviews[viewId].imgs[imageId].status = wamconsts.ERROR;
wamviews[viewId].imagesToLoad -= 1;
if (wamviews[viewId].imagesToLoad === 0) {
endImgLoadingEffect(viewId);
if (viewId === wamobj.viewid) {
/* If error in loading a single image, show error */
if (wamobj.error) {
wamDisplayError();
return;
}
wamSetBtState("pause");
sliderCmdPlay();
}
/* start timer play */
}
}
function abortLoadImage(viewId) {
/*  forget number of images already loaded, as this is automatically taken care by onload event next time */
wamviews[viewId].imagesToLoad = wamviews[viewId].maximgs;
/* this will also reset*/
endImgLoadingEffect(viewId);
}
function dispatchParallerImageLoad(viewId, max) {
for (var i = 0; (i < wamconsts.numImgThread) && (i < max) ; i++) {
/* even if wamconsts.numImgThread is less than nam number of images, LoadImage will not fire we can do a double check if required  */
loadImage(viewId, i, max);
}
}
/* Loads the specified image and triggers the next image load. */
function loadImage(viewId, index, max) {
if (viewId !== wamobj.viewid || !WeatherAppJS.Utilities.UI.MapsFlyout.isShown) {
abortLoadImage(viewId);
return;
}
if (index < max) {
msWriteProfilerMark("WeatherApp:MapsImageLoad:s");
var img = document.createElement('img');
WeatherAppJS.Utilities.UI.MapsFlyout.imageHolder.appendChild(img);
$(img).load(function () {
wamImgLoad(this, viewId, index);
msWriteProfilerMark("WeatherApp:MapsImageLoad:e");
loadImage(viewId, index + wamconsts.numImgThread, max);
});
$(img).error(function () {
// trigger the next image
wamImgError(this, viewId, index);
msWriteProfilerMark("WeatherApp:MapsImageLoad:e");
loadImage(viewId, index + wamconsts.numImgThread, max);
});
// bug 165062 crash fix : test if the image is available before accessing the 'url' property
if (wamviews[viewId] && wamviews[viewId].imgs && wamviews[viewId].imgs[index]) {
$(img).attr('src', wamviews[viewId].imgs[index].url);
}
}
}
function dispSpinner(viewId) {
var progress = CommonJS.Progress;
progress.showProgress(progress.centerProgressType);
}
function unDispSpinner(viewId) {
var progress = CommonJS.Progress;
progress.hideProgress(progress.centerProgressType);
}
function endImgLoadingEffect(viewId) {
unDispSpinner(viewId);
}
function startImgLoadingEffect(viewId) {
dispSpinner(viewId);
}
/* preloading/delay loading of images*/
function preLoadAnimImages(viewId) {
if (wamviews[viewId].imagesToLoad === 0) {
return;
}
else {
dispatchParallerImageLoad(viewId, wamviews[viewId].maximgs);
}
}
/* manages slider state, will impact button state if required*/
function setSliderState(state) {
if ((wamobj.slider_state === wamconsts.STOP) && (state === wamconsts.PLAY)) {
$("#slider-range-min").trigger("onsliderinit");
}
wamobj.slider_state = state;
if (state === wamconsts.STOP || state === wamconsts.PAUSE) {
/* change button to play, */
wamSetBtState("play");
}
else {
/* change button to pause */
wamSetBtState("pause");
}
}
function getSliderState() {
return (wamobj.slider_state);
}
/* send play command to slider, check for more than 1 images, no need of animation, if only 1 image is available*/
function sliderCmdPlay() {
$("#slider-range-min").slider("enable");
setSliderState(wamconsts.PLAY);
if (wamviews[wamobj.viewid].maximgs > 1) {
/* core function, controls animation*/
wam_timer_key = setInterval(triggerSlider, wamconsts.timer_tick);
}
}
/*process pause command by pausing the timer and changing slider state*/
function sliderCmdPause() {
setSliderState(wamconsts.PAUSE);
clearInterval(wam_timer_key);
}
})();
