﻿/********************************************************
*                                                        *
*   © Microsoft. All rights reserved.                    *
*                                                        *
*********************************************************/
(function () {
"use strict";
WinJS.Namespace.define("WeatherAppJS", {
ProcessListener: WinJS.Class.define(
/// <summary>
/// Constructor for your IProcessListener class
/// </summary>
function () {        
// variables to keep track of current times, views and the start time of view change.
this._curTime = 0;
this._chStartTime = 0;
this._curView = 0;
this._launchMode = null;
},
{
/// <summary>
/// Logs the dwell times in each view.
/// </summary>
/// <param name="event">
/// Layout change event
/// </param>
/// <returns>
/// Nothing
/// </returns>           
_logDwellTime: function _logDwellTime(event) {
var applicationViewState = Windows.UI.ViewManagement.ApplicationViewState;
this._curTime = new Date().getTime() / 1000;
if (this._curTime > this._chStartTime) {
var totalTime = this._curTime - this._chStartTime;
this._chStartTime = this._curTime;
var viewType;
if (this._curView === applicationViewState.snapped) {
viewType = "Snapped";
} else if (this._curView === applicationViewState.filled) {
viewType = "Filled";
} else {
viewType = "Fullscreen";
}
var eventAttr = {
viewType: viewType,
timeSpent: totalTime
};
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, "App", "Layout", "appViewed", Platform.Telemetry.UserActionMethod.unknown, 0, JSON.stringify(eventAttr));
}
this._curView = event.viewState;
},
/// <summary>
/// Handles layout change event
/// </summary>
/// <param name="event">
/// Layout change event
/// </param>
/// <returns>
/// Nothing
/// </returns>
onLayoutChange: function onLayoutChange(event) {
this._logDwellTime(event);
if (event && event.viewState) {
var applicationViewState = Windows.UI.ViewManagement.ApplicationViewState;
if (event.viewState === applicationViewState.fullScreenPortrait) {
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NumSwitchToPortrait");
} else if (event.viewState === applicationViewState.snapped) {
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NumSwitchToSnap");
} else if (event.viewState === applicationViewState.filled) {
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NumSwitchToFilled");
}
}
},
/// <summary>
/// Handles orientation change event
/// </summary>
/// <param name="event">
/// Orientation change event
/// </param>
/// <returns>
/// Nothing
/// </returns>
onOrientationChange: function onOrientationChange(event) {
Platform.Telemetry.FlightRecorder.logDisplayChange(Platform.Telemetry.LogLevel.normal, window.screen.width, window.screen.height);
},
/// <summary>
/// Handles Suspend event
/// </summary>
/// <param name="event">
/// Suspend event
/// </param>
/// <returns>
/// Nothing
/// </returns>
onSuspending: function onSuspending(event) {
this._logDwellTime(event);
WeatherAppJS.SettingsManager.setLastSuspendTimeAsync(new Date().getTime()).then(null, function (e) { });
},
/// <summary>
/// Handles Checkpoint event
/// </summary>
/// <param name="event">
/// Suspend suspend event
/// </param>
/// <returns>
/// Nothing
/// </returns>
onCheckpoint: function onCheckpoint(event) {
},
/// <summary>
/// Handles Resume event
/// </summary>
/// <param name="event">
/// resume event
/// </param>
/// <returns>
/// Nothing
/// </returns>
onResuming: function onResuming(event) {
// Check the current network status and update
WeatherAppJS.Networking.updateNetworkStatus();
//view mode and the time when the app is resumed
this._curView = Windows.UI.ViewManagement.ApplicationView.value;
this._chStartTime = new Date().getTime() / 1000;
var common = WeatherAppJS.Utilities.Common;
var currentPage = common.getCurrentPage();
var isStale = common.isAppStateStale();
// Navigate to home if the app state is stale.
if (!isStale && currentPage !== "WeatherAppJS.DailyForecast") { // if the app is not stale and page is daily drilldown, no need to refresh the page
// Refresh the currently viewed app page on app resume
var page = PlatformJS.Navigation.currentIPage;
if (page.onPageRefresh) {
page.onPageRefresh();
}
}
},
/// <summary>
/// Handles activation event
/// </summary>
/// <param name="event">
/// activation event
/// </param>
/// <returns>
/// Nothing
/// </returns>
onActivated: function onActivated(event) {
WeatherAppJS.Services.initialize(false);
//view mode and the time of activation
this._curView = Windows.UI.ViewManagement.ApplicationView.value;
this._chStartTime = new Date().getTime() / 1000;
WeatherAppJS.SettingsManager.onActivated();
var isSnapped = (Windows.UI.ViewManagement.ApplicationView.value === Windows.UI.ViewManagement.ApplicationViewState.snapped);
var defaultLocation = WeatherAppJS.LocationsManager.getDefaultLocation();
var isFRE = WeatherAppJS.SettingsManager.isFRE();
if (event && event.detail && event.detail.kind === Windows.ApplicationModel.Activation.ActivationKind.search) {
// Ignore the search query if app hasn't passed FRE yet
if (event.detail.queryText && !isFRE && defaultLocation) {
return {
pageInfo: {
fragment: "/panoramas/search/searchPage.html",
page: "WeatherAppJS.Search.SearchPage"
},
state: {
event: event,
query: event.detail.queryText,
searchAction: WeatherAppJS.Search.Actions.SearchLocation
}
};
}
} else if (event && event.detail && event.detail.kind === Windows.ApplicationModel.Activation.ActivationKind.protocol
&& (!isFRE && defaultLocation)
&& !WeatherAppJS.LocationsManager.isMarketBlockedByIP) {
// Handle protocol activation.
// Example: bingweather://controller/command#entityId?params
var uri = event.detail.uri.rawUri;
var action = PlatformJS.Navigation.createCommandFromUri(uri);
if (WinJS.Promise.is(action)) {
return action;
} else if (action.channelInfo) {
return { pageInfo: action.channelInfo.pageInfo, state: action.options };
} else {
action();
}
} else if ((event.type === "activated") && event.detail && (event.detail.tileId !== "App")
&& event.detail.arguments && WeatherAppJS.App.isTileSupportedInCurrentMarket(event.detail.arguments)
&& (!isFRE && defaultLocation)) {
var geoLoc = WeatherAppJS.App.getGeoLocFromTileArguments(event.detail.arguments);
if (geoLoc) {
this._launchMode = WeatherAppJS.Instrumentation.FormCodes.PinnedTile;
}
return {
pageInfo: {
channelId: 'Home',
fragment: (isSnapped) ? "/panoramas/snapmode/snapHome.html" : "/panoramas/home/home.html",
page: (isSnapped) ? "WeatherAppJS.SnapHome" : "WeatherAppJS.Pages.Home"
},
state: {
locID: geoLoc.getId(),
geocodeLocation: geoLoc,
searchingLocation: false
}
};
} else if (WeatherAppJS.Utilities.Common.isAppStateStale() && (WeatherAppJS.Utilities.Common.getCurrentPage() !== "WeatherAppJS.Pages.Home")) {
this._launchMode = WeatherAppJS.Instrumentation.FormCodes.AppLaunch;
return {
pageInfo: {
fragment: (isSnapped) ? "/panoramas/snapmode/snapHome.html" : "/panoramas/home/home.html",
page: (isSnapped) ? "WeatherAppJS.SnapHome" : "WeatherAppJS.Pages.Home"
},
state: {
}
};
} else {
// Let the platform navigate to the last opened page
var loc = WeatherAppJS.GeocodeCache.getLocation(WeatherAppJS.SettingsManager.getDefaultLocationId());
if (loc) {
this._launchMode = WeatherAppJS.Instrumentation.FormCodes.AppLaunch;
}
}
},
_isFirstViewRealized: false,
onFirstViewRealized: function onFirstViewRealized() {
if (!this._isFirstViewRealized) {
this._isFirstViewRealized = true;
WeatherAppJS.Search.initSearchCharm();
WeatherAppJS.UI.Settings.init();
WeatherAppJS.App.initAppTileManager();
//Set-up the level2 edgy
WeatherAppJS.topEdgy.instance.initializeEdgy();
// check for debug build
if (PlatformJS.isDebug) {
WinJS.UI.Fragments.render("/html/debug.html").then(
function () {
WeatherAppJS.Debug.init();
},
function (e) {
});
}
}
},
startPostActivateWork: function startPostActivateWork() {
},
/// <summary>
/// Invoked when the configuration file update event happens
/// </summary>
/// <param name="event">
/// Event object
/// </param>
/// <returns>
/// True, if the app needs to navigate to home after this event
/// </returns>
onUpdatedConfig: function onUpdatedConfig(event) {
// Reinitialize modules with cached config values
WeatherAppJS.App.reinitialize();
WeatherAppJS.LocationSuggest.reinitialize();
WeatherAppJS.Utilities.Common.reinitialize();
WeatherAppJS.GeocodePolygonSearch.reinitialize();
WeatherAppJS.Storage.Memory.removeItem("homeAdsConfig");
WeatherAppJS.Storage.Memory.removeItem("gsClusterConfig");
WeatherAppJS.Storage.Memory.removeItem("nsClusterConfig");
return true;
}
})
});
})();
