﻿/********************************************************
*                                                        *
*   © Microsoft. All rights reserved.                    *
*                                                        *
*********************************************************/
(function () {
"use strict";
var MAX_QUERY_SUGGESTIONS = 5;
WinJS.Namespace.define("WeatherAppJS.Search.Actions", {
SearchLocation: 0,
AddLocation: 1,
SetDefault: 2,
AddLocationDisambiguationPage: 3,
SetDefaultDisambiguationPage: 4,
DetectedLocationConfirm: 5
});
WinJS.Namespace.define("WeatherAppJS.Search",
{
/// <summary>
/// Suggestions list for add location flyout (quick search dialog)
/// </summary>
_quickSearchSuggestions: [],
/// <summary>
/// Suggestions list for search charm.
/// </summary>
_searchCharmSuggestions: [],
/// <summary>
/// Search Location action
/// </summary>
_searchAction: WeatherAppJS.Search.Actions.SearchLocation,
_context : "Add Location Flyout",
/// <summary>
/// Initialize Search Charm and set the required callback.
/// </summary>
initSearchCharm: function initSearchCharm() {
if (!this.searchCharmInitialized) {
var searchCharm = Windows.ApplicationModel.Search.SearchPane.getForCurrentView();
var resourceLoader = PlatformJS.Services.resourceLoader;
var config = PlatformJS.Services.appConfig;
searchCharm.onquerysubmitted = function (eventObject) {
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NumSearchCharmUsed");
if (WeatherAppJS.SettingsManager.isFRE() || !WeatherAppJS.LocationsManager.getDefaultLocation()) {
// Search charm isn't functional until user completes FRE 
return;
}
if (eventObject && eventObject.queryText && eventObject.queryText.trim()) {
var eventAttr = { searchQuery: eventObject.queryText, source: "Search Charm" };
Platform.Telemetry.FlightRecorder.logAppActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, "Search Results", "Search Disambiguation Page", Platform.Telemetry.AppActionOperation.open, 0, JSON.stringify(eventAttr));
if (WeatherAppJS.Networking.internetAvailable === true) {
WinJS.Navigation.navigate({ fragment: "/panoramas/search/searchPage.html", page: "WeatherAppJS.Search.SearchPage" }, { event: eventObject, searchAction: WeatherAppJS.Search.Actions.SearchLocation, query: eventObject.queryText.trim() });
}
else {
WeatherAppJS.Error.showMessageBar(PlatformJS.Services.resourceLoader.getString("NoInternetConnectionError"), false);
}
}
};
searchCharm.onresultsuggestionchosen = this.searchCharmSuggestionSelected;
searchCharm.onsuggestionsrequested = this.searchCharmSuggestionsRequested;
searchCharm.placeholderText = resourceLoader.getString("searchboxPlaceHolderTest");
searchCharm.searchHistoryEnabled = config.getBool("EnableSearchPaneHistory");
this.searchCharmInitialized = true;
}
},
/// <summary>
/// Try and fetch CC data and then navigate to target page. quicksearchObj is optional
/// </summary>
/// <param name="locationObj">
/// Geocode Location object
/// </param>
/// <param name="action" type="enum">
/// Enumerates one of the actions from WeatherAppJS.Search.Actions.
/// </param>
/// <param name="quickSearchObj" optional="true">
/// Quick search object. Needed to show error message/progress bars on the Add location flyout.
/// </param>
fetchLocationData: function fetchLocationData(locationObj, action, quickSearchObj) {
var that = this;
that._toggleProgressBar(true, quickSearchObj);
WeatherAppJS.LocationsManager.tryFetchingCCDataAsync(locationObj, true).then(
function (responseData) {
that._toggleProgressBar(false, quickSearchObj);
if (action === WeatherAppJS.Search.Actions.AddLocation || action === WeatherAppJS.Search.Actions.SetDefault) {
Platform.Telemetry.FlightRecorder.logSearch(Platform.Telemetry.LogLevel.normal, locationObj.getFullDisplayName(), "Bing", Platform.Telemetry.SearchMethod.form, true, -1, true, true, false, WeatherAppJS.Instrumentation.FormCodes.Search, locationObj.latitude + ' - ' + locationObj.longitude);
}
that.openLocationPage(responseData.locationObject, action, quickSearchObj);
msWriteProfilerMark("WeatherApp:LocationSuggestConfirmSelection:e");
},
function () {
that._toggleProgressBar(false, quickSearchObj);
if (WeatherAppJS.Networking.internetAvailable === true) {
that._showMessage("WeatherDataNotAvailable", quickSearchObj);
// Weather Data not available for location selected from Add New Location flyout dropdown or Search Charm
if (locationObj) {
WeatherAppJS.Utilities.Common.instrumentLocation("SearchByTextFailure", locationObj.latitude, locationObj.longitude, locationObj.fullName);
var eventAttr = { Location: locationObj.fullName, latlong: locationObj.latitude + " - " + locationObj.longitude};
Platform.Telemetry.FlightRecorder.logContentErrorWithJsonAttributes(Platform.Telemetry.LogLevel.normal, "", "No weather data for the location", 0, "", "", JSON.stringify(eventAttr));
}
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NoWeatherDataForLocation");
} else {
that._showMessage("NoInternetConnectionError", quickSearchObj);
}
msWriteProfilerMark("WeatherApp:LocationSuggestConfirmSelection:e");
});
},
/// <summary>
/// Callback function when selecting an item from the suggestion list in the Add location flyout (QuickSearchDialog).
/// </summary>
/// <param name="event">
/// Selection event object.
/// </param>
/// <param name="src">
/// Quick search object.
/// </param>
quickSearchSuggestionSelected: function quickSearchSuggestionSelected(event, src) {
msWriteProfilerMark("WeatherApp:LocationSuggestConfirmSelection:s");
var that = WeatherAppJS.Search;
var locationObj = src.detectedLocation;
var isFRE = src.isFRE;
var eventAttr;
if (locationObj && event.queryText === WeatherAppJS.GeocodeCache.getFullDisplayName(locationObj.getId())) { // if query text does not match detected location, user clicked on an autosuggest result or typed something and pressed enter
if (WeatherAppJS.Networking.internetAvailable === false) {
that._showMessage('NoInternetConnectionError', src);
return;
}
eventAttr = { detectedLocation: locationObj.getFullDisplayName(), accuracy: locationObj.locDetectAccuracy };
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, WeatherAppJS.Instrumentation.PageContext.ConfirmLocFlyout, "Confirm", Platform.Telemetry.UserActionOperation.click, Platform.Telemetry.UserActionMethod.unknown, 0, JSON.stringify(eventAttr));
if (WeatherAppJS.Search._searchAction === WeatherAppJS.Search.Actions.DetectedLocationConfirm) {
// for FRE, we need to set the detected location as default
if (isFRE) {
WeatherAppJS.SettingsManager.setDefaultLocationAsync(locationObj, true).then();
} else {
PlatformJS.Navigation.navigateToChannel('Home', { locID: locationObj.getId(), geocodeLocation: locationObj, method: Platform.Telemetry.SearchMethod.form, formCode: WeatherAppJS.Instrumentation.FormCodes.Search });
}
WeatherAppJS.Utilities.UI.dismissPlatformFlyout();
} else if ((WeatherAppJS.Search._searchAction === WeatherAppJS.Search.Actions.AddLocation) && src.detectedLocation) {
that.openLocationPage(src.detectedLocation, WeatherAppJS.Search.Actions.AddLocation, src);
} else if ((WeatherAppJS.Search._searchAction === WeatherAppJS.Search.Actions.AddLocationDisambiguationPage) && src.detectedLocation) {
that.openLocationPage(src.detectedLocation, WeatherAppJS.Search.Actions.AddLocationDisambiguationPage, src);
} else if ((WeatherAppJS.Search._searchAction === WeatherAppJS.Search.Actions.SetDefault) && src.detectedLocation) {
that.openLocationPage(src.detectedLocation, WeatherAppJS.Search.Actions.SetDefault, src);
} else if ((WeatherAppJS.Search._searchAction === WeatherAppJS.Search.Actions.SetDefaultDisambiguationPage) && src.detectedLocation) {
that.openLocationPage(src.detectedLocation, WeatherAppJS.Search.Actions.SetDefaultDisambiguationPage, src);
}
} else if (event.index !== undefined && event.index >= 0 && that._quickSearchSuggestions[event.index]) {
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NumAutoSuggestionUsed");
var loc = that._quickSearchSuggestions[event.index];
var curElement;
if (WeatherAppJS.Search._context === WeatherAppJS.Instrumentation.PageContext.ConfirmLocFlyout) {
eventAttr = { detectedLocation: locationObj.getFullDisplayName(), newLocation: event.queryText, detectedLocLatlong: locationObj.latitude + " - " + locationObj.longitude, newLocLatLong: loc.latitude + " - " + loc.longitude, detectedLocAccuracy: locationObj.locDetectAccuracy };
curElement = "Change Location";
} else {
eventAttr = { location: event.queryText };
curElement = "Auto-suggest City";
}
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, WeatherAppJS.Search._context, curElement, Platform.Telemetry.UserActionOperation.click, Platform.Telemetry.UserActionMethod.unknown, 0, JSON.stringify(eventAttr));
if (isFRE) {
WeatherAppJS.Search._searchAction = WeatherAppJS.Search.Actions.SetDefault;
}
WeatherAppJS.Search.fetchLocationData(loc, WeatherAppJS.Search._searchAction, src);
} else if (event.queryText && event.queryText.trim()) {
// User pressed enter and didn't select from the drop down list. Take user to search disambiguation page.
if (isFRE) {
WeatherAppJS.Search._searchAction = WeatherAppJS.Search.Actions.SetDefault;
}
if (WeatherAppJS.Networking.internetAvailable === true || WeatherAppJS.LocationsManager.getDefaultLocation()) {
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NumDisambiguationPageLaunchedFromAddLocFlyout");
eventAttr = { searchQuery: event.queryText, source: "Add Location Flyout" };
Platform.Telemetry.FlightRecorder.logAppActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, "Search Results", "Search Disambiguation Page", Platform.Telemetry.AppActionOperation.open, 0, JSON.stringify(eventAttr));
WinJS.Navigation.navigate({ fragment: "/panoramas/search/searchPage.html", page: "WeatherAppJS.Search.SearchPage" }, { event: {}, searchAction: WeatherAppJS.Search._searchAction, query: event.queryText.trim() });
}
else {
// Only until default location has been set and user is still in offline mode user would not be able to navigate to search disambiguation page
// If we allow user to navigate to disambiguation page, navigating back gives him a breaking experience.
that._showMessage("NoInternetConnectionError", src);
}
} else {
/* This would happen when user does not type anything and press enter */
that._toggleProgressBar(false, src);
that._showMessage("NoSearchResults", src);
msWriteProfilerMark("WeatherApp:LocationSuggestConfirmSelection:e");
}
},
/// <summary>
/// Callback function when selecting an item from the suggestion list in the Search Charm.
/// </summary>
/// <param name="event">
/// Selection event object.
/// </param>
searchCharmSuggestionSelected: function searchCharmSuggestionSelected(event) {
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NumSearchCharmUsed");
msWriteProfilerMark("WeatherApp:LocationSuggestConfirmSelection:s");
var that = WeatherAppJS.Search;
if (event.tag !== undefined && event.tag >= 0 && that._searchCharmSuggestions[event.tag]) {
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NumAutoSuggestionUsed");
var loc = that._searchCharmSuggestions[event.tag];
var eventAttr = { location: loc.getFullDisplayName() };
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, Platform.Telemetry.ActionContext.charms, "Auto-suggest City", Platform.Telemetry.UserActionOperation.select, Platform.Telemetry.UserActionMethod.unknown, 0, JSON.stringify(eventAttr));
that.fetchLocationData(loc, WeatherAppJS.Search.Actions.SearchLocation);
} else {
that._toggleProgressBar(false);
that._showMessage("NoSearchResults");
msWriteProfilerMark("WeatherApp:LocationSuggestConfirmSelection:e");
}
},
searchCharmSuggestionsRequested: function searchCharmSuggestionsRequested(event) {
/// <summary>
/// Callback function when requesting for suggestions. Used by Search Charm
/// </summary>
/// <param name="event">Selection event object</param>
if (WeatherAppJS.SettingsManager.isFRE() || !WeatherAppJS.LocationsManager.getDefaultLocation()) {
// Search charm isn't functional until user completes FRE 
return;
}
var that = WeatherAppJS.Search;
that.querySuggestionsRequested(event);
},
quickSearchSuggestionsRequested: function quickSearchSuggestionsRequested(event, src) {
/// <summary>
/// Callback function when requesting for suggestions. Used by Search Dialog
/// </summary>
/// <param name="event">Selection event object</param>
/// <param name="src">Quick search object</param>
var that = WeatherAppJS.Search;
that.querySuggestionsRequested(event, src);
},
/// <summary>
/// Callback function when requesting for suggestions. Used by both Search Dialog and Search Charm
/// </summary>
/// <param name="event">
/// Selection event object.
/// </param>
/// <param name="src">
/// Quick search object. Null in case of search charm.
/// </param>
querySuggestionsRequested: function querySuggestionsRequested(event, src) {
var that = WeatherAppJS.Search;
if (that._lastQuery) {
// The error handler shouldn't call the local autosuggest for the last query. 
// So, before cancelling the promise invalidate lastQuery. 
var lastQuery = that._lastQuery;
that._lastQuery = null;
lastQuery.cancel();
}
if (that._timeoutHandle) {
clearTimeout(that._timeoutHandle);
that._timeoutHandle = null;
}
if (!event || !event.queryText || !event.queryText.trim()) {
// Hide Search Dialog suggestions for empty query
if (event && src) {
src.searchSuggestions = { event: event, suggestions: [] };
}
return;
}
var searchText = event.queryText.trim();
var deferral = event.request.getDeferral();
msWriteProfilerMark("WeatherApp:LocationSuggestSearch:s");
function fetchLocalSuggestions() {
WeatherAppJS.LocationSuggest.getLocalSuggestionsAsync(searchText).then(
function (localSuggestions) {
that._processSuggestions(localSuggestions, event, src);
deferral.complete();
msWriteProfilerMark("WeatherApp:LocationSuggestSearch:e");
}, function () {
deferral.complete();
msWriteProfilerMark("WeatherApp:LocationSuggestSearch:e");
});
};
var isPredictEnabled = PlatformJS.Services.appConfig.getBool("UsePredict");
if (isPredictEnabled) {
that._timeoutHandle = setTimeout(function () {
that._lastQuery = WeatherAppJS.LocationSuggest.getPredictSuggestionsAsync(searchText).then(
function (suggestions) {
if (suggestions && suggestions.length > 0) {
that._lastQuery = null;
that._processSuggestions(suggestions, event, src);
} else {
/*
// No suggestions returned, fallback to local autosuggest.
fetchLocalSuggestions();
*/
}
deferral.complete();
msWriteProfilerMark("WeatherApp:LocationSuggestSearch:e");
},
function (err) {
/*
// An error or timeout occurred while fetching PredictSuggestion, fallback to local autosuggest.
// But, first check whether the last query is still valid or not.
if (that._lastQuery)
fetchLocalSuggestions();
*/
deferral.complete();
msWriteProfilerMark("WeatherApp:LocationSuggestSearch:e");
});
that._timeoutHandle = null;
}, 200);
} else {
fetchLocalSuggestions();
}
},
/// <summary>
/// Process suggestions and push them to the appropriate datasource (search charm or quick search dialog).
/// </summary>
_processSuggestions: function _processSuggestions(localSuggestions, event, src) {
var that = WeatherAppJS.Search;
localSuggestions = (localSuggestions.length > MAX_QUERY_SUGGESTIONS) ? localSuggestions.slice(0, MAX_QUERY_SUGGESTIONS) : localSuggestions;
if (src) {
// Set suggestions for add location flyout
that._quickSearchSuggestions = [];
var displaySuggestions = [];
for (var i in localSuggestions) {
// Save locations to retrieve later on selection
that._quickSearchSuggestions.push(localSuggestions[i]);
// Set the display name field
var displayName = localSuggestions[i].getFullDisplayName(true);
displaySuggestions.push({ Title: displayName });
}
src.searchSuggestions = { event: event, suggestions: displaySuggestions };
} else {
// Set suggestions for Search Charm
that._searchCharmSuggestions = [];
var searchSuggestionCollection = event.request.searchSuggestionCollection;
var cityUri = new Windows.Foundation.Uri("ms-appx:///images/autoSuggest/searchCharm.png");
var skiUri = new Windows.Foundation.Uri("ms-appx:///images/autoSuggest/searchCharmSki.png");
var cityImg = Windows.Storage.Streams.RandomAccessStreamReference.createFromUri(cityUri);
var skiImg = Windows.Storage.Streams.RandomAccessStreamReference.createFromUri(skiUri);
for (var suggestionIndex in localSuggestions) {
var displayNames = localSuggestions[suggestionIndex].getTwoLineDisplayName();
var img = localSuggestions[suggestionIndex].isSkiLocation ? skiImg : cityImg;
// Save locations to retrieve later on selection
that._searchCharmSuggestions.push(localSuggestions[suggestionIndex]);
// Append result suggestion 
searchSuggestionCollection.appendResultSuggestion(displayNames[0], displayNames[1], suggestionIndex, img, displayNames.join());
}
}
},
/// <summary>
/// This will be run when the flyout hides/ close button is clicked.
///  if detectAndFetchCurrentLocationDataAsync is still not complete, then cancel it.
/// </summary>
onAddLocationFlyoutHide: function onAddLocationFlyoutHide() {
msWriteProfilerMark("WeatherApp:AddLocationFlyout:Dismissed");
var that = WeatherAppJS.Search;
if (that._detectLocationPromise) {
that._detectLocationPromise.cancel();
that._detectLocationPromise = null;
}
that._isLocationDetectionInProgress = false;
},
/// <summary>
/// Handler for detect current location button on Add location flyout.
/// This function is invoked only by the Add location flyout (quick search dialog).
/// </summary>
/// <param name="quickSearchObj">Quick search object</param>
/// <param name="searchAction">Search action to be performed</param>
detectCurrentLocation: function detectCurrentLocation(quickSearchObj, searchAction) {
var that = WeatherAppJS.Search;
msWriteProfilerMark("WeatherApp:LocationSuggestDetectCurrent:s");
if (that._isLocationDetectionInProgress) { // Detection in progress
msWriteProfilerMark("WeatherApp:LocationSuggestDetectCurrent:e");
return;
}
that._toggleProgressBar(true, quickSearchObj);
/// Detect
that._isLocationDetectionInProgress = true;
that._detectLocationPromise = WeatherAppJS.LocationTracking.detectAndFetchCurrentLocationDataAsync()
.then(
function (locationObject) {
var eventAttr = { detectedLocation: locationObject.getFullDisplayName(), accuracy: locationObject.locDetectAccuracy };
WeatherAppJS.Search._context = WeatherAppJS.Instrumentation.PageContext.ConfirmLocFlyout;
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, WeatherAppJS.Search._context, "Detect Location", Platform.Telemetry.UserActionOperation.click, Platform.Telemetry.UserActionMethod.unknown, 0, JSON.stringify(eventAttr));
var currentLocation = WeatherAppJS.SettingsManager.getCurrentLocation();
if (!currentLocation || (currentLocation.getId() !== locationObject.getId())) {
WeatherAppJS.SettingsManager.setCurrentLocationAsync(locationObject).then();
} else {
var accuracy = locationObject.locDetectAccuracy;
locationObject = currentLocation;
locationObject.locDetectAccuracy = accuracy;
}
quickSearchObj.detectedLocation = locationObject;
that._toggleProgressBar(false, quickSearchObj);
that._isLocationDetectionInProgress = false;
// flyout is already open, we just need to set the required state
// todo: set the flyout title and button text through CUX api which is not yet implemented (bug 441816 on CUX) (arunrab)
var searchBox = document.getElementById("SearchTextBox");
if (searchBox) {
searchBox.focus();
searchBox.value = WeatherAppJS.GeocodeCache.getFullDisplayName(locationObject.getId());
}
var addButton = document.getElementById("SearchDialog_addButton");
if (addButton) {
addButton.innerText = PlatformJS.Services.resourceLoader.getString("ConfirmButtonTitle");
}
msWriteProfilerMark("WeatherApp:LocationSuggestDetectCurrent:e");
},
function (e) {
// Handle Errors
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NumDetectCurrentLocationFailed");
Platform.Telemetry.FlightRecorder.logContentError(Platform.Telemetry.LogLevel.high, "", "Current Location detection failed", 0, "", "");
that._toggleProgressBar(false, quickSearchObj);
that._isLocationDetectionInProgress = false;
var locationTracking = WeatherAppJS.ErrorCodes.LocationTracking;
if (e && e.errorcode === locationTracking.LOCATION_DETECTION_FAILED) {
that._showMessage("LocationDetectionFailed", quickSearchObj);
} else if (e && e.errorcode === locationTracking.LOCATION_TRACKING_DISABLED) {
that._showMessage("LocationDisabledError", quickSearchObj);
} else if (e && e.errorcode === locationTracking.NO_WEATHER_DATA) {
that._showMessage("CurrentLocDataNotAvailable", quickSearchObj);
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NoWeatherDataForLocation");
Platform.Telemetry.FlightRecorder.logContentError(Platform.Telemetry.LogLevel.normal, "", "No weather data for the current location", 0, "", "");
} else if (e && e.errorcode === locationTracking.NO_INTERNET_CONNECTION) {
that._showMessage("NoInternetConnectionError", quickSearchObj);
} else {
that._showMessage("LocDetectionUnknownError", quickSearchObj);
}
msWriteProfilerMark("WeatherApp:LocationSuggestDetectCurrent:e");
}
)
.done(
null,
function (e) {
that._isLocationDetectionInProgress = false;
}
);
},
_toggleProgressBar: function _toggleProgressBar(show, quickSearchObj) {
if (quickSearchObj) {
(show) ? quickSearchObj.showSearchProgress() : quickSearchObj.hideSearchProgress();
} else {
(show) ? CommonJS.Progress.showProgress(CommonJS.Progress.headerProgressType) : CommonJS.Progress.hideProgress(CommonJS.Progress.headerProgressType);
}
CommonJS.dismissAllEdgies();
},
_showMessage: function _showMessage(msgKey, quickSearchObj) {
if (quickSearchObj) {
quickSearchObj.message = (msgKey) ? PlatformJS.Services.resourceLoader.getString(msgKey) : "";
} else {
if (msgKey) {
WeatherAppJS.Error.showMessageBar(PlatformJS.Services.resourceLoader.getString(msgKey), false);
}
else {
// remove any old message bar.
if (CommonJS.messageBar) {
CommonJS.messageBar.hide();
}
}
}
CommonJS.dismissAllEdgies();
},
/// <summary>
/// Brings up the add location flyout
/// </summary>
/// <param name="headerId">Platform header control Id</param>
/// <param name="searchAction">Search action to be performed</param>
/// <param name="onDialogHide">An optional custom onHide handler</param>
/// <param name="locationObj">An optional location object</param>
/// <param name="isFRE">An optional isFRE flag</param>
showQuickSearchDialog: function showQuickSearchDialog(headerId, searchAction, onDialogHide, locationObj, isFRE) {
var that = WeatherAppJS.Search;
CommonJS.dismissAllEdgies();
msWriteProfilerMark("WeatherApp:AddLocationFlyout:Launched");
WeatherAppJS.Search._context = "Add Location Flyout";
var eventAttr;
if (searchAction === WeatherAppJS.Search.Actions.DetectedLocationConfirm) {
WeatherAppJS.Search._context = WeatherAppJS.Instrumentation.PageContext.ConfirmLocFlyout;
eventAttr = { detectedLocation: locationObj.getFullDisplayName(), accuracy: locationObj.locDetectAccuracy };
Platform.Telemetry.FlightRecorder.logAppActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, WeatherAppJS.Instrumentation.PageContext.ConfirmLocFlyout, "", Platform.Telemetry.AppActionOperation.open, 0, JSON.stringify(eventAttr));
} else {
Platform.Telemetry.FlightRecorder.logAppAction(Platform.Telemetry.LogLevel.normal, WeatherAppJS.Search._context, "", Platform.Telemetry.AppActionOperation.open, 0);
}
var quickSearchObj = CommonJS.SearchDialog.show({
onSearchTextEntered: WeatherAppJS.Search.quickSearchSuggestionSelected,
onSearchSuggestionsRequested: WeatherAppJS.Search.quickSearchSuggestionsRequested,
onHide: onDialogHide ? onDialogHide : that.onAddLocationFlyoutHide,
promptText: PlatformJS.Services.resourceLoader.getString("EnterLocation"),
extraButtons: Platform.Utilities.Globalization.isFeatureEnabled("CurrentLocationDetection") ? [
{
icon: "iconDetectCurrentAppEx",
theme: "dark",
title: PlatformJS.Services.resourceLoader.getString("Home"),
clickHandler: function clickHandler() {
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NumDetectCurrentLocationClicked");
WeatherAppJS.Search.detectCurrentLocation(quickSearchObj, searchAction);
}
}
] : [],
autoSuggestOptions: {
selectionMode: CommonJS.AutoSuggest.freeForm,
invalidEntryString: PlatformJS.Services.resourceLoader.getString("searchboxInvalidEntry")
},
// Set the placeholder text
searchPlaceholderText: PlatformJS.Services.resourceLoader.getString("searchboxPlaceHolderTest"),
detectedLocation: locationObj,
isFRE: isFRE
});
var cancelButton = document.getElementById("SearchDialog_closeButton");
cancelButton.addEventListener('click', function (event) {
eventAttr = { queryText: quickSearchObj._searchBox.value };
if (WeatherAppJS.Search._context === WeatherAppJS.Instrumentation.PageContext.ConfirmLocFlyout) {
eventAttr = { detectedLocation: quickSearchObj.detectedLocation.getFullDisplayName(), accuracy: quickSearchObj.detectedLocation.locDetectAccuracy };
}
if (isFRE) {
var loc = WeatherAppJS.SettingsManager.getDefaultLocation();
// In case of FRE, when user clicks cancel, the data is fetched overwriting the sample data for the default Location.
if (loc) {
Platform.Telemetry.FlightRecorder.logSearch(Platform.Telemetry.LogLevel.normal, loc.getFullDisplayName(), "Bing", Platform.Telemetry.SearchMethod.view, true, -1, false, true, false, WeatherAppJS.Instrumentation.FormCodes.AppLaunch, loc.latitude + ' - ' + loc.longitude);
}
}
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, WeatherAppJS.Search._context, "Cancel", Platform.Telemetry.UserActionOperation.click, Platform.Telemetry.UserActionMethod.unknown, 0, JSON.stringify(eventAttr));
});
WeatherAppJS.Search._searchAction = searchAction;
},
getPlatformHeaderControl: function getPlatformHeaderControl(headerId) {
var platformHeader = PlatformJS.Utilities.getControl(headerId);
if (platformHeader) {
// Check if this object is a pano control with header or just the header object.
return (platformHeader.header) ? platformHeader.header : platformHeader;
}
},
/// <summary>
/// Opens up the location page depending upon the search action being performed
/// </summary>
/// <param name="locObj">Location object</param>
/// <param name="searchAction">Search action to be performed</param>
/// <param name="quickSearchObj">The search object</param>
openLocationPage: function openLocationPage(locObj, searchAction, quickSearchObj) {
var defaultLoc = WeatherAppJS.SettingsManager.getDefaultLocationId();
// There is conditional use of Actions.SetDefault and Actions.SetDefaultDisambiguationPage to overwrite the current default location (in case of sample data being loaded)
var setDefaultOverwriteFlag = (WeatherAppJS.SettingsManager.isFRE() || !defaultLoc) ? true : false;
var searchActions = WeatherAppJS.Search.Actions;
if (searchAction === searchActions.SearchLocation) {
if (defaultLoc === locObj.getId()) {
PlatformJS.Navigation.navigateToChannel('Home', { formCode: WeatherAppJS.Instrumentation.FormCodes.Search, method: Platform.Telemetry.SearchMethod.charm });
return;
}
if (WeatherAppJS.SettingsManager.getEnableSearchHistory()) {
// Add to recently searched
WeatherAppJS.SettingsManager.addRecentLocationAsync(locObj).then(null, function () { });
}
PlatformJS.Navigation.navigateToChannel('Home', { locID: locObj.getId(), geocodeLocation: locObj, searchingLocation: true, formCode: WeatherAppJS.Instrumentation.FormCodes.Search, method: Platform.Telemetry.SearchMethod.charm });
}
else if (searchAction === searchActions.AddLocation) {
// Adding location
WeatherAppJS.SettingsManager.addFavoriteAsync(locObj)
.then(
null,
function (e) {
if (e.errorcode === WeatherAppJS.ErrorCodes.SettingsManager.FAVORITES_MAX_LIMIT_REACHED) {
var msg = PlatformJS.Services.resourceLoader.getString("MaxFavoritesLimitReached");
WeatherAppJS.Error.showMessageBar(msg, false);
if (quickSearchObj) {
quickSearchObj.searchVisible = false;
}
}
})
.then(
function () {
if (quickSearchObj) {
quickSearchObj.searchVisible = false;
}
});
}
else if (searchAction === searchActions.AddLocationDisambiguationPage) {
// Adding location
WeatherAppJS.SettingsManager.addFavoriteAsync(locObj)
.then(
function () {
PlatformJS.Navigation.navigateToChannel('MyPlaces', { disambigLocation : locObj });
},
function (e) {
if (e.errorcode === WeatherAppJS.ErrorCodes.SettingsManager.FAVORITES_MAX_LIMIT_REACHED) {
var msg = PlatformJS.Services.resourceLoader.getString("MaxFavoritesLimitReached");
WeatherAppJS.Error.showMessageBar(msg, true);
} else {
PlatformJS.Navigation.navigateToChannel('MyPlaces');
}
});
}
else if (searchAction === searchActions.SetDefault) {
WeatherAppJS.SettingsManager.setDefaultLocationAsync(locObj, setDefaultOverwriteFlag).then(
function () {
if (quickSearchObj) {
quickSearchObj.searchVisible = false;
}
},
function (e) {
if (quickSearchObj) {
quickSearchObj.searchVisible = false;
}
if (e.errorcode === WeatherAppJS.ErrorCodes.SettingsManager.FAVORITES_MAX_LIMIT_REACHED) {
var msg = PlatformJS.Services.resourceLoader.getString("MaxFavoritesLimitReached");
WeatherAppJS.Error.showMessageBar(msg, true);
}
else if (e.errorcode === WeatherAppJS.ErrorCodes.SettingsManager.LOCATION_PINNED_TO_START) {
WeatherAppJS.Error.showMessageBar(PlatformJS.Services.resourceLoader.getString("LocationPinnedToStart"), false);
}
});
}
else if (searchAction === searchActions.SetDefaultDisambiguationPage) {
WeatherAppJS.SettingsManager.setDefaultLocationAsync(locObj, setDefaultOverwriteFlag).then(
function () {
PlatformJS.Navigation.navigateToChannel('Home', { method: Platform.Telemetry.SearchMethod.form, formCode: WeatherAppJS.Instrumentation.FormCodes.Search } );
},
function (e) {
if (e.errorcode === WeatherAppJS.ErrorCodes.SettingsManager.FAVORITES_MAX_LIMIT_REACHED) {
var msg = PlatformJS.Services.resourceLoader.getString("MaxFavoritesLimitReached");
WeatherAppJS.Error.showMessageBar(msg, true);
}
else if (e.errorcode === WeatherAppJS.ErrorCodes.SettingsManager.LOCATION_PINNED_TO_START) {
WeatherAppJS.Error.showMessageBar(PlatformJS.Services.resourceLoader.getString("LocationPinnedToStart"), false);
}
else {
PlatformJS.Navigation.navigateToChannel('Home');
}
});
} else if (searchAction === searchActions.DetectedLocationConfirm) {
PlatformJS.Navigation.navigateToChannel('Home', {
locID: locObj.getId(), geocodeLocation: locObj,
searchingLocation: true, method: Platform.Telemetry.SearchMethod.form, formCode: WeatherAppJS.Instrumentation.FormCodes.Search
});
}
}
});
})();
