﻿/********************************************************
*                                                        *
*   © Microsoft. All rights reserved.                    *
*                                                        *
*********************************************************/
(function () {
'use strict';
var appData = Windows.Storage.ApplicationData.current;
WinJS.Namespace.define("WeatherAppJS", {
/// <summary>
/// Settings Manager class
/// </summary>
SettingsManager: new (WinJS.Class.mix(
/// <summary>
/// Settings Manager constructor. Initialize the settings objects.
/// </summary>
function () {
var that = this; 
if (appData.version === 0) {
// Clear any saved settings from previous version
that._clearUserSettings();
// Sets the version number of the application data in the app data store.
appData.setVersionAsync(
1, /* app data version number */
function (setVersionRequest) { /* ApplicationDataSetVersionHandler */
// For v1 we need not migrate any settings
}).done();
}
/// Can't use GetMarketLocation() as the app starts to load settings even before GetMarketLocation() is completely initialized.
/// Using isFeatureEnabled("UseTianQi") throws an exception at this point of time.
this._geographicRegion = new Windows.Globalization.GeographicRegion();
// Detect user's geographic region setting changes when the app starts
this._detectRegionSwitch();
// Initialize settings store based on market
this._initSettingsStore();
// Initialize vars
this._init();
//Read in the settings
this._readSettings();
},
{
/// <summary>
/// Setting names (keys) used to refer to the setting in the store.
/// </summary>
_settings: {
favorites: "favorites",
recentLocations: "recentlocations",
defaultlocation: "defaultlocation",
currentlocation: "currentlocation",
displayunit: "displayunit",
displayunitpending: "displayunitpending",
isFRE: "isFRE",
enableSearchHistory: "enablesearchhistory",
mapsEnvironment: "mapsenvironment",
isCCMinimized: "isCCMinimized",
lastSuspendTime: "lastSuspendTime",
geographicRegionISOCode: "geographicRegionISOCode",
userDefaultRegionISOCode: "userDefaultRegionISOCode"
},
/// <summary>
/// Favorite settings object, an array of locations
/// </summary>
_favorites: null,
/// <summary>
/// Recent location settings object, an array of locations
/// </summary>
_recentLocations: null,
/// <summary>
/// Default location object, string
/// </summary>
_defaultLocation: null,
/// <summary>
/// Default location object, string
/// </summary>
_currentLocation: null,
/// <summary>
/// Display unit string
/// </summary>
_displayUnit: null,
/// <summary>
/// Display unit string (pending change)
/// </summary>
_displayUnitPending: null,
/// <summary>
/// whether this run is an FRE run, boolean
/// set to true based on PM spec
/// </summary>
_isFRE: true,
/// <summary>
/// whether current conditions are minimized
/// set to true by default
/// </summary>
_isCCMinimized: true,
/// <summary>
/// User setting for enabling/disabling recent searches history
/// </summary>
_enableSearchHistory: null,
/// <summary>
/// Time when the app was last suspended (in milliseconds since 1970/01/01)
/// </summary>
_lastSuspendTime: null,
_localSettings: null,
_roamingSettings: null,
/// <summary>
/// Default Region ISOCode for the user detected via Rev-IP
/// </summary>
_userDefaultRegionISOCode: null,
/// <summary>
/// Flag to check if the Pre-FRE call for Rev-IP user region is completed
/// </summary>
_revIPComplete: false,
/// <summary>
/// Initialized to user's geographic region on app start
/// </summary>
_geographicRegion: null,
/// <summary>
/// Whether the user's geographic region has changed from last app start
/// </summary>
_geographicRegionSwitched: null,
/// <summary>
/// Flag for data changed handler registration
/// </summary>
_dataChangedHandlerRegistered: false,
/// <summary>
/// Initialize the vars to default values
/// </summary>
_init: function _init() {
this._favorites = [];
this._recentLocations = [];
this._defaultLocation = null;
this._currentLocation = null;
this._displayUnit = null;
this._displayUnitPending = null;
this._isFRE = null;
this._enableSearchHistory = null;
this._isCCMinimized = null;
this._lastSuspendTime = null;
this._dataChangedHandlerRegistered = false;
this._userDefaultRegionISOCode = null;
},
/// <summary>
/// Initializes the settings store based on the users geographic region.
/// </summary>
_initSettingsStore: function _initSettingsStore() {
var localSettings = appData.localSettings;
var roamingSettings = appData.roamingSettings;
if (this._geographicRegion.codeTwoLetter === "CN") {
// For China market use TianQiSettings container
localSettings.createContainer("TianQiSettings", Windows.Storage.ApplicationDataCreateDisposition.Always);
if (localSettings.containers.hasKey("TianQiSettings")) {
this._localSettings = localSettings.containers.lookup("TianQiSettings");
}
roamingSettings.createContainer("TianQiSettings", Windows.Storage.ApplicationDataCreateDisposition.Always);
if (roamingSettings.containers.hasKey("TianQiSettings")) {
this._roamingSettings = roamingSettings.containers.lookup("TianQiSettings");
}
} else {
this._localSettings = localSettings;
this._roamingSettings = roamingSettings;
}
},
/// <summary>
/// Clear all local and roaming settings of the app synchronously.
/// </summary>
_clearUserSettings: function _clearUserSettings() {
var localSettings = appData.localSettings;
var roamingSettings = appData.roamingSettings;
for (var setting in this._settings) {
roamingSettings.values.remove(this._settings[setting]);
localSettings.values.remove(this._settings[setting]);
}
for (var localIndex in localSettings.containers) {
if (localSettings.containers[localIndex] instanceof Windows.Storage.ApplicationDataContainer) {
localSettings.deleteContainer(localIndex);
}
}
for (var roamingIndex in roamingSettings.containers) {
if (roamingSettings.containers[roamingIndex] instanceof Windows.Storage.ApplicationDataContainer) {
roamingSettings.deleteContainer(roamingIndex);
}
}
},
/// <summary>
/// Detect if the users geographic region has switched from last app start.
/// </summary>
_detectRegionSwitch: function _detectRegionSwitch() {
var localSettings = appData.localSettings;
var oldGeographicRegion = localSettings.values[this._settings.geographicRegionISOCode];
if (oldGeographicRegion === undefined) {
localSettings.values[this._settings.geographicRegionISOCode] = this._geographicRegion.codeTwoLetter;
} else if (oldGeographicRegion !== this._geographicRegion.codeTwoLetter) { // User's geographic region changed
// Set flag to handle this asynchronously on activation.
this._geographicRegionSwitched = true;
localSettings.values[this._settings.geographicRegionISOCode] = this._geographicRegion.codeTwoLetter;
}
},
/// <summary>
/// Invoked when the roaming data changes. Reloads the settings
/// </summary>
_dataChangedHandler: function _dataChangedHandler(evt) {
try {
// Update all roaming settings
WeatherAppJS.SettingsManager._refreshRoamingSettings();
// Update all the tiles since settings just changed
WeatherAppJS.App.updateAllTiles();
}
catch (e) {
}
},
/// <summary>
/// Read in the settings from their appropriate location (roaming or local)
/// </summary>
_readSettings: function _readSettings() {
//Read in favorites
var favorites = this._deserializeSetting(this._settings.favorites, function (e) { throw e; });
if (favorites) {
if (Object.prototype.toString.call(favorites) === '[object Array]') {
for (var f in favorites) {
var favorite = favorites[f];
if (this._isValidLocation(favorite)) {
favorite = WeatherAppJS.GeocodeLocation.createGeocodeLocation(favorite);
this._favorites.push(favorite);
}
}
}
}
//Read in recents
var recentLocations = this._deserializeSetting(this._settings.recentLocations, function (e) { throw e; });
if (recentLocations) {
if (Object.prototype.toString.call(recentLocations) === '[object Array]') {
for (var r in recentLocations) {
var recentLoc = recentLocations[r];
if (this._isValidLocation(recentLoc)) {
recentLoc = WeatherAppJS.GeocodeLocation.createGeocodeLocation(recentLoc);
this._recentLocations.push(recentLoc);
}
}
}
}
//Read in default location
var defaultLocation = this._deserializeSetting(this._settings.defaultlocation, function (e) { throw e; });
if (defaultLocation) {
defaultLocation = WeatherAppJS.GeocodeLocation.createGeocodeLocation(defaultLocation);
this._defaultLocation = defaultLocation;
}
//Read in current location
var currentLocation = this._deserializeSetting(this._settings.currentlocation, function (e) { throw e; });
if (currentLocation) {
currentLocation = WeatherAppJS.GeocodeLocation.createGeocodeLocation(currentLocation);
this._currentLocation = currentLocation;
}
//Read in display unit
this._displayUnit = this._deserializeSetting(this._settings.displayunit);
this._displayUnitPending = this._deserializeSetting(this._settings.displayunitpending);
//Read in FRE
var isFRE = this._deserializeSetting(this._settings.isFRE, function (e) { throw e; });
if (isFRE !== true && isFRE !== false) {
// if deserialize fails for some reason or returns something unexpected, we'll assume FRE
this._isFRE = true;
} else {
this._isFRE = isFRE;
}
//Read in search history
this._enableSearchHistory = this._deserializeSetting(this._settings.enableSearchHistory);
//Read maps environment
this._mapsEnvironment = this._deserializeSetting(this._settings.mapsEnvironment);
//Read in current conditions minimized state
var isCCMinimized = this._deserializeSetting(this._settings.isCCMinimized, function (e) { throw e; });
if (isCCMinimized !== true && isCCMinimized !== false) {
// if deserialize fails for some reason or returns something unexpected, we'll assume minimized state
this._isCCMinimized = true;
} else {
this._isCCMinimized = isCCMinimized;
}
//Read in last suspend time
this._lastSuspendTime = this._deserializeSetting(this._settings.lastSuspendTime);
//Read in user's default region ISO Code
var userDefaultRegionISOCode = this._deserializeSetting(this._settings.userDefaultRegionISOCode);
if (userDefaultRegionISOCode) {
this._userDefaultRegionISOCode = userDefaultRegionISOCode;
} else {
var geographicRegion = this._geographicRegion ? this._geographicRegion : new Windows.Globalization.GeographicRegion();
this._userDefaultRegionISOCode = geographicRegion.codeTwoLetter;
// Start Prefetch
this.fetchUserDefaultRegionAsync();
}
},
/// <summary>
/// Update roaming settings by reading them from the store and raise events if required.
/// This function is called on 'datachanged' event.
/// </summary>
_refreshRoamingSettings: function _refreshRoamingSettings() {
// refresh favorites
var favorites = this._deserializeSetting(this._settings.favorites, function (e) { throw e; });
if (favorites) {
if (Object.prototype.toString.call(favorites) === '[object Array]') {
var addedFavorites = [];
var allFavorites = [];
for (var f in favorites) {
if (this._isValidLocation(favorites[f])) {
allFavorites.push(favorites[f]);
if (!this._isFavorite(favorites[f])) {
addedFavorites.push(favorites[f]);
}
}
if (this._isRecentLocation(favorites[f])) {
this.removeRecentLocationAsync(favorites[f]).done();
}
}
var oldFavorites = this._favorites;
this._favorites = allFavorites;
// Dispatch events - used by geocode cache and locations manager
for (var afIndex in addedFavorites) {
this.dispatchEvent('favoriteadded', this._copyLocation(addedFavorites[afIndex]));
}
for (var ofIndex in oldFavorites) {
if (!this._isFavorite(oldFavorites[ofIndex])) {
this.dispatchEvent('favoriteremoved', this._copyLocation(oldFavorites[ofIndex]));
}
}
}
}
// refresh default location
var defaultLocation = this._deserializeSetting(this._settings.defaultlocation, function (e) { throw e; });
if (defaultLocation && this._isValidLocation(defaultLocation) && !this._isDefaultLocation(defaultLocation)) {
if (this._isRecentLocation(defaultLocation)) {
this.removeRecentLocationAsync(defaultLocation).done();
}
this._defaultLocation = defaultLocation;
this.dispatchEvent('defaultlocationupdated', this._copyLocation(defaultLocation));
}
// refresh display unit
var displayUnit = this._deserializeSetting(this._settings.displayunit);
// Set only if the value is valid and different form existing value. Raises the settingchanged event.
if (displayUnit && (displayUnit === 'C' || displayUnit === 'F') && displayUnit !== this._displayUnit) {
this.setDisplayUnitAsync(displayUnit, "").done();
}
// refresh FRE
var isFRE = this._deserializeSetting(this._settings.isFRE, function (e) { throw e; });
// Set only if the value is valid and different form existing value.
if ((isFRE === true || isFRE === false) && isFRE !== this._isFRE) {
this._isFRE = isFRE;
if (this._isFRE === false) {
this.dispatchEvent('frecompleted');
}
}
// refresh search history
var enableSearchHistory = this._deserializeSetting(this._settings.enableSearchHistory);
// Set only if the value is valid and different form existing value.
if ((enableSearchHistory === true || enableSearchHistory === false) && enableSearchHistory !== this._enableSearchHistory) {
this._enableSearchHistory = enableSearchHistory;
if (this._enableSearchHistory === false) {
this.removeAllRecentLocationsAsync().done();
}
}
// refresh current conditions minimized state
var isCCMinimized = this._deserializeSetting(this._settings.isCCMinimized, function (e) { throw e; });
// Set only if the value is valid and different form existing value.
if ((isCCMinimized === true || isCCMinimized === false) && isCCMinimized !== this._isCCMinimized) {
this._isCCMinimized = isCCMinimized;
}
},
/// <summary>
/// Settings serializer, knows where and how to store a setting.
/// </summary>
_serializeSetting: function _serializeSetting(setting, /*@optional*/errorHandler) {
if (setting === this._settings.favorites) {
this._saveSettingAsJsonString(this._settings.favorites, this._favorites, this._roamingSettings, errorHandler);
} else if (setting === this._settings.defaultlocation) {
this._saveSettingAsJsonString(this._settings.defaultlocation, this._defaultLocation, this._roamingSettings, errorHandler);
} else if (setting === this._settings.currentlocation) {
this._saveSettingAsJsonString(this._settings.currentlocation, this._currentLocation, this._localSettings, errorHandler);
} else if (setting === this._settings.displayunit) {
this._roamingSettings.values[this._settings.displayunit] = this._displayUnit;
} else if (setting === this._settings.displayunitpending) {
this._localSettings.values[this._settings.displayunitpending] = this._displayUnitPending;
} else if (setting === this._settings.recentLocations) {
this._saveSettingAsJsonString(this._settings.recentLocations, this._recentLocations, this._localSettings, errorHandler);
} else if (setting === this._settings.isFRE) {
this._roamingSettings.values[this._settings.isFRE] = this._isFRE;
} else if (setting === this._settings.enableSearchHistory) {
this._roamingSettings.values[this._settings.enableSearchHistory] = this._enableSearchHistory;
} else if (setting === this._settings.mapsEnvironment) {
this._localSettings.values[this._settings.mapsEnvironment] = this._mapsEnvironment;
} else if (setting === this._settings.isCCMinimized) {
this._roamingSettings.values[this._settings.isCCMinimized] = this._isCCMinimized;
} else if (setting === this._settings.lastSuspendTime) {
this._localSettings.values[this._settings.lastSuspendTime] = this._lastSuspendTime;
} else if (setting === this._settings.userDefaultRegionISOCode) {
this._localSettings.values[this._settings.userDefaultRegionISOCode] = this._userDefaultRegionISOCode;
}
else {
if (errorHandler) {
errorHandler({ errorcode: WeatherAppJS.ErrorCodes.SettingsManager.UNKNOWN_SETTING_ERROR, errormessage: "Unknown setting error. Unable to serialize setting '" + setting + "'" });
}
}
},
_saveSettingAsJsonString: function _saveSettingAsJsonString(setting, settingObject, settingsStore, errorHandler){
try {
settingsStore.values[setting] = JSON.stringify(settingObject);
} catch (exp) {
if (errorHandler) { errorHandler({ errorcode: WeatherAppJS.ErrorCodes.SettingsManager.PARSE_ERROR, errormessage: exp.description }); }
}
},
/// <summary>
/// Settings deserializer, knows where and how to read a saved setting.
/// </summary>
_deserializeSetting: function _deserializeSetting(setting, /*@optional*/errorHandler) {
if (setting === this._settings.favorites) {
return this._readSettingFromJsonString(this._settings.favorites, this._roamingSettings, errorHandler);
} else if (setting === this._settings.defaultlocation) {
return this._readSettingFromJsonString(this._settings.defaultlocation, this._roamingSettings, errorHandler);
} else if (setting === this._settings.currentlocation) {
return this._readSettingFromJsonString(this._settings.currentlocation, this._localSettings, errorHandler);
} else if (setting === this._settings.displayunit) {
return this._roamingSettings.values[this._settings.displayunit];
} else if (setting === this._settings.displayunitpending) {
return this._localSettings.values[this._settings.displayunitpending];
} else if (setting === this._settings.recentLocations) {
return this._readSettingFromJsonString(this._settings.recentLocations, this._localSettings, errorHandler);
} else if (setting === this._settings.isFRE) {
return this._roamingSettings.values[this._settings.isFRE];
} else if (setting === this._settings.enableSearchHistory) {
return this._roamingSettings.values[this._settings.enableSearchHistory];
} else if (setting === this._settings.mapsEnvironment) {
return this._localSettings.values[this._settings.mapsEnvironment];
} else if (setting === this._settings.isCCMinimized) {
return this._roamingSettings.values[this._settings.isCCMinimized];
} else if (setting === this._settings.lastSuspendTime) {
return this._localSettings.values[this._settings.lastSuspendTime];
} else if (setting === this._settings.userDefaultRegionISOCode) {
return this._localSettings.values[this._settings.userDefaultRegionISOCode];
}
else {
if (errorHandler) {
errorHandler({ errorcode: WeatherAppJS.ErrorCodes.SettingsManager.UNKNOWN_SETTING_ERROR, errormessage: "Unknown setting error. Unable to deserialize setting '" + setting + "'" });
}
}
},
_readSettingFromJsonString: function _readSettingFromJsonString(setting, settingsStore, errorHandler) {
var settingBlob = settingsStore.values[setting];
if (settingBlob !== undefined) {
try {
return JSON.parse(settingBlob);
} catch (exp) {
if (errorHandler) { errorHandler({ errorcode: WeatherAppJS.ErrorCodes.SettingsManager.PARSE_ERROR, errormessage: exp.description }); }
}
}
},
_isFavorite: function _isFavorite(locationObj) {
if (locationObj) {
var locID = locationObj.getId();
for (var f in this._favorites) {
if (this._favorites[f].getId() === locID) {
return f;
}
}
}
},
_isRecentLocation: function _isRecentLocation(locationObj) {
if (locationObj) {
var locID = locationObj.getId();
for (var r in this._recentLocations) {
if (this._recentLocations[r].getId() === locID) {
return r;
}
}
}
},
_isDefaultLocation: function _isDefaultLocation(locationObj) {
if (locationObj && this._defaultLocation && this._defaultLocation.getId() === locationObj.getId()) {
return true;
}
return false;
},
_isValidLocation: function _isValidLocation(locationObj) {
if (locationObj) {
try {
var newLocation = new WeatherAppJS.GeocodeLocation(locationObj);
return true;
} catch (exp) {
}
}
return false;
},
_copyLocation: function _copyLocation(locationObj) {
if (locationObj) {
try {
var newLocation = new WeatherAppJS.GeocodeLocation(locationObj);
return newLocation;
} catch (exp) {
}
}
},
isFavorite: function isFavorite(locId) {
for (var f in this._favorites) {
if (this._favorites[f].getId() === locId) {
return true;
}
}
return false;
},
isRecentLocation: function isRecentLocation(locId) {
for (var r in this._recentLocations) {
if (this._recentLocations[r].getId() === locId) {
return true;
}
}
return false;
},
/// <summary>
/// Add a locaton as a favorite, and raise the 'favoriteadded' event
/// </summary>
/// <param name="location">
///  A location object with latitude, longitude and fullName properties.
/// </param>
/// <returns>
/// A Promise object
/// </returns>
addFavoriteAsync: function addFavoriteAsync(locationObj) {
var that = this;
var MAX_FAVORITES = PlatformJS.Services.appConfig.getInt32("MaxFavoriteLimit");
return new WinJS.Promise(
function (c, e) {
// Check if the location is not already a favorite
if (that._isFavorite(locationObj) === undefined) {
// Check it's not the default location
if (!that._isDefaultLocation(locationObj)) {
// Check if the limit has not reached.
if (that._favorites.length < MAX_FAVORITES) {
// Validate location
if (that._isValidLocation(locationObj)) {
// Check if recent location
var r, favorite;
if ((r = that._isRecentLocation(locationObj)) !== undefined) {
var recent = that._recentLocations.splice(r, 1);
favorite = recent[0];
that._serializeSetting(that._settings.recentLocations, e);
// Raise seperate event here, if required
that.dispatchEvent('recentlocationremoved', recent[0]);
} else {
// new location
favorite = that._copyLocation(locationObj);
}
that._favorites.push(favorite);
that._serializeSetting(that._settings.favorites, e);
that.dispatchEvent('favoriteadded', that._copyLocation(favorite));
c();
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NumFavorites");
Platform.Telemetry.FlightRecorder.logUserAction(Platform.Telemetry.LogLevel.normal, WeatherAppJS.Instrumentation.PageContext.Favorites, "", "appFavoriteAdded", "", 0);
} else {
e({ errorcode: WeatherAppJS.ErrorCodes.SettingsManager.INVALID_LOCATION, errormessage: 'Invalid location attribute' });
}
} else {
e({ errorcode: WeatherAppJS.ErrorCodes.SettingsManager.FAVORITES_MAX_LIMIT_REACHED, errormessage: 'Max ' + MAX_FAVORITES + ' favorites allowed' });
}
} else {
e({ errorcode: WeatherAppJS.ErrorCodes.SettingsManager.LOCATION_ALREADY_A_DEFAULT, errormessage: 'Location is already the default location' });
}
} else {
e({ errorcode: WeatherAppJS.ErrorCodes.SettingsManager.FAVORITE_ALREADY_EXISTS, errormessage: 'Location already exists as a favorite' });
}
});
},
/// <summary>
/// Remove a favorite, and raise the 'favoriteremoved' event
/// </summary>
/// <param name="location">
///  A location object with latitude, longitude and fullName properties.
/// </param>
/// <returns>
/// A Promise object
/// </returns>
removeFavoriteAsync: function removeFavoriteAsync(locationObj) {
var that = this;
return new WinJS.Promise(
function (c, e) {
// Check if the location is a favorite
var f;
if ((f = that._isFavorite(locationObj)) !== undefined) {
var removed = that._favorites.splice(f, 1);
that._serializeSetting(that._settings.favorites, e);
that.dispatchEvent('favoriteremoved', removed[0]);
c();
} else {
e({ errorcode: WeatherAppJS.ErrorCodes.SettingsManager.FAVORITE_DOES_NOT_EXIST, errormessage: 'Location is not a favorite' });
}
});
},
/// <summary>
/// Remove all favorites, and raise the 'favoriteremoved' event for each favorite
/// </summary>
/// <returns>
/// A Promise object
/// </returns>
removeAllFavoritesAsync: function removeAllFavoritesAsync() {
var that = this;
return new WinJS.Promise(
function (c, e) {
var favorites = that._favorites;
that._favorites = [];
that._serializeSetting(that._settings.favorites, e);
while (favorites.length) {
var f = favorites.pop();
that.dispatchEvent('favoriteremoved', f);
}
c();
});
},
/// <summary>
/// Get a list of favorites. Passes the favorites list to the oncomplete handler.
/// </summary>
/// <returns>
/// A Promise object
/// </returns>
getFavoritesAsync: function getFavoritesAsync() {
var that = this;
return new WinJS.Promise(
function (c, e) {
try {
// make a copy of the favorites
var favorites = [];
for (var f in that._favorites) {
var favorite = that._favorites[f];
favorite = WeatherAppJS.GeocodeLocation.createGeocodeLocation(favorite);
favorites.push(favorite);
}
c(favorites);
} catch (exp) {
e({ errorcode: WeatherAppJS.ErrorCodes.SettingsManager.PARSE_ERROR, errormessage: exp.description });
}
});
},
/// <summary>
/// Add a locaton as a recentlocation, and raise the 'recentlocationadded' event if the location is new. 
/// 'recentlocationupdated' event if it is already a recent location.
/// </summary>
/// <param name="location">
///  A location object with latitude, longitude and fullName properties.
/// </param>
/// <returns>
/// A Promise object
/// </returns>
addRecentLocationAsync: function addRecentLocationAsync(locationObj) {
var that = this;
var MAX_RECENT_LOCATIONS = PlatformJS.Services.appConfig.getInt32("MaxRecentLocationsLimit");
return new WinJS.Promise(
function (c, e) {
if (that.getEnableSearchHistory()) {
// Validate location
if (that._isValidLocation(locationObj)) {
// Check if the location is not already a favorite
if (that._isFavorite(locationObj) === undefined) {
// Check it's not the default location
if (!that._isDefaultLocation(locationObj)) {
var r;
if ((r = that._isRecentLocation(locationObj)) !== undefined) {
// already a recent location, bring to front if not already at front
if (r !== 0) {
var recent = that._recentLocations.splice(r, 1);
that._recentLocations.unshift(recent[0]);
that._serializeSetting(that._settings.recentLocations, e);
}
that.dispatchEvent('recentlocationupdated', { location: that._copyLocation(recent[0]), oldPosition: r, newPosition: 0 });
c();
} else {
// add new recent location
var oldRecents;
var oldRecent = that._copyLocation(locationObj);
that._recentLocations.unshift(oldRecent);
if (that._recentLocations.length > MAX_RECENT_LOCATIONS) {
oldRecents = that._recentLocations.splice(MAX_RECENT_LOCATIONS, that._recentLocations.length - MAX_RECENT_LOCATIONS);
}
that._serializeSetting(that._settings.recentLocations, e);
that.dispatchEvent('recentlocationadded',locationObj);
if (oldRecents) {
for (var o in oldRecents) {
that.dispatchEvent('recentlocationremoved', oldRecents[o]);
}
}
c();
Platform.Telemetry.FlightRecorder.logAppAction(Platform.Telemetry.LogLevel.normal, WeatherAppJS.Instrumentation.PageContext.Recent, "", "appRecentlySearchedLocAdded", "", 0);
}
} else {
e({ errorcode: WeatherAppJS.ErrorCodes.SettingsManager.LOCATION_ALREADY_A_DEFAULT, errormessage: 'Location is already the default location' });
}
} else {
e({ errorcode: WeatherAppJS.ErrorCodes.SettingsManager.LOCATION_ALREADY_A_FAVORITE, errormessage: 'Location is already a favorite' });
}
} else {
e({ errorcode: WeatherAppJS.ErrorCodes.SettingsManager.INVALID_LOCATION, errormessage: 'Invalid location attribute' });
}
} else {
c();
}
});
},
/// <summary>
/// Remove a recent location, and raise the 'recentlocationremoved' event
/// </summary>
/// <param name="locationObj">
///  A location object with latitude, longitude and fullName properties.
/// </param>
/// <returns>
/// A Promise object
/// </returns>
removeRecentLocationAsync: function removeRecentLocationAsync(locationObj) {
var that = this;
return new WinJS.Promise(
function (c, e) {
// Check if the location is a recent location
var r;
if ((r = that._isRecentLocation(locationObj)) !== undefined) {
var removed = that._recentLocations.splice(r, 1);
that._serializeSetting(that._settings.recentLocations, e);
that.dispatchEvent('recentlocationremoved', removed[0]);
c();
} else {
e({ errorcode: WeatherAppJS.ErrorCodes.SettingsManager.NOT_A_RECENT_LOCATION, errormessage: 'Location is not a recent location' });
}
});
},
/// <summary>
/// Remove all recent locations, and raise the 'recentlocationscleared'
/// </summary>
/// <returns>
/// A Promise object
/// </returns>
removeAllRecentLocationsAsync: function removeAllRecentLocationsAsync() {
var that = this;
return new WinJS.Promise(
function (c, e) {
var recentLocations = that._recentLocations;
that._recentLocations = [];
that._serializeSetting(that._settings.recentLocations, e);
that.dispatchEvent('recentlocationscleared', recentLocations);
c();
});
},
/// <summary>
/// Get a list of recent locations. Passes the recent locations list to the oncomplete handler.
/// </summary>
/// <returns>
/// A Promise object
/// </returns>
getRecentLocationsAsync: function getRecentLocationsAsync() {
var that = this;
return new WinJS.Promise(
function (c, e) {
try {
// make a copy of the recent Locations
var recentLocations = [];
for (var r in that._recentLocations) {
var recentLoc = that._recentLocations[r];
recentLoc = WeatherAppJS.GeocodeLocation.createGeocodeLocation(recentLoc);
recentLocations.push(recentLoc);
}
c(recentLocations);
} catch (exp) {
e({ errorcode: WeatherAppJS.ErrorCodes.SettingsManager.PARSE_ERROR, errormessage: exp.description });
}
});
},
/// <summary>
/// Returns the default location.
/// </summary>
/// <returns>
/// Default location string
/// </returns>
getDefaultLocation: function getDefaultLocation() {
return this._defaultLocation;
},
/// <summary>
/// Returns the id of default location.
/// </summary>
/// <returns>
/// Default location
/// </returns>
getDefaultLocationId: function getDefaultLocationId() {
var defaultLoc;
if (this._defaultLocation) {
defaultLoc = this._defaultLocation.getId();
}
return defaultLoc;
},
/// <summary>
/// Returns the current location.
/// </summary>
/// <returns>
/// Current location
/// </returns>
getCurrentLocation: function getCurrentLocation() {
return this._currentLocation;
},
/// <summary>
/// Clears the current location.
/// </summary>
clearCurrentLocation: function clearCurrentLocation() {
this._currentLocation = null;
this._serializeSetting(this._settings.currentlocation);
},
/// <summary>
/// Function to check whether max limit of favorite locations is reached
/// </summary>
/// <returns>
/// returns true if max limit is reached
/// </returns>
isFavoritesMaxLimitReached: function isFavoritesMaxLimitReached() {
return (this._favorites.length >= PlatformJS.Services.appConfig.getInt32("MaxFavoriteLimit"));
},
/// <summary>
/// Sets the param 'location' as the current location, 
/// </summary>
/// <returns>
/// A promise object
/// </returns>
setCurrentLocationAsync: function setCurrentLocationAsync(locationObj) {
var that = this;
return new WinJS.Promise(
function (c, e) {
if (that._isValidLocation(locationObj)) {
that._currentLocation = locationObj;
that._serializeSetting(that._settings.currentlocation, e);
c(locationObj);
}
else {
that.clearCurrentLocation();
}
});
},
/// <summary>
/// Sets the param 'location' as the default location, and adds the old default as a favorite. 
/// If location is not a favorite/recent and 'overwrite' is true, then the old default location is discarded.
/// Raises 'defaultlocationupdated' and 'favoriteadded' events.
/// </summary>
/// <returns>
/// A promise object
/// </returns>
setDefaultLocationAsync: function setDefaultLocationAsync(locationObj, overwrite) {
var that = this;
overwrite = (overwrite) ? true : false;
var MAX_FAVORITES = PlatformJS.Services.appConfig.getInt32("MaxFavoriteLimit");
return new WinJS.Promise(
function (c, e) {
if (that._isValidLocation(locationObj)) {
if (!WeatherAppJS.App.isLocationPinned(locationObj.getId())) {
if (that._isFavorite(locationObj) !== undefined) {
that.setFavoriteAsDefaultLocationAsync(locationObj).then(c, e);
} else if (that._isRecentLocation(locationObj) !== undefined) {
that.setRecentAsDefaultLocationAsync(locationObj).then(c, e);
} else if (!that._isDefaultLocation(locationObj)) {
var oldDefault = that.getDefaultLocation();
if (!overwrite) {
// Check if the favorites limit has not reached.
if (that._favorites.length < MAX_FAVORITES) {
that._favorites.push(oldDefault);
that._serializeSetting(that._settings.favorites, e);
var newFav = that._copyLocation(oldDefault);
that.dispatchEvent('favoriteadded', newFav);
} else {
e({ errorcode: WeatherAppJS.ErrorCodes.SettingsManager.FAVORITES_MAX_LIMIT_REACHED, errormessage: 'Max ' + MAX_FAVORITES + ' favorites limit reached.' });
return;
}
}
var newDefault = that._copyLocation(locationObj);
that._defaultLocation = newDefault;
that._serializeSetting(that._settings.defaultlocation, e);
that.dispatchEvent('defaultlocationupdated', locationObj);
c();
} else {
c(); // already a default
}
} else {
e({ errorcode: WeatherAppJS.ErrorCodes.SettingsManager.LOCATION_PINNED_TO_START, errormessage: 'Cannot set pinned location as default' });
}
} else {
e({ errorcode: WeatherAppJS.ErrorCodes.SettingsManager.INVALID_LOCATION, errormessage: 'Invalid default location' });
}
});
},
setFavoriteAsDefaultLocationAsync: function setFavoriteAsDefaultLocationAsync(favorite) {
var that = this;
return new WinJS.Promise(
function (c, e) {
// Validate location
if (that._isValidLocation(favorite)) {
if (!WeatherAppJS.App.isLocationPinned(favorite.getId())) {
// Check if the location is a favorite
var f;
if ((f = that._isFavorite(favorite)) !== undefined) {
var newDefault = that._favorites[f];
var oldDefault = that.getDefaultLocation();
that._favorites[f] = oldDefault;
that._defaultLocation = newDefault;
that._serializeSetting(that._settings.favorites, e);
that._serializeSetting(that._settings.defaultlocation, e);
var newFav = that._copyLocation(that._favorites[f]);
var newDefaultLocation = that._copyLocation(that.getDefaultLocation());
that.dispatchEvent('favoriteremoved', newDefaultLocation);
that.dispatchEvent('defaultlocationupdated', newDefaultLocation);
that.dispatchEvent('favoriteadded', newFav);
c(newDefaultLocation, newFav);
} else {
e({ errorcode: WeatherAppJS.ErrorCodes.SettingsManager.FAVORITE_DOES_NOT_EXIST, errormessage: 'Location is not a favorite' });
}
} else {
e({ errorcode: WeatherAppJS.ErrorCodes.SettingsManager.LOCATION_PINNED_TO_START, errormessage: 'Cannot set pinned location as default' });
}
} else {
e({ errorcode: WeatherAppJS.ErrorCodes.SettingsManager.INVALID_LOCATION, errormessage: 'Invalid location' });
}
});
},
ReorderFavorites: function ReorderFavorites(reorderedFavorites) {
var that = this;
return new WinJS.Promise(
function (c, e) {
that._favorites = [];
for (var locID in reorderedFavorites) {
var reorderedFav = WeatherAppJS.GeocodeCache.getLocation(reorderedFavorites[locID]);
var fav = that._copyLocation(reorderedFav);
that._favorites.push(fav);
}
that._serializeSetting(that._settings.favorites, e);
c();
});
},
setRecentAsDefaultLocationAsync: function setRecentAsDefaultLocationAsync(recent) {
var that = this;
var MAX_FAVORITES = PlatformJS.Services.appConfig.getInt32("MaxFavoriteLimit");
return new WinJS.Promise(
function (c, e) {
if (that.getEnableSearchHistory()) {
// Validate location
if (that._isValidLocation(recent)) {
if (!WeatherAppJS.App.isLocationPinned(recent.getId())) {
// Check if the favorites limit has not reached.
if (that._favorites.length < MAX_FAVORITES) {
// Check if the location is a recent location
var f;
if ((f = that._isRecentLocation(recent)) !== undefined) {
var newDefault = that._recentLocations[f];
var oldDefault = that.getDefaultLocation();
that._recentLocations.splice(f, 1);
// Validate if some old default already existed. One such scenario could be when a city has been searched through search charm
// before a default location has been set in FRE experience, and then user goes on to add same location as default through the 
// add location flyout shown in FRE on home page.
if (that._isValidLocation(oldDefault)) {
that._favorites.push(oldDefault);
}
that._defaultLocation = newDefault;
that._serializeSetting(that._settings.recentLocations, e);
that._serializeSetting(that._settings.defaultlocation, e);
that._serializeSetting(that._settings.favorites, e);
var newDefaultLocation = that._copyLocation(that.getDefaultLocation());
that.dispatchEvent('recentlocationremoved', newDefaultLocation);
that.dispatchEvent('defaultlocationupdated', newDefaultLocation);
if (that._isValidLocation(oldDefault)) {
var newFav = that._copyLocation(oldDefault);
that.dispatchEvent('favoriteadded', newFav);
};
c(newDefaultLocation, newFav);
} else {
e({ errorcode: WeatherAppJS.ErrorCodes.SettingsManager.NOT_A_RECENT_LOCATION, errormessage: 'Location is not a recent location' });
}
} else {
e({ errorcode: WeatherAppJS.ErrorCodes.SettingsManager.FAVORITES_MAX_LIMIT_REACHED, errormessage: 'Max ' + MAX_FAVORITES + ' favorites limit reached.' });
}
} else {
e({ errorcode: WeatherAppJS.ErrorCodes.SettingsManager.LOCATION_PINNED_TO_START, errormessage: 'Cannot set pinned location as default' });
}
} else {
e({ errorcode: WeatherAppJS.ErrorCodes.SettingsManager.INVALID_LOCATION, errormessage: 'Invalid location' });
}
} else {
e({ errorcode: WeatherAppJS.ErrorCodes.SettingsManager.SEARCH_HISTORY_DISABLED, errormessage: 'Cannot set recent location as default because search history is disabled.' });
}
});
},
ReorderRecent: function ReorderRecent(reorderedRecent) {
var that = this;
return new WinJS.Promise(
function (c, e) { 
that._recentLocations = [];
reorderedRecent.reverse();
for (var locID in reorderedRecent) {
var reorderRecent = WeatherAppJS.GeocodeCache.getLocation(reorderedRecent[locID]);
var oldRecent = that._copyLocation(reorderRecent);
that._recentLocations.unshift(oldRecent);
}
that._serializeSetting(that._settings.recentLocations, e);
c();
});
},
/// <summary>
/// Sets the display unit for the app and raise the settings changed event
/// </summary>
/// <returns>
/// A promise object
/// </returns>
setDisplayUnitAsync: function setDisplayUnitAsync(newUnit, context) {
var that = this;
return new WinJS.Promise(
function (c, e) {
if (newUnit === 'F' || newUnit === 'C') {
if (that.getDisplayUnit() !== newUnit) {
if ((WeatherAppJS.Networking.internetAvailable === true) || (that._isFRE || !that._defaultLocation)) {
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NumTimesDisplayUnitSettingChanged");
if (context !== "") {
var eventAttr = { change: newUnit === 'C' ? "F to C" : "C to F" };
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, context, "Change Unit", Platform.Telemetry.UserActionOperation.click, Platform.Telemetry.UserActionMethod.unknown, 0, JSON.stringify(eventAttr));
}
that._displayUnit = newUnit;
that._displayUnitPending = null;
WeatherAppJS.Networking.removeEventListener('networkonline', that.onHandlePendingDisplayUnitChange);
that._serializeSetting(that._settings.displayunit, e);
that._serializeSetting(that._settings.displayunitpending, e);
var eventObj = {};
eventObj.setting = "unitChange";
eventObj.unit = that._displayUnit;
that.dispatchEvent('settingchanged', eventObj);
} else {
if (!that._displayUnitPending || that._displayUnitPending !== newUnit) {
that._displayUnitPending = newUnit;
that._serializeSetting(that._settings.displayunitpending, e);
WeatherAppJS.Networking.addEventListener('networkonline', that.onHandlePendingDisplayUnitChange, false);
}
WeatherAppJS.Error.showMessageBar(PlatformJS.Services.resourceLoader.getString("NoInternetOnSettingChange"), false);
}
} else {
that._displayUnitPending = null;
that._serializeSetting(that._settings.displayunitpending, e);
WeatherAppJS.Networking.removeEventListener('networkonline', that.onHandlePendingDisplayUnitChange);
}
c();
} else {
e();
}
});
},
/// <summary>
/// Returns the display unit.
/// </summary>
/// <returns>
/// Display Unit
/// </returns>
getDisplayUnit: function getDisplayUnit() {
if (this._displayUnit !== 'F' && this._displayUnit !== 'C') {
var defaultValue = 'C';
try {
defaultValue = WeatherAppJS.Services.Utilities.isRegionMetric(this._userDefaultRegionISOCode) ? 'C' : 'F';
} catch (err) {
}
this._displayUnit = defaultValue;
}
return this._displayUnit;
},
/// <summary>
/// Returns _isFRE used to determine whether this is FRE or not
/// </summary>
/// <returns>
/// A bool
/// </returns>
isFRE: function isFRE() {
return this._isFRE;
},
/// <summary>
/// sets 
/// </summary>
/// <returns>
/// A bool
/// </returns>
setIsFREAsync: function setIsFREAsync(updatedIsFRE) {
var that = this;
return new WinJS.Promise(
function (c, e) {
if (updatedIsFRE === true || updatedIsFRE === false) { 
if (that._isFRE !== updatedIsFRE) {
that._isFRE = updatedIsFRE;
that._serializeSetting(that._settings.isFRE, e);
if (that._isFRE === false) {
that.dispatchEvent('frecompleted');
}
}
c();
} else {
e();
}
});
},
getEnableSearchHistory: function getEnableSearchHistory() {
if (this._enableSearchHistory !== true && this._enableSearchHistory !== false) {
var defaultValue = PlatformJS.Services.appConfig.getBool('EnableSearchHistoryDefault');
this._enableSearchHistory = defaultValue;
}
return this._enableSearchHistory;
},
setEnableSearchHistory: function setEnableSearchHistory(enableFlag) {
var that = this;
return new WinJS.Promise(
function (c, e) {
if (enableFlag === true) {
that._enableSearchHistory = true;
that._serializeSetting(that._settings.enableSearchHistory, e);
c();
} else if (enableFlag === false) {
that.removeAllRecentLocationsAsync().then(function () {
that._enableSearchHistory = false;
that._serializeSetting(that._settings.enableSearchHistory, e);
c();
}, e);
} else {
e();
}
});
},
setMapsEnvironment: function setMapsEnvironment(value) {
var that = this;
return new WinJS.Promise(function (c, e) {
that._mapsEnvironment = value;
that._serializeSetting(that._settings.mapsEnvironment, e);
that.dispatchEvent("mapsenvchanged", null);
c();
});
},
getMapsEnvironment: function getMapsEnvironment(value) {
var that = this;
if (!this._mapsEnvironment || !PlatformJS.Services.appConfig.getDictionary("MapEnvironments").hasKey(this._mapsEnvironment)) {
this._mapsEnvironment = PlatformJS.Services.appConfig.getString("DefaultMapsEnv");
}
return that._mapsEnvironment;
},
/// <summary>
/// Returns _isCCMinimized used to determine current conditions are maximized or not
/// </summary>
/// <returns>
/// A bool
/// </returns>
isCCMinimized: function isCCMinimized() {
return this._isCCMinimized;
},
/// <summary>
/// sets isCCMinimized
/// </summary>
/// <returns>
/// A bool
/// </returns>
setIsCCMinimizedAsync: function setIsCCMinimizedAsync(updatedIsCCMinimized) {
var that = this;
return new WinJS.Promise(
function (c, e) {
if ((updatedIsCCMinimized !== undefined) && (that._isCCMinimized !== updatedIsCCMinimized)) {
that._isCCMinimized = updatedIsCCMinimized;
that._serializeSetting(that._settings.isCCMinimized, e);
}
c();
});
},
/// <summary>
/// Returns the time the app was last suspended (in milliseconds since 1970/01/01)
/// </summary>
/// <returns>
/// Last suspend time
/// </returns>
getLastSuspendTime: function getLastSuspendTime() {
return this._lastSuspendTime;
},
/// <summary>
/// sets lastSuspendTime (in milliseconds since 1970/01/01)
/// </summary>
/// <returns>
/// A promise object
/// </returns>
setLastSuspendTimeAsync: function setLastSuspendTimeAsync(lastSuspendTime) {
var that = this;
return new WinJS.Promise(
function (c, e) {
that._lastSuspendTime = lastSuspendTime;
that._serializeSetting(that._settings.lastSuspendTime, e);
c();
});
},
onHandlePendingDisplayUnitChange: function onHandlePendingDisplayUnitChange() {
var that = WeatherAppJS.SettingsManager;
if (that._displayUnitPending) {
that.setDisplayUnitAsync(that._displayUnitPending, "").then();
}
},
getUserDefaultRegionISOCode: function getUserDefaultRegionISOCode() {
return this._userDefaultRegionISOCode;
},
setUserDefaultRegionISOCodeAsync: function setUserDefaultRegionISOCodeAsync(isoCode) {
var that = this;
return new WinJS.Promise(
function (c) {
if (isoCode && WeatherAppJS.Services.Utilities.isValidCountryCode(isoCode)) {
that._userDefaultRegionISOCode = isoCode.toUpperCase();
that._serializeSetting(that._settings.userDefaultRegionISOCode);
c();
}
});
},
// Fetch and Update User's default region based on Rev-IP call
// The Rev-IP call should not be made for China Market
fetchUserDefaultRegionAsync: function fetchUserDefaultRegionAsync() {
var that = this;
if (!that._revIPComplete) {
var geographicRegion = that._geographicRegion ? that._geographicRegion : new Windows.Globalization.GeographicRegion;
if (geographicRegion.codeTwoLetter !== "CN") {
Platform.Globalization.Marketization.getLocationFromServerAsync()
.then(function (isoCode) {
return that.setUserDefaultRegionISOCodeAsync(isoCode);
}).done(function () {
that._revIPComplete = true;
}, function (err) {
});
}
}
},
// This function is invoked once Pre-FRE for checking if the Rev-IP call is complete or else wait on it for some time
// The call to this happens before the first call to getDisplayUnit() so that if the user default region changes, the change would be reflected in the unit as well
preFREUserDefaultRegionFetchTimeoutAsync: function preFREUserDefaultRegionFetchTimeoutAsync() {
var that = this;
if (that._revIPComplete) {
return WinJS.Promise.wrap(true);
}
var timeoutPeriod = PlatformJS.Services.appConfig.getInt32("PreFRERevIPTimeout", 0);
return WinJS.Promise.timeout(timeoutPeriod)
.then(function () {
return WinJS.Promise.wrap(true);
}, function (err) {
return WinJS.Promise.wrap(true);
});
},
/// <summary>
/// Performs any deferred initialization on activation
/// </summary>
onActivated: function onActivated() {
if (this._displayUnitPending) {
if (WeatherAppJS.Networking.internetAvailable === true) {
this.setDisplayUnitAsync(this._displayUnitPending, "").then();
} else {
WeatherAppJS.Networking.addEventListener('networkonline', this.onHandlePendingDisplayUnitChange, false);
}
}
if (this._geographicRegionSwitched) {
// Handle user geographic region switching
try {
// To provide correct data according to user's region, we need to clear all tiles. 
// These will be intialized again by the app.js
WeatherAppJS.App.clearAllTiles();                        
} catch (ex) {
}
this._geographicRegionSwitched = false;
}
if (!this._dataChangedHandlerRegistered) {
//Setup the event handler for when the roaming settings change
appData.addEventListener("datachanged", this._dataChangedHandler);
this._dataChangedHandlerRegistered = true;
}
}
},
WinJS.Utilities.eventMixin,
WinJS.Utilities.createEventProperties(
'favoriteadded', 
'favoriteremoved', 
'recentlocationadded',
'recentlocationremoved',
'recentlocationupdated',
'recentlocationscleared',
'defaultlocationupdated',
'frecompleted',
'settingchanged',
'mapsenvchanged')
))
});
})();
