﻿/********************************************************
*                                                        *
*   © Microsoft. All rights reserved.                    *
*                                                        *
*********************************************************/
(function () {
'use strict';
/// <summary>
/// Weather App Settings UI namespace.
/// </summary>
WinJS.Namespace.define("WeatherAppJS.UI.Settings", {
/// <summary>
/// Property that stores the app version
/// </summary>
_appVersion: "",
/// <summary>
/// Initializes the display unit setting on the settings flyout before the flyout shows up
/// </summary>
_initDisplayUnitSettings: function _initDisplayUnitSettings() {
var useTianQi = Platform.Utilities.Globalization.isFeatureEnabled("UseTianQi");
if (useTianQi) {
var unitChangeElement = document.querySelector('#displayUnit');
if (unitChangeElement) {
unitChangeElement.style.display = "none";
}
} else {
var displayUnit = WeatherAppJS.SettingsManager.getDisplayUnit();
var cunit = document.getElementById('cunit');
var funit = document.getElementById('funit');
if (displayUnit === "C") {
funit.checked = "false";
cunit.checked = "true";
}
else {
cunit.checked = "false";
funit.checked = "true";
}
}
},
/// <summary>
/// Initializes the 'Enable Search History' setting on the settings flyout before the flyout shows up
/// </summary>
_initEnableSearchHistorySetting: function _initEnableSearchHistorySetting(settingsDiv) {
var enableSearchHistory = WeatherAppJS.SettingsManager.getEnableSearchHistory();
var searchHistoryToggleSwitchControl = settingsDiv.querySelector('#enableSearchHistory').winControl;
searchHistoryToggleSwitchControl.checked = enableSearchHistory;
},
/// <summary>
/// Initializes the 'Maps environment' setting on the settings flyout before the flyout shows up
/// </summary>
_initMapsEnvironmentList: function _initMapsEnvironmentList(settingsDiv) {
var mapsEnvListElem = document.getElementById('mapsEnvironmentList');
var mapEnvs = WeatherAppJS.UI.Settings.tryGetMapsEnvironmentsList();
var currentEnv = WeatherAppJS.SettingsManager.getMapsEnvironment();
if (mapsEnvListElem && mapEnvs && mapEnvs.length > 0) {
var envLength = mapEnvs.length;
for (var j = 0; j < envLength; j++) {
var envOptionElem = document.createElement("option");
envOptionElem.value = j;
envOptionElem.innerText = mapEnvs[j];
if (mapEnvs[j] === currentEnv) {
envOptionElem.selected = true;
}
mapsEnvListElem.appendChild(envOptionElem);
}
}
},
tryGetMapsEnvironmentsList: function tryGetMapsEnvironmentsList() {
return WeatherAppJS.UI.Settings.getDictionaryKeys("MapEnvironments");
},
getDictionaryKeys: function getDictionaryKeys(dictionaryName)
{
var keys = [];
if (!dictionaryName) {
return keys;
}
var dictionary = PlatformJS.Services.appConfig.getDictionary(dictionaryName);
if (!dictionary) {
return keys;
}
var dictKeys = Object.keys(dictionary);
for (var key in dictKeys) {
keys.push(dictKeys[key]);
}
return keys;
},
/// <summary>
/// Readies the app settings flyout before showing
/// </summary>
onBeforeShowSettingsCmd: function onBeforeShowSettingsCmd(event) {
if (event && event.target) {
var settingsDiv = event.target;
WeatherAppJS.UI.Settings._initEnableSearchHistorySetting(settingsDiv);
WeatherAppJS.UI.Settings._initDisplayUnitSettings();
WeatherAppJS.UI.Settings._initMapsEnvironmentList();
WinJS.UI.processAll(settingsDiv);
WinJS.Resources.processAll(settingsDiv);
}
},
/// <summary>
/// Readies the about flyout before showing
/// </summary>
onBeforeShowAboutCmd: function onBeforeShowAboutCmd(event) {
if (event && event.target) {
var aboutDiv = event.target;
var displayTianQiAttrib = "none";
var useTianQi = Platform.Utilities.Globalization.isFeatureEnabled("UseTianQi");
if (useTianQi) {
displayTianQiAttrib = "";
}
var tianQiAttributionDivs = aboutDiv.querySelectorAll(".tianqiAttribution");
for (var i = 0; i < tianQiAttributionDivs.length; i++) {
tianQiAttributionDivs[i].style.display = displayTianQiAttrib;
}
WeatherAppJS.UI.Settings.addMarketSpecificProvider(aboutDiv.querySelector("#providerAttribution"));
var weatherDotComUrl = aboutDiv.querySelector("#weatherDotComUrl");
weatherDotComUrl.innerText = WeatherAppJS.Utilities.Common.getWeatherDotComBaseUrl();
weatherDotComUrl.setAttribute("href", WeatherAppJS.Utilities.Common.getWeatherDotComHomeUrl());
if (!PlatformJS.Services.appConfig.getBool("ShowWeatherTrendsAttribution")) {
var weatherTrendsContainer = aboutDiv.querySelector('.weatherTrendsContainer');
WinJS.Utilities.addClass(weatherTrendsContainer, "hideWeatherTrendsContainer");
}
var imageCreditdiv = aboutDiv.querySelector('.imageCreditsContainer');
WeatherAppJS.UI.Settings.displayImageCredits(imageCreditdiv);
WeatherAppJS.UI.Settings.setAppVersion();
WinJS.UI.processAll(aboutDiv);
WinJS.Resources.processAll(aboutDiv);
}
},
/// <summary>
/// Adds the provider attribution for single provider markets
/// </summary>
addMarketSpecificProvider: function addMarketSpecificProvider(aboutNode) {
var appConfig = PlatformJS.Services.appConfig;
var resourceLoader = PlatformJS.Services.resourceLoader;
var singleProviderOnly = appConfig.getBool("SingleProviderOnly");
var bindingData = null;
var providerNode = aboutNode.querySelector(".localProvider");
if (singleProviderOnly && !providerNode) {
var providerData = appConfig.getDictionary("LocalProvider");
// if there is no local provider for a given market, just return
if (providerData && providerData.size === 0) {
return;
}
var templateId = null;
var header = providerData.getString("Header");
var cssClass = providerData.getString("CssClass");
var providerName = providerData.getString("Name");
var logo = providerData.getString("Logo");
if (!logo) {
templateId = "providerTemplate";
bindingData = {
attributionHeading: resourceLoader.getString(header),
providerName: resourceLoader.getString(providerName)
};
}
else {
templateId = "providerTemplateLink";
bindingData = {
attributionHeading: resourceLoader.getString(header),
providerUrl: resourceLoader.getString(providerName + "Url"),
providerLogo: logo
};
}
CommonJS.loadModule({ fragmentPath: "/html/secondaryTemplates.html", templateId: templateId }, bindingData, null, null, null).then(function (providerTemplate) {
if (cssClass) {
WinJS.Utilities.addClass(providerTemplate, cssClass);
}
aboutNode.appendChild(providerTemplate);
});
}
},
/// <summary>
/// Pushes the setting commands specific to app into the application commands vector.
/// </summary>
onCommandsRequested: function onCommandsRequested(e) {
var appSettings = Windows.UI.ApplicationSettings;
var vector = e.request.applicationCommands;
//Add the default buttons for all apps
CommonJS.Settings.addCommonButtons(vector);
},
/// <summary>
/// Updates the display unit
/// <param>
/// unit - display unit as set in the settings flyout
/// </param>
/// </summary>
onDisplayUnitSet: function onDisplayUnitSet(unit) {
WeatherAppJS.SettingsManager.setDisplayUnitAsync(unit, Platform.Telemetry.ActionContext.charms).then(
function () { },
function (e) {
//TODO: Set Display unit failed. Handle error.
}
);
},
/// <summary>
/// Clears the recently searched locations history
/// </summary>
onClearSearchHistory: function onClearSearchHistory() {
Platform.Telemetry.FlightRecorder.logUserAction(Platform.Telemetry.LogLevel.normal, Platform.Telemetry.ActionContext.charms, "Clear History Setting", Platform.Telemetry.UserActionOperation.select, Platform.Telemetry.UserActionMethod.unknown, 0);
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NumTimesClearHistorySettingChanged");
WeatherAppJS.SettingsManager.removeAllRecentLocationsAsync().then();
},
/// <summary>
/// Enable/Disable the recently searched locations history
/// </summary>
onEnableSearchHistory: function onEnableSearchHistory(event) {
if (event) {
var toggleControl = PlatformJS.Utilities.getControl(event.currentTarget);
if (toggleControl && (toggleControl.checked === true || toggleControl.checked === false)) {
WeatherAppJS.Utilities.Instrumentation.incrementInt32("EnableSearchHistorySettingChanged");
var eventAttr = { change: toggleControl.checked ? "ON" : "OFF" };
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, Platform.Telemetry.ActionContext.charms, "Search History Setting", Platform.Telemetry.UserActionOperation.select, Platform.Telemetry.UserActionMethod.unknown, 0,JSON.stringify(eventAttr));
WeatherAppJS.SettingsManager.setEnableSearchHistory(toggleControl.checked).then(
function () { },
function () {
event.preventDefault();
});
} else {
event.preventDefault();
}
// toggle clear history button states and styles
var clearHistoryButton = document.querySelector('#searchHistory .buttonContainer');
if (!toggleControl.checked) {
clearHistoryButton.disabled = "disabled"; // sets the disabled attribute which pushes the button into the disabled state
WinJS.Utilities.addClass(clearHistoryButton, "disabled");
} else {
clearHistoryButton.disabled = "";
WinJS.Utilities.removeClass(clearHistoryButton, "disabled");
}
}
},
onMapsEnvironmentChange: WinJS.UI.eventHandler(function (value) {
var mapEnvs = WeatherAppJS.UI.Settings.tryGetMapsEnvironmentsList();
if (mapEnvs && mapEnvs[value]) {
WeatherAppJS.SettingsManager.setMapsEnvironment(mapEnvs[value]);
PlatformJS.Navigation.navigateToChannel("Home");
}
}),
/// <summary>
/// Get app version once and store it
/// </summary>
getAppVersion: function getAppVersion() {
var that = WeatherAppJS.UI.Settings;
if (!that._appVersion) {
var version = Windows.ApplicationModel.Package.current.id.version;
var versionString = PlatformJS.Services.resourceLoader.getString("/platform/appVersion").format(version.major + "." + version.minor + "." + version.build + "." + version.revision);
that._appVersion = versionString;
}
return that._appVersion;
},
/// <summary>
/// Sets the app version string in the settings UX
/// </summary>
setAppVersion: function setAppVersion() {
var appVersion = document.getElementById('appVersion');
if (appVersion) {
appVersion.innerText = WeatherAppJS.UI.Settings.getAppVersion();
}
},
getImageCredit: function getImageCredit(formatStringKey, attributionValue) {
var credit = document.createElement('div');
credit.innerText = PlatformJS.Services.resourceLoader.getString(formatStringKey).format(attributionValue);
return credit;
},
/// <summary>
/// Shows the image credits ux
/// </summary>
displayImageCredits: function displayImageCredits(imageCreditsPage) {
var domContentDiv = imageCreditsPage.querySelector(".imageAttribution");
var hasChildNodes = domContentDiv.hasChildNodes();
if (!hasChildNodes) {
// create a div element and append the dom elements to it
// when ready, attach the div to the dom. This avoid acessing the dom for every append operation.
var contentDiv = document.createElement("div");
contentDiv.appendChild(this.getImageCredit("ThunderstormsCredits", "(© Julie Fletcher/ Flickr);"));
contentDiv.appendChild(this.getImageCredit("ThunderstormsCredits", "(© Lionel Brown/The Image Bank/Getty Images);"));
contentDiv.appendChild(this.getImageCredit("RainSnowCredits", "(© Image Source/Corbis);"));
contentDiv.appendChild(this.getImageCredit("RainSnowCredits", "(© Tricia Shay Photography/Workbook Stock);"));
contentDiv.appendChild(this.getImageCredit("SnowCredits", "(© GO ITO/amanaimages/Corbis © Tiit Veermae/Alamy);"));
contentDiv.appendChild(this.getImageCredit("SnowCredits", "(© Krischel/mauritius);"));
contentDiv.appendChild(this.getImageCredit("LightShowersCredits", "(© Hornbil Images/Alamy);"));
contentDiv.appendChild(this.getImageCredit("LightShowersCredits", "(© Ekaterina Nosenko/Flickr/Getty Images © Anthony Redpath/Corbis);"));
contentDiv.appendChild(this.getImageCredit("LightShowersCredits", "(© Diane Deaton-Street/Flickr RF/Getty Images © Vitaliy Piltser/Flickr RF/Getty Images);"));
contentDiv.appendChild(this.getImageCredit("DrizzleCredits", "(© Luciano Gaudenzio/SIME/4Corners Images © Charles Briscoe-Knight/Photographer's Choice RF/Getty Images);"));
contentDiv.appendChild(this.getImageCredit("DrizzleCredits", "(© Christina Kilgour/Flickr Open);"));
contentDiv.appendChild(this.getImageCredit("RainCredits", "(© Natalia Campbell of NC Photography/Flickr);"));
contentDiv.appendChild(this.getImageCredit("RainCredits", "(© Jill Ferry/Flickr RF/Getty Images);"));
contentDiv.appendChild(this.getImageCredit("BlowingSnowCredits", "(© Dave Greenwood/Getty Images © David Dear/Photographer's Choice/Getty Images © Exactostock/Superstock);"));
contentDiv.appendChild(this.getImageCredit("BlowingSnowCredits", "(© Krischel/mauritius images);"));
contentDiv.appendChild(this.getImageCredit("ScatteredFlurriesCredits", "(© Staffan Andersson/Getty Images);"));
contentDiv.appendChild(this.getImageCredit("FreezingRainCredits", "(© Alan Majchrowicz/The Image Bank);"));
contentDiv.appendChild(this.getImageCredit("FreezingRainCredits", "(© John Koinberg/Flickr);"));
contentDiv.appendChild(this.getImageCredit("DustyClearCredits", "(© Ted Foxx/Alamy);"));
contentDiv.appendChild(this.getImageCredit("DustyClearCredits", "(© Amit Basu Photography/Getty images);"));
contentDiv.appendChild(this.getImageCredit("DustCloudyCredits", "(© Martin Harvey/Peter Arnold);"));
contentDiv.appendChild(this.getImageCredit("DustCloudyCredits", "(©Roger Coulam/Oxford Scientific);"));
contentDiv.appendChild(this.getImageCredit("FogCredits", "(© peter zelei/Vetta);"));
contentDiv.appendChild(this.getImageCredit("FogCredits", "(© by dio5/Flickr);"));
contentDiv.appendChild(this.getImageCredit("FogCredits", "(© Martin Barraud/Getty Images);"));
contentDiv.appendChild(this.getImageCredit("WindyCredits", "(© Biddiboo/Photographer's Choice/Getty Images);"));
contentDiv.appendChild(this.getImageCredit("WindyCredits", "(© Moodboard/Jupiterimages);"));
contentDiv.appendChild(this.getImageCredit("FrigidCredits", "(© Harold Lloyd/Getty images);"));
contentDiv.appendChild(this.getImageCredit("FrigidCredits", "(© Werner Van Steen/Getty images);"));
contentDiv.appendChild(this.getImageCredit("FrigidCredits", "(© Ross M. Horowitz/Iconica/Getty Images);"));
contentDiv.appendChild(this.getImageCredit("CloudyCredits", "(© Trevor Mein/Riser/Getty Images);"));
contentDiv.appendChild(this.getImageCredit("CloudyCredits", "(© Steve Cicero/Photolibrar);"));
contentDiv.appendChild(this.getImageCredit("MostlyCloudyCredits", "(© Asa Franck/age fotostock);"));
contentDiv.appendChild(this.getImageCredit("MostlyCloudyCredits", "(© David Lyons/Alamy);"));
contentDiv.appendChild(this.getImageCredit("PartlyCloudyCredits", "(© Image Source/Getty Images);"));
contentDiv.appendChild(this.getImageCredit("PartlyCloudyCredits", "(© Paul Conrath/Getty Images);"));
contentDiv.appendChild(this.getImageCredit("ClearCredits", "(© Radius Images/Getty Images © James French/Photographer's Choice/Getty Images © Tetra Images/Getty Images);"));
contentDiv.appendChild(this.getImageCredit("ClearCredits", "(© Patrick Eden/Alamy);"));
contentDiv.appendChild(this.getImageCredit("ScatteredCloudsCredits", "(© Arctic-Images/Riser/Getty Images);"));
contentDiv.appendChild(this.getImageCredit("HeavyRainCredits", "(© Corbis Cusp/Alamy);"));
contentDiv.appendChild(this.getImageCredit("BlizzardCredits", "(© Ian Evans/Alamy);"));
contentDiv.appendChild(this.getImageCredit("UnknownCredits", "(© Stephen Mallon/Photonica/Getty Images);"));
domContentDiv.appendChild(contentDiv);
}
},
init: function init() {
// On application load, register the application commands and settings callback. When the settings charm comes up in application context, 
// these callbacks populates the setting commands necessary to show up on the setting charm.
var settingsPane = Windows.UI.ApplicationSettings.SettingsPane.getForCurrentView();
settingsPane.oncommandsrequested = WeatherAppJS.UI.Settings.onCommandsRequested;
/// Handler for laying out the application settings and defining the fragments to load for each application command.
WinJS.Application.onsettings = function (e) {
e.detail.applicationcommands = {
SettingsPage: { title: PlatformJS.Services.resourceLoader.getString("/platform/optionsTitle"), href: "/html/settingsFlyout.html" },
AboutPage: { title: PlatformJS.Services.resourceLoader.getString("About"), href: "/html/settingsFlyout.html" }
};
WinJS.UI.SettingsFlyout.populateSettings(e);
};
}
});
var markSupportedForProcessing = WinJS.Utilities.markSupportedForProcessing;
var weatherSettings = WeatherAppJS.UI.Settings;
markSupportedForProcessing(weatherSettings.onBeforeShowSettingsCmd);
markSupportedForProcessing(weatherSettings.onBeforeShowAboutCmd);
markSupportedForProcessing(weatherSettings.onEnableSearchHistory);
})();
