﻿/********************************************************
*                                                        *
*   © Microsoft. All rights reserved.                    *
*                                                        *
*********************************************************/
(function () {
'use strict';
WinJS.Namespace.define("WeatherAppJS.Share", {
init: function init() {
var that = this;
// Get the instance of DataTransferManager class that's been assigned to the current windows
var dataTransferManager = Windows.ApplicationModel.DataTransfer.DataTransferManager.getForCurrentView();
// datarequested event is fired when a user presses the share charm
dataTransferManager.addEventListener("datarequested", that.shareDataRequestedEventHandler);
// Register 'bingweather://' protocol scheme
Platform.Utilities.AppExUri.addScheme("bingweather");
// Create controllers
var applicationController = {
default: function (args) {
return PlatformJS.Navigation.createNavigationAction("Home");
},
view: function view(args) {
if (args) {
if (args.entitytype === "partnerPano") {
if (!args.sourceid) {
return PlatformJS.Navigation.createNavigationAction("Home");
}
/* Match the sourceid in the sources data and create the dynamicInfo accordingly */
var sources = new PlatformJS.FeaturedPartners.Sources();
return sources.getSourceForID(args.sourceid).then(function (sourceItem) {
return sourceItem;
}, function () {
return;
});
} else {
// For backward compatibilty, check both 'entityType' and 'entitytype' url parameters
if (args.entityType && PlatformJS.Navigation.entityViewMap[args.entityType]) {
return PlatformJS.Navigation.entityViewMap[args.entityType](args);
} else if (args.entitytype && PlatformJS.Navigation.entityViewMap[args.entitytype]) {
return PlatformJS.Navigation.entityViewMap[args.entitytype](args);
}
}
}
return PlatformJS.Navigation.createNoopAction();
}                
};
var placesController = {
default: function (args) {
return PlatformJS.Navigation.createNavigationAction("MyPlaces");
}
};
var weatherMapsController = {
default: function (args) {
var channelInfo = PlatformJS.Navigation.mainNavigator.channelManager.resolveChannel("WeatherMaps", "standard");
if (channelInfo && channelInfo.pageInfo) {
return PlatformJS.Navigation.createNavigationAction(channelInfo.id, channelInfo.state);
}
else {
return PlatformJS.Navigation.createCommand("application", "default");
}
}
};
var skiResortsController = {
default: function (args) {
var channelInfo = PlatformJS.Navigation.mainNavigator.channelManager.resolveChannel("SkiResorts", "standard");
if (channelInfo && channelInfo.pageInfo) {
return PlatformJS.Navigation.createNavigationAction(channelInfo.id, channelInfo.state);
}
else {
return PlatformJS.Navigation.createCommand("application", "default");
}
}
};
var worldWeatherController = {
default: function (args) {
return PlatformJS.Navigation.createNavigationAction("WorldWeather");
}
};
var helpController = {
default: function (args) {
return CommonJS.Settings.onHelpCmd;
}
};
// Register controllers and commands
PlatformJS.Navigation.addController("application", applicationController);
PlatformJS.Navigation.addController("places", placesController);
PlatformJS.Navigation.addController("weathermaps", weatherMapsController);
PlatformJS.Navigation.addController("skiresorts", skiResortsController);
PlatformJS.Navigation.addController("worldweather", worldWeatherController);
PlatformJS.Navigation.addController("help", helpController);
PlatformJS.Navigation.addEntityView("location", this.getLocationViewCommand());
PlatformJS.Navigation.addEntityView("ski", this.getLocationViewCommand());
PlatformJS.Navigation.addEntityView("article", this.articleEntityView);
PlatformJS.Navigation.addEntityView("pano", this.panoView);
// call PlatformJS.Navigation.addCloudPanoEntityView(); to register CloudPano entity type.
// the parameter of addCloudPanoEntityView is the cloudpano channel ID set in AppConfiguration.xml
// example uri: "bingweather://application/view?entitytype=cloudPano&datasource=AdPanoDataSource&filename=AdPano.internal.json"
PlatformJS.Navigation.addCloudPanoEntityView("CloudPano");
PlatformJS.Navigation.addCommonEntityViews();
},
panoView: function panoView(args) {
// default navigation action
var navAction = PlatformJS.Navigation.createCommand("application", "default");
var channelInfo;
var panoId = args.panoid ? args.panoid : null;
if (panoId) {
switch (panoId) {
case "places":
navAction = PlatformJS.Navigation.createNavigationAction("MyPlaces");
break;
case "weathermaps":
channelInfo = PlatformJS.Navigation.mainNavigator.channelManager.resolveChannel("WeatherMaps", "standard");
if (channelInfo && channelInfo.pageInfo) {
navAction = PlatformJS.Navigation.createNavigationAction(channelInfo.id, channelInfo.state);
}
else {
navAction = PlatformJS.Navigation.createCommand("application", "default");
}
break;
case "skiresorts":
channelInfo = PlatformJS.Navigation.mainNavigator.channelManager.resolveChannel("SkiResorts", "standard");
if (channelInfo && channelInfo.pageInfo) {
navAction = PlatformJS.Navigation.createNavigationAction(channelInfo.id, channelInfo.state);
}
else {
navAction = PlatformJS.Navigation.createCommand("application", "default");
}
break;
case "worldweather":
navAction = PlatformJS.Navigation.createNavigationAction("WorldWeather");
break;
case "help":
navAction = CommonJS.Settings.onHelpCmd;
break;
case "addfavorite":
navAction = PlatformJS.Navigation.createNavigationAction("MyPlaces", { action: 'onAddFavoriteClick' });
break;
case "changehome":
navAction = PlatformJS.Navigation.createNavigationAction("MyPlaces", { action: 'onChangeHomeClick' });
break;
default:
break;
}
}
return navAction;
},
articleEntityView: function articleEntityView(args) {
var providerConfiguration = PlatformJS.Collections.createStringDictionary();
var mkt = args.market ? args.market : Platform.Globalization.Marketization.getMarketLocation().toLowerCase();
providerConfiguration.insert("queryServiceId", "RenderProxy_CustomPanoDataSource");
providerConfiguration.insert("fallbackQueryServiceId", "CustomPanoDataSource");
providerConfiguration.insert("market", mkt);
var arConfig = {
providerType: "AppEx.Common.ArticleReader.BedrockArticleProvider",
providerConfiguration: providerConfiguration
};
var articleId = null;
if (args.articleid) {   // Editorial link
articleId = args.articleid;
}
else if (args.pageid) { // Share charm link
articleId = args.pageid + (args.contentid ? "/" + args.contentid : "");
}
if (articleId) {
var entityItem = {
providerType: arConfig.providerType,
providerConfiguration: arConfig.providerConfiguration,
initialArticleId: articleId,
market: args.market,
source: 2,
enableSharing: true
};
return PlatformJS.Navigation.createNavigationAction("ArticleReader", entityItem);
}
return PlatformJS.Navigation.createCommand("application", "default");
}, 
// <summary>
// Function to be executed when the location view link is launched.
// </summary>
getLocationViewCommand: function getLocationViewCommand() {
var that = this;
// return command function
return (function (args) {
// return action function
return function () {
CommonJS.Progress.showProgress(CommonJS.Progress.headerProgressType);
that.parseLocationViewArgsAsync(args).then(function (responseData) {
CommonJS.Progress.hideProgress(CommonJS.Progress.headerProgressType);
if (responseData && responseData.locationObject) {
var geoLoc = responseData.locationObject;
var state, targetPage = "Home";
if (geoLoc.getId() !== WeatherAppJS.SettingsManager.getDefaultLocationId()) {
targetPage = "Home";
state = {
locID: geoLoc.getId(),
geocodeLocation: geoLoc,
searchingLocation: false,
isHome: false
};
}
PlatformJS.Navigation.navigateToChannel(targetPage, state);
} else {
WeatherAppJS.Error.showMessageBar(PlatformJS.Services.resourceLoader.getString("ShareDataNotAvailableError"), false);
}
}, function (err) {
CommonJS.Progress.hideProgress(CommonJS.Progress.headerProgressType);
if (PlatformJS.Utilities.isPromiseCanceled(err)) {
return;
} else if (!WeatherAppJS.Networking.internetAvailable) {
WeatherAppJS.Error.showMessageBar(PlatformJS.Services.resourceLoader.getString("NoInternetConnectionError"), false);
} else {
WeatherAppJS.Error.showMessageBar(PlatformJS.Services.resourceLoader.getString("ShareDataNotAvailableError"), false);
}
});
};
});
},
// <summary>
// Event handler for datarequested event. Generates the URI link.
// </summary>
shareDataRequestedEventHandler: function shareDataRequestedEventHandler(e) {
// Populate the request data object with the information that weather app wants to share. App shares across a link to the app on microsoft store
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NumTimesDataShared");
Platform.Telemetry.FlightRecorder.logUserAction(Platform.Telemetry.LogLevel.normal, Platform.Telemetry.ActionContext.charms, "Share", Platform.Telemetry.UserActionOperation.click, Platform.Telemetry.UserActionMethod.unknown, 0);
var request = e.request;
var currentIPage = PlatformJS.Navigation.currentIPage;
var pageName = WeatherAppJS.Utilities.Common.getCurrentPage();
var pageState = currentIPage ? currentIPage.getPageState() : null;
if ((pageName === "WeatherAppJS.DailyForecast" || pageName === "WeatherAppJS.Pages.Home") // Valid page
&& (!WeatherAppJS.SettingsManager.isFRE() && WeatherAppJS.LocationsManager.getDefaultLocation()) // FRE complete ?
&& !WeatherAppJS.LocationsManager.isMarketBlockedByIP
&& currentIPage) {
var locID = null;
if (pageState && pageState.locID) {
locID = pageState.locID;
} else {
locID = WeatherAppJS.SettingsManager.getDefaultLocationId();
}
var geoLoc = WeatherAppJS.GeocodeCache.getLocation(locID); // Get geocode location obj from cache
if (geoLoc) {
var that = WeatherAppJS.Share;
var standardDataFormats = Windows.ApplicationModel.DataTransfer.StandardDataFormats;
var title = PlatformJS.Services.resourceLoader.getString("SharePageTitle").format(geoLoc.getShortDisplayName());
// Set the title and description so that the header is not empty
request.data.properties.title = title;
request.data.properties.description = "";
// Set a delegate to return the html, when the target app requests
request.data.setDataProvider(standardDataFormats.html, function (providerRequest) { that.onDeferredDataRequested(providerRequest, geoLoc, standardDataFormats.html); });
request.data.setDataProvider(standardDataFormats.text, function (providerRequest) { that.onDeferredDataRequested(providerRequest, geoLoc, standardDataFormats.text); });
request.data.setDataProvider(standardDataFormats.uri, function (providerRequest) { that.onDeferredDataRequested(providerRequest, geoLoc, standardDataFormats.uri); });
// Perf: In general, the target app would call the delegate of only one format.
// However, in case the target app invokes more than one delegate, we might end
// up making the getGeocodeLocationForSharingAsync call more than once. So, making
// the call here with the hope that we will get the response back and cached before
// any delegate is invoked.
that.getGeocodeLocationForSharingAsync(geoLoc).then();
}
} else if (pageName === "WeatherAppJS.ArticleReaderPage") {
if (currentIPage && currentIPage.handleShareRequest) {
if (!currentIPage.canShareRequest || currentIPage.canShareRequest()) {
currentIPage.handleShareRequest(request);
}
}
} else {
request.failWithDisplayText(PlatformJS.Services.resourceLoader.getString("ShareInvalidSharePageMessage"));
}
},
onDeferredDataRequested: function onDeferredDataRequested(request, geoLoc, format) {
var deferral = request.getDeferral();
function setShareData(uriString) {
try {
if (!uriString) {
var uriBuilder = new Platform.Utilities.AppExUriBuilder();
uriBuilder.controllerId = "application";
uriString = uriBuilder.toString();
}
if (format === Windows.ApplicationModel.DataTransfer.StandardDataFormats.html) {
CommonJS.loadModule({ fragmentPath: "/html/secondaryTemplates.html", templateId: "shareTemplate" }, {})
.then(function (template) {
// Generate the HTML to be shared.
var resourceFile = PlatformJS.Services.resourceLoader;
var locationNameText = geoLoc.getShortDisplayName();
var snippetText = resourceFile.getString("ShareSnippet").format(geoLoc.getShortDisplayName());
var elem = document.createElement("a");
elem.href = uriString;
elem.innerText = resourceFile.getString("/platform/win8ShareLink").format(resourceFile.getString("AppTitle"));
elem.style.cssText = "color: rgb(0, 130, 153);";
var deepLinkHTML = resourceFile.getString("/platform/win8Share").format(resourceFile.getString("/platform/win8Brand"), elem.outerHTML);
template.querySelector("#locationName").innerText = locationNameText;
template.querySelector("#snippet").innerText = snippetText;
template.querySelector("#deepLink").innerHTML = deepLinkHTML;
var html = Windows.ApplicationModel.DataTransfer.HtmlFormatHelper.createHtmlFormat(template.innerHTML);
request.setData(html);
deferral.complete();
}, function (error) {
deferral.complete();
});
} else if (format === Windows.ApplicationModel.DataTransfer.StandardDataFormats.text) {
request.setData(uriString);
deferral.complete();
} else if (format === Windows.ApplicationModel.DataTransfer.StandardDataFormats.uri) {
var uri = new Windows.Foundation.Uri(uriString);
request.setData(uri);
deferral.complete();
} else {
deferral.complete();
}
} catch (error) {
deferral.complete();
}
}
WeatherAppJS.Share.getGeocodeLocationForSharingAsync(geoLoc)
.then(function (invariantLoc) {
if (invariantLoc) {
// Generate the URI to be shared.
// Example: bingweather:///view?entityType=location&id=Hyderabad%2C%20Andhra%20Pradesh%2C%20India&la=17.401159286499023&lo=78.47042083740234&iso2=IN
// The latitude and longitude params are not strictly required (except for a market where VE geocoding calls are disabled).
var uriBuilder = new Platform.Utilities.AppExUriBuilder();
uriBuilder.controllerId = "application";
uriBuilder.commandId = "view";
uriBuilder.queryParameters.insert("entityType", invariantLoc.getLocTypeString());
if (invariantLoc.locType === WeatherAppJS.GeocodeLocation.locationType.skiLocation) {
uriBuilder.queryParameters.insert("id", invariantLoc.getId());
} else {
uriBuilder.queryParameters.insert("id", invariantLoc.getQueryName());
}
if (invariantLoc.locType !== WeatherAppJS.GeocodeLocation.locationType.skiLocation) {
var trimLatLong = WeatherAppJS.Utilities.Common.trimLatLong;
var la = trimLatLong(invariantLoc.latitude);
var lo = trimLatLong(invariantLoc.longitude);
uriBuilder.queryParameters.insert("la", la);
uriBuilder.queryParameters.insert("lo", lo);
}
uriBuilder.queryParameters.insert("iso2", invariantLoc.isoCode);
var uriString = uriBuilder.toString();
setShareData(uriString);
} else {
setShareData(null);
}
}, function (error) {
setShareData(null);
});
},
// <summary>
// Parse and validate the URI arguments. Returns a promise to get a geocode location object on success.
// </summary>
parseLocationViewArgsAsync: function parseLocationViewArgsAsync(args) {
try {
if (args && args.id) {
var latitude = args.la || "";
var longitude = args.lo || "";
var isoCode = args.iso2 || "";
var locName = args.id;
var locType = WeatherAppJS.GeocodeLocation.getLocTypeFromString(args.entityType);
return this.getGeocodeLocationFromSharedLinkAsync(locName, latitude, longitude, isoCode, locType)
.then(function (geoLoc) {
if (geoLoc) {
return WeatherAppJS.LocationsManager.tryFetchingCCDataAsync(geoLoc, false);
} else {
return WinJS.Promise.wrap(null);
}
});
}
} catch (e) {
}
return WinJS.Promise.wrap(null);
},
/// <summary>
/// Returns the geocode location object using the location info. provided in the shared URI link. 
/// The latitude and longitude are required  when VE geocoding API is disabled.
/// </summary>
getGeocodeLocationFromSharedLinkAsync: function getGeocodeLocationFromSharedLinkAsync(locName, latitude, longitude, isoCode, locType) {
var culture = WeatherAppJS.LocationSuggest.getVECultureCode();
var useTianQi = Platform.Utilities.Globalization.isFeatureEnabled("UseTianQi");
if (useTianQi && (locType === WeatherAppJS.GeocodeLocation.locationType.skiLocation)) {
return WinJS.Promise.wrap(null);
}
if (locType !== WeatherAppJS.GeocodeLocation.locationType.skiLocation) {
// Check if we can make Virtual Earth geocoding API call
var isGeocodingAPIEnabled = Platform.Utilities.Globalization.isFeatureEnabled("VEGeocodingAPISuggestions");
if (isGeocodingAPIEnabled && locName) {
// Get localized geocode location from VE.
return WeatherAppJS.LocationSuggest.getLocalizedGeocodeLocationAync(locName, latitude, longitude, isoCode)
.then(function (loc) {
if (!loc) {
// VE didn't return a location, try autosuggest reverse lookup. For Singapore, Hong Kong etc.
return WeatherAppJS.LocationSuggest.getLocationByLatLongAsync(latitude, longitude, locName, locType, culture);
} else {
return WinJS.Promise.wrap(loc);
}
});
} else if (!isGeocodingAPIEnabled && latitude !== undefined && longitude !== undefined) {
// Do autosuggest reverse lat-long search when VE geocoding API is disabled.
return WeatherAppJS.LocationSuggest.getLocationByLatLongAsync(latitude, longitude, locName, locType, culture);
}
} else {
return WeatherAppJS.LocationSuggest.getLocationByLatLongAsync(latitude, longitude, locName, locType, culture);
}
return WinJS.Promise.wrap(null);
},
/// <summary>
/// Returns the invariant geocode location object which will be used to generate the shared link
/// </summary>
getGeocodeLocationForSharingAsync: function getGeocodeLocationForSharingAsync(geoLoc) {
if (geoLoc) {
if (geoLoc.locType !== WeatherAppJS.GeocodeLocation.locationType.skiLocation) {
// Check if we can make Virtual Earth geocoding API call
var isGeocodingAPIEnabled = Platform.Utilities.Globalization.isFeatureEnabled("VEGeocodingAPISuggestions");
if (isGeocodingAPIEnabled) {
// Get invariant geocode location.
return WeatherAppJS.LocationSuggest.getInvariantGeocodeLocationAync(geoLoc.getQueryName(), geoLoc.latitude, geoLoc.longitude, geoLoc.isoCode)
.then(function (iLoc) {
if (!iLoc) {
return WeatherAppJS.LocationSuggest.getInvariantLocationByLatLongAsync(geoLoc.latitude, geoLoc.longitude, geoLoc.id, geoLoc.locType);
} else {
return WinJS.Promise.wrap(iLoc);
}
}, function (err) {
return WinJS.Promise.wrap(null);
});
} else {
return WeatherAppJS.LocationSuggest.getInvariantLocationByLatLongAsync(geoLoc.latitude, geoLoc.longitude, geoLoc.id, geoLoc.locType);
}
} else {
return WeatherAppJS.LocationSuggest.getInvariantLocationByLatLongAsync(geoLoc.latitude, geoLoc.longitude, geoLoc.id, geoLoc.locType);
}
}
return WinJS.Promise.wrap(null);
},
});
WeatherAppJS.Share.init();
})();