﻿/********************************************************
*                                                        *
*   © Microsoft. All rights reserved.                    *
*                                                        *
*********************************************************/
(function () {
'use strict';
WinJS.Namespace.define("WeatherAppJS.PageLoadTypes", {
loadPageByDefault: "Default",
loadPageByRefresh: "AutoRefresh",
loadPageBySettingsChange: "SettingsChange",
loadPageByInternetChange: "NetworkChangeCallback"
});
WinJS.Namespace.define("WeatherAppJS", {
Networking: new (WinJS.Class.mix(
function () {},
{
internetAvailable: null,
networkChangeCallback: null,
init: function init() {
this.internetAvailable = Platform.Networking.NetworkManager.instance.isNetworkAvailable;
Platform.Networking.NetworkManager.addEventListener("networkstatuschanged", WeatherAppJS.Networking.onNetworkStatusChange);
},
onNetworkStatusChange: function onNetworkStatusChange(event) {
var currentStatus = Platform.Networking.NetworkManager.instance.isNetworkAvailable;
var weatherAppNetworking = WeatherAppJS.Networking;
//app goes offline from online.
if (weatherAppNetworking.internetAvailable === true) {
if (currentStatus === false) {
weatherAppNetworking.internetAvailable = currentStatus;
weatherAppNetworking.dispatchEvent("networkoffline", true);
}
}
else {
// app comes online from offline.
if (currentStatus === true) {
weatherAppNetworking.internetAvailable = currentStatus;
if (weatherAppNetworking.networkChangeCallback) {
try {
weatherAppNetworking.networkChangeCallback();
}
catch (err) {
}
}
weatherAppNetworking.dispatchEvent("networkonline", true);
}
}
},
updateNetworkStatus: function updateNetworkStatus() {
this.internetAvailable = Platform.Networking.NetworkManager.instance.isNetworkAvailable;
},
},
WinJS.Utilities.eventMixin,
WinJS.Utilities.createEventProperties(
'networkonline', 
'networkoffline')
))
});
WeatherAppJS.Networking.init();
WinJS.Namespace.define("WeatherAppJS.Error", {
showMessageBar: function showMessageBar(message, isAutoHide) {
var messageBar = new CommonJS.MessageBar(message, { level: CommonJS.MessageBarLevelWarning, autoHide: isAutoHide });
var bLabel = PlatformJS.Services.resourceLoader.getString("CloseButtonLabel");
messageBar.addButton(bLabel, function () { messageBar.hide(); });
messageBar.show();
if (message === PlatformJS.Services.resourceLoader.getString("MaxFavoritesLimitReached") || message === PlatformJS.Services.resourceLoader.getString("SetDefaultMaxLimitReached")) {
Platform.Telemetry.FlightRecorder.logAppAction(Platform.Telemetry.LogLevel.normal, WeatherAppJS.Instrumentation.PageContext.MessageBar, "Max Favorites Limit Reached Message", "appShow", 0);
}
},
showErrorMessageBar: function showErrorMessageBar(message, isAutoHide) {
var messageBar = new CommonJS.MessageBar(message, { level: CommonJS.MessageBarLevelError, autoHide: isAutoHide });
var bLabel = PlatformJS.Services.resourceLoader.getString("CloseButtonLabel");
messageBar.addButton(bLabel, function () { messageBar.hide(); });
messageBar.show();
},
showMessageBarWithButton: function showMessageBarWithButton(message, isAutoHide, buttonLabel, buttonClickHandler) {
var messageBar = new CommonJS.MessageBar(message, { level: CommonJS.MessageBarLevelWarning, autoHide: isAutoHide });
var bLabel = PlatformJS.Services.resourceLoader.getString("CloseButtonLabel");
messageBar.addButton(buttonLabel, buttonClickHandler);
messageBar.addButton(bLabel, function () { messageBar.hide(); });
messageBar.show();
},
isMarketBlockedError: function isMarketBlockedError(e) {
// "104" == NetworkFailure403HttpStatusCode
// -2146233088 == COR_E_EXCEPTION
return (e && e.message && e.message === "104" && e.number && e.number === -2146233088);
},
/// <summary>Creates an error module item</summary>
/// <param name="retryCallback">Function to invoke when retry button is pressed</param>
/// <param name="width">Optional width of module</param>
/// <param name="height">Optional height of module</param>
/// <param name="domElementToRender">Optional dom element to be cleared and used to show the error</param>
createErrorModule: function createErrorModule(retryCallback, width, height, domElementToRender) {
var that = this;
var data;
var retryFunction = WinJS.Utilities.markSupportedForProcessing(retryCallback);
if (!PlatformJS.Utilities.hasInternetConnection()) {
data = CommonJS.Error.getErrorModuleItem(retryFunction, width, height, "/platform/offline_noContent", "platformOfflineErrorModule");
} else {
data = CommonJS.Error.getErrorModuleItem(retryFunction, width, height);
}
var moduleInfo = data.moduleInfo;
var errorElt = document.createElement("div");
errorElt.className = "platformHide";
errorElt.style.width = moduleInfo.width;
errorElt.style.height = moduleInfo.height;
if (domElementToRender) {
domElementToRender.innerHTML = "";
domElementToRender.appendChild(errorElt);
}
CommonJS.loadModule(moduleInfo, data, errorElt);
return errorElt;
},
createProgressModule: function createProgressModule(width, height, domElementToRender) {
var data = CommonJS.Error.getProgressModuleItem(width, height);
var moduleInfo = data.moduleInfo;
var progressElt = document.createElement("div");
WinJS.Utilities.addClass(progressElt, "platformHide");
WinJS.Utilities.addClass(progressElt, "weatherProgressModule");
progressElt.style.width = moduleInfo.width;
progressElt.style.height = moduleInfo.height;
if (domElementToRender) {
domElementToRender.appendChild(progressElt);
}
CommonJS.loadModule(moduleInfo, data, progressElt);
return progressElt;
}
});
WinJS.Namespace.define("WeatherAppJS.Utilities.Templates", {
// current conditions moduleinfo template
getCurrentConditionsTemplate: function getCurrentConditionsTemplate(item) {
var that = WeatherAppJS.Utilities.Templates;
var ui = WeatherAppJS.Utilities.UI;
var viewManagement = Windows.UI.ViewManagement;
var winjsAddClass = WinJS.Utilities.addClass;
var resourceLoader = PlatformJS.Services.resourceLoader;
var appConfig = PlatformJS.Services.appConfig;
var ccCluster = document.createElement("div");
winjsAddClass(ccCluster, "win-interactive currentConditionsModule");
var currentConditionsData = item.data.loc.getCurrentConditions();
var singleProviderOnly = appConfig.getBool("SingleProviderOnly");
var ccProviderOverride = appConfig.getString("CCProviderOverride");
var showDailyConditionsProvider = appConfig.getBool("ShowDailyForecastAttribution");
if (!item.data.loc) {
return null;
}
var locationID = item.data.loc.id;
var showLimitedExperience = ui.showLimitedExperience(locationID);
if (showLimitedExperience) {
winjsAddClass(ccCluster, "showLimitedExperience");
var showLimitedExperienceOverride = Platform.Utilities.Globalization.isFeatureEnabled("showLimitedExperienceOverride");
if (showLimitedExperienceOverride) {
winjsAddClass(ccCluster, "showLimitedExperienceOverride");
}
var showLimitedExperiencePrecipOverride = Platform.Utilities.Globalization.isFeatureEnabled("showLimitedExperiencePrecipOverride");
if (showLimitedExperiencePrecipOverride) {
winjsAddClass(ccCluster, "showLimitedExperiencePrecipOverride");
}
}
// add compare text
currentConditionsData.showMoreDailyConditionsDataText = resourceLoader.getString('moreDailyConditionsDataText');
currentConditionsData.showLessDailyConditionsDataText = resourceLoader.getString('lessDailyConditionsDataText');
CommonJS.loadModule({fragmentPath:"/html/templates.html", templateId:"ccModule"}, currentConditionsData, null, null, item).then(function (ccTemplate) {
ccCluster.appendChild(ccTemplate);
// Set aria-label
var alertNode = ccCluster.querySelector(".infoWrapper div");
if (alertNode) {
alertNode.setAttribute("aria-labelledby", alertNode.querySelector(".alertInfo")["id"]);
}
var ariaText = resourceLoader.getString("currentConditionsAriaText").format(currentConditionsData.tempCC + " °", currentConditionsData.caption,
currentConditionsData.feelslikeTemp, currentConditionsData.provider);
// Disable the unit for china market.
var useTianQi = Platform.Utilities.Globalization.isFeatureEnabled("UseTianQi");
//try to get ski hero data
var isSkiPano = false;
isSkiPano = WeatherAppJS.GeocodeCache.getLocation(locationID).isSkiLocation;
if (!useTianQi) {
var ccElemCont = ccTemplate.querySelector(".bTemp");
winjsAddClass(ccElemCont, "bigTempContainer");
// Bind Click and KeyUP Event.
ccElemCont.addEventListener("click", WeatherAppJS.Utilities.UI.ChangeDisplayUnitonClick);
ccElemCont.onkeyup = function (event) { WeatherAppJS.Utilities.UI.ChangeDisplayUnitonClick(event); };
// set aria label on current conditions
var unitAriaText = (currentConditionsData.tempunit === "F") ? resourceLoader.getString("AriaTextF") : resourceLoader.getString("AriaTextC");
unitAriaText = unitAriaText + ". ";
var toolTipAriaText = (currentConditionsData.tempunit === "F") ? resourceLoader.getString("ToolTipTextC") : resourceLoader.getString("ToolTipTextF");
ariaText = resourceLoader.getString("currentConditionsAriaText").format(currentConditionsData.tempCC + " °" + unitAriaText + toolTipAriaText, currentConditionsData.caption,
currentConditionsData.feelslikeTemp, currentConditionsData.provider);
}
else {
var ccElemBigTempUnit = ccTemplate.querySelector(".bigTempUnit");
ccElemBigTempUnit.style.display = "none";
}
var ccElem = ccTemplate.querySelector(".bTemp");
ccElem.setAttribute("aria-label", ariaText);
var isMinimized = WeatherAppJS.SettingsManager.isCCMinimized();
if (!isMinimized) {
ccElem.setAttribute("aria-describedby", "ccExpanded");
}
var dailyData = ccTemplate.querySelector('.dailyModule');
var otherProviderDailyDataTemplates = ccTemplate.querySelectorAll('.dailyModuleSmall');
var mainProviderName = ccTemplate.querySelector('.ccProvider');
var otherProviderNames = ccTemplate.querySelectorAll('.dailyModuleOtherProviderName');
var otherProviderDailyData1 = otherProviderDailyDataTemplates[0];
var otherProviderDailyData2 = otherProviderDailyDataTemplates[1];
var otherProviderName1 = otherProviderNames[0];
var otherProviderName2 = otherProviderNames[1];
var forecastsWrapper = ccTemplate.querySelectorAll('.forecastsWrapper');
var precipitationWrapper = ccTemplate.querySelector('.precipitationWrapper');
var mainProviderForecastWrapper = forecastsWrapper[0];
var otherProviderForecastWrapper1 = forecastsWrapper[1];
var otherProviderForecastWrapper2 = forecastsWrapper[2];
var pDailyConditionsWrapper = ccTemplate.querySelector('.portraitDailyConditionsWrapper');
var pPrimaryProvider = pDailyConditionsWrapper.querySelector('.primaryProvider');
var pOtherProvider1 = pDailyConditionsWrapper.querySelector('.secondaryProvider1');
var pOtherProvider2 = pDailyConditionsWrapper.querySelector('.secondaryProvider2');
var pPrimaryProviderName = pDailyConditionsWrapper.querySelector('.primaryProviderName');
var pOtherProviderName1 = pDailyConditionsWrapper.querySelector('.secondaryProviderName1');
var pOtherProviderName2 = pDailyConditionsWrapper.querySelector('.secondaryProviderName2');
var otherProvidersWrapper = ccTemplate.querySelector('.otherProvidersWrapper');
var otherProvider1 = otherProvidersWrapper.querySelector('.primaryProvider');
var otherProvider2 = otherProvidersWrapper.querySelector('.secondaryProvider1');
//on the snow attribution
var OTSAttribution = ccTemplate.querySelector('#autoID_SnowProvider');
if (isSkiPano) {
// hack to align platform attribution button with the more forecast days button for all resolutions
var setImgAttributionClick = function (button) {
button.addEventListener('click', function (event) {
var attributionFlyout = document.querySelector('.platformAttributionFlyout');
if (attributionFlyout) {
attributionFlyout.setAttribute('aria-label', attributionFlyout.textContent);
}
});
};
setImgAttributionClick(document.querySelector('.platformInfoButtonOff')); // original attribution button
// create a new attribution button and place it for landscape and hide the original. For portrait, show the original and hide the new button.
var ccWrapper = ccTemplate.querySelector('.ccWrapper');
if (ccWrapper) {
var skiAttributionButton = document.createElement("div");
skiAttributionButton.className = "platformHeroImageAttribution alignPlatformHeroAttribution";
var skiAttrLabel = WeatherAppJS.Utilities.Common.getSkiAttributionData(currentConditionsData);
var skiAttribution = new CommonJS.ImageAttribution(skiAttributionButton, {
flyout: {
label: skiAttrLabel,
placement: "top",
alignment: (window.getComputedStyle(document.body).direction === "rtl") ? "left" : "right"
}
});
setImgAttributionClick(skiAttributionButton.querySelector('.platformInfoButtonOff')); // new attribution button
ccWrapper.appendChild(skiAttributionButton);
}
}
var getProviderUrlHtml = function (provName, provUrl) { // get the display HTML given provider name, provider url
if (provName === "Weather Service Ltd") {
provName = resourceLoader.getString("JWSAttributionValue");
winjsAddClass(pPrimaryProviderName, "primaryProviderNameJPmkt");
} else {
provName = resourceLoader.getString(provName);
}
if (provName === "weather.com") {
provName = WeatherAppJS.Utilities.Common.getWeatherDotComBaseUrl();
}
var displayProvUrl = '';
// provider url is optional and can be empty
if (provUrl) {
displayProvUrl = '<a class="dailyConditionsProviderLink" href="' + provUrl + '" tabIndex="0" >' + provName + '</a>';
} else {
displayProvUrl = provName;
}
return displayProvUrl;
};
var dailyConditions = item.data.loc.getDailyConditions();
var dailyConditionsLength = Object.keys(dailyConditions).length;
var primaryDisplayProviderUrl = "";
var providerName = "", providerUrl = "";
// Set current conditions provider
// For specific markets, we need to override the data provider
if (ccProviderOverride && showLimitedExperience) {
providerName = ccProviderOverride;
providerUrl = resourceLoader.getString(ccProviderOverride + "Url");
}
else if (dailyConditionsLength > 0) {
providerName = dailyConditions[0].provider;
providerUrl = dailyConditions[0].url;
}
primaryDisplayProviderUrl = getProviderUrlHtml(providerName, providerUrl);
mainProviderName.innerHTML = primaryDisplayProviderUrl;
pPrimaryProviderName.innerHTML = primaryDisplayProviderUrl;
/*
Go through all providers and based on the provider ranking, load different templates
The counter keeps track of the rank.
*/
var counter = 0;
var isWeatherDotCom = false;
var weatherDotComPosition = 0;
var weatherDotComUrl = '';
var overlayContainer = ccCluster.querySelector(".overlayContainer");
ui._totalDailyForecasts = item.data.loc.maxDailyForecastDays;
// add event listener for bringing into view the other days
var forecastSlider = ccTemplate.querySelector('.forecastSlider');
forecastSlider.addEventListener('click', (function (locName) {
return function () {
WeatherAppJS.Utilities.UI.AnimateForecastSlide(locName);
};
}(item.data.loc.id)), false);
var pForecastSlider = ccTemplate.querySelector('.pForecastSlider');
pForecastSlider.addEventListener('click', (function (locName) {
return function () {
WeatherAppJS.Utilities.UI.AnimatePortraitForecastSlide(locName);
};
}(item.data.loc.id)), false);
//Configure event listeners for attribution links
mainProviderName.addEventListener('click', (function () {
WeatherAppJS.Utilities.Instrumentation.onClickAttribution(mainProviderName.textContent, WeatherAppJS.Instrumentation.PageContext.DailyForecast);
}));
otherProvider1.addEventListener('click', (function () {
WeatherAppJS.Utilities.Instrumentation.onClickAttribution(otherProvider1.textContent, WeatherAppJS.Instrumentation.PageContext.DailyForecast);
}));
otherProvider2.addEventListener('click', (function () {
WeatherAppJS.Utilities.Instrumentation.onClickAttribution(otherProvider2.textContent, WeatherAppJS.Instrumentation.PageContext.DailyForecast);
}));
pPrimaryProviderName.addEventListener('click', (function () {
WeatherAppJS.Utilities.Instrumentation.onClickAttribution(pPrimaryProviderName.textContent, WeatherAppJS.Instrumentation.PageContext.DailyForecast);
}));
pOtherProviderName1.addEventListener('click', (function () {
WeatherAppJS.Utilities.Instrumentation.onClickAttribution(pOtherProviderName1.textContent, WeatherAppJS.Instrumentation.PageContext.DailyForecast);
}));
pOtherProviderName2.addEventListener('click', (function () {
WeatherAppJS.Utilities.Instrumentation.onClickAttribution(pOtherProviderName2.textContent, WeatherAppJS.Instrumentation.PageContext.DailyForecast);
}));
OTSAttribution.addEventListener('click', (function () {
WeatherAppJS.Utilities.Instrumentation.onClickAttribution(OTSAttribution.textContent, WeatherAppJS.Instrumentation.PageContext.DailyForecast);
}));
// get skiforecast data if skipano.
var skiForecastCurrentIndex = 0;
var pskiForecastCurrentIndex = 0;
var skiForecastMaxCount = 0;
var skiForeCastData = null;
if (isSkiPano) {
skiForeCastData = currentConditionsData.heroData.dailyForeCast;
if (skiForeCastData) {
skiForecastMaxCount = Object.keys(skiForeCastData).length;
}
ariaText = resourceLoader.getString("currentConditionsAriaText").format(currentConditionsData.tempCC + " °" + unitAriaText + toolTipAriaText, currentConditionsData.caption,
'', currentConditionsData.AttrProvider);
ccElem.setAttribute("aria-label", ariaText);
// Override dailyConditionsLength to only populate single provider data.
showLimitedExperience = true;
}
for (var index = 0; index < dailyConditionsLength; index++) {
// for few markets we consider only the first provider.
// For ski panos we consider only first provider.
if (counter > 0 && showLimitedExperience) {
break;
}
// daily conditions provider
var displayProviderUrl = getProviderUrlHtml(dailyConditions[index].provider, dailyConditions[index].url);
var showLimitedExperienceOverrideFlag = Platform.Utilities.Globalization.isFeatureEnabled("showLimitedExperienceOverride");
if (showLimitedExperienceOverrideFlag && showLimitedExperience) { // over-ride to show correct provider name
pPrimaryProviderName.innerHTML = displayProviderUrl;
}
if (dailyConditions[index].provider === "weather.com") {
isWeatherDotCom = true;
weatherDotComPosition = counter;
weatherDotComUrl = WeatherAppJS.Utilities.Common.getWeatherDotComUrl(dailyConditions[index].baseurl, "WeatherDotComTenDayUrl");
}
var totalConditions = 0;
if (dailyConditions[index].forecasts) {
totalConditions = dailyConditions[index].forecasts.length;
}
var eachDailyModuleTemplate = null; // for landscape
var pEachDailyModuleTemplate = null; // for portrait
var otherProviderDailyModuleTemplate = null; // for the smaller daily module
var isDailyDrilldownEnabled = WeatherAppJS.Utilities.UI.isDailyDrilldownEnabled(item.data.loc.id);                   
for (var i = 0; i < totalConditions; i++) {
var forecastData = dailyConditions[index].forecasts[i];
if (counter === 0) {
var ariaTextValues = that._getAriaTextForDailyConditions(dailyConditions, i);
if (skiForeCastData && currentConditionsData.heroData.snowData[forecastData.date]) {
ariaTextValues.ariaText += ", " + resourceLoader.getString("FreshSnow") + ", " + currentConditionsData.heroData.snowData[forecastData.date];
ariaTextValues.ariaTextExpanded += ", " + resourceLoader.getString("FreshSnow") + ", " + currentConditionsData.heroData.snowData[forecastData.date];
}
if (isDailyDrilldownEnabled) {
// add overlay div
var overlay = document.createElement("div");
WinJS.Utilities.addClass(overlay, "overlay");
// hide all other overlays when hovering over this one
// css takes care of styling this one when hovering
overlay.onmspointerover = function () {
ui.hideAllForecastOverlays();
};
overlayContainer.appendChild(overlay);
}
if (!eachDailyModuleTemplate) {
CommonJS.loadModule({ fragmentPath: "/html/templates.html", templateId: "eachDailyModule" }, forecastData, null, null, item).then(function (dailyTemplate) {
eachDailyModuleTemplate = dailyTemplate;
ui.setForecastNodeDisplayState(dailyTemplate, showDailyConditionsProvider, showLimitedExperience, displayProviderUrl, i, isSkiPano);
var fNode = dailyTemplate.querySelector('.dailyModuleFirstProvider'); // first node
ui.setForecastNodeAriaLabel(fNode, isMinimized, showLimitedExperience, ariaTextValues);
if (isDailyDrilldownEnabled) {
ui.attachFocusHandlerForDailyForecast(fNode, i, locationID);
}
mainProviderForecastWrapper.appendChild(dailyTemplate);
});
} else {
var thisDailyTemplate = eachDailyModuleTemplate.cloneNode(true);
thisDailyTemplate.querySelector('.date').innerText = forecastData.date;
thisDailyTemplate.querySelector('.tempHigh').innerText = forecastData.temphigh;
thisDailyTemplate.querySelector('.tempLow').innerText = forecastData.templow;
thisDailyTemplate.querySelector('.caption').innerText = forecastData.caption;
var skyCodeImage = thisDailyTemplate.querySelector('.skycodeImage');
skyCodeImage.alt = forecastData.caption;
skyCodeImage.src = forecastData.skycode;
ui.setForecastNodeDisplayState(thisDailyTemplate, showDailyConditionsProvider, showLimitedExperience, displayProviderUrl, i, isSkiPano);
var oNode = thisDailyTemplate.querySelector('.dailyModuleFirstProvider'); // other nodes
ui.setForecastNodeAriaLabel(oNode, isMinimized, showLimitedExperience, ariaTextValues);
if (isDailyDrilldownEnabled) {
ui.attachFocusHandlerForDailyForecast(oNode, i, locationID);
}
mainProviderForecastWrapper.appendChild(thisDailyTemplate);
if (i === totalConditions - 1) {
var floater = document.createElement('div');
winjsAddClass(floater, "clearFloats");
mainProviderForecastWrapper.appendChild(floater);
// based on max forecast days, show / hide the slide animation button
if (ui._totalDailyForecasts <= ui._totalDailyForecastsShown) {  // total daily conditions we show now: _totalDailyForecastsShown
forecastSlider.style.display = "none";
} else {
forecastSlider.style.display = "block";
}
}
}
if (!pEachDailyModuleTemplate) {
CommonJS.loadModule({ fragmentPath: "/html/templates.html", templateId: "pEachDailyModule" }, forecastData, null, null, item).then(function (pDailyTemplate) {
pEachDailyModuleTemplate = pDailyTemplate;
var pDailyNode = pEachDailyModuleTemplate.querySelector('.precip');
if (pDailyNode.innerText === "--") {
pDailyNode.style.display = 'none';
pEachDailyModuleTemplate.querySelector('.raindrop').style.display = 'none';
}
var pSnowDailyNode = pEachDailyModuleTemplate.querySelector('.snowFreshText');
// Bind todays data for snowfresh. Override the value for first instance.
if (skiForeCastData) {
if (skiForeCastData[pskiForecastCurrentIndex] && pskiForecastCurrentIndex < skiForecastMaxCount && skiForeCastData[pskiForecastCurrentIndex].date === forecastData.date) {
pSnowDailyNode.innerHTML = skiForeCastData[pskiForecastCurrentIndex].freshSnow;
pskiForecastCurrentIndex++;
} else {
pSnowDailyNode.innerHTML = "";
pSnowDailyNode.style.display = 'none';
}
}
var node = pDailyTemplate.querySelector('.pDailyModuleFirstProvider');
ui.setForecastNodeAriaLabel(node, isMinimized, showLimitedExperience, ariaTextValues);
if (isDailyDrilldownEnabled) {
WinJS.Utilities.addClass(node, 'enablePortraitHoverEffect');
node.onmspointerover = function () {
ui.hideAllForecastOverlays();
};
ui.attachFocusHandlerForDailyForecast(node, i, locationID);
}
pPrimaryProvider.appendChild(pDailyTemplate);
});
} else {
var pThisDailyTemplate = pEachDailyModuleTemplate.cloneNode(true);
pThisDailyTemplate.querySelector('.day').innerText = forecastData.day;
pThisDailyTemplate.querySelector('.dayName').innerText = forecastData.dayName;
pThisDailyTemplate.querySelector('.tempHigh').innerText = forecastData.temphigh;
pThisDailyTemplate.querySelector('.tempLow').innerText = forecastData.templow;
pThisDailyTemplate.querySelector('.caption').innerText = forecastData.caption;
var pSkyCodeImage = pThisDailyTemplate.querySelector('.skycodeImage');
pSkyCodeImage.alt = forecastData.caption;
pSkyCodeImage.src = forecastData.skycode;
if (forecastData.precipitation === "--") {
pThisDailyTemplate.querySelector('.precip').style.display = 'none';
pThisDailyTemplate.querySelector('.raindrop').style.display = 'none';
} else {
pThisDailyTemplate.querySelector('.precip').innerText = forecastData.precipitation;
}
// bind skiforecast data if available. Check for Date before binding.
if (skiForeCastData) {
var snowData = pThisDailyTemplate.querySelector('.snowFreshText');
if (skiForeCastData[pskiForecastCurrentIndex] && pskiForecastCurrentIndex < skiForecastMaxCount && skiForeCastData[pskiForecastCurrentIndex].date === forecastData.date) {
snowData.innerHTML = skiForeCastData[pskiForecastCurrentIndex].freshSnow;
pskiForecastCurrentIndex++;
snowData.style.display = "inline";
} else {
snowData.innerHTML = "";
snowData.style.display = 'none';
}
}
var pNode = pThisDailyTemplate.querySelector('.pDailyModuleFirstProvider');
ui.setForecastNodeAriaLabel(pNode, isMinimized, showLimitedExperience, ariaTextValues);
if (isDailyDrilldownEnabled) {
WinJS.Utilities.addClass(pNode, 'enablePortraitHoverEffect');
pNode.onmspointerover = function () {
ui.hideAllForecastOverlays();
};
ui.attachFocusHandlerForDailyForecast(pNode, i, locationID);
}
if (i === totalConditions - 1) {
// based on max forecast days, show / hide the slide animation button
if (ui._totalDailyForecasts <= 5) {  // total daily conditions we show now: 5
pForecastSlider.style.display = "none";
} else {
pForecastSlider.style.display = "block";
}
}
pPrimaryProvider.appendChild(pThisDailyTemplate);
}
}
else if (counter === 1) {
var displayProviderUrl1 = displayProviderUrl;
if (!otherProviderDailyModuleTemplate) {
CommonJS.loadModule({ fragmentPath: "/html/templates.html", templateId: "eachDailyModuleSmall" }, forecastData, null, null, item).then(function (otherProviderDailyTemplateNext) {
otherProviderDailyModuleTemplate = otherProviderDailyTemplateNext;
otherProviderForecastWrapper1.appendChild(otherProviderDailyTemplateNext);
var tmpTemplate1 = otherProviderDailyTemplateNext.cloneNode(true); // for portrait
pOtherProvider1.appendChild(tmpTemplate1);
});
} else {
var otherProviderDailyTemplate = otherProviderDailyModuleTemplate.cloneNode(true);
otherProviderDailyTemplate.querySelector('.tempHigh').innerText = forecastData.temphigh;
otherProviderDailyTemplate.querySelector('.tempLow').innerText = forecastData.templow;
otherProviderDailyTemplate.querySelector('.precip').innerText = forecastData.precipitation;
var skyCodeImageProvider1 = otherProviderDailyTemplate.querySelector('.skycodeImage');
skyCodeImageProvider1.src = forecastData.skycodeSmall;
skyCodeImageProvider1.alt = forecastData.caption;
otherProviderForecastWrapper1.appendChild(otherProviderDailyTemplate);
if (i === totalConditions - 1) {
var otherProviderFloater1 = document.createElement('div');
winjsAddClass(otherProviderFloater1, "clearFloats");
otherProviderForecastWrapper1.appendChild(otherProviderFloater1);
otherProviderName1.innerHTML = displayProviderUrl1;
pOtherProviderName1.innerHTML = displayProviderUrl1;
otherProviderName1.disabled = isMinimized;
}
var tmpTemplate2 = otherProviderDailyTemplate.cloneNode(true);
pOtherProvider1.appendChild(tmpTemplate2);
}
}
else {
var displayProviderUrl2 = displayProviderUrl;
if (!otherProviderDailyModuleTemplate) {
CommonJS.loadModule({ fragmentPath: "/html/templates.html", templateId: "eachDailyModuleSmall" }, forecastData, null, null, item).then(function (otherProviderDailyTemplateSmall) {
otherProviderDailyModuleTemplate = otherProviderDailyTemplateSmall;
otherProviderForecastWrapper2.appendChild(otherProviderDailyTemplateSmall);
var tmpTemplate = otherProviderDailyTemplateSmall.cloneNode(true); // for portrait
pOtherProvider2.appendChild(tmpTemplate);
});
} else {
var otherProviderDailyTemplate2 = otherProviderDailyModuleTemplate.cloneNode(true);
otherProviderDailyTemplate2.querySelector('.tempHigh').innerText = forecastData.temphigh;
otherProviderDailyTemplate2.querySelector('.tempLow').innerText = forecastData.templow;
otherProviderDailyTemplate2.querySelector('.precip').innerText = forecastData.precipitation;
var skyCodeImageProvider2 = otherProviderDailyTemplate2.querySelector('.skycodeImage');
skyCodeImageProvider2.src = forecastData.skycodeSmall;
skyCodeImageProvider2.alt = forecastData.caption;
otherProviderForecastWrapper2.appendChild(otherProviderDailyTemplate2);
if (i === totalConditions - 1) {
var otherProviderFloater2 = document.createElement('div');
winjsAddClass(otherProviderFloater2, "clearFloats");
otherProviderForecastWrapper2.appendChild(otherProviderFloater2);
otherProviderName2.innerHTML = displayProviderUrl2;
pOtherProviderName2.innerHTML = displayProviderUrl2;
otherProviderName2.disabled = isMinimized;
}
var pTemplate = otherProviderDailyTemplate2.cloneNode(true);
pOtherProvider2.appendChild(pTemplate);
}
}
}
counter++;
}
var providers = Object.keys(dailyConditions);
var primaryProvider = providers[0];
var totalForecasts = dailyConditions[primaryProvider].forecasts.length;
msWriteProfilerMark("WeatherApp:HomePageLoad:PrecipModule:s");                
CommonJS.loadModule({ fragmentPath: "/html/templates.html", templateId: "precipModule" }, dailyConditions[primaryProvider].forecasts[0], null, null, item).then(function (precipTemplate) {
var precipFragment = document.createDocumentFragment();
precipTemplate.querySelector(".precip").setAttribute("dir", currentConditionsData.symbolPosition);
for (var i = 0; i < totalForecasts; i++) {
var currentPrecipTemplate = precipTemplate.cloneNode(true);
var precip = currentPrecipTemplate.querySelector('.precip');
if (dailyConditions[primaryProvider].forecasts[i].precipitation === "--") {
precip.style.display = 'none';
currentPrecipTemplate.querySelector('.raindrop').style.display = 'none';
} else {
precip.innerText = dailyConditions[primaryProvider].forecasts[i].precipitation;
}
// bind skiforecast data if available. Check for Date before binding.
if (skiForeCastData) {
var psnowData = currentPrecipTemplate.querySelector('.snowFreshText');
if (skiForeCastData[skiForecastCurrentIndex] && skiForecastCurrentIndex < skiForecastMaxCount && skiForeCastData[skiForecastCurrentIndex].date === dailyConditions[primaryProvider].forecasts[i].date) {
psnowData.innerHTML = skiForeCastData[skiForecastCurrentIndex].freshSnow;
skiForecastCurrentIndex++;
} else {
psnowData.innerHTML = "";
psnowData.style.display = 'none';
}                            
}
precipFragment.appendChild(currentPrecipTemplate);
}
var precipFloater = document.createElement('div');
winjsAddClass(precipFloater, "clearFloats");
precipFragment.appendChild(precipFloater);
precipitationWrapper.appendChild(precipFragment);
msWriteProfilerMark("WeatherApp:HomePageLoad:PrecipModule:e");
});
/// set the hero cluster state based on the view type - portrait, landscape etc.
/// pass in the template so state changes can be done by querying on the template instead of the entire document
// no animation required in limited experience, where we show only one provider data.
if (!ui.showLimitedExperience(locationID)) {
ui._isDailyForecastScrolled = false;
ui.setHeroClusterState(viewManagement.ApplicationView.value, ccTemplate, isSkiPano);
}
// for landscape and filled, adjust hero based on resolution
var viewState = viewManagement.ApplicationViewState;
if (viewManagement.ApplicationView.value === viewState.fullScreenLandscape || viewManagement.ApplicationView.value === viewState.filled) {
ui.adjustHeroClusterForResolution(ccTemplate, locationID);
}
// set tab indexes after adjustHeroCluster so that the global _totalDailyForecastsShown will be available
if (!ui.showLimitedExperience(locationID) && isDailyDrilldownEnabled) {
WeatherAppJS.Utilities.TabIndexManager.setTabIndexOnDailyConditions(ccCluster, true); // set page load to true
}
var animateButton = ccCluster.querySelector(".forecastSlider .forecastAnimateButton");
var animateButtonPortrait = ccCluster.querySelector(".portraitSliders .pDailyConditionsAnimateButton ");
winjsAddClass(animateButton, "buttonRight");
winjsAddClass(animateButtonPortrait, "buttonDown");
/* If weather.com is available as a provider, get all the nodes of the daily conditions div
and then add the 'see more' text 
*/
if (isWeatherDotCom) {
CommonJS.loadModule({fragmentPath:"/html/templates.html", templateId:"seeMoreTemplate"}, {}, null, null, null).then(function (seeMoreTemplate) {
var children = null;
var seeMore = seeMoreTemplate.querySelector('.weatherDotComSeeMoreText');
seeMore.setAttribute('href', weatherDotComUrl);
if (weatherDotComPosition === 0) {
children = mainProviderForecastWrapper.childNodes;
mainProviderForecastWrapper.insertBefore(seeMoreTemplate, children[children.length - 1]);
} else if (weatherDotComPosition === 1) {
children = otherProviderForecastWrapper1.childNodes;
otherProviderForecastWrapper1.insertBefore(seeMoreTemplate, children[children.length - 1]);
} else {
children = otherProviderForecastWrapper2.childNodes;
otherProviderForecastWrapper2.insertBefore(seeMoreTemplate, children[children.length - 1]);
}
// determine the total daily conditions
var dailyConditionsDefaultShown = ui.getDefaultDailyConditionsTotal();
if (children.length > dailyConditionsDefaultShown) {
seeMore.innerText = resourceLoader.getString("SeeMoreTruncatedText");
} else {
seeMore.innerText = WeatherAppJS.Utilities.Common.getWeatherDotComSeeMoreText();
}
// enable/disable see more text based on whether it's visible or not
ui.setSeeMoreStatus(ccCluster, seeMore, isMinimized, viewManagement.ApplicationViewState.fullScreenLandscape);
});
// portrait - see more for weather.com. This is loaded differently because the behavior for landscape is different based on animation 
CommonJS.loadModule({ fragmentPath: "/html/templates.html", templateId: "pSeeMoreTemplate" }, {}, null, null, null).then(function (pSeeMoreTemplate) {
var seeMore = pSeeMoreTemplate.querySelector('.weatherDotComSeeMoreText');
seeMore.setAttribute('href', weatherDotComUrl);
seeMore.innerText = WeatherAppJS.Utilities.Common.getWeatherDotComSeeMoreText();
if (weatherDotComPosition === 0) {
pPrimaryProvider.appendChild(pSeeMoreTemplate);
} else if (weatherDotComPosition === 1) {
pOtherProvider1.appendChild(pSeeMoreTemplate);
} else {
pOtherProvider2.appendChild(pSeeMoreTemplate);
}
// enable/disable see more text based on whether it's visible or not
ui.setSeeMoreStatus(ccCluster, seeMore, isMinimized, viewManagement.ApplicationViewState.fullScreenPortrait);
});
}
// determine if we want to show hover status and click handler to enable daily drilldown
if (isDailyDrilldownEnabled) {
var ccSlider = ccCluster.querySelector('.currentTemp');
if (ccSlider) {
ui.attachClickHandlerForForecast(ccSlider, locationID);
}
var portraitWrapper = ccCluster.querySelector('.portraitMultiProviderWrapper');
if (portraitWrapper) {
ui.attachClickHandlerForForecast(portraitWrapper, locationID);
}
}
});
return ccCluster;
},
_getAriaTextForDailyConditions: function _getAriaTextForDailyConditions(dailyConditions, index) {
if (!dailyConditions || index < 0) {
return "";
}
var providers = Object.keys(dailyConditions);
if (providers.length === 0 || dailyConditions[providers[0]].forecasts.length <= index) {
return "";
}
var firstProvider = dailyConditions[providers[0]].forecasts[index];
var resourceLoader = PlatformJS.Services.resourceLoader;
var ariaText = resourceLoader.getString("dailyForecast").format(firstProvider.fullDate, firstProvider.temphighWithoutUnit + " °",
firstProvider.templowWithoutUnit + " °", firstProvider.caption, firstProvider.precipitation);
WeatherAppJS.Utilities.Instrumentation.dayOfWeek[index] = ((new Date()).getDay() + index) % 7;
var ariaTextExpanded = ariaText;
var secondProvider;
var thirdProvider;
if (providers.length >= 3
&& dailyConditions[providers[1]].forecasts.length > index
&& dailyConditions[providers[2]].forecasts.length > index) {
secondProvider = dailyConditions[providers[1]].forecasts[index];
thirdProvider = dailyConditions[providers[2]].forecasts[index];
ariaTextExpanded = resourceLoader.getString("dailyForecastExpanded2").format(firstProvider.fullDate, firstProvider.temphighWithoutUnit + " °",
firstProvider.templowWithoutUnit + " °", firstProvider.caption, firstProvider.precipitation,
secondProvider.temphighWithoutUnit + " °", secondProvider.templowWithoutUnit + " °", providers[1],
thirdProvider.temphighWithoutUnit + " °", thirdProvider.templowWithoutUnit + " °", providers[2]);
}
else if (providers.length >= 2
&& dailyConditions[providers[1]].forecasts.length > index) {
secondProvider = dailyConditions[providers[1]].forecasts[index];
ariaTextExpanded = resourceLoader.getString("dailyForecastExpanded1").format(firstProvider.fullDate, firstProvider.temphighWithoutUnit + " °",
firstProvider.templowWithoutUnit + " °", firstProvider.caption, firstProvider.precipitation,
secondProvider.temphighWithoutUnit + " °", secondProvider.templowWithoutUnit + " °", providers[1]);
}
return { ariaText: ariaText, ariaTextExpanded: ariaTextExpanded };
},
// hourly forecast cluster template
getHourlyForecastTemplate: function getHourlyForecastTemplate(item) {
var hourlyConditionsData = item.data.hourlyConditionsData;
if (!hourlyConditionsData) {
return;
}
var resourceLoader = PlatformJS.Services.resourceLoader;
var winjsAddClass = WinJS.Utilities.addClass;
var weatherUI = WeatherAppJS.Utilities.UI;
var hfCluster = document.createElement("div");
winjsAddClass(hfCluster, "win-interactive hourlyForecastCluster");
var hCluster = document.createElement("div");
winjsAddClass(hCluster, "hourlyForecastModule");
var hHeader = document.createElement("div");
winjsAddClass(hHeader, "hourlyForecastHeader");
var hRows = document.createElement("div");
winjsAddClass(hRows, "hourlyForecastRows");
hRows.addEventListener("mousewheel", CommonJS.Immersive.PanoramaPanel.verticalScrollHandler, false);
//scroll event
var isHourlyForecastScrolledDown = false;
var isHourlyForecastScrolledUp = false;
var lastScrollTop = 0;
hRows.addEventListener('scroll',
function handleScroll(event) {
var eventAttr = { location: WeatherAppJS.GeocodeCache.getLocation(item.data.location).getFullDisplayName(), dayIndex: item.data.index, dayOfWeek: item.data.dayOfWeek};
//consecutive downward or consecutive upward scrolls are logged as single scroll.
if (this.scrollTop > lastScrollTop) {
if (!isHourlyForecastScrolledDown) {
isHourlyForecastScrolledDown = true;
isHourlyForecastScrolledUp = false;
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, "Day Drilldown", "Hourly Forecast", Platform.Telemetry.UserActionOperation.scroll, Platform.Telemetry.UserActionMethod.unknown, 0, JSON.stringify(eventAttr));
}
}
else if (!isHourlyForecastScrolledUp) {
isHourlyForecastScrolledUp = true;
isHourlyForecastScrolledDown = false;
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, "Day Drilldown", "Hourly Forecast", Platform.Telemetry.UserActionOperation.scroll, Platform.Telemetry.UserActionMethod.unknown, 0, JSON.stringify(eventAttr));
}
lastScrollTop = this.scrollTop;
});
var hAttributions = document.createElement("div");
winjsAddClass(hAttributions, "hourlyForecastAttributions");
var hourlyConditionsDataPerProvider;
for (var provider in hourlyConditionsData) {
hourlyConditionsDataPerProvider = hourlyConditionsData[provider];
break; // We pick only the first providers data.
}
CommonJS.loadModule({ fragmentPath: item.data.fragmentPath, templateId: item.data.hourlyHeaderTemplate }, {}, null, null, item).then(function (hhTemplate) {
hHeader.appendChild(hhTemplate); /// append the header
var isSkiPano = WeatherAppJS.GeocodeCache.getLocation(item.data.location).isSkiLocation;
weatherUI.setHourlyHeaderCellWidth(hhTemplate.querySelector('.precip'), 'hourlyHeaderPrecipCalculator'); // for precip
var feelsLikeHeader = hhTemplate.querySelector('.feelslike');
if (feelsLikeHeader) {
weatherUI.setHourlyHeaderCellWidth(feelsLikeHeader, 'hourlyHeaderFeelsLikeCalculator'); // for feelslike  
}
var templateId = item.data.hourlyRowTemplate;
var symbolPosition = WeatherAppJS.Utilities.Formatting.getSymbolPosition();
var hourlyFragment = document.createDocumentFragment();
for (var i = 0, j = hourlyConditionsDataPerProvider.length; i < j ; i++) {
if (i % 2 === 0) {
hourlyConditionsDataPerProvider[i].className = "hourlyConditions row1";
} else {
hourlyConditionsDataPerProvider[i].className = "hourlyConditions row2";
}
hourlyConditionsDataPerProvider[i].symbolPosition = symbolPosition;
var regex;
if (WeatherAppJS.UI[templateId] && WeatherAppJS.UI[templateId].querySelector('.hourlyConditions')) {
var hTemplate1 = WeatherAppJS.UI[templateId].cloneNode(true);
hTemplate1.querySelector('.hourlyConditions').className = hourlyConditionsDataPerProvider[i].className;
hTemplate1.querySelector('.hour').innerText = hourlyConditionsDataPerProvider[i].time;
hTemplate1.querySelector('.skycodeImage').src = hourlyConditionsDataPerProvider[i].skycode;
hTemplate1.querySelector('.skycodeImage').alt = hourlyConditionsDataPerProvider[i].caption;
hTemplate1.querySelector('.temperature').innerText = hourlyConditionsDataPerProvider[i].temperature;
hTemplate1.querySelector('.temperature').dir = hourlyConditionsDataPerProvider[i].symbolPosition;
hTemplate1.querySelector('.caption').innerText = hourlyConditionsDataPerProvider[i].caption;
var feelsLikeNode = hTemplate1.querySelector('.feelslike');
if (feelsLikeNode) {
feelsLikeNode.innerText = hourlyConditionsDataPerProvider[i].feelslike;
feelsLikeNode.dir = hourlyConditionsDataPerProvider[i].symbolPosition;
}
hTemplate1.querySelector('.precipValue').innerText = hourlyConditionsDataPerProvider[i].precipitation;
var ariaText1 = resourceLoader.getString("hourlyForecastAriaText").format(hourlyConditionsDataPerProvider[i].time, hourlyConditionsDataPerProvider[i].temperatureWithoutUnit + " °",
hourlyConditionsDataPerProvider[i].caption, hourlyConditionsDataPerProvider[i].feelslike, hourlyConditionsDataPerProvider[i].precipitation);
if (isSkiPano) {
regex = new RegExp(resourceLoader.getString("HourlyForecastFeelsLikeTitle") + " " + hourlyConditionsDataPerProvider[i].feelslike + ",", 'i');
ariaText1 = ariaText1.replace(regex, "");
}
hTemplate1.querySelector(".hourlyConditions").setAttribute("aria-label", ariaText1);
hTemplate1.querySelector(".precipValue").setAttribute("dir", symbolPosition);
if (feelsLikeHeader) {
weatherUI.setHourlyHeaderCellWidth(hTemplate1.querySelector('.feelslike'), 'hourlyHeaderFeelsLikeCalculator'); //for feelslike row text
}
hTemplate1.removeAttribute('tabindex');
hTemplate1.addEventListener('keydown', WeatherAppJS.Utilities.TabIndexManager.handleArrowKeyForAccessibility, false);
hourlyFragment.appendChild(hTemplate1);
} else {
CommonJS.loadModule({ fragmentPath: "/html/templates.html", templateId: item.data.hourlyRowTemplate }, hourlyConditionsDataPerProvider[i], null, null, item).then(function (hTemplate) {
WeatherAppJS.UI[templateId] = hTemplate;
var ariaText = resourceLoader.getString("hourlyForecastAriaText").format(hourlyConditionsDataPerProvider[i].time, hourlyConditionsDataPerProvider[i].temperatureWithoutUnit + " °",
hourlyConditionsDataPerProvider[i].caption, hourlyConditionsDataPerProvider[i].feelslike, hourlyConditionsDataPerProvider[i].precipitation);
if (isSkiPano) {
regex = new RegExp(resourceLoader.getString("HourlyForecastFeelsLikeTitle") + " " + hourlyConditionsDataPerProvider[i].feelslike + ",", 'i');
ariaText = ariaText.replace(regex, "");
}
hTemplate.querySelector(".hourlyConditions").setAttribute("aria-label", ariaText);
hTemplate.querySelector(".precipValue").setAttribute("dir", symbolPosition);
if (feelsLikeHeader) {
weatherUI.setHourlyHeaderCellWidth(hTemplate.querySelector('.feelslike'), 'hourlyHeaderFeelsLikeCalculator'); //for feelslike row text
}
hTemplate.setAttribute('tabindex', '0');
hTemplate.addEventListener('keydown', WeatherAppJS.Utilities.TabIndexManager.handleArrowKeyForAccessibility, false);
hourlyFragment.appendChild(hTemplate);
});
}
}
hRows.appendChild(hourlyFragment);
});
if (item.data.showHourlyAttribution) {
CommonJS.loadModule({ fragmentPath: "/html/templates.html", templateId: "hourlyAttributions" }, hourlyConditionsDataPerProvider, null, null, item).then(function (hTemplate) {
var ariaText = resourceLoader.getString("DataAttributionAriaText").format(hourlyConditionsDataPerProvider.providerName);
hTemplate.querySelector("a").setAttribute("aria-label", ariaText);
hAttributions.appendChild(hTemplate);
});
}
hCluster.appendChild(hHeader);
hCluster.appendChild(hRows);
hfCluster.appendChild(hCluster);
hfCluster.appendChild(hAttributions);
return hfCluster;
},
// hourly forecast cluster template
getHourlyForecastTemplate2: function getHourlyForecastTemplate2(data, options) {
var isChinaExp = options.isChinaExp || false;
var hourlyConditionsData = data.getHourlyConditions();
if (!hourlyConditionsData) {
return;
}
var resourceLoader = PlatformJS.Services.resourceLoader;
var winjsAddClass = WinJS.Utilities.addClass;
var weatherUI = WeatherAppJS.Utilities.UI;
var hfCluster = document.createElement("div");
winjsAddClass(hfCluster, "win-interactive hourlyForecastCluster");
var hCluster = document.createElement("div");
winjsAddClass(hCluster, "hourlyForecastModule");
var hHeader = document.createElement("div");
winjsAddClass(hHeader, "hourlyForecastHeader");
var hRows = document.createElement("div");
winjsAddClass(hRows, "hourlyForecastRows");
hRows.addEventListener("mousewheel", CommonJS.Immersive.PanoramaPanel.verticalScrollHandler, false);
//scroll event
var isHourlyForecastScrolledDown = false;
var isHourlyForecastScrolledUp = false;
var lastScrollTop = 0;
hRows.addEventListener('scroll',
function handleScroll(event) {
var eventAttr = { location: WeatherAppJS.GeocodeCache.getLocation(data.id).getFullDisplayName() };
//consecutive downward or consecutive upward scrolls are logged as single scroll.
if (this.scrollTop > lastScrollTop) {
if (!isHourlyForecastScrolledDown) {
isHourlyForecastScrolledDown = true;
isHourlyForecastScrolledUp = false;
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, "Hourly Forecast", "Hourly Forecast", Platform.Telemetry.UserActionOperation.scroll, Platform.Telemetry.UserActionMethod.unknown, 0, JSON.stringify(eventAttr));
}
}
else if (!isHourlyForecastScrolledUp) {
isHourlyForecastScrolledUp = true;
isHourlyForecastScrolledDown = false;
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, "Hourly Forecast", "Hourly Forecast", Platform.Telemetry.UserActionOperation.scroll, Platform.Telemetry.UserActionMethod.unknown, 0, JSON.stringify(eventAttr));
}
lastScrollTop = this.scrollTop;
});
var hAttributions = document.createElement("div");
winjsAddClass(hAttributions, "hourlyForecastAttributions");
var hourlyConditionsDataPerProvider;
for (var provider in hourlyConditionsData) {
hourlyConditionsDataPerProvider = hourlyConditionsData[provider];
break; // We pick only the first providers data.
}
// here we calculate the wind header length based on longest value provided
var hourlyConditionsDataLength = hourlyConditionsDataPerProvider.length;
var windHeaderWidth = hourlyConditionsDataPerProvider[hourlyConditionsDataLength - 1].wind;
for (var i = hourlyConditionsDataLength - 2; i >= 0 ; i--) {
if (windHeaderWidth.length < hourlyConditionsDataPerProvider[i].wind.length) {
windHeaderWidth = hourlyConditionsDataPerProvider[i].wind;
}
}
var windHeaderCellWidth = WeatherAppJS.Utilities.UI.getPixelWidthFromString(windHeaderWidth, 'hourlyHeaderWindCalculator');
CommonJS.loadModule({ fragmentPath: "/html/templates.html", templateId: "hourlyHeaderModule" }, {}, null, null, null).then(function (hhTemplate) {
hHeader.appendChild(hhTemplate); /// append the header
weatherUI.setHourlyHeaderCellWidth(hhTemplate.querySelector('.precip'), 'hourlyHeaderPrecipCalculator'); // for precip                
var isSkiPano = WeatherAppJS.GeocodeCache.getLocation(data.id).isSkiLocation;
var windHeader = hhTemplate.querySelector('.wind');
windHeader.style.width = windHeaderCellWidth;
var feelsLikeHeader = hhTemplate.querySelector('.feelslike');
if (feelsLikeHeader) {
weatherUI.setHourlyHeaderCellWidth(feelsLikeHeader, 'hourlyHeaderFeelsLikeCalculator'); // for feelslike  
}
var templateId = options.hourlyRowTemplate;
if (isChinaExp) {
templateId = templateId + "_China";
}
var symbolPosition = WeatherAppJS.Utilities.Formatting.getSymbolPosition();
var hourlyFragment = document.createDocumentFragment();
for (var i = 0, j = hourlyConditionsDataPerProvider.length; i < j ; i++) {
if (i % 2 === 0) {
hourlyConditionsDataPerProvider[i].className = "hourlyConditions row1";
} else {
hourlyConditionsDataPerProvider[i].className = "hourlyConditions row2";
}
hourlyConditionsDataPerProvider[i].symbolPosition = symbolPosition;
var regex;
if (WeatherAppJS.UI[templateId] && WeatherAppJS.UI[templateId].querySelector('.hourlyConditions')) {
var hTemplate1 = WeatherAppJS.UI[templateId].cloneNode(true);
hTemplate1.querySelector('.hourlyConditions').className = hourlyConditionsDataPerProvider[i].className;
hTemplate1.querySelector('.hour').innerText = hourlyConditionsDataPerProvider[i].time;
hTemplate1.querySelector('.skycodeImage').src = hourlyConditionsDataPerProvider[i].skycode;
hTemplate1.querySelector('.skycodeImage').alt = hourlyConditionsDataPerProvider[i].caption;
hTemplate1.querySelector('.temperature').innerText = hourlyConditionsDataPerProvider[i].temperature;
hTemplate1.querySelector('.temperature').dir = hourlyConditionsDataPerProvider[i].symbolPosition;
hTemplate1.querySelector('.caption').innerText = hourlyConditionsDataPerProvider[i].caption;
var feelsLikeNode = hTemplate1.querySelector('.feelslike');
if (feelsLikeNode) {
feelsLikeNode.innerText = hourlyConditionsDataPerProvider[i].feelslike;
feelsLikeNode.dir = hourlyConditionsDataPerProvider[i].symbolPosition;
}
hTemplate1.querySelector('.precipValue').innerText = hourlyConditionsDataPerProvider[i].precipitation;
hTemplate1.querySelector('.wind').innerText = hourlyConditionsDataPerProvider[i].wind;                       
hTemplate1.querySelector(".precipValue").setAttribute("dir", symbolPosition);
if (feelsLikeHeader) {
weatherUI.setHourlyHeaderCellWidth(hTemplate1.querySelector('.feelslike'), 'hourlyHeaderFeelsLikeCalculator'); //for feelslike row text
}
var ariaText1;
if (isChinaExp) {
// Overrides for China Experience (TianQi)
var temp = hTemplate1.querySelectorAll('.temperature');
temp[0].innerText = hourlyConditionsDataPerProvider[i].tempLow;
temp[1].innerText = hourlyConditionsDataPerProvider[i].tempHigh;
ariaText1 = resourceLoader.getString("hourlyForecastAriaText").format(hourlyConditionsDataPerProvider[i].time, hourlyConditionsDataPerProvider[i].tempLow,
hourlyConditionsDataPerProvider[i].tempHigh, hourlyConditionsDataPerProvider[i].caption, hourlyConditionsDataPerProvider[i].wind);
} else {
ariaText1 = resourceLoader.getString("hourlyForecastAriaText").format(hourlyConditionsDataPerProvider[i].time, hourlyConditionsDataPerProvider[i].temperatureWithoutUnit + " °",
hourlyConditionsDataPerProvider[i].caption, hourlyConditionsDataPerProvider[i].feelslike, hourlyConditionsDataPerProvider[i].precipitation);
}
if (isSkiPano) {
regex = new RegExp(resourceLoader.getString("HourlyForecastFeelsLikeTitle") + " " + hourlyConditionsDataPerProvider[i].feelslike + ",", 'i');
ariaText1 = ariaText1.replace(regex, "");
}
hTemplate1.querySelector(".hourlyConditions").setAttribute("aria-label", ariaText1);
hTemplate1.removeAttribute('tabindex');
hTemplate1.addEventListener('keydown', WeatherAppJS.Utilities.TabIndexManager.handleArrowKeyForAccessibility, false);
hourlyFragment.appendChild(hTemplate1);
} else {
CommonJS.loadModule({ fragmentPath: "/html/templates.html", templateId: "hourlyModule" }, hourlyConditionsDataPerProvider[i], null, null, null).then(function (hTemplate) {
WeatherAppJS.UI[templateId] = hTemplate;
var ariaText;
if (!isChinaExp) {
ariaText = resourceLoader.getString("hourlyForecastAriaText").format(hourlyConditionsDataPerProvider[i].time, hourlyConditionsDataPerProvider[i].temperatureWithoutUnit + " °",
hourlyConditionsDataPerProvider[i].caption, hourlyConditionsDataPerProvider[i].feelslike, hourlyConditionsDataPerProvider[i].precipitation);
if (isSkiPano) {
regex = new RegExp(resourceLoader.getString("HourlyForecastFeelsLikeTitle") + " " + hourlyConditionsDataPerProvider[i].feelslike + ",", 'i');
ariaText = ariaText.replace(regex, "");
}
hTemplate.querySelector(".hourlyConditions").setAttribute("aria-label", ariaText);
hTemplate.querySelector(".precipValue").setAttribute("dir", symbolPosition);
hTemplate.querySelector(".wind").style.width = windHeaderCellWidth;
if (feelsLikeHeader) {
weatherUI.setHourlyHeaderCellWidth(hTemplate.querySelector('.feelslike'), 'hourlyHeaderFeelsLikeCalculator'); //for feelslike row text
}
} else {
ariaText = resourceLoader.getString("hourlyForecastAriaText").format(hourlyConditionsDataPerProvider[i].time, hourlyConditionsDataPerProvider[i].tempLow,
hourlyConditionsDataPerProvider[i].tempHigh, hourlyConditionsDataPerProvider[i].caption, hourlyConditionsDataPerProvider[i].wind);
hTemplate.querySelector(".hourlyConditions").setAttribute("aria-label", ariaText);
hTemplate.querySelector(".wind").style.width = windHeaderCellWidth;
var tempWrapper = hTemplate.querySelector('.tempContainer');
var tempLowSpan = document.createElement('span');
winjsAddClass(tempLowSpan, 'temperature');
tempLowSpan.innerText = hourlyConditionsDataPerProvider[i].tempLow;
tempLowSpan.dir = symbolPosition;
var tempHighSpan = hTemplate.querySelector('.temperature');
tempHighSpan.innerText = hourlyConditionsDataPerProvider[i].tempHigh;
var tilda = document.createElement('span');
tilda.innerText = '~';
tempWrapper.insertBefore(tilda, tempHighSpan);
tempWrapper.insertBefore(tempLowSpan, tilda);
}
hTemplate.setAttribute('tabindex', '0');
hTemplate.addEventListener('keydown', WeatherAppJS.Utilities.TabIndexManager.handleArrowKeyForAccessibility, false);
hourlyFragment.appendChild(hTemplate);
});
}
}
hRows.appendChild(hourlyFragment);
});
if (options.showHourlyAttribution) {
CommonJS.loadModule({ fragmentPath: "/html/templates.html", templateId: "hourlyAttributions" }, hourlyConditionsDataPerProvider, null, null, null).then(function (hTemplate) {
var ariaText = resourceLoader.getString("DataAttributionAriaText").format(hourlyConditionsDataPerProvider.providerName);
hTemplate.querySelector("a").setAttribute("aria-label", ariaText);
hAttributions.appendChild(hTemplate);
});
}
hCluster.appendChild(hHeader);
hCluster.appendChild(hRows);
hfCluster.appendChild(hCluster);
hfCluster.appendChild(hAttributions);
return hfCluster;
},
getHistoricalTemplate: function getHistoricalTemplate(loc) {
var hwData = loc.getHistoricalWeather();
var hwCluster = document.createElement('div');
WinJS.Utilities.addClass(hwCluster, "historicWrapper");
// Get the historical weather template from store.
CommonJS.loadModule({ fragmentPath: "/html/templates.html", templateId: "historicalWeather" }, {}, null, null, null).then(function (hTemplate) {
hwCluster.appendChild(hTemplate);
msSetImmediate(function () {
WeatherAppJS.UI.HWChartManager.init(hTemplate, hwData, WeatherAppJS.GeocodeCache.getLocation(loc.id).getFullDisplayName());
});
});
return hwCluster;
},
getSemanticZoomTemplate: WinJS.UI.eventHandler(function (itemPromise) {
return {
element: itemPromise.then(function (itemObj) {
var div = document.createElement("div");
var clusterTitle = itemObj.data.clusterTitle;
if (!clusterTitle) {
clusterTitle = PlatformJS.Services.resourceLoader.getString("Overview");
}
WinJS.Utilities.addClass(div, "zoomedOutTemplate");
// seperate styles for daily drilldown semantic zoom, so set a seperate class for that
var currentPage = WeatherAppJS.Utilities.Common.getCurrentPage();
if (currentPage === "WeatherAppJS.DailyForecast") {
WinJS.Utilities.addClass(div, "ddSemanticZoom");
}
var uxInfo = {
clusterTitle: clusterTitle,
moduleInfo: {
fragmentPath: "/html/templates.html",
templateId: "zoomedOutTemplate"
}
};
CommonJS.loadModule(uxInfo.moduleInfo, uxInfo, div).then();
return div;
})
};
}),
getTianQiTemplate: function getTianQiTemplate(item) {
var that = WeatherAppJS.Utilities.Templates;
var ui = WeatherAppJS.Utilities.UI;
var viewManagement = Windows.UI.ViewManagement;
var winjsAddClass = WinJS.Utilities.addClass;
var resourceLoader = PlatformJS.Services.resourceLoader;
var appConfig = PlatformJS.Services.appConfig;
var ccCluster = document.createElement('div');
winjsAddClass(ccCluster, "win-interactive currentConditionsModule tianqiHeroCluster");
var currentConditionsData = item.data.loc.getCurrentConditions();
var dailyConditionsData = item.data.loc.getDailyConditions();
var locationID = item.data.loc.id;
ui._totalDailyForecasts = item.data.loc.maxDailyForecastDays;
CommonJS.loadModule({ fragmentPath: "/html/templates.html", templateId: "ccModule" }, currentConditionsData, null, null, item).then(function (ccTemplate) {
// add event listener for bringing into view the other days
var forecastSlider = ccTemplate.querySelector('.forecastSlider');
forecastSlider.addEventListener('click', (function (locName) {
return function () {
WeatherAppJS.Utilities.UI.AnimateForecastSlide(locName);
};
}(item.data.loc.id)), false);
var pForecastSlider = ccTemplate.querySelector('.pForecastSlider');
pForecastSlider.addEventListener('click', (function (locName) {
return function () {
WeatherAppJS.Utilities.UI.AnimatePortraitForecastSlide(locName);
};
}(item.data.loc.id)), false);
// set aria-text - needs update
var ariaText = resourceLoader.getString("currentConditionsAriaText").format(currentConditionsData.tempCC + " °", currentConditionsData.caption, currentConditionsData.feelslikeTemp, currentConditionsData.provider);
var ccElem = ccTemplate.querySelector(".bTemp");
ccElem.setAttribute("aria-label", ariaText);
var feelsLikeWrapper = ccTemplate.querySelector('.feelsLikeWrapper');
var wind = document.createElement('div');
winjsAddClass(wind, 'ccWind');
wind.innerText = currentConditionsData.wind;
feelsLikeWrapper.appendChild(wind);
var humidity = document.createElement('div');
winjsAddClass(humidity, 'ccHumidity');
humidity.innerText = currentConditionsData.humidity;
feelsLikeWrapper.appendChild(humidity);
// Over-write ProviderWrapper to add Precipitation
if (currentConditionsData.precipitation !== WeatherAppJS.Utilities.Common.getNoDataString()) {
var precipitation = document.createElement('div');
winjsAddClass(precipitation, 'ccPrecipitation');
precipitation.innerText = currentConditionsData.precipitation;
ccTemplate.querySelector('.providerWrapper').appendChild(precipitation);
}
var dailyData = ccTemplate.querySelector('.dailyModule');
var forecastWrapper = ccTemplate.querySelector('.forecastsWrapper');
var pPrimaryProvider = ccTemplate.querySelector('.portraitDailyConditionsWrapper .primaryProvider');
var precipitationWrapper = ccTemplate.querySelector('.precipitationWrapper');
var dailyConditions = dailyConditionsData[0];
var eachDailyModuleTemplate = null; // for landscape
var pEachDailyModuleTemplate = null; // for portrait
var totalConditions = 0;
if (dailyConditions.forecasts) {
totalConditions = dailyConditions.forecasts.length;
}
var mainProviderName = document.createElement('div');
winjsAddClass(mainProviderName, 'tianQiProviderWrapper');
var pPrimaryProviderName = ccTemplate.querySelector('.primaryProviderName');
var providerName = dailyConditions.provider, providerUrl = dailyConditions.url;
var displayProvUrl;
if (providerUrl) {
displayProvUrl = '<a class="dailyConditionsProviderLink" href="' + providerUrl + '" tabIndex="0" >' + providerName + '</a>';
} else {
displayProvUrl = providerName;
}
mainProviderName.setAttribute('role', 'link');
mainProviderName.setAttribute('id', 'autoID_PrimaryProviderChina');
mainProviderName.innerHTML = displayProvUrl;
pPrimaryProviderName.innerHTML = displayProvUrl;
mainProviderName.addEventListener('click', function (event) {
WeatherAppJS.Utilities.Instrumentation.onClickAttribution(mainProviderName.textContent, WeatherAppJS.Instrumentation.PageContext.DailyForecast);
});
pPrimaryProvider.addEventListener('click', function (event) {
WeatherAppJS.Utilities.Instrumentation.onClickAttribution(pPrimaryProvider.textContent, WeatherAppJS.Instrumentation.PageContext.DailyForecast);
});
document.querySelector('.currentConditionsHeroModule').appendChild(mainProviderName);
for (var i = 0; i < totalConditions; i++) {
var forecastData = dailyConditions.forecasts[i];
// aria texts
if (!eachDailyModuleTemplate) {
CommonJS.loadModule({ fragmentPath: "/html/templates.html", templateId: "eachDailyModule" }, forecastData, null, null, item).then(function (dailyTemplate) {
eachDailyModuleTemplate = dailyTemplate;
forecastWrapper.appendChild(dailyTemplate);
});
} else {
var thisDailyTemplate = eachDailyModuleTemplate.cloneNode(true);
thisDailyTemplate.querySelector('.date').innerText = forecastData.date;
thisDailyTemplate.querySelector('.tempHigh').innerText = forecastData.temphigh;
thisDailyTemplate.querySelector('.tempLow').innerText = forecastData.templow;
thisDailyTemplate.querySelector('.caption').innerText = forecastData.caption;
var skyCodeImage = thisDailyTemplate.querySelector('.skycodeImage');
skyCodeImage.alt = forecastData.caption;
skyCodeImage.src = forecastData.skycode;
forecastWrapper.appendChild(thisDailyTemplate);
}
if (!pEachDailyModuleTemplate) {
CommonJS.loadModule({ fragmentPath: "/html/templates.html", templateId: "pEachDailyModule" }, forecastData, null, null, item).then(function (pDailyTemplate) {
pEachDailyModuleTemplate = pDailyTemplate;
pDailyTemplate.querySelector('.precipWrapper').innerText = forecastData.windspeed;
pPrimaryProvider.appendChild(pDailyTemplate);
});
} else {
var thispDailyTemplate = pEachDailyModuleTemplate.cloneNode(true);
thispDailyTemplate.querySelector('.day').innerText = forecastData.day;
thispDailyTemplate.querySelector('.dayName').innerText = forecastData.dayName;
thispDailyTemplate.querySelector('.caption').innerText = forecastData.caption;
thispDailyTemplate.querySelector('.tempHigh').innerText = forecastData.temphigh;
thispDailyTemplate.querySelector('.tempLow').innerText = forecastData.templow;
thispDailyTemplate.querySelector('.precipWrapper').innerText = forecastData.windspeed;
var skycodeImage = thispDailyTemplate.querySelector('.skycodeImage');
skycodeImage.src = forecastData.skycode;
skycodeImage.alt = forecastData.caption;
pPrimaryProvider.appendChild(thispDailyTemplate);
}
if (i === totalConditions - 1) {
if (ui._totalDailyForecasts <= ui._totalDailyForecastsShown) {  // total daily conditions we show now: 5
pForecastSlider.style.display = "none";
forecastSlider.style.display = "none";
} else {
pForecastSlider.style.display = "block";
forecastSlider.style.display = "block";
}
}
}
if (totalConditions > 0) {
// precipitation -> wind
CommonJS.loadModule({ fragmentPath: "/html/templates.html", templateId: "precipModule" }, dailyConditions.forecasts[0], null, null, item).then(function (precipTemplate) {
var precipFragment = document.createDocumentFragment();
for (var i = 0; i < totalConditions; i++) {
var currentPrecipTemplate = precipTemplate.cloneNode(true);
var precip = currentPrecipTemplate.querySelector('.precipWrapper');
precip.innerText = dailyConditions.forecasts[i].windspeed;
precipFragment.appendChild(precip);
}
var precipFloater = document.createElement('div');
winjsAddClass(precipFloater, "clearFloats");
precipFragment.appendChild(precipFloater);
precipitationWrapper.appendChild(precipFragment);
});
}
ui.setHeroClusterState(viewManagement.ApplicationView.value, ccTemplate, true);
ccCluster.appendChild(ccTemplate);
var viewState = viewManagement.ApplicationViewState;
if (viewManagement.ApplicationView.value === viewState.fullScreenLandscape || viewManagement.ApplicationView.value === viewState.filled) {
ui.adjustHeroClusterForResolution(ccTemplate, locationID);
}
var animateButton = ccCluster.querySelector(".forecastSlider .forecastAnimateButton");
var animateButtonPortrait = ccCluster.querySelector(".portraitSliders .pDailyConditionsAnimateButton ");
winjsAddClass(animateButton, "buttonRight");
winjsAddClass(animateButtonPortrait, "buttonDown");
});
return ccCluster;
},
// historic module on daily drilldown forecast
getHistoricDailyDataTemplate: function getHistoricDailyDataTemplate(item) {
var addlModulesClass = item.data.groupKey;
var addlModules = document.querySelector('.' + addlModulesClass);
if (addlModules) {
addlModules.innerHTML = '';
} else {
addlModules = document.createElement('div');
WinJS.Utilities.addClass(addlModules, 'win-interactive addlModules ' + addlModulesClass);
}
var historicData = item.data.historicData;
var resLoader = PlatformJS.Services.resourceLoader;
if (historicData) {
var historicalString = historicData.historicalString;
var historicWrapper = document.createElement('div');
WinJS.Utilities.addClass(historicWrapper, 'historicWrapper');
var historicModule = document.createElement('div');
WinJS.Utilities.addClass(historicModule, 'historicModule');
var historicalHeading = document.createElement('div');
historicalHeading.innerText = resLoader.getString("HistoricalWeather");
WinJS.Utilities.addClass(historicalHeading, 'heading');
var historicalStringElem = null;
if (historicalString) {
historicalStringElem = document.createElement('div');
WinJS.Utilities.addClass(historicalStringElem, 'historicString');
historicalStringElem.innerText = historicalString;
}
var formatting = WeatherAppJS.Utilities.Formatting;
var resourceFile = PlatformJS.Services.resourceLoader;
// There are only max 4 properties to set which are always available or show '--'. So we can bind this directly.
var noData = WeatherAppJS.Utilities.Common.getNoDataString();
var formattedData = {};
if (historicData.MaxTemp === '') {
formattedData.MaxTemp = noData;
} else {
formattedData.MaxTemp = formatting.getTemperatureWithDegreeUnit(historicData.MaxTemp);
}
if (historicData.MinTemp === '') {
formattedData.MinTemp = noData;
} else {
formattedData.MinTemp = formatting.getTemperatureWithDegreeUnit(historicData.MinTemp);
}
if (historicData.MaxRecordedTemp === '') {
formattedData.MaxRecordedTemp = noData;
} else {
formattedData.MaxRecordedTemp = formatting.getTemperatureWithDegreeUnit(historicData.MaxRecordedTemp) + "<span class=\"recordedYear\">(" + historicData.MaxRecordedTempDate + ")</span>";
}
if (historicData.MinRecordedTemp === '') {
formattedData.MinRecordedTemp = noData;
} else {
formattedData.MinRecordedTemp = formatting.getTemperatureWithDegreeUnit(historicData.MinRecordedTemp) + "<span class=\"recordedYear\">(" + historicData.MinRecordedTempDate + ")</span>";
}
CommonJS.loadModule({ fragmentPath: '/html/secondaryTemplates.html', templateId: 'historicColumnTemplate' }, formattedData, null, null, item).then(function (template) {
historicModule.appendChild(template);
// add precipitation values if present
var displayUnit = WeatherAppJS.SettingsManager.getDisplayUnit();
if (historicData.Precipitation !== '') {
var heading1 = template.querySelector('.optionalKey1');
if (heading1) {
heading1.innerText = resourceFile.getString("AverageRainfall");
}
var value1 = template.querySelector('.optionalValue1');
if (value1) {
value1.innerText = formatting.getLengthUnitForHW(historicData.Precipitation, displayUnit);
}
if (historicData.MaxPrecipitation !== '') {
var heading2 = template.querySelector('.optionalKey2');
if (heading2) {
heading2.innerText = resourceFile.getString("MaximumRecordedRainfall");
}
var value2 = template.querySelector('.optionalValue2');
if (value2) {
value2.innerHTML = formatting.getLengthUnitForHW(historicData.MaxPrecipitation, displayUnit) + "<span class=\"recordedYear\">(" + historicData.MaxRecordedPrecipitationDate + ")</span>";
}
}
}
if (addlModules) {
historicWrapper.appendChild(historicalHeading);
if (historicalStringElem) {
historicWrapper.appendChild(historicalStringElem);
}
historicWrapper.appendChild(historicModule);
addlModules.appendChild(historicWrapper);
WinJS.UI.Animation.enterContent(historicWrapper);
}
});
}
var hourlyCluster = WeatherAppJS.Utilities.Templates.getHourlyForecastTemplate(item);
if (hourlyCluster) {
var hourlyWrapper = document.createElement('div');
WinJS.Utilities.addClass(hourlyWrapper, 'hourlyWrapper');
var hourlyHeading = document.createElement('div');
hourlyHeading.innerText = resLoader.getString("HourlyForecast");
WinJS.Utilities.addClass(hourlyHeading, 'heading');
hourlyWrapper.appendChild(hourlyHeading);
// if there's no historic data, hourly module should occupy full height
var hourlyModule = hourlyCluster.querySelector('.hourlyForecastModule');
WinJS.Utilities.addClass(hourlyModule, 'hourlyForecastFullModule');
if (historicData) {
if (historicalString) {
WinJS.Utilities.addClass(hourlyModule, 'hourlyForecastRestrictedModule');
} else {
WinJS.Utilities.addClass(hourlyModule, 'hourlyForecastMediumModule');
}                        
} else {
WinJS.Utilities.addClass(hourlyModule, 'hourlyForecastShortModule');
}
var ui = WeatherAppJS.Utilities.UI;
if (!ui._historicDataDailyDrilldownHeight) {
var hd = document.querySelector('.historicalData');
if (hd) {
ui._historicDataDailyDrilldownHeight = hd.clientHeight;
}
}
hourlyWrapper.appendChild(hourlyCluster);
addlModules.appendChild(hourlyWrapper);
if (item.data.showEnterContentAnimation) {
WinJS.UI.Animation.enterContent(hourlyWrapper);
}
}
return addlModules;
},
getDailyConditionsTemplate: function getDailyConditionsTemplate(item) {
var wrapper = document.createElement('div');
if (item.data && item.data.forecastData) {
var forecastData = item.data.forecastData;
var histClass = item.data.forecastData.histModuleClassName;
CommonJS.loadModule({ fragmentPath: "/html/secondaryTemplates.html", templateId: "dailyConditionTemplate" }, forecastData, null, null, null).then(function (dailyConditionsTemplate) {
var histModule = dailyConditionsTemplate.querySelector('.histModule');
if (histModule) {
WinJS.Utilities.addClass(histModule, histClass);
}
wrapper.appendChild(dailyConditionsTemplate);
WinJS.UI.Animation.enterContent(dailyConditionsTemplate);
});
}
return wrapper;
}
});
/// <summary>
/// All cluster definitions are present in this namespace
/// Existing cluster definitions have pre-defined properties which can be over-ridden using options
/// New properties and events can be added by passing in an options object 
/// </summary>
WinJS.Namespace.define("WeatherAppJS.Utilities.ClusterDefinition", {
StandardListView: function StandardListView(meta, options) {
this.clusterKey = meta.clusterKey;
this.clusterTitle = meta.clusterTitle;
this.clusterContent = {
contentControl: "WinJS.UI.ListView",
contentOptions: {
itemDataSource: meta.itemDataSource,
itemTemplate: meta.itemTemplate,
}
};
this.gridLayout = {
multiSize: false,
cellWidth: 20,
cellHeight: 20,
verticalSpacing: 0,
horizontalSpacing: 0
};
this.selectionMode = "multi";
this.tapBehavior = "invoke";
if (options) {
WinJS.UI.setOptions(this.clusterContent.contentOptions, options);
}
},
AdvancedListView: function AdvancedListView(meta, options) {
this.clusterKey = meta.clusterKey;
this.clusterTitle = meta.clusterTitle;
this.clusterContent = {
contentControl: "CommonJS.UI.ListView2",
contentOptions: {
itemDataSource: meta.itemDataSource,
itemTemplate: meta.itemTemplate,
enabledFeatures: meta.enabledFeatures,
addTileItemTemplate: meta.addTileItemTemplate
}
};
this.gridLayout = {
multiSize: false,
cellWidth: 20,
cellHeight: 20,
verticalSpacing: 0,
horizontalSpacing: 0
};
this.selectionMode = "multi";
this.tapBehavior = "invoke";
if (options) {
WinJS.UI.setOptions(this.clusterContent.contentOptions, options);
}
},
Hero: function Hero(meta, options) {
this.clusterKey = meta.clusterKey;
this.clusterTitle = meta.clusterTitle;
this.hideTitle = true;
this.isSticky = true;
this.titleClass = "headerFontLight";
this.clusterContent = {
contentControl: "CommonJS.Immersive.ParallaxItemsContainer",
contentOptions: {
itemDataSource: meta.dataSource,
className: meta.className,
imageData: meta.imageData
}
};
if (options) {
WinJS.UI.setOptions(this.clusterContent.contentOptions, options);
}
},
ItemsContainer: function ItemsContainer(meta, options) {            
this.clusterKey = meta.clusterKey;
this.clusterTitle = meta.clusterTitle;
this.supressTitleInHeader = meta.supressTitleInHeader;
this.clusterContent = {
contentControl: "CommonJS.Immersive.ItemsContainer",
contentOptions: {
itemDataSource: meta.dataSource
}
};
if (options) {
WinJS.UI.setOptions(this.clusterContent.contentOptions, options);
}
}
});
WinJS.Namespace.define("WeatherAppJS.Utilities.UI", {
setSeeMoreStatus: function setSeeMoreStatus(parentNode, seeMoreElement, isMinimized, layoutState) {
var isDisabled = isMinimized;
var viewState = Windows.UI.ViewManagement.ApplicationViewState;
parentNode = parentNode || document;
if (!layoutState) {
layoutState = Windows.UI.ViewManagement.ApplicationView.value;
}
if (!seeMoreElement) {
if (layoutState === viewState.filled || layoutState === viewState.fullScreenLandscape) {
seeMoreElement = seeMoreElement || parentNode.querySelector(".seeMore .weatherDotComSeeMoreText");
}
else if (layoutState === viewState.fullScreenPortrait) {
seeMoreElement = seeMoreElement || parentNode.querySelector(".pSeeMore .weatherDotComSeeMoreText");
}
if (!seeMoreElement) {
return;
}
}
// if hero cluster is not minimized, then do further checks to see if it's visible or not.
if (!isDisabled) {
var forecastWrapper = null;
var childCount = -1;
var animateButton = null;
// check if see more text is visible in landscape mode
if (layoutState === viewState.filled || layoutState === viewState.fullScreenLandscape) {
forecastWrapper = seeMoreElement.parentNode.parentNode.parentNode;
if (forecastWrapper) {
childCount = forecastWrapper.childNodes.length - 1;
animateButton = parentNode.querySelector(".forecastAnimateButton");
if (animateButton) {
var isButtonRight = WinJS.Utilities.hasClass(animateButton, "buttonRight");
// if isButtonRight is true, then it means we are in the first 5 days forecast
// else the user is viewing next 5days forecast
if ((childCount <= WeatherAppJS.Utilities.UI._totalDailyForecastsShown && !isButtonRight)
|| (childCount > WeatherAppJS.Utilities.UI._totalDailyForecastsShown && isButtonRight)) {
isDisabled = true;
}
}
}
}
else if (layoutState === viewState.fullScreenPortrait) {
forecastWrapper = seeMoreElement.parentNode.parentNode.parentNode.parentNode;
if (forecastWrapper) {
childCount = forecastWrapper.childNodes.length;
animateButton = parentNode.querySelector(".pDailyConditionsAnimateButton");
if (animateButton) {
var isButtonDown = WinJS.Utilities.hasClass(animateButton, "buttonDown");
// if isButtonDown is true, then it means we are in the first 5 days forecast
// else the user is viewing next 5days forecast
if ((childCount <= 5 && !isButtonDown)
|| (childCount > 5 && isButtonDown)) {
isDisabled = true;
}
}
}
}
}
seeMoreElement.disabled = isDisabled;
},
// toggle enabledness of other provider links
setSecondaryProviderLinkStatus: function setSecondaryProviderLinkStatus(parentNode, isMinimized) {
parentNode = parentNode || document;
var viewState = Windows.UI.ViewManagement.ApplicationViewState;
var layoutState = Windows.UI.ViewManagement.ApplicationView.value;
var otherProviderNames = null;
if (layoutState === viewState.filled || layoutState === viewState.fullScreenLandscape) {
otherProviderNames = parentNode.querySelectorAll(".otherProvidersWrapper .dailyConditionsProviderLink");
if (otherProviderNames[0]) {
otherProviderNames[0].disabled = isMinimized;
}
if (otherProviderNames[1]) {
otherProviderNames[1].disabled = isMinimized;
}
}
else if (layoutState === viewState.fullScreenPortrait) {
otherProviderNames = parentNode.querySelectorAll(".portraitOtherProviderNameWrapper .dailyConditionsProviderLink");
if (otherProviderNames[0]) {
otherProviderNames[0].disabled = isMinimized;
}
if (otherProviderNames[1]) {
otherProviderNames[1].disabled = isMinimized;
}
}
},
imageCardSource: WinJS.Binding.converter(function (url) {
return {
url: url,
cacheId: "PlatformImageCache"
};
}),
// For few countries we have limited functionality exposed.
showLimitedExperience: function showLimitedExperience(locID) {
if (!locID) {
return false;
}
var singleProviderOnly = PlatformJS.Services.appConfig.getBool("SingleProviderOnly");
var showLimitedExperience = false;
var locationInfo = WeatherAppJS.GeocodeCache.getLocation(locID);
if (singleProviderOnly) {
if (locationInfo.isoCode) {
if (locationInfo.isoCode === Platform.Globalization.Marketization.getMarketLocation()) {
showLimitedExperience = true;
}
}
else {
var geographicRegion = new Windows.Globalization.GeographicRegion();
if (geographicRegion.nativeName === locationInfo.country || geographicRegion.displayName === locationInfo.country) {
showLimitedExperience = true;
}
}
}
return showLimitedExperience;
},
ShowFlyoutBackground: function ShowFlyoutBackground(show) {
var flyoutBackground = document.getElementById("flyoutBackground");
if (flyoutBackground) {
flyoutBackground.style.display = show ? "block" : "none";
}
},
LoadAlertsFlyout: function LoadAlertsFlyout(locationID) {
msWriteProfilerMark("WeatherApp:AlertsFlyoutLoad:s");
WeatherAppJS.Utilities.UI.ShowFlyoutBackground(false);
var winjsAddClass = WinJS.Utilities.addClass;
/// get the location, read the alerts from it and display
var loc = WeatherAppJS.LocationsManager.getLocation(locationID);
var alertsData = loc.getAlerts();
var alertsCount = alertsData.length;
var alertsFlyout = document.getElementById('alertsFlyout');
var alertsContainer = alertsFlyout.querySelector('#alertsContainer');
alertsContainer.innerText = '';
var alertsFlyoutHeading = document.createElement('div');
winjsAddClass(alertsFlyoutHeading, "alertsFlyoutHeading");
var resourceFile = PlatformJS.Services.resourceLoader;
alertsFlyoutHeading.innerText = resourceFile.getString((alertsCount === 1) ? "Single Alert Heading" : "Multiple Alerts Heading");
alertsContainer.appendChild(alertsFlyoutHeading);
if (alertsCount === 1) {
WeatherAppJS.Utilities.Instrumentation.incrementInt32("AlertFlyoutOpenedOnSingleAlert");
}
else {
WeatherAppJS.Utilities.Instrumentation.incrementInt32("AlertFlyoutOpenedOnMultipleAlerts");
}
var eventAttr = { location: WeatherAppJS.GeocodeCache.getLocation(loc.id).getFullDisplayName(), numOfAlerts: alertsCount };
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, "Alerts", "Alerts Flyout", Platform.Telemetry.AppActionOperation.open, Platform.Telemetry.UserActionMethod.unknown, 0, JSON.stringify(eventAttr));
var alertHeadings = document.createElement('div');
winjsAddClass(alertHeadings, "alertHeadings");
// create a list of alert titles to be shown seperately
for (var i = 0; i < alertsCount; i++) {
var alertHeadingWrapper = document.createElement('div');
winjsAddClass(alertHeadingWrapper, "alertHeadingWrapper");
alertHeadingWrapper.setAttribute("tabindex", "0");
alertHeadingWrapper.setAttribute("role", "document");
alertHeadingWrapper.onkeyup = WeatherAppJS.Utilities.TabIndexManager.HandleKeyboardInvocation;
var alertHeading = document.createElement('span');
winjsAddClass(alertHeading, "alertHeading");
alertHeading.innerText = alertsData[i].title;
alertHeading.id = 'alert' + i;
alertHeadingWrapper.addEventListener('click',
function (event) {
WeatherAppJS.Utilities.UI.ViewAlert(event, WeatherAppJS.GeocodeCache.getLocation(loc.id).getFullDisplayName());
},false);
if (i === 0) {
winjsAddClass(alertHeadingWrapper, "addAlertHighlight"); // highlight the first heading when flyout opens
}
if (alertsData[i].title.toLowerCase().indexOf("warning") !== -1) {
winjsAddClass(alertHeadingWrapper, 'warningAlertFormat');
} else {
winjsAddClass(alertHeadingWrapper, 'otherAlertFormat');
}
alertHeadingWrapper.appendChild(alertHeading);
alertHeadings.appendChild(alertHeadingWrapper);
}
alertsContainer.appendChild(alertHeadings);
/// wrapper for alert descriptions
var alertDescriptions = document.createElement('div');
winjsAddClass(alertDescriptions, "alertDescriptions");
/// load the bound alert template for each alert and append it to the parent
for (var j = 0; j < alertsCount; j++) {
var alertTime = resourceFile.getString("FromToText").format(alertsData[j].createTime, alertsData[j].expirationTime);
alertsData[j].formattedAlertTime = alertTime;
if (j === 0) {
alertsData[j].displayType = 'block';
}
CommonJS.loadModule({ fragmentPath: '/html/templates.html', templateId: 'alertModule' }, alertsData[j], null, null, null).then(function (eachAlertTemplate) {
var desc = eachAlertTemplate.getElementsByClassName('alertDesc')[0].innerHTML;
desc = desc.replace(/&lt;br&gt;/g, "<br />"); // toStaticHTML converts all <br /> tags to <br>, and IE doesn't render it as expected. Working around this limitation by this regex replacement.
eachAlertTemplate.getElementsByClassName('alertDesc')[0].innerHTML = desc;
var isAlertDescScrolledDown = false;
var isAlertDescScrolledUp = false;
var lastScrollTop = 0;
//event listener for scroll.
eachAlertTemplate.getElementsByClassName('alertDesc')[0].addEventListener("scroll",
function handleScroll(event) {
eventAttr = { location: WeatherAppJS.GeocodeCache.getLocation(loc.id).getFullDisplayName(), alertTitle : eachAlertTemplate.getElementsByClassName('alertTitle')[0].innerText};
//consecutive downward or consecutive upward scrolls are logged as single scroll.
if (this.scrollTop > lastScrollTop) {
if (!isAlertDescScrolledDown) {
isAlertDescScrolledDown = true;
isAlertDescScrolledUp = false;
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, "Alerts", "Alert Description", Platform.Telemetry.UserActionOperation.scroll, Platform.Telemetry.UserActionMethod.unknown, 0, JSON.stringify(eventAttr));
}
}
else if (!isAlertDescScrolledUp) {
isAlertDescScrolledUp = true;
isAlertDescScrolledDown = false;
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, "Alerts", "Alert Description", Platform.Telemetry.UserActionOperation.scroll, Platform.Telemetry.UserActionMethod.unknown, 0, JSON.stringify(eventAttr));
}
lastScrollTop = this.scrollTop;
});
alertDescriptions.appendChild(eachAlertTemplate);
});
}
alertsContainer.appendChild(alertDescriptions);
// get the flyout
var alertsFlyoutControl = alertsFlyout.winControl;
alertsFlyoutControl.addEventListener("afterhide", function () {
WeatherAppJS.Utilities.UI.ShowFlyoutBackground(false);
var alertIndicator = document.querySelector(".alertIndicator");
if (alertIndicator) {
alertIndicator.focus();
}
});
/// get the flyout control
var alertsAnchor = document.querySelector('#flyoutAnchor');
if (alertsFlyoutControl && alertsFlyoutControl.hidden) {
alertsFlyoutControl.show(alertsAnchor);
WinJS.UI.Animation.showPopup(alertsFlyout);
}
msWriteProfilerMark("WeatherApp:AlertsFlyoutLoad:e");
},
ViewAlert: function ViewAlert(event, locn) {
// get the id of the alert title clicked on and use that to display the required alert description
var alertTitle = event.srcElement.querySelector(".alertHeading");
if (!alertTitle) {
alertTitle = event.srcElement;
}
WeatherAppJS.Utilities.Instrumentation.incrementInt32("AlertTabsSwitch");
var eventAttr = { location: locn, alertTitle: alertTitle.textContent };
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, "Alerts", "Alert Tab", Platform.Telemetry.UserActionOperation.click, Platform.Telemetry.UserActionMethod.unknown, 0, JSON.stringify(eventAttr));
var alertNumber = parseInt(alertTitle.id.replace('alert', ''));
// loop through all alerts, show the one required, hide the rest
var alertsContainer = document.getElementById('alertsContainer');
var allAlerts = alertsContainer.getElementsByClassName('eachAlert');
var allAlertHeadings = alertsContainer.getElementsByClassName('alertHeadingWrapper');
var winjsUtils = WinJS.Utilities;
for (var i = 0; i < allAlerts.length; i++) {
// remove all highlight classes and add fresh ones everytime
winjsUtils.removeClass(allAlertHeadings[i], "addAlertHighlight");
if (i === alertNumber) {
winjsUtils.addClass(allAlertHeadings[i], "addAlertHighlight");
allAlerts[i].style.display = "block";
}
else {
allAlerts[i].style.display = "none";
}
}
},
/// navigate to city details page when a favorite is clicked
handleFavoritesClick: function handleFavoritesClick(locationObj) {
PlatformJS.Navigation.navigateToChannel('Home', { locID: locationObj, geocodeLocation: WeatherAppJS.GeocodeCache.getLocation(locationObj), formCode: WeatherAppJS.Instrumentation.FormCodes.MyPlacesTile });
},
/// used in world weather panorama to set location weather data when it changes
bindWorldWeatherProperties: function bindWorldWeatherProperties(source, sourceProperties, dest, destProperties) {
if (sourceProperties[0] === 'skycode') {
if (source[sourceProperties[0]]) {
dest[destProperties[0]][destProperties[1]] = 'block';
}
else {
dest[destProperties[0]][destProperties[1]] = 'none';
}
}
},
disableNavigationBar: function disableNavigationBar() {
var navBar = document.querySelector("#platformNavigationBar").winControl;
if (navBar) {
navBar.disabled = true;
}
},
enableNavigationBar: function enableNavigationBar() {
var navBar = document.querySelector("#platformNavigationBar").winControl;
if (navBar) {
navBar.disabled = false;
}
},
disableBottomEdgy: function disableBottomEdgy(edgyId) {
var bEdgy = PlatformJS.Utilities.getControl(edgyId);
if (bEdgy) {
bEdgy.disabled = true;
}
},
enableBottomEdgy: function enableBottomEdgy(edgyId) {
var bEdgy = PlatformJS.Utilities.getControl(edgyId);
if (bEdgy) {
bEdgy.disabled = false;
}
},
// identify which forecast day is the hover on
// if the hover is not on 
getHoveredForecastIndex: function getHoveredForecastIndex(e) {
var ui = WeatherAppJS.Utilities.UI;
var layoutState = Windows.UI.ViewManagement.ApplicationView.value;
var hoveredForecastIndex = null;
var direction = WeatherAppJS.Utilities.Common.getDirection();
// for portrait
if (layoutState === Windows.UI.ViewManagement.ApplicationViewState.fullScreenPortrait) {
var hoverPointY = 0;
hoverPointY = e ? e.clientY : 0;
// for supporting all resolutions, where height changes, we need to get the hover point from bottom since the box is fixed to the page bottom
hoverPointY = ui.getWindowHeight() - hoverPointY;
var hoverPointX = e.clientX;
if (direction === "rtl") {
hoverPointX = window.innerWidth - hoverPointX;
}
if ((hoverPointY > 640 || hoverPointY < 76) || (hoverPointX > 375 || hoverPointX < 99)) { // bounding box within which the forecast box is present
return null;
}
// from forecasts bounding box, get the specific forecast hovered upon
hoveredForecastIndex = Math.floor((640 - hoverPointY) / 113); // 640 - offset of forecast box from bottom, 113 - width of each forecast box
if (ui._isDailyForecastScrolled) { // on animation, increment the index counter by 5 since we show 5 elements by default
var pdailyModuleFirstProviders = document.querySelectorAll(".pDailyModuleFirstProvider");
var pTotalElements = 0;
var pAlreadyShownElements = 5;
if (pdailyModuleFirstProviders) {
pTotalElements = pdailyModuleFirstProviders.length;
}
var pOffset = pTotalElements - pAlreadyShownElements;
hoveredForecastIndex = hoveredForecastIndex + pOffset;
}
return hoveredForecastIndex;
} else {
var hoverPoint = 0;
hoverPoint = e ? e.clientX : 0;
if (direction === "rtl") {
hoverPoint = window.innerWidth - hoverPoint;
}
// pano initial scroll position is 300
var initialPanoScrollPosition = 300;
var panoScrollPosition = ui.getPanoScrollPosition();
// pano scroll position changes after semantic zoom in some resolutions. For our purposes, we reset it to 300 and then calculate hover point
if (panoScrollPosition < initialPanoScrollPosition) {
panoScrollPosition = initialPanoScrollPosition;
}
// bounding box within which the forecast box is present
hoverPoint = hoverPoint + (panoScrollPosition - initialPanoScrollPosition);
if (hoverPoint < 370) {
return null;
}
// from forecasts bounding box, get the specific forecast hovered upon
hoveredForecastIndex = Math.floor((hoverPoint - 370) / 147); // 370 - std. offset from left, 147 - width of each forecast box
if (ui._isDailyForecastScrolled) { // on scroll animation, increment the index counter by _totalDailyForecastsShown
var dailyModuleFirstProviders = document.querySelectorAll(".dailyModuleFirstProvider");
var totalElements = 0;
var alreadyShownElements = WeatherAppJS.Utilities.UI._totalDailyForecastsShown;
if (dailyModuleFirstProviders) {
totalElements = dailyModuleFirstProviders.length;
}
var offset = totalElements - alreadyShownElements;
hoveredForecastIndex = hoveredForecastIndex + offset;
}
return hoveredForecastIndex;
}
},
// get pano scroll position from the panorama element
getPanoScrollPosition: function getPanoScrollPosition() {
var scrollPosition = 0;
var panoElem = document.getElementById('mainPanorama');
if (panoElem) {
var panoControl = panoElem.winControl;
if (panoControl && panoControl._panel && panoControl._panel._scrollPosition) {
scrollPosition = panoControl._panel._scrollPosition;
}
}
return scrollPosition;
},
// given an index and a bool to specify whether to create pressedIn effect or pressedOut effect, create that effect on forecast elements under that index
createBounceEffectOnForecastElement: function createBounceEffectOnForecastElement(forecastIndex, isBounceIn) {
var ui = WeatherAppJS.Utilities.UI;
var layoutState = Windows.UI.ViewManagement.ApplicationView.value;
if (layoutState === Windows.UI.ViewManagement.ApplicationViewState.fullScreenPortrait) {
var pDailyModuleFirstProvider = document.querySelectorAll(".pDailyModuleFirstProvider");
if (pDailyModuleFirstProvider) {
var pCurrentDailyModule = pDailyModuleFirstProvider[forecastIndex];
ui.bounceInOutElement(pCurrentDailyModule, isBounceIn);
}
} else {
var dailyModuleFirstProviders = document.querySelectorAll(".dailyModuleFirstProvider");
if (dailyModuleFirstProviders) {
var currentDailyModule = dailyModuleFirstProviders[forecastIndex];
ui.bounceInOutElement(currentDailyModule, isBounceIn);
}
var dailyPrecipWrapper = document.querySelectorAll(".precipWrapper");
if (dailyPrecipWrapper) {
var currentPrecip = dailyPrecipWrapper[forecastIndex];
ui.bounceInOutElement(currentPrecip, isBounceIn);
}
}
},
// Attach event handlers to detect hover on forecast elements
// Platform pano expects scrollPosition and count of clusters to scroll to a cluster
// on click of a day, pass scrollPosition and total cluster count to daily forecast pano to set right scroll position
attachClickHandlerForForecast: function attachClickHandlerForForecast(element, locationID) {
var ui = WeatherAppJS.Utilities.UI;
if (element) {
// click down or touch press
element.onmspointerdown = function (e) {
// do not respond to right or middle mouse clicks
if (e.button && (e.button === 1 || e.button === 2)) {
return;
}
var forecastIndex = ui.getHoveredForecastIndex(e);
if (forecastIndex !== undefined && forecastIndex !== null) {
ui.createBounceEffectOnForecastElement(forecastIndex, true);
}
};
(function (locID, numClusters) {
// click or touch remove
element.onmspointerup = function (e) {
// do not respond to right or middle mouse clicks
if (e.button && (e.button === 1 || e.button === 2)) {
return;
}
var forecastIndex = ui.getHoveredForecastIndex(e);
if (forecastIndex === undefined || forecastIndex === null || forecastIndex >= ui._totalDailyForecasts) {
return;
}
ui.createBounceEffectOnForecastElement(forecastIndex, false);
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NumDailyForecastClick");
WeatherAppJS.Utilities.Instrumentation.incrementInt32Array("NumDailyDrilldownDayIndexTapped", forecastIndex, 10); // pass max array size as fixed always since it's easier for instrumentation and report generation as per nowyang
//Log the index of the day that is tapped by the user.
var eventAttr = { location: WeatherAppJS.GeocodeCache.getLocation(locID).getFullDisplayName(), dayIndex: forecastIndex, dayOfWeek: WeatherAppJS.Utilities.Instrumentation.dayOfWeek[forecastIndex] };
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, "Hero", "Day Drilldown Day", Platform.Telemetry.UserActionOperation.click, Platform.Telemetry.UserActionMethod.unknown, 0, JSON.stringify(eventAttr));
var clusterWidth = ui.getDailyDrilldownClusterWidth();
WinJS.Navigation.navigate({ fragment: "/panoramas/dailyforecast/dailyForecast.html", page: "WeatherAppJS.DailyForecast" }, { locID: locID, scrollPosition: clusterWidth * forecastIndex, count: numClusters });
};
})(locationID, ui._totalDailyForecasts);
// click or touch move out of the element 
element.onmspointerout = function (e) {
// do not respond to right or middle mouse clicks
if (e.button && (e.button === 1 || e.button === 2)) {
return;
}
// prevent flickering effect on hovering. It calls multiple mouseOuts if we move the mouse over an already highlighted area since the overlay gains prominence
var relatedTarget = e ? e.relatedTarget : null;
if (relatedTarget && (relatedTarget.className === "hoverEffect" || relatedTarget.className === "phoverEffect")) {
return;
}
var forecastIndex = ui.getHoveredForecastIndex(e);
if (forecastIndex === undefined || forecastIndex === null) {
ui.toggleForecastHoverOverlay(forecastIndex, false);
return;
}
ui.createBounceEffectOnForecastElement(forecastIndex, false);
ui.toggleForecastHoverOverlay(forecastIndex, false);
};
}
},
attachFocusHandlerForDailyForecast: function attachFocusHandlerForDailyForecast(dailyForecastNode, index, locationID) {
var ui = WeatherAppJS.Utilities.UI;
if (dailyForecastNode && locationID) {
if (!index) {
index = 0;
}
(function (nodeIndex) {
dailyForecastNode.onfocus = function () {
ui.toggleForecastHoverOverlay(nodeIndex, true);
};
})(index);
(function (nodeIndex, locID, totalClusters) {
// on keyboard enter press. onkeypress isn't working for 'Enter' key sometimes though works for other keys
dailyForecastNode.onkeydown = function (e) {
// respond only to keyboard enter or spacebar
if (e.keyCode && (e.keyCode === WinJS.Utilities.Key.enter || e.keyCode === WinJS.Utilities.Key.space)) {
ui.toggleForecastHoverOverlay(nodeIndex, true);
ui.createBounceEffectOnForecastElement(nodeIndex, true);
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NumDailyForecastClick");
WeatherAppJS.Utilities.Instrumentation.incrementInt32Array("NumDailyDrilldownDayIndexTapped", nodeIndex, 10);  // pass max array size as fixed always since it's easier for instrumentation and report generation as per nowyang
var clusterWidth = ui.getDailyDrilldownClusterWidth();
WinJS.Navigation.navigate({ fragment: "/panoramas/dailyforecast/dailyForecast.html", page: "WeatherAppJS.DailyForecast" }, { locID: locID, scrollPosition: clusterWidth * nodeIndex, count: totalClusters });
}
};
})(index, locationID, ui._totalDailyForecasts);
dailyForecastNode.onblur = function () {
ui.toggleForecastHoverOverlay(index, false);
};
}
},
_isDailyForecastScrolled: false, // state stored to figure out if we've scrolled forecast to show next set of days
_totalDailyForecastsShown: 0, // state stored to figure out how many forecast days are being shown in the available width
_totalDailyForecasts: 0, // state stored to figure out how many forecast days are available
_historicDataDailyDrilldownHeight: 0, // state stored per app session, so we don't have to calculate each time. This is used to determine height of hourly data.
// given a forecast element index, show or hide the hover effect
toggleForecastHoverOverlay: function toggleForecastHoverOverlay(index, show) {
var ui = WeatherAppJS.Utilities.UI;
if (index === undefined) {
return;
}
var layoutState = Windows.UI.ViewManagement.ApplicationView.value;
var overlays = null;
if (layoutState === Windows.UI.ViewManagement.ApplicationViewState.fullScreenPortrait) {
overlays = document.querySelectorAll('.pDailyModuleFirstProvider');
} else {
var overlayContainer = document.querySelector('.overlayContainer');
if (overlayContainer) {
overlays = overlayContainer.querySelectorAll('.overlay');
}
}
if (overlays && overlays[index]) {
if (show) {
WinJS.Utilities.addClass(overlays[index], 'hoverEffect');
} else {
WinJS.Utilities.removeClass(overlays[index], 'hoverEffect');
}
}
},
hideAllForecastOverlays: function hideAllForecastOverlays() {
var layoutState = Windows.UI.ViewManagement.ApplicationView.value;
var overlays = null;
if (layoutState === Windows.UI.ViewManagement.ApplicationViewState.fullScreenPortrait) {
overlays = document.querySelectorAll('.pDailyModuleFirstProvider');
} else {
overlays = document.querySelectorAll('.overlay');
}
for (var i = 0; i < overlays.length; i++) {
if (overlays[i]) {
WinJS.Utilities.removeClass(overlays[i], 'hoverEffect');
}
}
},
// enable bounce-in effect on element
bounceInOutElement: function bounceInOutElement(element, isBounceIn) {
if (element) {
if (isBounceIn) {
WinJS.UI.Animation.pointerDown(element);
} else {
WinJS.UI.Animation.pointerUp(element);
}
}
},
_getOffset: function _getOffset(forecastTables) {
var offset = '';
var maxChildren = 0;
for (var i = 0, j = forecastTables.length; i < j; i++) {
// Subtract one from the children since the last child is a div with clear:floats style and shouldn't be counted as a forecast container
var numCurrentChildren = forecastTables[i].childNodes.length - 1;
if (numCurrentChildren > maxChildren) {
maxChildren = numCurrentChildren;
}
}
var singleElemOffset = 152; // 147 + 5 (width + marginRight, @eachDailyForecastWidth + @spacingBetweenForecasts) of each forecast. More performant to hard-code than calculate each time unnecessarily
var currentlyShownChildrenNum = WeatherAppJS.Utilities.UI._totalDailyForecastsShown; // 760 (@totalDailyForecastWidth) is the constant width for the daily forecast window
// calculate offset subtracting the currently shown children that are already shown
offset = ((maxChildren - currentlyShownChildrenNum) * singleElemOffset) + 'px';
return { offset: offset, count: currentlyShownChildrenNum };
},
// get window height
_windowHeight: 0, 
getWindowHeight: function getWindowHeight() {
var ui = WeatherAppJS.Utilities.UI;
if (!ui._windowHeight) {
ui._windowHeight = window.innerHeight;
}
return ui._windowHeight;
},
// set window height when resolution changes
setWindowHeight: function setWindowHeight() {
WeatherAppJS.Utilities.UI._windowHeight = window.innerHeight;
},
getDailyDrilldownClusterWidth: function getDailyDrilldownClusterWidth() {
var layoutState = Windows.UI.ViewManagement.ApplicationView.value;
var clusterWidth = 0;
if (layoutState === Windows.UI.ViewManagement.ApplicationViewState.fullScreenPortrait) {
clusterWidth = 680; // 680 is the fixed width of each cluster in portrait
} else {
clusterWidth = 1220; // 1220 is the fixed width of each cluster in landscape
}
return clusterWidth;
},
ChangeDisplayUnitonClick: function ChangeDisplayUnitonClick(e) {
/*Disable the c/f settings*/
var useTianQi = Platform.Utilities.Globalization.isFeatureEnabled("UseTianQi");
if (useTianQi) {
return;
}
// checkKeyBoard pressed.
if (e.keyCode && (e.keyCode !== WinJS.Utilities.Key.enter && e.keyCode !== WinJS.Utilities.Key.space)) {
return;
}
var defUnit = WeatherAppJS.SettingsManager.getDisplayUnit();
if (defUnit === "F") {
CommonJS.dismissAllEdgies();
WeatherAppJS.SettingsManager.setDisplayUnitAsync('C', "Hero").then();
} else {
CommonJS.dismissAllEdgies();
WeatherAppJS.SettingsManager.setDisplayUnitAsync('F', "Hero").then();
}
},
AnimateForecastSlide: function AnimateForecastSlide(locationId, /*@optional*/direction, /*@optional*/animationRequired, /*@optional*/template) {
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NumMoreDaysClick");
animationRequired = (animationRequired !== false) ? true : false;
if (animationRequired) {
var eventAttr = { location: WeatherAppJS.GeocodeCache.getLocation(locationId).getFullDisplayName() };
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, "Hero", "More Days Forecast", Platform.Telemetry.UserActionOperation.click, Platform.Telemetry.UserActionMethod.unknown, 0, JSON.stringify(eventAttr));
}
if (!template) {
template = document;
}
var forecastTables = template.querySelectorAll('.animateSlide');
var overlayViewport = template.querySelector(".overlayViewport");
var winjsUtilities = WinJS.Utilities;
var uiUtilities = WeatherAppJS.Utilities.UI;
var isDailyDrilldownEnabled = uiUtilities.isDailyDrilldownEnabled(locationId);
if (forecastTables) {
var nextFocusItem = 0;
var windowDirection = WeatherAppJS.Utilities.Common.getDirection();
// get the max number of pixels for which we need to scroll
var offsetList = uiUtilities._getOffset(forecastTables);
var maxOffset = offsetList.offset;
// used for storing the actual number of pixels to scroll
var offset = "";
var buttonState1 = "block";
var buttonState2 = "none";
var animateButton = template.querySelector(".forecastSlider .forecastAnimateButton");
var leftButtonClass = "buttonLeft";
var rightButtonClass = "buttonRight";
// we need to figure out the animation state - whether we have already animated to the next 5 days or not
// instead of maintaining a state variable, the easiest way to get that is to see which button is being displayed
// the 'direction' variable is the state variable
if (!direction) {
direction = winjsUtilities.hasClass(animateButton, leftButtonClass) ? "right" : "left";
}
var resultClass = (direction && direction === "right") ? rightButtonClass : leftButtonClass;
if (direction === "left") {
if (windowDirection === "rtl") {
offset = maxOffset;
} else {
offset = '-' + maxOffset;
}
uiUtilities._isDailyForecastScrolled = true;
var wrapper = template.querySelectorAll(".dailyModuleFirstProvider");
var totalCount = wrapper.length;
nextFocusItem = totalCount - uiUtilities._totalDailyForecastsShown;
} else if (direction === "right") {
offset = "0px";
uiUtilities._isDailyForecastScrolled = false;
}
if (animationRequired) {
// slide all forecasts based on calculated offset
WinJS.UI.executeTransition(
forecastTables,
{
property: '-ms-transform',
delay: 0,
duration: 1000,
timing: "ease-out",
to: "translateX(" + offset + ")"
}
).then(function () {
if (isDailyDrilldownEnabled) {
uiUtilities.hideAllForecastOverlays();
var dailyModuleFirstProvider = template.querySelectorAll(".dailyModuleFirstProvider");
if (dailyModuleFirstProvider && dailyModuleFirstProvider[nextFocusItem]) {
dailyModuleFirstProvider[nextFocusItem].focus();
}
}
});
} else {
for (var sliderIndex = 0; sliderIndex < forecastTables.length; sliderIndex++) {
forecastTables[sliderIndex].style['-ms-transform'] = "translateX(" + offset + ")";
}
if (isDailyDrilldownEnabled) {
//no need to set focuses
uiUtilities.hideAllForecastOverlays();
}
}
// toggle buttons
if (animateButton) {
winjsUtilities.removeClass(animateButton, leftButtonClass);
winjsUtilities.removeClass(animateButton, rightButtonClass);
winjsUtilities.addClass(animateButton, resultClass);
}
// Truncate or show full 'see more' attribution text for weather.com only
var seeMoreText = template.querySelector('.weatherDotComSeeMoreText');
var dailyModule = null;
if (seeMoreText) { // if this element is not there, weather.com isn't a provider
dailyModule = seeMoreText.parentNode.parentNode.parentNode;
var dailyConditionsDefaultShown = uiUtilities.getDefaultDailyConditionsTotal();
var showFullAttribution = false;
if (dailyConditionsDefaultShown === 5) {
if ((dailyModule.childNodes.length - 1) > dailyConditionsDefaultShown) {
showFullAttribution = true;
}
} else {
if ((dailyModule.childNodes.length - 1) >= dailyConditionsDefaultShown) {
showFullAttribution = true;
} else {
showFullAttribution = false;
}
}
if (direction === "left" && showFullAttribution) { // 6 daily conditions by default are shown
seeMoreText.innerText = WeatherAppJS.Utilities.Common.getWeatherDotComSeeMoreText();
}
else if (direction === "right" && dailyModule.childNodes.length > dailyConditionsDefaultShown) {
seeMoreText.innerText = PlatformJS.Services.resourceLoader.getString("SeeMoreTruncatedText");
}
uiUtilities.setSeeMoreStatus(template, seeMoreText, WeatherAppJS.SettingsManager.isCCMinimized());
}
uiUtilities.setCCAriaLabels(template);
if (isDailyDrilldownEnabled) {
WeatherAppJS.Utilities.TabIndexManager.setTabIndexOnDailyConditions(template);
}
}
},
// vertical animation to reveal/hide additional data in portrait: more daily forecasts
AnimatePortraitForecastSlide: function AnimatePortraitForecastSlide(locationId) {
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NumMoreDaysClick");
var eventAttr = { location: WeatherAppJS.GeocodeCache.getLocation(locationId).getFullDisplayName() };
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, "Hero", "More Days Forecast", Platform.Telemetry.UserActionOperation.click, Platform.Telemetry.UserActionMethod.unknown, 0, JSON.stringify(eventAttr));
var winjsUtilities = WinJS.Utilities;
var uiUtilities = WeatherAppJS.Utilities.UI;
var isDailyDrilldownEnabled = uiUtilities.isDailyDrilldownEnabled(locationId);
var forecastTables = document.querySelector('.portraitMultiProviderWrapper');
var animateButton = document.querySelector(".pDailyConditionsAnimateButton");
var buttonUpClass = "buttonUp";
var buttonDownClass = "buttonDown";
var nextFocusItem = 0;
var resultClass = null;
var offset = '0px';
if (winjsUtilities.hasClass(animateButton, buttonDownClass)) {
var maxChildren = forecastTables.querySelector('.primaryProvider').childNodes.length;
offset = '-' + ((maxChildren - 5) * 113) + 'px'; // calculate translation offset by subtracting the currently shown 5 children. 113 px is the standard height of each row.
resultClass = buttonUpClass;
uiUtilities._isDailyForecastScrolled = true;
nextFocusItem = maxChildren - 5;
} else {
resultClass = buttonDownClass;
uiUtilities._isDailyForecastScrolled = false;
}
if (forecastTables) {
WinJS.UI.executeTransition(
forecastTables,
{
property: '-ms-transform',
delay: 0,
duration: 1300,
timing: "ease-out",
to: "translateY(" + offset + ")"
}
).then(function () {
if (isDailyDrilldownEnabled && forecastTables) {
uiUtilities.hideAllForecastOverlays();
var forecastNodes = forecastTables.querySelectorAll('.pDailyModuleFirstProvider');
if (forecastNodes && forecastNodes[nextFocusItem]) {
forecastNodes[nextFocusItem].focus();
}
}
});
}
if (animateButton) {
winjsUtilities.removeClass(animateButton, buttonUpClass);
winjsUtilities.removeClass(animateButton, buttonDownClass);
winjsUtilities.addClass(animateButton, resultClass);
}
uiUtilities.setSeeMoreStatus(document, null, WeatherAppJS.SettingsManager.isCCMinimized());
uiUtilities.setCCAriaLabels(document);
var direction = WeatherAppJS.Utilities.UI._isDailyForecastScrolled;
if (isDailyDrilldownEnabled) {
WeatherAppJS.Utilities.TabIndexManager.setTabIndexOnDailyConditions(document);
}
},
// calculate the offset required for animation in portrait mode
_getCCOffsetForPortrait: function _getCCOffsetForPortrait(template, /*optional*/forceMinimized) {
if (!template) {
template = document;
}
var winjsUtilities = WinJS.Utilities;
var windowDirection = WeatherAppJS.Utilities.Common.getDirection();
var buttonState1 = "block";
var buttonState2 = "none";
var animateButton = template.querySelector(".portraitSliders .forecastAnimateButton");
var leftButtonClass = "buttonLeft";
var rightButtonClass = "buttonRight";
var isMinimized = forceMinimized ? true : WeatherAppJS.SettingsManager.isCCMinimized();
var direction = '';
var resultClass = null;
resultClass = (isMinimized) ? rightButtonClass : leftButtonClass;
direction = (isMinimized) ? "right" : "left";
if (animateButton) {
winjsUtilities.removeClass(animateButton, leftButtonClass);
winjsUtilities.removeClass(animateButton, rightButtonClass);
winjsUtilities.addClass(animateButton, resultClass);
}
var maxProviderOffset = '225px'; // 225px is the standard translation-X distance to be moved to minimize the additional providers
var maxCCKeyValueOffset = '250px'; // 250px is the standard translation-X distance to be moved to minimize the current conditions key/values table
var offset = {};
if (direction === "left") {
offset["maxProviderOffset"] = '0px';
offset["maxCCKeyValueOffset"] = '0px';
} else if (direction === "right") {
if (windowDirection === "rtl") {
offset["maxProviderOffset"] = maxProviderOffset;
offset["maxCCKeyValueOffset"] = maxCCKeyValueOffset;
} else {
offset["maxProviderOffset"] = '-' + maxProviderOffset;
offset["maxCCKeyValueOffset"] = '-' + maxCCKeyValueOffset;
}
}
return offset;
},
// horizontal animation to reveal/hide additional data in portrait: multiple providers daily forecast and additional current conditions
AnimatePortraitCurrentConditionsSlide: function AnimatePortraitCurrentConditionsSlide() {
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NumMultiProviderClick");
Platform.Telemetry.FlightRecorder.logUserAction(Platform.Telemetry.LogLevel.normal, "Hero", "Multi Provider Forecast", Platform.Telemetry.UserActionOperation.click, Platform.Telemetry.UserActionMethod.unknown, 0);
var uiUtilities = WeatherAppJS.Utilities.UI;
var ccElem = document.querySelector(".bigTempWrapper");
var isMinimized = WeatherAppJS.SettingsManager.isCCMinimized();
if (isMinimized) {
WeatherAppJS.SettingsManager.setIsCCMinimizedAsync(false).then();
ccElem.setAttribute("aria-describedby", "ccExpanded");
} else {
WeatherAppJS.SettingsManager.setIsCCMinimizedAsync(true).then();
ccElem.removeAttribute("aria-describedby");
}
var offset = uiUtilities._getCCOffsetForPortrait();
var tables = document.querySelectorAll('.portraitCCSlide');
if (tables) {
WinJS.UI.executeTransition(
tables,
{
property: '-ms-transform',
delay: 0,
duration: 500,
timing: "ease-out",
to: "translateX(" + offset['maxProviderOffset'] + ")" // standard width for slide
}
).then();
}
// a different animation translationx for current conditions key/value pair table
var ccKeyValueTable = document.querySelectorAll('.conditionsCollection');
if (ccKeyValueTable) {
WinJS.UI.executeTransition(
ccKeyValueTable,
{
property: '-ms-transform',
delay: 0,
duration: 500,
timing: "ease-out",
to: "translateX(" + offset['maxCCKeyValueOffset'] + ")" // standard width for slide
}
).then();
}
// toggle enabledness of other provider links
uiUtilities.setSecondaryProviderLinkStatus(document, !isMinimized);
// enable/disable see more link
uiUtilities.setSeeMoreStatus(document, null, !isMinimized);
uiUtilities.setCCAriaLabels(document);
},
AnimateCurrentConditionsSlide: function AnimateCurrentConditionsSlide(event) {
/// if the click is on a provider url, don't animate
if (event) {
if (event.target && event.target.nodeName === "A") {
return;
}
}
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NumMultiProviderClick");
Platform.Telemetry.FlightRecorder.logUserAction(Platform.Telemetry.LogLevel.normal, "Hero", "Multi Provider Forecast", Platform.Telemetry.UserActionOperation.click, Platform.Telemetry.UserActionMethod.unknown, 0);
var uiUtilities = WeatherAppJS.Utilities.UI;
var sliders = document.querySelectorAll('.ccSlider');
if (sliders) {
var offset = "";
var ccElem = document.querySelector(".bigTempWrapper");
var buttonState1 = "block";
var buttonState2 = "none";
var isMinimized = WeatherAppJS.SettingsManager.isCCMinimized();
if (!isMinimized) {
offset = "135px";
WeatherAppJS.SettingsManager.setIsCCMinimizedAsync(true).then();
if (ccElem) {
ccElem.removeAttribute("aria-describedby");
}
} else {
offset = "0px";
WeatherAppJS.SettingsManager.setIsCCMinimizedAsync(false).then();
ccElem.setAttribute("aria-describedby", "ccExpanded");
}
// toggle buttons
uiUtilities.toggleHeroButtons(!isMinimized);
// toggle enabledness of other provider links
uiUtilities.setSecondaryProviderLinkStatus(document, !isMinimized);
// enable/disable see more link
uiUtilities.setSeeMoreStatus(document, null, !isMinimized);
// slide all forecasts based on calculated offset
WinJS.UI.executeTransition(
sliders,
{
property: '-ms-transform',
delay: 0,
duration: 500,
timing: "ease-out",
to: "translateY(" + offset + ")"
}
).then();
}
uiUtilities.setCCAriaLabels(document);
},
toggleStickyEdgy: function toggleStickyEdgy(edgy, show) {
if (edgy) {
if (show) {
edgy.sticky = true;
if (edgy.hidden) { edgy.show(); }
} else {
edgy.sticky = false;
if (!edgy.hidden) { edgy.hide(); }
}
}
},
toggleShowButton: function toggleShowButton(b, show) {
if (b) {
if (show) {
b.style.display = "";
} else {
b.style.display = "none";
}
}
},
// various elements in the hero cluster are re-used in different modes like portrait, landscape etc. 
// this function manages the animation states of those elements based on view mode, minimized / maximized, ltr / rtl modes etc.
// the template is passed in for cases where the elements wouldn't be yet in the document. This is a condition that can happen when the cluster is getting built
setHeroClusterState: function setHeroClusterState(layoutState, /*@optional*/template, /*@optional*/forceMinimized) {
if (!template) {
template = document;
}
var currentConditionsCluster = template.querySelector('.currentConditions');
if (currentConditionsCluster) {
var uiUtilities = WeatherAppJS.Utilities.UI;
var conditionsCollection = template.querySelector('.conditionsCollection');
var isMinimized = forceMinimized ? true : WeatherAppJS.SettingsManager.isCCMinimized();
if (layoutState === Windows.UI.ViewManagement.ApplicationViewState.fullScreenPortrait) {
var offset = uiUtilities._getCCOffsetForPortrait(template, forceMinimized);
var pSliders = template.querySelectorAll('.portraitCCSlide');
for (var i = 0; i < pSliders.length; i++) {
pSliders[i].style['-ms-transform'] = "translateX( " + offset['maxProviderOffset'] + ")";
}
if (isMinimized) {
var pCCSliders = template.querySelectorAll('.ccSlider');
for (var pSliderIndex = 0; pSliderIndex < pCCSliders.length; pSliderIndex++) {
pCCSliders[pSliderIndex].style['-ms-transform'] = "translateY(0px)";
}
if (conditionsCollection) {
conditionsCollection.style['-ms-transform'] = "translateX(" + offset['maxCCKeyValueOffset'] + ")";
}
}
uiUtilities.toggleHeroButtons(isMinimized, template);
} else if (layoutState === Windows.UI.ViewManagement.ApplicationViewState.filled || layoutState === Windows.UI.ViewManagement.ApplicationViewState.fullScreenLandscape) {
if (isMinimized) {
var ccSliders = template.querySelectorAll('.ccSlider');
for (var sliderIndex = 0; sliderIndex < ccSliders.length; sliderIndex++) {
ccSliders[sliderIndex].style['-ms-transform'] = "translateY(135px)";
}
}
if (conditionsCollection) {
conditionsCollection.style['-ms-transform'] = "translateX(0px)";
}
uiUtilities.toggleHeroButtons(isMinimized, template);
}
// toggle enabledness of other provider links
uiUtilities.setSecondaryProviderLinkStatus(template, isMinimized);
uiUtilities.setCCAriaLabels(template);
}
},
adjustHeroClusterForResolution: function adjustHeroClusterForResolution(template, locID) {
if (!template) {
template = document;
}
var currentConditionsCluster = template.querySelector('.currentConditions');
if (currentConditionsCluster) {
var wrapper = template.querySelector('.dailyModuleWrapper');
var precipWrapper = template.querySelector('.dailyPrecipWrapper');
var otherProvidersWrapper = template.querySelector('.otherProvidersWrapper');
var overlayViewport = template.querySelector('.overlayViewport');
var animateButton = template.querySelector('.dailyConditionsAnimateButton');
var screenWidth = WinJS.Utilities.getTotalWidth(document.getElementById("platformPageArea"));
var totalDaysVisible = 0; // this is the total number of forecasts we can show in the available width
// for lesser resolutions, go with fixed widths defined in css classes and show 5 days data
// for higher resolutions, determine how much width is available and layout days and position buttons accordingly
if (screenWidth <= 1366) {
totalDaysVisible = 5;
if (animateButton) {
WinJS.Utilities.removeClass(animateButton, 'dailyConditionsAnimateButtonAdjusted');
WinJS.Utilities.removeClass(animateButton, 'dailyConditionsAnimateButtonAdjustedLesser');
}
} else {
var availableWidth = (screenWidth * 0.87) - (60 + 120 + 252); // 87% of window width minus (left gutter + current conditions + animation buttons widths)
totalDaysVisible = Math.floor(availableWidth / 152); // each day forecast width is 152. Round this off to get exact number of days we can show without any chopping.
if (animateButton) {
if (totalDaysVisible > 7) {
WinJS.Utilities.addClass(animateButton, 'dailyConditionsAnimateButtonAdjusted');
WinJS.Utilities.removeClass(animateButton, 'dailyConditionsAnimateButtonAdjustedLesser');
} else {
WinJS.Utilities.addClass(animateButton, 'dailyConditionsAnimateButtonAdjustedLesser');
}
}
}
// reset animation if it has already happened
var translateDirection = "right";
var ui = WeatherAppJS.Utilities.UI;
ui.AnimateForecastSlide(locID, translateDirection, false, template);
// hide the animation button if we are showing all forecast days
var sliderButton = template.querySelector('.forecastSlider');
if (totalDaysVisible >= ui._totalDailyForecasts) {
totalDaysVisible = ui._totalDailyForecasts;
if (sliderButton) {
sliderButton.style.display = "none";
}
} else {
if (sliderButton) {
sliderButton.style.display = "block";
}
}
// store available days so we don't have to recalculate all the time
ui._totalDailyForecastsShown = totalDaysVisible;
var finalWidth = 152 * totalDaysVisible; // each day forecast width is 152. 
wrapper.style.width = finalWidth + 'px';
precipWrapper.style.width = finalWidth + 'px';
otherProvidersWrapper.style.width = finalWidth + 'px';
overlayViewport.style.width = finalWidth + 'px';
}
},
setCCAriaLabels: function setCCAriaLabels(template) {
var viewState = Windows.UI.ViewManagement.ApplicationViewState;
var layoutState = Windows.UI.ViewManagement.ApplicationView.value;
var resourceLoader = CommonJS.resourceLoader;
var windowDirection = WeatherAppJS.Utilities.Common.getDirection();
var isMinimized = WeatherAppJS.SettingsManager.isCCMinimized();
var dailyConditions = null;
var utilities = WinJS.Utilities;
if (!template) {
template = document;
}
if (layoutState === viewState.fullScreenPortrait) {
// more providers button
var pProviderButton = template.querySelector(".portraitSliders .forecastAnimateButton");
if (pProviderButton) {
var pProviderAriaText = isMinimized ? "moreDailyConditionsAriaText" : "lessDailyConditionsAriaText";
pProviderButton.setAttribute("aria-label", resourceLoader.getString(pProviderAriaText));
}
// forecast button
var pForecastButton = template.querySelector(".portraitSliders .pDailyConditionsAnimateButton");
if (pForecastButton) {
var pForecastAriaText = utilities.hasClass(pForecastButton, "buttonUp") ? "prevDailyConditionsDataText" : "nextDailyConditionsDataText";
pForecastButton.setAttribute("aria-label", resourceLoader.getString(pForecastAriaText));
}
dailyConditions = template.querySelectorAll(".pDailyModuleFirstProvider");
}
else if (layoutState === viewState.filled || layoutState === viewState.fullScreenLandscape) {
// more providers button
var providerButton = template.querySelector(".providerSlider .dailyConditionsAnimateButton");
if (providerButton) {
var providerAriaText = isMinimized ? "moreDailyConditionsAriaText" : "lessDailyConditionsAriaText";
providerButton.setAttribute("aria-label", resourceLoader.getString(providerAriaText));
}
// forecast button
var forecastButton = template.querySelector(".forecastSlider .forecastAnimateButton");
if (forecastButton) {
var direction = utilities.hasClass(forecastButton, windowDirection === "ltr" ? "buttonLeft" : "buttonRight");
var forecastAriaText = direction ? "prevDailyConditionsDataText" : "nextDailyConditionsDataText";
forecastButton.setAttribute("aria-label", resourceLoader.getString(forecastAriaText));
}
dailyConditions = template.querySelectorAll(".dailyModuleFirstProvider");
}
// update aria labels on daily conditions
if (dailyConditions) {
var ariaKey = isMinimized ? "ariaText" : "ariaTextExpanded";
var ariaLabel = "";
for (var index = 0, len = dailyConditions.length; index < len; index++) {
ariaLabel = utilities.data(dailyConditions[index])[ariaKey];
dailyConditions[index].setAttribute("aria-label", ariaLabel);
}
}
},
// this function toggles the state of the hero cluster more/less buttons irrespective of whether show/hide is called in portrait, landscape or filled modes
// the template is passed in for cases where the elements wouldn't be yet in the document. This is a condition that can happen when the cluster is getting built
toggleHeroButtons: function toggleHeroButtons(isMinimized, template) {
if (!template) {
template = document;
}
var winjsUtilities = WinJS.Utilities;
var animateButton = template.querySelector(".dailyConditionsAnimateButton");
var buttonUpClass = "buttonUp";
var buttonDownClass = "buttonDown";
if (animateButton) {
winjsUtilities.removeClass(animateButton, buttonUpClass);
winjsUtilities.removeClass(animateButton, buttonDownClass);
winjsUtilities.addClass(animateButton, isMinimized ? buttonDownClass : buttonUpClass);
}
},
_isScrollCurrentTargetPanorama: true, // if scroll start on hourly set this to False
_hourlyScrollDetectTimer: null, // after an interval of 1 second, this will reset the above flag
// get total daily conditions shown by default
getDefaultDailyConditionsTotal: function getDefaultDailyConditionsTotal() {
// For all other languages, we show 5 by default.
// Get the total forecasts displayed at a time
var dailyConditionsDisplayTotal = 5;
/*
For all regions it's 5 now, but retaining the code in case we need it again
var currentDisplayLanguage = Platform.Globalization.Marketization.getQualifiedLanguage();
if (currentDisplayLanguage.languageTag.indexOf('en') === 0) {
dailyConditionsDisplayTotal = 6;
}
*/
return dailyConditionsDisplayTotal;
},
// calculates the width to be set for the hourly header cell (in different languages, strings have different lengths)
// two parameters - the first is the div for which the width has to be calc. and set. The second is an id to differentiate among elements since we re-use widths once created.
setHourlyHeaderCellWidth: function setHourlyHeaderCellWidth(hourlyHeaderCellElem, elementId) {
if (hourlyHeaderCellElem) { //"hourlyHeaderPrecipCalculator", "hourlyHeaderFeelsLikeCalculator"
var hourlyHeaderCellWidth = WeatherAppJS.Utilities.UI.getPixelWidthFromString(hourlyHeaderCellElem.innerText, elementId);
hourlyHeaderCellElem.style.width = hourlyHeaderCellWidth;
}
},
// to calculate the width to which a div has to be set without strings wrapping
// create a div with required styles, set the string as innerHTML and get the div's width
// cache the width since we need to calculate this only once per language
getPixelWidthFromString: function getPixelWidthFromString(str, id) {
var stringId = id + "Width";
var ui = WeatherAppJS.Utilities.UI;
if (ui[stringId]) {
return ui[stringId];
}
// create an absolutely positioned element and set the string as innerHTML
// once created, just keep re-using the same element
var elem = document.querySelector('#' + id);
if (!elem) {
elem = document.createElement('div');
elem.id = id;
var body = document.getElementsByTagName('body')[0];
body.appendChild(elem);
elem.innerText = str;
}
ui[stringId] = (elem.clientWidth + 1) + 'px'; // add a pixel because clientWidth rounds off decimals to a lower length
return ui[stringId];
},
// given a daily forecast node in hero cluster, shows or hides the node based on market
setForecastNodeDisplayState: function setForecastNodeDisplayState(template, showDailyConditionsProvider, showLimitedExperience, displayProviderUrl, dailyForecastIndex, isSkiPano) {
// This is used only when one provider data is shown. 
if (template) {
var forecastNode = template.querySelector('.forecastProvider');
if (forecastNode) {
if (showDailyConditionsProvider) {
if (!showLimitedExperience || isSkiPano) {
forecastNode.style.display = "none";
}
else if (dailyForecastIndex === 0) {
forecastNode.innerHTML = displayProviderUrl;
forecastNode.style.display = "block";
} else if (showLimitedExperience && dailyForecastIndex > 0) {
forecastNode.innerHTML = '';
}
}
else {
forecastNode.style.display = "none";
}
}
}
},
// given a daily forecast node, sets the aria label
setForecastNodeAriaLabel: function setForecastNodeAriaLabel(node, isMinimized, showLimitedExperience, ariaTextValues) {
if (node) {
var utilities = WinJS.Utilities;
utilities.data(node).ariaText = ariaTextValues.ariaText;
utilities.data(node).ariaTextExpanded = ariaTextValues.ariaTextExpanded;
if (isMinimized || showLimitedExperience) {
node.setAttribute("aria-label", ariaTextValues.ariaText);
}
else {
node.setAttribute("aria-label", ariaTextValues.ariaTextExpanded);
}
}
},
// fade in promise - nodeList can be a single node or a list of nodes
fadeInAsync: function fadeInAsync(nodeList, durationOfAnimation) {
if (nodeList) {
if (!durationOfAnimation) {
durationOfAnimation = 500;
}
return WinJS.UI.executeTransition(
nodeList,
{
property: 'opacity',
delay: 0,
duration: durationOfAnimation,
timing: "linear",
from: 0,
to: 1
});
}
},
// fade out promise - nodeList can be a single node or a list of nodes
fadeOutAsync: function fadeOutAsync(nodeList, durationOfAnimation) {
if (nodeList) {
if (!durationOfAnimation) {
durationOfAnimation = 500;
}
return WinJS.UI.executeTransition(
nodeList,
{
property: 'opacity',
delay: 0,
duration: durationOfAnimation,
timing: "linear",
from: 1,
to: 0
});
}
},
/// <summary>
/// Determine if daily drilldown is to be enabled per location based on various factors
/// </summary>
isDailyDrilldownEnabled: function isDailyDrilldownEnabled(locationId) {
if (!WeatherAppJS.SettingsManager.isFRE() && WeatherAppJS.LocationsManager.getDefaultLocation()) { // if FRE is not over, do not enable daily drilldown since we don't have sample data for it
if (locationId) {
var locationObj = WeatherAppJS.LocationsManager.getLocation(locationId);
var cc = locationObj ? locationObj.getCurrentConditions() : null;
if (cc && cc.IsDailyDrilldownEnabled) { // whether the feature is set to enabled from the server-side for the location
var isEnabledInConfig = Platform.Utilities.Globalization.isFeatureEnabled("DailyDrilldownPanorama"); // whether the feature is enabled in the app config or not (market-based)
if (isEnabledInConfig) {
return true;
}
}
}
}
return false;
},
// decide whether to fetch reduced data for hourly to improve the day drilldown page perf
// return 1 for full data, 0 for reduced data
fetchReducedHourlyForecastData: function fetchReducedHourlyForecastData() {
var getReducedHourlyDataConfig = PlatformJS.Services.appConfig.getBool("FetchReducedHourlyData");
if (getReducedHourlyDataConfig) { // if set to true, check for architecture type
var currentArchType = Windows.ApplicationModel.Package.current.id.architecture;
var archObject = Windows.System.ProcessorArchitecture;
if ((currentArchType === archObject.ARM) || (currentArchType === archObject.arm)) { // if required architecture type, check if high res or not
var isHighRes = WeatherAppJS.Utilities.UI.isHighRes();
var getFullHourlyDataForHighRes = PlatformJS.Services.appConfig.getBool("FetchFullHourlyDataForHighRes");
if (isHighRes && getFullHourlyDataForHighRes) { // for high res, check if config permits showing full data
return 1;
} else {
return 0;
}
}
return 1;
} else {
return 1;
}
},
_isPerfOptimizationEnabled: null,
isPerfOptimizedExperience: function isPerfOptimizedExperience() {
if (this._isPerfOptimizationEnabled === null) {
var allowReducedExperience = PlatformJS.Services.appConfig.getBool("allowReducedExperience");
var currentArchType = Windows.ApplicationModel.Package.current.id.architecture;
var archObject = Windows.System.ProcessorArchitecture;
this._isPerfOptimizationEnabled = allowReducedExperience && ((currentArchType === archObject.ARM) || (currentArchType === archObject.arm));
}
return this._isPerfOptimizationEnabled;
},
// if the screen height is greater than a threshold, then it is considered high-resolution
isHighRes: function isHighRes() {
var isHighRes = null;
var heightToCheck = 768;
var viewState = Windows.UI.ViewManagement.ApplicationViewState;
var layoutState = Windows.UI.ViewManagement.ApplicationView.value;
if (layoutState === viewState.fullScreenPortrait) {
heightToCheck = 1366;
}
isHighRes = (window.screen.height > heightToCheck) ? true : false;
return isHighRes;
},
dismissPlatformFlyout: function dismissPlatformFlyout() {
// Hide Platform Flyouts (Add Location Flyout) if it already exists
var platformFlyout = document.getElementById("platformFlyout");
if (platformFlyout && platformFlyout.winControl) {
platformFlyout.winControl.hide();
//Workaround for Bug#388676 (TypetoSearch) Since we do not save an instance of the Add Location Flyout and hence do not properly hide the search Dialog, the showOnKeyboardInput parameter is not set to true after the dialog closes
try {
Windows.ApplicationModel.Search.SearchPane.getForCurrentView().showOnKeyboardInput = true;
} catch (e) { }
}
},
openInAppHelp: function openInAppHelp() {
Platform.Telemetry.FlightRecorder.setImpressionNavMethod(Platform.Telemetry.ImpressionNavMethod.appBar);
Platform.Telemetry.FlightRecorder.logUserAction(Platform.Telemetry.LogLevel.normal, Platform.Telemetry.ActionContext.bottomEdgy, "Help", Platform.Telemetry.UserActionOperation.click, Platform.Telemetry.UserActionMethod.unknown, 0);
CommonJS.Settings.onHelpCmd();
},
addNewHomeLoc: function addNewHomeLoc() {
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NumChangeHomeAdd");
var eventAttr = { currentHome: WeatherAppJS.SettingsManager.getDefaultLocation().getFullDisplayName() };
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, "Change Home Flyout", "Add Location", Platform.Telemetry.UserActionOperation.click, Platform.Telemetry.UserActionMethod.unknown, 0, JSON.stringify(eventAttr));
if (WeatherAppJS.SettingsManager.isFavoritesMaxLimitReached()) {
WeatherAppJS.Error.showMessageBar(PlatformJS.Services.resourceLoader.getString("MaxFavoritesLimitReached"), false);
}
else if (WeatherAppJS.Networking.internetAvailable === false) {
WeatherAppJS.Error.showMessageBar(PlatformJS.Services.resourceLoader.getString("NoInternetConnectionError"), false);
}
else {
WeatherAppJS.Search.showQuickSearchDialog("mainPanorama", WeatherAppJS.Search.Actions.SetDefault, WeatherAppJS.Search.onAddLocationFlyoutHide);
var addButton = document.getElementById("SearchDialog_addButton");
if (addButton) {
addButton.innerText = PlatformJS.Services.resourceLoader.getString("ConfirmButtonTitle");
}
var title = document.querySelector('.platformSearchDialogPrompt');
if (title) {
title.innerText = PlatformJS.Services.resourceLoader.getString("ChangeHomeButtonTitle");
}
}
},
/// show add location flyout with detected location name pre-set to confirm from the user if the detected location is accurate
showAddLocationFlyout: function showAddLocationFlyout(locationObj, /*@optional*/isFRE, /*@optional*/flyoutHideCallback) {
CommonJS.Progress.hideProgress(CommonJS.Progress.headerProgressType);
// todo: set the flyout title and button text through CUX api which is not yet implemented (bug 441816 on CUX) (arunrab)
WeatherAppJS.Search.showQuickSearchDialog("mainPanorama", WeatherAppJS.Search.Actions.DetectedLocationConfirm, flyoutHideCallback, locationObj, isFRE);
var searchBox = document.getElementById("SearchTextBox");
if (searchBox) {
searchBox.focus();
searchBox.value = WeatherAppJS.GeocodeCache.getFullDisplayName(locationObj.getId());
}
var addButton = document.getElementById("SearchDialog_addButton");
if (addButton) {
addButton.innerText = PlatformJS.Services.resourceLoader.getString("ConfirmButtonTitle");
}
// change the title of the search box only for FRE
if (isFRE) {
var title = document.querySelector('.platformSearchDialogPrompt');
if (title) {
title.innerText = PlatformJS.Services.resourceLoader.getString("SetHomeLocationTitle");
}
}
},
getMapTypesForRegion: function getMapTypesForRegion(region) {
var mapTypes = PlatformJS.Services.appConfig.getDictionary("MapTypesPerRegion");
try {
var regionData = mapTypes[region];
return JSON.parse(regionData.value);
} catch (ex) {
return null;
}
},
_allWeatherMapTypes: null,
_mapTypeToIdMapping: null,
_regionFetchIdToMapRegionMapping: null,
_initWeatherMapTypesSubTypes: function _initWeatherMapTypesSubTypes() {
var allMapTypes = JSON.parse(PlatformJS.Services.appConfig.getString("AllWeatherMapTypesSubTypes"));
this._allWeatherMapTypes = {};
this._mapTypeToIdMapping = {};
for (var m in allMapTypes) {
var map = allMapTypes[m];
if (map.id && map.mapType && map.mapSubTypes) {
if (!this._allWeatherMapTypes[map.id]) {
this._allWeatherMapTypes[map.id] = {};
this._mapTypeToIdMapping[map.mapType] = {};
}
for (var st in map.mapSubTypes) {
var mapSubType = map.mapSubTypes[st];
this._allWeatherMapTypes[map.id][mapSubType.id] = { id: map.id, mapType: map.mapType, subTypeId: mapSubType.id, mapSubType: mapSubType.subType };
this._mapTypeToIdMapping[map.mapType][mapSubType.subType] = this._allWeatherMapTypes[map.id][mapSubType.id];
if (st === "0") { // Save the first subtype as the default for the maptype
this._allWeatherMapTypes[map.id + "_default"] = { id: map.id, mapType: map.mapType, subTypeId: mapSubType.id, mapSubType: mapSubType.subType };
this._mapTypeToIdMapping[map.mapType + "_default"] = this._allWeatherMapTypes[map.id + "_default"];
}
}
}
}
},
getPageStateForMapType: function getPageStateForMapType(mapTypeId, mapSubTypeId) {
if (!this._allWeatherMapTypes) {
this._initWeatherMapTypesSubTypes();
}
if (mapTypeId) {
return (mapSubTypeId) ? this._allWeatherMapTypes[mapTypeId][mapSubTypeId] : this._allWeatherMapTypes[mapTypeId + "_default"];
}
},
getIdForMapTypeSubType: function getIdForMapTypeSubType(mapType, mapSubType) {
if (!this._mapTypeToIdMapping) {
this._initWeatherMapTypesSubTypes();
}
if (mapType) {
return (mapSubType) ? this._mapTypeToIdMapping[mapType][mapSubType] : this._mapTypeToIdMapping[mapType + "_default"];
}
},
getMapRegionByFetchId: function getMapRegionByFetchId(fetchId) {
if (!this._regionFetchIdToMapRegionMapping) {
this._regionFetchIdToMapRegionMapping = {};
var baseMapRegionsConfig = PlatformJS.Services.appConfig.getDictionary("BaseMapRegions");
for (var regionName in baseMapRegionsConfig) {
if (baseMapRegionsConfig.hasItem(regionName)) {
var regionData = JSON.parse(baseMapRegionsConfig[regionName].value);
this._regionFetchIdToMapRegionMapping[regionData.fetchId] = regionData;
}
}
}
return this._regionFetchIdToMapRegionMapping[fetchId];
},
_isoCodeToRegionMap: null,
// Async'ly return the continent data for an isocode. Returns null if not found.
getMapRegionDataForIsoCodeAsync: function getMapRegionDataForIsoCodeAsync(isocode) {
var that = this;
if (WeatherAppJS.Utilities.UI._isoCodeToRegionMap) {
return WinJS.Promise.wrap(WeatherAppJS.Utilities.UI._isoCodeToRegionMap[isocode]);
} else {
return that.createMapRegionToIsoCodeMappingAsync()
.then(function () {
if (that._isoCodeToRegionMap) {
return that._isoCodeToRegionMap[isocode];
}
},
function () {
return null;
});
}
},
createMapRegionToIsoCodeMappingAsync: function createMapRegionToIsoCodeMappingAsync() {
var ui = WeatherAppJS.Utilities.UI;
var isoCodeRegionMap = ui._isoCodeToRegionMap;
if (!isoCodeRegionMap) {
var countryRegionCodesFile = PlatformJS.Services.appConfig.getString("CountryToMapRegionMappingFile");
return WinJS.xhr({ url: countryRegionCodesFile }).then(
function (request) {
try {
ui._isoCodeToRegionMap = JSON.parse(request.responseText);
return WinJS.Promise.wrap(null);
} catch (ex) {
ui._isoCodeToRegionMap = {};
return WinJS.Promise.wrapError(null);
}
},
function (ex) {
ui._isoCodeToRegionMap = {};
return WinJS.Promise.wrapError(null);
});
} else {
return WinJS.Promise.wrap(null);
}
}        
});
WinJS.Namespace.define("WeatherAppJS.Services", {
Utilities: AppEx.WeatherApp.Services.Utilities,
});
WinJS.Namespace.define("WeatherAppJS.Utilities.Common", {
/// <summary>
/// caches window direction per app load based on langauge
/// </summary>
_windowDirection: null,
_latlongTrimValue: null,
/// <summary>
/// The input value is parsed and trimmed to a specified decimal places. 
/// This is done to improve caching on the server side.
/// </summary>
/// <param name="value">latitude or longitude</param>
/// <param name="decimalPlaces">Optional : Number of decimal places to keep after formatting</param>
/// <returns>trimmed value of the input</returns>
trimLatLong: function trimLatLong(value, decimalPlaces) {
var that = WeatherAppJS.Utilities.Common;
if (!that._latlongTrimValue) {
that._latlongTrimValue = PlatformJS.Services.appConfig.getInt32("LatLongDecimalPlaces");
}
var places = (decimalPlaces) ? decimalPlaces : that._latlongTrimValue;
if (value) {
value = parseFloat(value).toFixed(places);
}
return value || "";
},
/// <summary>
/// returns 'ltr' or 'rtl'
/// </summary>
getDirection: function getDirection() {
var common = WeatherAppJS.Utilities.Common;
var direction = common._windowDirection;
if (!direction) {
common._windowDirection = window.getComputedStyle(document.body).direction;
}
return common._windowDirection;
},
/// <summary>
/// Returns whether the country is same as user's geographic region. 
/// </summary>
/// <param name="country">Country to be tested</param>
/// <returns>Flag indicating whether the input country is the current region</returns>
isCurrentRegion: function isCurrentRegion(country) {
try {
var geographicRegion = new Windows.Globalization.GeographicRegion();
country = country.toUpperCase();
var nativeName = geographicRegion.nativeName.toUpperCase();
var displayName = geographicRegion.displayName.toUpperCase();
return (country === nativeName) || (country === displayName);
}
catch (Exception) {
return false;
}
},
/// <summary>
/// string to be used in case optional fields are absent
/// </summary>
getNoDataString: function getNoDataString() {
return '--';
},
/// <summary>
/// Checks whether the data is stale or not
/// </summary>
/// <param name="lastUpdatedTimeInMs">Last updated time of the data in Milli seconds</param>
/// <param name="freshtimeInMs">Data freshness time in milliseconds after which data would be stale</param>
/// <returns>True/False</returns>
isStale: function isStale(lastUpdatedTimeInMs, freshTimeInMs) {
var currentTime = new Date();
var currentTimeInMs = currentTime.getTime();
if ((currentTimeInMs > lastUpdatedTimeInMs) && (currentTimeInMs - lastUpdatedTimeInMs >= freshTimeInMs)) {
return true;
}
return false;
},
/// every location data fetch will update this variable. This is set in common because we don't call Utilities.UI anywhere in fetch and process code
enableDailyDrilldown: false,
/// <summary>
/// Removes a cluster from the list
/// </summary>
/// <param name="list">Data list</param>
/// <param name="clusterKey">Key of the Cluster</param>
removeCluster: function removeCluster(list, clusterKey) {
if (!list) {
return;
}
for (var i = 0; i < list.length; i++) {
var element = list.getItem(i);
if (element.data.clusterKey === clusterKey) {
list.splice(i, 1);
}
}
},
formatAlertDescription: function formatAlertDescription(alertDescription) {
if (alertDescription) {
// pattern removes the break tags at the start of the alert which aren't required
alertDescription = alertDescription.replace(/^(\s*<br \/>\s*)*/g, '');
}
return alertDescription;
},
// used for hourly and historic attributions
// adding the rest of the providers for comprehensiveness
getProviderAttributionUrl: function getProviderAttributionUrl(providerName) {
var providerUrl = '';
try {
if (providerName) {
var providerNameTmp = providerName.toLowerCase();
providerUrl = PlatformJS.Services.resourceLoader.getString(providerNameTmp + 'Url');
}
} catch (err) {
}
return providerUrl;
},
/// <summary>
/// Retruns the preset default location from config. Throws exception on failure
/// </summary>
/// <returns>Geocode location object</returns>
getPresetDefaultLocation: function getPresetDefaultLocation() {
var resourceLoader = PlatformJS.Services.resourceLoader;
var presetDefaultLocation = PlatformJS.Services.appConfig.getString("DefaultLocation");
var defaultLocation = JSON.parse(presetDefaultLocation);
// Get localized location names if available, else just return the location name
var city = resourceLoader.getString(defaultLocation.city);
if (city) {
defaultLocation.city = city;
}
var state = defaultLocation.state === "" ? "" : resourceLoader.getString(defaultLocation.state);
if (state) {
defaultLocation.state = state;
}
defaultLocation.country = defaultLocation.countryORregion;
var country = resourceLoader.getString(defaultLocation.country);
if (country) {
defaultLocation.country = country;
}
defaultLocation.fullName = [defaultLocation.city, defaultLocation.state, defaultLocation.country].join(", ");
// Add location to cache first, as it has not been added to Settings Manager yet.
var geoLoc = new WeatherAppJS.GeocodeLocation(defaultLocation);
WeatherAppJS.GeocodeCache.addLocation(geoLoc);
return geoLoc;
},
/// <summary>
/// Retuns true if app has been suspended for a long time (defined in config)
/// </summary>
/// <returns>Bool flag</returns>
isAppStateStale: function isAppStateStale() {
var currentTimeInMs = new Date().getTime();
var lastSuspendTimeInMs = WeatherAppJS.SettingsManager.getLastSuspendTime();
var appStateStaleTime = PlatformJS.Services.appConfig.getString("AppStateStaleTime");
return (lastSuspendTimeInMs && ((currentTimeInMs - lastSuspendTimeInMs) >= appStateStaleTime)) ? true : false;
},
/// <summary>
/// Retuns the current page
/// </summary>
/// <returns>Current page</returns>
getCurrentPage: function getCurrentPage() {
var page = "";
try {
page = WinJS.Navigation.history.current.location.page;
} catch (e) {
}
return page;
},
/// <summary>
/// Function to get the safe icon code value, any value which isn't recognized by the app is defaulted to 44
/// </summary>
/// <param name="iconCode">Icon code</param>
/// <returns>safe icon code</returns>
getSafeIconCode: function getSafeIconCode(iconCode) {
switch (iconCode) {
case "1": return "1";
case "1b": return "1b";
case "5": return "5";
case "7": return "7";
case "9": return "9";
case "9b": return "9b";
case "9c": return "9c";
case "11": return "11";
case "12": return "12";
case "13_41_46": return "13_41_46";
case "17": return "17";
case "19": return "19";
case "19b": return "19b";
case "19c": return "19c";
case "20": return "20";
case "20b": return "20b";
case "20c": return "20c";
case "23": return "23";
case "25": return "25";
case "25b": return "25b";
case "26": return "26";
case "27": return "27";
case "27b": return "27b";
case "29": return "29";
case "29b": return "29b";
case "31": return "31";
case "31b": return "31b";
case "34_33": return "34_33";
case "40": return "40";
case "43": return "43";
case "44": return "44";
default: return "44";
}
},
// weather.com region
weatherDotComRegion: null,
/// <summary>
/// Function to get the weather.com region 
/// </summary>
/// <returns>region code</returns>
getWeatherDotComRegion: function getWeatherDotComRegion() {
var weatherCommon = WeatherAppJS.Utilities.Common;
if (weatherCommon.weatherDotComRegion) {
return weatherCommon.weatherDotComRegion;
}
var currentRegion = Windows.System.UserProfile.GlobalizationPreferences.homeGeographicRegion;
if (currentRegion) {
var regionList = PlatformJS.Services.appConfig.getList("WeatherDotComRegionList");
var regionDict = PlatformJS.Services.appConfig.getDictionary("WeatherDotComRegionsDict");
if (regionList && regionDict) {
for (var i = 0; i < regionList.size; i++) {
var region = regionList[i].value;
if (region === currentRegion) {
weatherCommon.weatherDotComRegion = region;
break;
}
else {
var regionValue = regionDict.getString(region);
var pos = regionValue.search(currentRegion);
if (pos !== -1) {
weatherCommon.weatherDotComRegion = region;
break;
}
}
}
}
}
if (!weatherCommon.weatherDotComRegion) {
weatherCommon.weatherDotComRegion = "US";
}
return weatherCommon.weatherDotComRegion;
},
/// <summary>
/// Function to get the market-dependent weather.com base url
/// </summary>
getWeatherDotComBaseUrl: function getWeatherDotComBaseUrl() {
var region = WeatherAppJS.Utilities.Common.getWeatherDotComRegion();
return PlatformJS.Services.appConfig.getString("WeatherDotComBaseUrl-" + region);
},
/// <summary>
/// Function to get the market-dependent weather.com home page url
/// </summary>
getWeatherDotComHomeUrl: function getWeatherDotComHomeUrl() {
var region = WeatherAppJS.Utilities.Common.getWeatherDotComRegion();
return PlatformJS.Services.appConfig.getString("WeatherDotComUrl-" + region);
},
/// <summary>
/// Function to get the market-dependent weather.com base url
/// </summary>
getWeatherDotComSeeMoreText: function getWeatherDotComSeeMoreText() {
var seeMoreText = PlatformJS.Services.resourceLoader.getString("WeatherDotComSeeMoreText");
return seeMoreText.replace("weather.com", WeatherAppJS.Utilities.Common.getWeatherDotComBaseUrl());
},
/// <summary>
/// Function to get market-dependent weather.com URL for a given location
/// </summary>
/// <param name="weatherDotComUrl">base weather.com url</param>
/// <param name="urlName">name of the url type to retrieve</param>
/// <returns>market dependent weather.com url</returns>
getWeatherDotComUrl: function getWeatherDotComUrl(weatherDotComUrl, urlName) {
if (weatherDotComUrl && weatherDotComUrl.length > 0) {
var baseUrl = weatherDotComUrl.replace(/\?.*$/, '');
var locationIdIndex = baseUrl.lastIndexOf("/");
if (locationIdIndex !== -1) {
var locationId = baseUrl.substr(locationIdIndex + 1);
if (locationId) {
var urlConfigName = urlName + "-" + WeatherAppJS.Utilities.Common.getWeatherDotComRegion();
var mktUrl = PlatformJS.Services.appConfig.getString(urlConfigName);
return mktUrl.format(locationId);
}
}
}
// fallback to the home-page URL for the market
return WeatherAppJS.Utilities.Common.getWeatherDotComHomeUrl();
},
/// <summary>
/// Function to get market-dependent weather.com interactive map URL for a given location
/// </summary>
/// <param name="locationId">location id</param>
/// <returns>market dependent weather.com interactive map url</returns>
getWeatherDotComIMapUrl: function getWeatherDotComIMapUrl(locationId) {
var appConfig = PlatformJS.Services.appConfig;
var weatherCommon = WeatherAppJS.Utilities.Common;
if (locationId) {
var mapsUrlPrefix = appConfig.getString("WeatherDotComMapsUrlPrefix-" + weatherCommon.getWeatherDotComRegion());
var mapsUrlSuffix = appConfig.getString("WeatherDotComMapsUrlSuffix-" + weatherCommon.getWeatherDotComRegion());
if (mapsUrlSuffix && mapsUrlSuffix.length > 0) {
return mapsUrlPrefix + locationId + mapsUrlSuffix;
}
if (mapsUrlPrefix && mapsUrlPrefix.length > 0) {
return mapsUrlPrefix;
}
}
// fallback to the home-page URL for the market
return weatherCommon.getWeatherDotComHomeUrl();
},
/// <summary>
/// Clean up the promises stack
/// </summary>
/// <param name="promises">Array of Promises</param>
cleanupPromiseStack: function cleanupPromiseStack(promises) {
if (!promises) {
return;
}
while (promises.length > 0) {
var p = promises.pop();
if (p && p.cancel) {
p.cancel();
p = null;
}
}
},
/// <summary>
/// Reinitialize and reset cached app config values
/// </summary>
reinitialize: function reinitialize() {
this.weatherDotComRegion = null;
this._latlongTrimValue = null;
},
instrumentLocation: function instrumentLocation(datapoint, latitude, longitude, locationName) {
var sliceParameter = PlatformJS.Services.appConfig.getInt32("LocationInstrumentationStringLength");
var trimLatLong = WeatherAppJS.Utilities.Common.trimLatLong;
latitude = (latitude) ? trimLatLong(latitude, 2) : "";
longitude = (longitude) ? trimLatLong(longitude, 2) : "";
var value = latitude + '*' + longitude + '*' + locationName;
WeatherAppJS.Utilities.Instrumentation.addString(datapoint, value.slice(0, sliceParameter));
},
/// <summary>
/// Navigate to map app with lat and long in case map app is present on client else open it in browser.
/// </summary>
navigateToMapApp: function navigateToMapApp(lat, long) {
var latLong = "{0}~{1}".format(lat, long);
//Get map App Url to Launch. And format the url paramters. 
var mapAppUrl = PlatformJS.Services.appConfig.getString("MapAppUrl");
mapAppUrl = mapAppUrl.format(latLong);
var uri = new Windows.Foundation.Uri(mapAppUrl);
// Launch the URI.
Windows.System.Launcher.launchUriAsync(uri);
// TODO FallBack Scenario where Map app is not installed. Need to check with PM for Fallback
},
/// <summary>
/// Navigate to Iframe reader.
/// </summary>
/// <param name="destinationUrl">destination url to open</param>
navigateToIFrameReader: WinJS.UI.eventHandler(function (destinationUrl, page) {
if (destinationUrl && destinationUrl !== undefined) {
PlatformJS.Navigation.navigateToChannel('IFrameReader', { articleUrl: destinationUrl, page: page});
}
}),
/// <summary>
/// Converts a lat long point to pixel on a mercator map. Returns null on failure.
/// The longitude is assumed to range from -180 to +180 degrees, and the latitude must be clipped to range from -85.05112878 to 85.05112878
/// </summary>
tryLatLongToPixel: function tryLatLongToPixel(latitude, longitude, zoomLevel) {
var mercatorLatitudeLimit = 85.05112878;
var pixelX, pixelY;
if (latitude > this.MercatorLatitudeLimit || latitude < -this.MercatorLatitudeLimit) {
return null;
}
try{
var sinLatitude = Math.sin(latitude * Math.PI / 180);
pixelX = ((longitude + 180) / 360) * 256 * Math.pow(2, zoomLevel);
pixelY = (0.5 - (Math.log((1 + sinLatitude) / (1 - sinLatitude)) / (4.0 * Math.PI))) * 256 * Math.pow(2, zoomLevel);
return { x: pixelX, y: pixelY };
} catch (e) {
}
},
_fetchImageAsync: function _fetchImageAsync(imageSource, networkCallRequired) {
var imageUrl = imageSource.url;
var cacheId = imageSource.cacheId;
// Check to see if this is a local image
if (typeof imageUrl !== "string") {
return WinJS.Promise.wrap("");
}
else if (imageUrl.indexOf("/") === 0 || imageUrl.indexOf("ms-appdata://") === 0 || imageUrl.indexOf("ms-appx://") === 0) {
return WinJS.Promise.wrap(imageUrl);
} else {
return PlatformJS.Utilities.fetchImage(cacheId, imageUrl, networkCallRequired);
}
},
_fetchImageFromCacheAsync: function _fetchImageFromCacheAsync(imageSource) {
var that = this;
var cacheId = imageSource.cacheId;
return PlatformJS.Cache.CacheService.getInstance(cacheId).findEntry(imageSource.url, { fileNameOnly: true })
.then(function image_cache_completion(response) {
return response;
});
},
fetchSingleResolutionImageAsync: function fetchSingleResolutionImageAsync(imageSource, bypassCache) {
var that = this;
if (!imageSource || !imageSource.url || !imageSource.cacheId) {
return WinJS.Promise.wrap(null);
}
var promise = (!bypassCache) ? this._fetchImageFromCacheAsync(imageSource) : WinJS.Promise.wrap(null);
return promise
.then(function image_refresh(response) {
if (!response || response.isStale()) {
return that._fetchImageAsync(imageSource, null);
}
return WinJS.Promise.wrap(null);
}).then(function loadImageComplete(localUrl) {
return localUrl;
});
},
fetchDualResolutionImageAsync: function fetchDualResolutionImageAsync(dualImageSource, bypassCache) {
var that = this;
var lowResImageSource = dualImageSource.lowRes;
var highResImageSource = dualImageSource.highRes;
var promise = (!bypassCache) ? this._fetchImageFromCacheAsync(highResImageSource) : WinJS.Promise.wrap(null);
return promise
.then(function image_refresh(response) {
if (!response || response.isStale()) {
// High res image not cached, load low res then high res
return that._loadSingleResolutionAsync(lowResImageSource, bypassCache);
}
})
.then(function loadSingleResolutionComplete() {
return that._fetchImageAsync(highResImageSource, null);
})
.then(function fetchImage_complete(localUrl) {
return localUrl;
});
},
prefetchImageService: null,
prefetchImage: function prefetchImage(imageSource, queryServiceOptions) {
if (!imageSource || !imageSource.url || !imageSource.cacheId) {
return;
}
if (!this.prefetchImageService) {
this.prefetchImageService = {};
}
if (!this.prefetchImageService[imageSource.cacheId]) {
this.prefetchImageService[imageSource.cacheId] = new Platform.ImageService(imageSource.cacheId);
}
if (!queryServiceOptions) {
queryServiceOptions = new Platform.DataServices.PrefetchQueryServiceOptions();
queryServiceOptions.priority = Platform.DataServices.QueryServicePriority.high;
}
this.prefetchImageService[imageSource.cacheId].prefetchImage(imageSource.url, queryServiceOptions);
},
getSkiAttributionData: function getSkiAttributionData(currentConditionsData) {
var resourceLoader = PlatformJS.Services.resourceLoader;
var skiAttrLabel = "";
var skiImageAttrText = currentConditionsData.heroImageData ? currentConditionsData.heroImageData.attribution : '';
if ((skiImageAttrText) && (skiImageAttrText !== '')) {
skiAttrLabel += '<span class="skiAttributionText">' + resourceLoader.getString("SkiDetailsImageAttribution").format(skiImageAttrText) + '</span>';
}
var skiSnowProvider = currentConditionsData.heroData ? currentConditionsData.heroData.snowProviderData : '';
if ((skiSnowProvider) && (skiSnowProvider !== '')) {
skiAttrLabel += '<span class="skiAttributionText"><span>' + resourceLoader.getString("SnowDataAttribution").format('') + '</span>';
var skiSnowProviderUrl = currentConditionsData.heroData.snowProviderDataUrl;
if (skiSnowProviderUrl) {
skiAttrLabel += '<a class="skiAttributionLink" href="' + skiSnowProviderUrl + '">' + skiSnowProvider + '</a>';
} else {
skiAttrLabel += skiSnowProvider;
}
skiAttrLabel += '</span>';
}
var skiCurrentCondProvider = (currentConditionsData.AttrProvider !== undefined) ? currentConditionsData.AttrProvider : '';
var skiForecastProvider = (currentConditionsData.dailyForecastProvider !== undefined) ? currentConditionsData.dailyForecastProvider : '';
if ((skiCurrentCondProvider !== '') && (skiForecastProvider !== '')) {
skiAttrLabel += '<span class="skiAttributionText"><span>' + resourceLoader.getString("CCDailyAttribution") + ' </span>';
var langDirection = WeatherAppJS.Utilities.Common.getDirection();
var currentConditionProviderLink = "";
var skiCurrentCondProviderUrl = (currentConditionsData.AttrProviderUrl !== undefined) ? currentConditionsData.AttrProviderUrl : '';
if (skiCurrentCondProviderUrl !== '') {
currentConditionProviderLink += '<a class="skiAttributionLink" href="' + skiCurrentCondProviderUrl + '">' + skiCurrentCondProvider + '</a>';
} else {
currentConditionProviderLink += skiCurrentCondProvider;
}
var dailyForecastProviderLink = "";
var skiForecastProviderUrl = (currentConditionsData.dailyForecastProviderUrl !== undefined) ? currentConditionsData.dailyForecastProviderUrl : '';
if (skiForecastProviderUrl !== '') {
dailyForecastProviderLink += '<a class="skiAttributionLink" href="' + skiForecastProviderUrl + '">' + skiForecastProvider + '</a>';
} else {
dailyForecastProviderLink += skiForecastProvider;
}
if (skiCurrentCondProvider !== skiForecastProvider) {
if (langDirection === 'rtl') {
skiAttrLabel += dailyForecastProviderLink + ", " + currentConditionProviderLink;
} else {
skiAttrLabel += currentConditionProviderLink + ", " + dailyForecastProviderLink;
}
} else {
skiAttrLabel += currentConditionProviderLink;
}
skiAttrLabel += '</span>';
}
return skiAttrLabel;
}
});
WinJS.Namespace.define("WeatherAppJS.Utilities.Instrumentation", {
dayOfWeek:[],
onClickAttribution: function onClickAttribution(attribution, type) {
var eventAttr = { providerName: attribution };
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, type, "Attribution", Platform.Telemetry.UserActionOperation.click, Platform.Telemetry.UserActionMethod.unknown, 0, JSON.stringify(eventAttr));
},
onClickHWFilter: function onClickHWFilter(filter, locationName) {
var eventAttr = { location: locationName, filter: filter };
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, "Historical Weather", "Filter", Platform.Telemetry.UserActionOperation.select, Platform.Telemetry.UserActionMethod.unknown, 0, JSON.stringify(eventAttr));
},
incrementInt32: function incrementInt32(datapoint) {
PlatformJS.Services.instrumentation.incrementInt32(
PlatformJS.Services.appConfig.getDictionary("Instrumentation").getInt32("DataSetID"),
PlatformJS.Services.appConfig.getDictionary("Instrumentation").getDictionary("DataPoints").getInt32(datapoint), 1);
},
addString: function addString(datapoint, value) {
PlatformJS.Services.instrumentation.addString(
PlatformJS.Services.appConfig.getDictionary("Instrumentation").getInt32("NewDataSetID"),
PlatformJS.Services.appConfig.getDictionary("Instrumentation").getDictionary("DataPoints").getInt32(datapoint), value);
},
incrementInt32Array: function incrementInt32Array(datapoint, index, arraySize) {
PlatformJS.Services.instrumentation.incrementInt32Array(
PlatformJS.Services.appConfig.getDictionary("Instrumentation").getInt32("DataSetID"),
PlatformJS.Services.appConfig.getDictionary("Instrumentation").getDictionary("DataPoints").getInt32(datapoint), index, arraySize);
},
});
WinJS.Namespace.define("WeatherAppJS.Utilities.ThemeManager", {
_skyCodeMapping: {
"1b": "1b+n",   // Default = Night
"5": "5+n",     // Default = Night
"9c": "9c+n",   // Default = Night
"11": "11+n",   // Default = Night
"12": "12+n",   // Default = Night
"17": "17+n",   // Default = Night
"19c": "19c+n", // Default = Night
"23": "23+d",   // Default = Day
"26": "26+n",   // Default = Night
},
_skyCodeToThemeMapping: {
"1b+d": "ThunderstormsDayTheme",
"1b+n": "ThunderstormsNightTheme",
"5+d": "RainSnowDayTheme",
"5+n": "RainSnowNightTheme",
"7": "SnowDayTheme",
"9": "LightShowersDayTheme",
"9b": "LightShowersNightTheme",
"9c+d": "DrizzleDayTheme",
"9c+n": "DrizzleNightTheme",
"11+d": "RainDayTheme",
"11+n": "RainNightTheme",
"12+d": "BlowingSnowDayTheme",
"12+n": "BlowingSnowNightTheme",
"13_41_46": "ScatteredFlurriesTheme",
"17+d": "FreezingRainDayTheme",
"17+n": "FreezingRainNightTheme",
"19": "DustyClearDayTheme",
"19b": "DustyClearNightTheme",
"19c+d": "DustCloudDayTheme",
"19c+n": "DustCloudNightTheme",
"20": "FogDayTheme",
"20b": "FogNightTheme",
"20c": "FogNeutralTheme",
"23+d": "WindyDayTheme",
"23+n": "WindyNightTheme",
"25": "FrigidDayTheme",
"25b": "FrigidNightTheme",
"26+d": "CloudyDayTheme",
"26+n": "CloudyNightTheme",
"27": "MostlyCloudyDayTheme",
"27b": "MostlyCloudyNightTheme",
"29": "PartlyCloudyDayTheme",
"29b": "PartlyCloudyNightTheme",
"31": "ClearDayTheme",
"31b": "ClearNightTheme",
"34_33": "ScatteredCloudsTheme",
"40": "HeavyRainTheme",
"43": "BlizzardTheme",
"44": "DefaultTheme"
},
_skyCodeToAnchorMapping: {
"1b+d": "anchorLeft",
"1b+n": "anchorLeft",
"5+d": "anchorLeft",
"5+n": "anchorLeft",
"7": "anchorRight",
"9": "anchorLeft",
"9b": "anchorMiddle",
"9c+d": "anchorRight",
"9c+n": "anchorLeft",
"11+d": "anchorMiddle",
"11+n": "anchorMiddle",
"12+d": "anchorMiddle",
"12+n": "anchorMiddle",
"13_41_46": "anchorLeft",
"17+d": "anchorMiddle",
"17+n": "anchorMiddle",
"19": "anchorMiddle",
"19b": "anchorMiddle",
"19c+d": "anchorLeft",
"19c+n": "anchorRight",
"20": "anchorLeft",
"20b": "anchorRight",
"20c": "anchorRight",
"23+d": "anchorMiddle",
"23+n": "anchorLeft",
"25": "anchorMiddle",
"25b": "anchorLeft",
"26+d": "anchorMiddle",
"26+n": "anchorMiddle",
"27": "anchorMiddle",
"27b": "anchorRight",
"29": "anchorLeft",
"29b": "anchorMiddle",
"31": "anchorMiddle",
"31b": "anchorRight",
"34_33": "anchorMiddle",
"40": "anchorMiddle",
"43": "anchorRight",
"44": "anchorMiddle"
},
_skyCodeToAnchorMappingForPortrait: {
"1b+d": "anchorLeft",
"1b+n": "anchorLeft",
"5+d": "anchorRight",
"5+n": "anchorLeft",
"7+d": "anchorRight",
"7+n": "anchorLeft",
"9": "anchorRight",
"9b": "anchorLeft",
"9c+d": "anchorLeft",
"9c+n": "anchorLeft",
"11+d": "anchorRight",
"11+n": "anchorLeft",
"12+d": "anchorMiddle",
"12+n": "anchorMiddle",
"13_41_46": "anchorLeft",
"17+d": "anchorLeft",
"17+n": "anchorLeft",
"19": "anchorMiddle",
"19b": "anchorMiddle",
"19c+d": "anchorRight",
"19c+n": "anchorLeft",
"20": "anchorLeft",
"20b": "anchorRight",
"20c": "anchorMiddle",
"23+d": "anchorMiddle",
"23+n": "anchorLeft",
"25": "anchorLeft",
"25b": "anchorMiddle",
"26+d": "anchorMiddle",
"26+n": "anchorRight",
"27": "anchorRight",
"27b": "anchorRight",
"29": "anchorLeft",
"29b": "anchorMiddle",
"31": "anchorMiddle",
"31b": "anchorRight",
"34_33": "anchorLeft",
"40": "anchorMiddle",
"43": "anchorRight",
"44": "anchorMiddle"
},
_skyCodeToThemeTypeMapping: {
"1b+d": "darkTheme",
"1b+n": "darkTheme",
"5+d": "darkTheme",
"5+n": "darkTheme",
"7+d": "darkTheme",
"7+n": "darkTheme",
"9": "darkTheme",
"9b": "darkTheme",
"9c+d": "darkTheme",
"9c+n": "darkTheme",
"11+d": "darkTheme",
"11+n": "darkTheme",
"12+d": "darkTheme",
"12+n": "darkTheme",
"13_41_46": "darkTheme",
"17+d": "darkTheme",
"17+n": "darkTheme",
"19": "darkTheme",
"19b": "darkTheme",
"19c+d": "darkTheme",
"19c+n": "darkTheme",
"20": "darkTheme",
"20b": "darkTheme",
"20c": "darkTheme",
"23+d": "darkTheme",
"23+n": "darkTheme",
"25": "darkTheme",
"25b": "darkTheme",
"26+d": "darkTheme",
"26+n": "darkTheme",
"27": "darkTheme",
"27b": "darkTheme",
"29": "darkTheme",
"29b": "darkTheme",
"31": "darkTheme",
"31b": "darkTheme",
"34_33": "darkTheme",
"40": "darkTheme",
"43": "darkTheme",
"44": "darkTheme"
},
_getMorphedSkyCode: function _getMorphedSkyCode(skycode, dayNightIndicator) {
var defaultSkyCode = this._skyCodeMapping[skycode];
if (defaultSkyCode) {
return dayNightIndicator ? skycode + "+" + dayNightIndicator : defaultSkyCode;
} else {
return skycode;
}
},
getHeroImageForSkyCode: function getHeroImageForSkyCode(skycode, dayNightIndicator) {
var morphedSkyCode = this._getMorphedSkyCode(skycode, dayNightIndicator);
return "/images/panoramaBackgrounds/" + morphedSkyCode + ".jpg";
},
getTileImageForSkyCode: function getTileImageForSkyCode(skycode, dayNightIndicator) {
var morphedSkyCode = this._getMorphedSkyCode(skycode, dayNightIndicator);
return 'url(/images/favoritesBackgrounds/' + morphedSkyCode + '.jpg)';
},
getAnchorForSkyCode: function getAnchorForSkyCode(skycode, dayNightIndicator) {
var anchorPoint = "";
var morphedSkyCode = this._getMorphedSkyCode(skycode, dayNightIndicator);
var viewManagement = Windows.UI.ViewManagement;
if (viewManagement.ApplicationView.value === viewManagement.ApplicationViewState.fullScreenPortrait) {
anchorPoint = this._skyCodeToAnchorMappingForPortrait[morphedSkyCode];
} else {
anchorPoint = this._skyCodeToAnchorMapping[morphedSkyCode];
}
if (!anchorPoint) {
anchorPoint = "anchorTop"; // default
}
// for RTL, we need to reverse the left/right anchors
var windowDirection = WeatherAppJS.Utilities.Common.getDirection();
if (windowDirection === "rtl") {
if (anchorPoint === "anchorLeft") {
anchorPoint = "anchorRight";
} else if (anchorPoint === "anchorRight") {
anchorPoint = "anchorLeft";
}
}
return anchorPoint;
},
_themesList: [],
/// get the theme mapped to the skycode. Return default theme if skycode is not available or not present in the mapping
_getThemeForSkyCode: function _getThemeForSkyCode(skycode) {
if (this._skyCodeToThemeMapping[skycode]) {
return this._skyCodeToThemeMapping[skycode];
}
else {
return this._skyCodeToThemeMapping["44"];
}
},
/// get the theme type mapped to the skycode. Return default type theme if skycode is not available or not present in the mapping
_getThemeTypeForSkyCode: function _getThemeTypeForSkyCode(skycode) {
if (this._skyCodeToThemeTypeMapping[skycode]) {
return this._skyCodeToThemeTypeMapping[skycode];
}
else {
return this._skyCodeToThemeTypeMapping["44"];
}
},
/// remove the current theme from the given element
_clearTheme: function _clearTheme(elem) {
var classList = "darkTheme lightTheme ";
var themes = this._getAllThemes();
if (themes && themes.length > 0) {
classList += themes.join(" ");
}
WinJS.Utilities.removeClass(elem, classList);
},
/// applys a theme on the given element
_applyTheme: function _applyTheme(elem, theme) {
WinJS.Utilities.addClass(elem, theme);
},
/// we first run through this list to remove all themes set before applying a new theme
_getAllThemes: function _getAllThemes() {
if (this._themesList.length === 0) {
for (var themeName in this._skyCodeToThemeMapping) {
this._themesList.push(this._skyCodeToThemeMapping[themeName]);
}
}
return this._themesList;
},
/// given an element and a skycode, apply the theme on that element
applyTheme: function applyTheme(elementNode, skycode, dayNightIndicator) {
if (elementNode) {
var elem = elementNode;
var morphedSkyCode = this._getMorphedSkyCode(skycode, dayNightIndicator);
this._clearTheme(elem);
this._applyTheme(elem, this._getThemeForSkyCode(morphedSkyCode));
this._applyTheme(elem, this._getThemeTypeForSkyCode(morphedSkyCode));
}
}
});
WinJS.Namespace.define("WeatherAppJS.Utilities.TabIndexManager", {
HandleKeyboardInvocation: WinJS.UI.eventHandler(function (event) {
if (event) {
switch (event.keyCode) {
case WinJS.Utilities.Key.enter:
case WinJS.Utilities.Key.space:
if (event.target) {
event.target.click();
}
break;
default:
break;
}
}
})
});
WinJS.Namespace.define("WeatherAppJS.Utilities.TianQi", {
getEncodedLocationNameForQuery: function getEncodedLocationNameForQuery(locationName) {
var encodedLocationName = encodeURI(locationName.replace(/,.*$/, ''));
return (encodedLocationName + "%20" + "%e5%a4%a9%e6%b0%94"); // %e5%a4%a9%e6%b0%94 is the uriencoded version of 天气; the word 'weather' in simplified chinese
},
getEncodedLocationName: function getEncodedLocationName(locationName) {
var encodedLocationName = encodeURI(locationName.replace(/,.*$/, ''));
return encodedLocationName;
}
});
WinJS.Namespace.define("WeatherAppJS.Utilities.Formatting", {
_formatPattern: PlatformJS.Services.resourceLoader.getString('ShortDateTimeFormatString'),
_dayFormatter: null,
dayFormatter: {
get: function get() {
if (!this._dayFormatter) {
this._dayFormatter = new Windows.Globalization.DateTimeFormatting.DateTimeFormatter("day");
}
return this._dayFormatter;
}
},
// date format used by reviews: 04 Feb, 2013
_reviewDateFormatter: null,
reviewDateFormatter: {
get: function get() {
if (!this._reviewDateFormatter) {
this._reviewDateFormatter = new Windows.Globalization.DateTimeFormatting.DateTimeFormatter("{day.integer(2)} {month.abbreviated}, {year.full}");
}
return this._reviewDateFormatter;
}
},
_dayOfWeekFormatterShort: null,
dayOfWeekFormatterShort: {
get: function get() {
if (!this._dayOfWeekFormatterShort) {
this._dayOfWeekFormatterShort = new Windows.Globalization.DateTimeFormatting.DateTimeFormatter("dayofweek.abbreviated");
}
return this._dayOfWeekFormatterShort;
}
},
_dayFormatReversed: null,
dayFormatReversed: {
get: function get() {
if (!this._dayFormatReversed) {
this._dayFormatReversed = PlatformJS.Services.appConfig.getBool("DayFormatReversed");
}
return this._dayFormatReversed;
}
},
_dayOfWeekFormatter: null,
dayOfWeekFormatter: {
get: function get() {
if (!this._dayOfWeekFormatter) {
this._dayOfWeekFormatter = new Windows.Globalization.DateTimeFormatting.DateTimeFormatter("dayofweek");
}
return this._dayOfWeekFormatter;
}
},
_yearFormatter: null,
yearFormatter: {
get: function get() {
if (!this._yearFormatter) {
this._yearFormatter = new Windows.Globalization.DateTimeFormatting.DateTimeFormatter("year");
}
return this._yearFormatter;
}
},
_hourFormatter: null,
hourFormatter: {
get: function get() {
var formatting = Windows.Globalization.DateTimeFormatting;
if (!this._hourFormatter) {
this._hourFormatter = new formatting.DateTimeFormatter(formatting.HourFormat.default, formatting.MinuteFormat.default, formatting.SecondFormat.none);
}
return this._hourFormatter;
}
},
_intFormatter: null,
intFormatter: {
get: function get() {
if (!this._intFormatter) {
this._intFormatter = new Windows.Globalization.NumberFormatting.DecimalFormatter();
this._intFormatter.fractionDigits = 0;
}
return this._intFormatter;
}
},
_forecastClusterKeyFormatter: null,
forecastClusterKeyFormatter: {
get: function get() {
var formatting = Windows.Globalization.DateTimeFormatting;
if (!this._forecastClusterKeyFormatter) {
this._forecastClusterKeyFormatter = new Windows.Globalization.DateTimeFormatting.DateTimeFormatter("dayofweek month day");
}
return this._forecastClusterKeyFormatter;
}
},
// This is the time difference between 1601/1/1 and 1970/1/1 in miliseconds
_timeDiff: 11644473600000,
/// <summary>
/// converts kif time to javascript date object
/// </summary>
/// <param name="kifTime">number of 100-nanosecond ticks from 1601/1/1</param>
/// <returns>Date object equivalent to the input</returns>
_convertKifToJSDate: function _convertKifToJSDate(kifTime) {
// number of miliseconds from 1970/1/1
var jsMiliSeconds = (kifTime / 10000) - this._timeDiff;
var jsDate = new Date();
jsDate.setTime(jsMiliSeconds);
jsDate.setMinutes(jsDate.getMinutes() + jsDate.getTimezoneOffset());
return jsDate;
},
convertUTCToLocalDate: function convertUTCToLocalDate(utcDateObj) {
var localDateObj = new Date();
localDateObj.setUTCFullYear(utcDateObj.getFullYear());
localDateObj.setUTCMonth(utcDateObj.getMonth());
localDateObj.setUTCDate(utcDateObj.getDate());
localDateObj.setUTCHours(utcDateObj.getHours());
localDateObj.setUTCMinutes(utcDateObj.getMinutes());
localDateObj.setUTCSeconds(utcDateObj.getSeconds());
return localDateObj;
},
/// <summary>
/// if the user's language uses native digits instead of latin, then it returns number in that format.
/// This is mostly required for arabic. most of the languages use latin digits.
/// </summary>
/// <param name="val">the input number</param>
/// <returns>the input number in the user's native language</returns>
formatInt: function formatInt(val) {
return WeatherAppJS.Utilities.Formatting.intFormatter.format(val);
},
isRtlNumberFormat: function isRtlNumberFormat() {
var numeralSystem = WeatherAppJS.Utilities.Formatting.intFormatter.numeralSystem;
if (numeralSystem === "Arab" || numeralSystem === "ArabExt") {
return true;
}
return false;
},
_languageString: "",
getQualifiedLanguageString: function getQualifiedLanguageString() {
if (this._languageString === "") {
this._languageString = Platform.Globalization.Marketization.getQualifiedLanguageString().toLowerCase();
}
return this._languageString;
},
getSymbolPosition: function getSymbolPosition() {
var tag = this.getQualifiedLanguageString();
var result = "ltr";
// Apart from the exception list, all other arab numeral systems have the degree symbol to the left of the number.
if (this.isRtlNumberFormat() && (tag !== "fa" && tag !== "sd-arab" && tag !== "pa-arab")) {
result = "rtl";
}
return result;
},
_percentFormatter: null,
percentFormatter: {
get: function get() {
if (!this._percentFormatter) {
this._percentFormatter = new Windows.Globalization.NumberFormatting.PercentFormatter();
this._percentFormatter.fractionDigits = 0;
}
return this._percentFormatter;
}
},
/// <summary>
/// Appends the '%' symbol based on user's market settings
/// </summary>
/// <param name="val">the input number</param>
/// <returns>the input number appended with '%' symbol</returns>
formatPercent: function formatPercent(val) {
return this.percentFormatter.format(val / 100);
},
/// <summary>
/// Formats the last updated time in short date time format
/// </summary>
/// <param name="year">last updated year</param>
/// <param name="month">last updated month</param>
/// <param name="date">last updated date</param>
/// <param name="hours">last updated hours</param>
/// <param name="minutes">last updated minutes</param>
/// <param name="secs">last updated seconds</param>
/// <param name="formatPattern">format string for the date-time</param>
/// <returns>Formatted last updated time string</returns>
getFormattedLastUpdateTimeShort: function getFormattedLastUpdateTimeShort(year, month, date, hour, minutes, secs, formatPattern) {
var dateToFormat = new Date(year, month, date, hour, minutes, secs);
var dateTimeFormatter = Windows.Globalization.DateTimeFormatting.DateTimeFormatter;
formatPattern = formatPattern || this._formatPattern;
return formatPattern.format(dateTimeFormatter.shortDate.format(dateToFormat), dateTimeFormatter.shortTime.format(dateToFormat));
},
/// <summary>
/// Formats the last updated time
/// </summary>
/// <param name="year">last updated year</param>
/// <param name="month">last updated month</param>
/// <param name="date">last updated date</param>
/// <param name="hours">last updated hours</param>
/// <param name="minutes">last updated minutes</param>
/// <param name="secs">last updated seconds</param>
/// <returns>Formatted last updated time string</returns>
getFormattedLastUpdateTime: function getFormattedLastUpdateTime(year, month, date, hour, minutes, secs) {
var dateToFormat = new Date(year, month, date, hour, minutes, secs);
var dateTimeFormatter = Windows.Globalization.DateTimeFormatting.DateTimeFormatter;
var formatPattern = this._formatPattern;
return formatPattern.format(dateTimeFormatter.longDate.format(dateToFormat), dateTimeFormatter.longTime.format(dateToFormat));
},
/// <summary>
/// Formats the display hour for hourly cluster
/// </summary>
/// <param name="hourlyTime">timestamp string</param>
/// <returns>Formatted time string</returns>
getFormattedHourlyTime: function getFormattedHourlyTime(hourlytime) {
var jsDate = this._convertKifToJSDate(hourlytime);
return this.hourFormatter.format(jsDate);
},
getFormattedDailyTime: function getFormattedDailyTime(dailyTime) {
var formattedDateTime = "";
try {
var jsDate = this._convertKifToJSDate(dailyTime);
if (this.dayFormatReversed) {
formattedDateTime = this._formatPattern.format(this.dayFormatter.format(jsDate), this.dayOfWeekFormatter.format(jsDate));
}
else {
formattedDateTime = this._formatPattern.format(this.dayOfWeekFormatter.format(jsDate), this.dayFormatter.format(jsDate));
}
}
catch (ex) {
}
return formattedDateTime;
},
getFormattedDailyTimeShort: function getFormattedDailyTimeShort(dailyTime) {
var formattedDateTime = "";
try {
var jsDate = this._convertKifToJSDate(dailyTime);
if (this.dayFormatReversed) {
formattedDateTime = this._formatPattern.format(this.dayFormatter.format(jsDate), this.dayOfWeekFormatterShort.format(jsDate));
}
else {
formattedDateTime = this._formatPattern.format(this.dayOfWeekFormatterShort.format(jsDate), this.dayFormatter.format(jsDate));
}
}
catch (ex) {
}
return formattedDateTime;
},
getFormattedYearFromKifTime: function getFormattedYearFromKifTime(kifTime) {
var jsDate = this._convertKifToJSDate(kifTime);
var result = this.yearFormatter.format(jsDate);
return result;
},
getFullDateTimeString: function getFullDateTimeString(kifTime, formatPattern) {
var dateTimeFormatter = Windows.Globalization.DateTimeFormatting.DateTimeFormatter;
formatPattern = formatPattern || this._formatPattern;
var jsDate = this._convertKifToJSDate(kifTime);
return formatPattern.format(dateTimeFormatter.longDate.format(jsDate), dateTimeFormatter.longTime.format(jsDate));
},
getFormattedMapSnapTime: function getFormattedMapSnapTime(kifTime, formatPattern) {
var dateTimeFormatter = Windows.Globalization.DateTimeFormatting.DateTimeFormatter;
formatPattern = formatPattern || this._formatPattern;
var jsDate = this._convertKifToJSDate(kifTime);
var result = formatPattern.format(dateTimeFormatter.longDate.format(jsDate), dateTimeFormatter.shortTime.format(jsDate));
return result;
},
getShortDateTimeString: function getShortDateTimeString(val, isKifTime) {
var dateTimeFormatter = Windows.Globalization.DateTimeFormatting.DateTimeFormatter;
var formatPattern = this._formatPattern;
var jsDate;
if (isKifTime) {
jsDate = this._convertKifToJSDate(val);
}
else {
jsDate = new Date(val);
}
return formatPattern.format(dateTimeFormatter.shortDate.format(jsDate), dateTimeFormatter.shortTime.format(jsDate));
},
getFormattedDayName: function getFormattedDayName(kifTime) {
var jsDate = this._convertKifToJSDate(kifTime);
return this.dayFormatter.format(jsDate) + "|" + this.dayOfWeekFormatterShort.format(jsDate);
},
getFormattedForecastClusterKey: function getFormattedForecastClusterKey(kifTime) {
var jsDate = this._convertKifToJSDate(kifTime);
return this.forecastClusterKeyFormatter.format(jsDate);
},
getFormattedForecastSkiClusterKey: function getFormattedForecastSkiClusterKey(jsTime) {
var jsDate = new Date(jsTime);
var formattedDateTime = "";
try {
if (this.dayFormatReversed) {
formattedDateTime = this._formatPattern.format(this.dayFormatter.format(jsDate), this.dayOfWeekFormatterShort.format(jsDate));
}
else {
formattedDateTime = this._formatPattern.format(this.dayOfWeekFormatterShort.format(jsDate), this.dayFormatter.format(jsDate));
}
} 
catch (ex) {
}
return formattedDateTime;
},
getDayOfYear: function getDayOfYear(kifTime) {
var jsDate = this._convertKifToJSDate(kifTime);
var dayOne = new Date(jsDate.getFullYear(), 0, 1);
return Math.ceil((jsDate - dayOne) / 86400000);
},
// get the year from kifTime, no globalization required since this is for internal use
getYearFromKifTime: function getYearFromKifTime(kifTime) {
var jsDate = this._convertKifToJSDate(kifTime);
return jsDate.getFullYear();
},
getDistanceUnit: function getDistanceUnit(val, displayUnit) {
var formattedVal = this.formatInt(val);
var distUnit = (displayUnit === "F") ? PlatformJS.Services.resourceLoader.getString('UnitInMiles').format(formattedVal) : PlatformJS.Services.resourceLoader.getString('UnitInKilometres').format(formattedVal);
return distUnit;
},
getSpeedUnit: function getSpeedUnit(val, displayUnit) {
var formattedVal = this.formatInt(val);
var speedUnit = (displayUnit === "F") ? PlatformJS.Services.resourceLoader.getString('MilesPerHour').format(formattedVal) : PlatformJS.Services.resourceLoader.getString('KilometresPerHour').format(formattedVal);
return speedUnit;
},
getSpeedUnitAndDirection: function getSpeedUnitAndDirection(val, direction, displayUnit) {
var formattedVal = this.formatInt(val);
var speedUnit = (displayUnit === "F") ? PlatformJS.Services.resourceLoader.getString('SpeedUnitsInMPHWithDirection').format(direction, formattedVal) : PlatformJS.Services.resourceLoader.getString('SpeedUnitsInKMPHWithDirection').format(direction, formattedVal);
return speedUnit;
},
getPressureUnit: function getPressureUnit(val, displayUnit) {
var formattedVal = this.formatInt(val);
var lenUnit = (displayUnit === "F") ? PlatformJS.Services.resourceLoader.getString('UnitInInches').format(formattedVal) : PlatformJS.Services.resourceLoader.getString('UnitInMillibars').format(formattedVal);
return lenUnit;
},
getLengthUnitForHW: function getLengthUnitForHW(val, displayUnit) {
var formattedVal = this.formatInt(val);
var lenUnit = (displayUnit === "F") ? PlatformJS.Services.resourceLoader.getString('UnitInInches').format(formattedVal) : PlatformJS.Services.resourceLoader.getString('UnitInCentimetres').format(formattedVal);
return lenUnit;
},
getTemperatureWithDegreeUnit: function getTemperatureWithDegreeUnit(val) {
var formattedVal = this.formatInt(val);
var tempValue = PlatformJS.Services.resourceLoader.getString('TemperatureWithDegreeUnit').format(formattedVal);
return tempValue;
},
getDaysUnit: function getDaysUnit(val) {
var daysValue = '';
var formattedVal = this.formatInt(val);
if (val === 1) {
daysValue = PlatformJS.Services.resourceLoader.getString('UnitInDay').format(formattedVal);
} else {
daysValue = PlatformJS.Services.resourceLoader.getString('UnitInDays').format(formattedVal);
}
return daysValue;
},
getHoursPerDayUnit: function getHoursPerDayUnit(val) {
var hoursPerDayValue = '';
var formattedVal = this.formatInt(val);
if (val > 1) {
hoursPerDayValue = PlatformJS.Services.resourceLoader.getString('UnitInHoursPerDay').format(formattedVal);
} else {
hoursPerDayValue = PlatformJS.Services.resourceLoader.getString('UnitInHourPerDay').format(formattedVal);
}
return hoursPerDayValue;
},
getPercentageUnit: function getPercentageUnit(val) {
return this.formatPercent(val);
},
getPercentageValue: function getPercentageValue(val,totVal) {
var pctVal = Math.round(( parseInt(val) / parseInt(totVal)) * 100);            
return pctVal;
},
getBarFill: function getBarFill(ArrStar) {
ArrStar.sort(function (a, b) { return b - a; });
var ArrColor = ["data firstColor", "data secondColor", "data thirdColor", "data fourthColor"];
var arrBarColor = [];
for (var i = 0; i < ArrStar.length; i++) {
arrBarColor[ArrStar[i]] = ArrColor[i];
if (ArrStar[i] === ArrStar[i + 1]) {
i++;
}
}            
return arrBarColor;
},
getBarHeight: WinJS.Binding.converter(function (ht) {
ht = (ht / 100) * 145;
ht = ht + "px";
return ht;          
}),
constructBarHeight: WinJS.Binding.converter(function (ht) {
ht = 145 - ((ht / 100) * 145);
ht = ht + "px 0 0 0";
return ht;
}),
// format hourly row data into a format understood by the ux template
formatHourlyConditions: function formatHourlyConditions(hourlyData) {
var formattedHourlyData = {};
formattedHourlyData.time = WeatherAppJS.Utilities.Formatting.getFormattedHourlyTime(hourlyData.Time);
var caption = hourlyData.ProviderCaption;
if (!caption) {
caption = hourlyData.Caption;
}
if (caption) {
var hrCaption = PlatformJS.Services.resourceLoader.getString(caption);
formattedHourlyData.caption = hrCaption ? hrCaption : "";
} else {
formattedHourlyData.caption = "";
}
var formatting = WeatherAppJS.Utilities.Formatting;
var common = WeatherAppJS.Utilities.Common;
var noDataString = common.getNoDataString();
formattedHourlyData.temperature = (hourlyData.Temp !== undefined && hourlyData.Temp !== null) ? formatting.getTemperatureWithDegreeUnit(hourlyData.Temp) : noDataString;
formattedHourlyData.temperatureWithoutUnit = (hourlyData.Temp !== undefined && hourlyData.Temp !== null) ? formatting.formatInt(hourlyData.Temp) : noDataString;
var safeIconCode = common.getSafeIconCode(hourlyData.IconCode);
formattedHourlyData.skycode = "/images/skycodes/48x48/" + safeIconCode + '.png';
formattedHourlyData.precipitation = (hourlyData.PrecipChance !== undefined && hourlyData.PrecipChance !== null) ? formatting.getPercentageUnit(hourlyData.PrecipChance) : noDataString;
formattedHourlyData.feelslike = (hourlyData.FeelsLike !== undefined && hourlyData.FeelsLike !== null) ? formatting.getTemperatureWithDegreeUnit(hourlyData.FeelsLike) : noDataString;
formattedHourlyData.wind = (hourlyData.WindSpeed !== undefined && hourlyData.WindSpeed !== null) ? hourlyData.WindSpeed : noDataString;
return formattedHourlyData;
},
// format snow unit as per ux
formatSnowOpenLabel: function formatSnowOpenLabel(displayUnit) {
var label;
var resourceLoader = PlatformJS.Services.resourceLoader;
label = (displayUnit === "F") ? resourceLoader.getString("AcresOpen") : resourceLoader.getString("HectaresOpen");
return label;
},
// format snow unit as per ux
formatSnowUnit: function formatSnowUnit(displayUnit) {
var unit;
var resourceLoader = PlatformJS.Services.resourceLoader;
unit = (displayUnit === "F") ? resourceLoader.getString('UnitInInches').format("") : resourceLoader.getString('UnitInCentimetres').format("");
return unit;
},
// format snow data as per ux
formatSnowData: function formatSnowData(todaySnowData, snowData, displayUnit) {
var formattedSnowData = {};
var listindex = 0;
var constSnowData = "<span class='snowdaysGlyph'></span><span class='snowdaysText'>{0}</span>";
if (snowData) {
for (var index = 0; index < snowData.length; index++) {
var item = {};
item.freshSnow = (snowData[index].snfl !== undefined) ? constSnowData.format(WeatherAppJS.Utilities.Formatting.getLengthUnitForHW(snowData[index].snfl, displayUnit)) : '';
item.date = WeatherAppJS.Utilities.Formatting.getFormattedForecastSkiClusterKey(snowData[index].date);
formattedSnowData[listindex] = item;
listindex++;
}
}
return formattedSnowData;
},
// Visibity Convertor where object are need not to be displayed.
visibilityConverter: WinJS.Binding.converter(function (obj) {
if (!(obj === null || typeof obj === 'undefined' || obj === "")) {
return 'visible';
}
else {
return 'hidden';
}
}),
displayConverter: WinJS.Binding.converter(function (obj) {
if (!(obj === null || typeof obj === 'undefined' || obj === "")) {
return 'block';
}
else {
return 'none';
}
}),
displayInlineConverter: WinJS.Binding.converter(function (obj) {
if (!(obj === null || typeof obj === 'undefined' || obj === "")) {
return 'inline';
}
else {
return 'none';
}
}),
resortStatusConverter: WinJS.Binding.converter(function(resortStatus) {
var className = "resortStatus ";
// TODO: localization. Server needs to send some Id to be able to compare against.
if (resortStatus !== WeatherAppJS.Utilities.Common.getNoDataString()) {
if (resortStatus === "Open") {
className += "open";
}
else if (resortStatus === "WeekendsOnly") {
className += "open weekendOnly";
}
else if (resortStatus === "TemporarilyClosed" || resortStatus === "TempClosed") {
className += "closed tempClosed";
}
else {
className += "closed";
}
}
return className;
}),
resortStatusStringConverter: WinJS.Binding.converter(function (resortStatus) {
var resStatus = resortStatus;
if (resortStatus !== WeatherAppJS.Utilities.Common.getNoDataString()) {
if (resortStatus === "TempClosed") {
resStatus = PlatformJS.Services.resourceLoader.getString(resortStatus);
} else {
resStatus = PlatformJS.Services.resourceLoader.getString("ResortStatus" + resortStatus);
}
}
return resStatus;
}),
// format elevation for trails data and returns with proper unit
// also removing data after decimal points
getElevationUnit: function getElevationUnit(val, displayUnit) {
var unit;
var resourceLoader = PlatformJS.Services.resourceLoader;
if (val) {
val = this.formatInt(Math.floor(val));
}
try {
var unitContainer = "{0} <span>{1}</span>";
unit = (displayUnit === "F") ? unitContainer.format(val, resourceLoader.getString("Feet")) : unitContainer.format(val, resourceLoader.getString("Metres"));
} catch (err) {
}
return unit;
},
// formatted Review Date usDailyConditionsed in SkiDetails
getFormattedReviewDate: function getFormattedReviewDate(val) {
var jsDate = new Date(val);
return this.reviewDateFormatter.format(jsDate);
},
// Used to make sure that external links have proper schema so that the app does not crash
getSanitizedUrl: function getSanitizedUrl(url) {
if (url) {
var staticURL = toStaticHTML(url);
if (staticURL.indexOf("http://") !== 0) {
staticURL = "http://" + staticURL;
}
return staticURL;
}
return null;
},
});
WinJS.Namespace.define("WeatherAppJS.Utilities.TabIndexManager", {
setTabIndexOnDailyConditions: function setTabIndexOnDailyConditions(parentNode, isPageLoad) {
var tm = WeatherAppJS.Utilities.TabIndexManager;
var lWrapperClass = '.dailyModuleWrapper';
var lEachForecastNodeClass = '.dailyModuleFirstProvider';
var pWrapperClass = '.portraitMultiProviderWrapper';
var pEachForecastNodeClass = '.pDailyModuleFirstProvider';
// on page load, set indexes for both landscape and portrait, so that on every layout change we need not
if (isPageLoad) {
tm._setTabIndexes(parentNode, lWrapperClass, lEachForecastNodeClass);
tm._setTabIndexes(parentNode, pWrapperClass, pEachForecastNodeClass);
} else {
var layoutState = Windows.UI.ViewManagement.ApplicationView.value;
// for portrait
if (layoutState === Windows.UI.ViewManagement.ApplicationViewState.fullScreenPortrait) {
tm._setTabIndexes(parentNode, pWrapperClass, pEachForecastNodeClass);
} else {
tm._setTabIndexes(parentNode, lWrapperClass, lEachForecastNodeClass);
}
}
},
handleArrowKeyForAccessibility: function handleArrowKeyForAccessibility(event, /*@optional*/xAxis) {
if (!event || !event.srcElement) {
return;
}
function focusItem(elem) {
if (elem) {
event.srcElement.tabIndex = -1;
elem.tabIndex = 0;
elem.focus();
}
}
var isRTL = (WeatherAppJS.Utilities.Common.getDirection() === 'rtl') ? true : false;
var keyCode = event.keyCode;
// if keycode is one of the below, do not allow the event to propagate, the rest should pass through
if (keyCode === WinJS.Utilities.Key.rightArrow || 
keyCode === WinJS.Utilities.Key.leftArrow || 
keyCode === WinJS.Utilities.Key.upArrow || 
keyCode === WinJS.Utilities.Key.downArrow || 
keyCode === WinJS.Utilities.Key.pageUp || 
keyCode === WinJS.Utilities.Key.pageDown) {
event.stopImmediatePropagation();
event.preventDefault();
}
if (xAxis) { // for the DOM sub-tree where elements are horizontally arranged
switch (keyCode) {
case WinJS.Utilities.Key.rightArrow: {
var nextElem = isRTL ? event.srcElement.previousElementSibling : event.srcElement.nextElementSibling;
if (!nextElem) {
CommonJS.NavigableView.dispatchBoundaryEvent(event.srcElement, CommonJS.NavigableView.Direction.RIGHT);
} else {
focusItem(nextElem);
}
}
break;
case WinJS.Utilities.Key.leftArrow: {
var nextElement = isRTL ? event.srcElement.nextElementSibling : event.srcElement.previousElementSibling;
if (!nextElement) {
CommonJS.NavigableView.dispatchBoundaryEvent(event.srcElement, CommonJS.NavigableView.Direction.LEFT);
} else {
focusItem(nextElement);
}
}
break;
case WinJS.Utilities.Key.downArrow:
case WinJS.Utilities.Key.pageDown:
CommonJS.NavigableView.dispatchBoundaryEvent(event.srcElement, CommonJS.NavigableView.Direction.RIGHT);
break;
case WinJS.Utilities.Key.upArrow:
case WinJS.Utilities.Key.pageUp:
CommonJS.NavigableView.dispatchBoundaryEvent(event.srcElement, CommonJS.NavigableView.Direction.LEFT);
break;
}
} else { // for the DOM sub-tree where elements are vertically arranged
switch (keyCode) {
case WinJS.Utilities.Key.downArrow:
focusItem(event.srcElement.nextElementSibling);
break;
case WinJS.Utilities.Key.upArrow:
focusItem(event.srcElement.previousElementSibling);
break;
case WinJS.Utilities.Key.rightArrow:
case WinJS.Utilities.Key.pageDown:
CommonJS.NavigableView.dispatchBoundaryEvent(event.srcElement, CommonJS.NavigableView.Direction.RIGHT);
break;
case WinJS.Utilities.Key.leftArrow:
case WinJS.Utilities.Key.pageUp:
CommonJS.NavigableView.dispatchBoundaryEvent(event.srcElement, CommonJS.NavigableView.Direction.LEFT);
break;
}
}
},
_setTabIndexes: function _setTabIndexes(parentNode, wrapperIdentifier, forecastNodesIdentifier) {
var dailyConditionsWrapper = parentNode.querySelector(wrapperIdentifier);
if (dailyConditionsWrapper) {
var isScrolled = WeatherAppJS.Utilities.UI._isDailyForecastScrolled;
// Total number of daily conditions
var dailyConditionsAll = dailyConditionsWrapper.querySelectorAll(forecastNodesIdentifier);
var dailyConditionsViewCount = WeatherAppJS.Utilities.UI._totalDailyForecastsShown;
var startIndex = 0;
var endIndex = (dailyConditionsAll.length < dailyConditionsViewCount) ? dailyConditionsAll.length : dailyConditionsViewCount;
if (isScrolled) {
endIndex = dailyConditionsAll.length;
startIndex = endIndex - dailyConditionsViewCount;
}
for (var hiddenIndex = 0, len = dailyConditionsAll.length; hiddenIndex < len; hiddenIndex++) {
dailyConditionsAll[hiddenIndex].setAttribute("tabIndex", "-1");
}
for (var index = startIndex; index < endIndex; index++) {
dailyConditionsAll[index].setAttribute("tabIndex", "0");
}
}
}
});
WinJS.Namespace.define("WeatherAppJS.Storage.Memory", {
_dataStore: {},
cacheItem: function cacheItem(key, value) {
this._dataStore[key] = value;
},
getItem: function getItem(key) {
if (this._dataStore[key]) {
return this._dataStore[key];
}
return null;
},
removeItem: function removeItem(key) {
if (this._dataStore[key]) {
delete this._dataStore[key];
}
},
});
WinJS.Namespace.define("WeatherAppJS.Instrumentation.PageContext", {
AddLocFlyout : "Add Location Flyout",
ConfirmLocFlyout: "Confirm Location Flyout",
SkiTrailMap: "/Ski Resorts/Trail Map",
SkiOverviewOTS: "/Ski Resorts/OTS Overview Site",
SkiResortWebsite: "/Ski Resorts/Website",
SkiResortSlideshow: "/Ski Resorts/Slideshow",
SkiResort360Pano: "/Ski Resorts/360Pano",
SkiResortNews: "/Ski Resorts/Deals and News",
SkiResortReview: "/Ski Resorts/Users Review",
ArticleReader: "/Article Reader",
Home: "/Home",
DayDrilldown: "/Day Drilldown",
CityDetails: "/City Details",
SkiResorts: "/Ski Resorts",
IFramePage: "/IFrame Page",
MyPlaces: "/My Places",
PhotoSynth: "/360Pano",
SearchDisambiguation: "/Search Disambiguation",
SkiResortsECV: "/Ski Resorts ECV",
SlideShow: "/Slideshow",
Snap: "/Snap",
WorldWeather: "/World Weather",
PartnerPano: "/Partner Pano",
WeatherMapsChrome: "Weather Maps Chrome",
WeatherMapsChromeTopBar: "Weather Maps Chrome top-bar",
DailyForecast: "Daily Forecast",
SkiOverview: "Ski Resorts Overview",
MessageBar: "Message Bar",
Favorites: "Favorites",
Recent: "Recent"
});
WinJS.Namespace.define("WeatherAppJS.Instrumentation.FormCodes", {
AppLaunch: "APXW02",
Search: "APXW03",
PinnedTile: "APXW04",
MyPlacesTile: "APXW05",
TopEdgy: "APXW06",
Default: "APXW01"
});
})();
