﻿/********************************************************
*                                                        *
*   © Microsoft. All rights reserved.                    *
*                                                        *
*********************************************************/
(function SkiResortsInit() {
"use strict";
WinJS.Namespace.define("WeatherAppJS.SkiResorts", {
EntityCollection: WinJS.Class.derive(CommonJS.EntityCollection.Collection, function entityCollection_ctor(options) {
msWriteProfilerMark("WeatherApp:SkiResorts:PageLoad:s");
CommonJS.EntityCollection.Collection.call(this, options);
var that = this;
this._options = options || {};
this._data = [];
this._cacheInstance = PlatformJS.Cache.CacheService.getInstance("ECVProcessedDataCache");
msWriteProfilerMark("WeatherApp:SkiResorts:PageLoad:e");
this.impressionContext = WeatherAppJS.Instrumentation.PageContext.SkiResortsECV;
},
{
_tempunit: null,
_userRegion: null,
_processedData: null,
_cacheInstance: null,
/*CommonJS.EntityCollection.Collection Members*/
Initialize: function Initialize(config) {
msWriteProfilerMark("WeatherApp:SkiResorts:Initialize:s");
CommonJS.setTheme("skiECVBackground", true);
config = config || {};
this.UIConfig = new WeatherAppJS.SkiResorts.UIConfig({ collection: this });
this.DataConfig = config.DataConfig || new CommonJS.EntityCollection.DataConfig();
this.Provider = WeatherAppJS.SkiResorts._provider;
var unit = this._tempunit = WeatherAppJS.SettingsManager.getDisplayUnit();
var isoCode = this._userRegion = WeatherAppJS.SettingsManager.getUserDefaultRegionISOCode();
var providerConfiguration = PlatformJS.Collections.createStringDictionary();
providerConfiguration.insert("unit", unit);
providerConfiguration.insert("userRegion", isoCode);
return this.Provider.initializeAsync(providerConfiguration);
},
_getCacheKey: function _getCacheKey() {
if (WeatherAppJS.SkiResorts._isFirstLaunch) {
var prefix = 'pd';
var lang = Platform.Globalization.Marketization.getQualifiedLanguageString();
var isNearbyEnabled = Platform.Utilities.Globalization.isFeatureEnabled("CurrentLocationDetection"); // required so that we do not accidentally show SearchType filter when it is not enabled
var args = [prefix, this._tempunit, this._userRegion, lang, isNearbyEnabled];
return args.join('_');
}
return null;
},
_updateCacheIfRequired: function _updateCacheIfRequired(dataString) {
var that = this;
var cacheKey = this._getCacheKey();
if (dataString && cacheKey && WeatherAppJS.SkiResorts._isFirstLaunch) {
WeatherAppJS.SkiResorts._isFirstLaunch = false;
this._cacheInstance.findEntry(cacheKey).then(
function (cachedData) {
if (cachedData && cachedData.dataValue !== dataString) {
that._cacheInstance.addEntry(cacheKey, dataString).done();
that.invalidateView();
} else if (!cachedData) {
that._cacheInstance.addEntry(cacheKey, dataString).done();
}
},
function (err) {
// do nothing
});
}
},
_preprocessFilters: function _preprocessFilters(filters) {
// When filter bar items change,  CUX does not update the filter bar DOM. so sending less filters does not ensure that the DOM is cleaned for the existing filters
// this is a workaround for the same.
var filterBarElem = document.getElementById("filterBar");
if (filterBarElem && filterBarElem.winControl) {
var filterBar = filterBarElem.winControl;
if (filterBar && filterBar._filterBar) {
filterBar._filterBar.dispose();
filterBar._filterBar.filterGroups = {};
filterBar._filterBar.initialize();
}
}
filters = filters || [];
for (var i = 0, l = filters.length; i < l; i++) {
// Bug fix 534872 - if region filter has only one item; it is not supposed to be displayed
// removing this filter as the result dataset is not affected by it for this scenario
if (filters[i].attribute === "Region" && filters[i].items.length === 1) {
filters.splice(i, 1);
}
}
return filters;
},
_showErrorMessage: function _showErrorMessage(filters)
{
msSetImmediate(function () {
var appBar = document.querySelector(".entityappbar");
if (appBar && appBar.winControl) {
appBar.winControl.disabled = false;
}
});
if (WeatherAppJS.Networking.internetAvailable === false) {
WeatherAppJS.Error.showMessageBar(PlatformJS.Services.resourceLoader.getString("NoInternetConnectionError"), false);
} else {
for (var f in filters) {
if (filters[f].attribute === "SearchType") {
for (var fi in filters[f].items) {
if (filters[f].items[fi].value === "Nearby" && filters[f].items[fi].selected === true) {
if (this.Provider.isLocationDetectionDisabled()) {
WeatherAppJS.Error.showMessageBar(PlatformJS.Services.resourceLoader.getString("LocationDisabledError"), false);
}
return;
}
}
}
}
}
},
onBeforeQuery: function onBeforeQuery(query) {
var that = this;
var cacheKey = this._getCacheKey();
return CommonJS.EntityCollection.Collection.prototype.onBeforeQuery.call(this).then(
function () {
if (!cacheKey) {
that._processedData = null;
return WinJS.Promise.wrap(true);
}
return that._cacheInstance.findEntry(cacheKey).then(
function (cachedData) {
if (cachedData) {
that._processedData = cachedData.dataValue;
}
}, function (err) {
that._processedData = null;
});
});
},  
onQueryReturned: function onQueryReturned(entities, sorts, groups, filters) {
if (!entities || (entities instanceof Array && entities.length === 0)) {
this._showErrorMessage(filters);
}
msWriteProfilerMark("WeatherApp:SkiResorts:Initialize:e");
msWriteProfilerMark("WeatherApp:SkiResorts:OnQueryReturned:s");
return CommonJS.EntityCollection.Collection.prototype.onQueryReturned.call(this).then(function () {
msWriteProfilerMark("WeatherApp:SkiResorts:OnQueryReturned:e");
});
},
Query: function Query(sort, filters) {
var that = this;
var isQueryComplete = false;
var logStr = "Query(): Selected Sort: " + (sort ? sort.identifier : " none") + ", Selected Filters: ";
var p = new WinJS.Promise(function collection_queryPromiseInit(complete, error, progress) {
var onQueryUpdate = function (jsonStr) {
var result = {};
var dataset = null;
try {
dataset = JSON.parse(jsonStr);
jsonStr = null;
}
catch (e) {
error(e);
return;
}
var e = dataset.entities;
that.Sorts = dataset.sorts;
var g = that.Groups = dataset.groups;
that.Filters = that._preprocessFilters(dataset.filters);
var groupsDictionary = {};
for (var i = 0; i < g.length; i++) {
var key = g[i].identifier;
groupsDictionary[key] = g[i].count;
}
for (var j = 0; j < e.length; j++) {
e[j].groupCount = groupsDictionary[e[j].group];
}
try {
// allow caller to enrich dataset on the javascript side
that.onQueryReturned(e, that.Sorts, that.Groups, that.Filters).done(function collection_onQueryReturnedComplete() {
that.Entities = e;
if (isQueryComplete) {
if (that.onQueryComplete) {
that.onQueryComplete(e);
}
complete();
} else {
progress();
}
});
}
catch (e) {
error(e);
}
};
var onQueryProgress = function (dataset) {
isQueryComplete = false;
onQueryUpdate(dataset);
};
var onQueryComplete = function (dataset) {
isQueryComplete = true;
msWriteProfilerMark("CommonControls:EntityCollection:providerQuery:e");
onQueryUpdate(dataset);
};
var onDeferedQueryComplete = function (dataset) {
if (!isQueryComplete) {
isQueryComplete = true;
msWriteProfilerMark("CommonControls:EntityCollection:providerQuery:e");
onQueryUpdate(dataset);
}
that._updateCacheIfRequired(dataset);
};
var q = new AppEx.Common.EntityCollection.Query.EntityQuery();
if (filters) {
for (var ii = 0; ii < filters.length; ii++) {
q.filters.append(filters[ii]);
logStr += "[" + filters[ii].attribute + ":";
for (var jj = 0; jj < filters[ii].value.length; jj++) {
var val = filters[ii].value[jj];
logStr += val + ";";
}
logStr += "] ";
}
}
if (sort && sort.identifier) {
q.sortIdentifier = sort.identifier;
q.sortDirection = sort.sortDirection;
}
//  console.log(logStr);
// allow caller to modify query before sending to provider
that.onBeforeQuery(q).done(function collection_onBeforeQueryComplete() {
msWriteProfilerMark("CommonControls:EntityCollection:providerQuery:s");
var jsonShim = new AppEx.Common.EntityCollection.JSONCollectionProvider(that.Provider);
if (!that._processedData) {
jsonShim.queryAsync(q).done(onDeferedQueryComplete, error, onQueryProgress);
} else {
onQueryComplete(that._processedData);
jsonShim.queryAsync(q).done(onDeferedQueryComplete);
}
});
});
return WinJS.Promise.wrap(p);
},
}),
_provider: new AppEx.WeatherApp.Services.SkiResorts.SkiResortsProvider(),
_isFirstLaunch: true,
});
})();
(function () {
WinJS.Namespace.define("WeatherAppJS.SkiResorts", {
UIConfig: WinJS.Class.derive(CommonJS.EntityCollection.UIConfig, function (options) {
CommonJS.EntityCollection.UIConfig.call(this, options);
this._collection = options.collection;
this._bindableSettingChanged = this.onSettingsChanged.bind(this);
WeatherAppJS.SettingsManager.addEventListener('settingchanged', this._bindableSettingChanged, false);
}, {
// current instance of WeatherAppJS.SkiResorts.EntityCollection
_collection: null,
OnItemClicked: function OnItemClicked(entity) {
var geocodeLocation = WeatherAppJS.LocationSuggest.getSkiResortGeocodeLocation(entity);
var eventAttr = { resort: geocodeLocation.getFullDisplayName(), index: entity.index };
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, "Ski Resorts ECV", "Ski Resort", Platform.Telemetry.UserActionOperation.click, Platform.Telemetry.UserActionMethod.unknown, 0, JSON.stringify(eventAttr));
WinJS.Navigation.navigate({ fragment: "/panoramas/home/home.html", page: "WeatherAppJS.Pages.Home" }, { locID: geocodeLocation.getId(), geocodeLocation: geocodeLocation });
},
GetViewOptions: function GetViewOptions(view) {
var options = {};
options.disableSemanticZoom = true;
return options;
},
GetSupportedViews: function GetSupportedViews() {
return {
list: [CommonJS.EntityCollection.Views.List],
defaultView: CommonJS.EntityCollection.Views.List
};
},
SetFilterType: function SetFilterType(filter) {
var resourceLoader = PlatformJS.Services.resourceLoader;
if (filter.attribute === "Country") {
filter.selectionMode = "Single";
}
if (filter.attribute === "SearchType") {
filter.selectionMode = "Single";
filter.useWideTile = true;
filter.name = resourceLoader.getString(filter.name);
if (filter.items) {
for (var i = 0, l = filter.items.length; i < l; i++) {
filter.items[i].label = resourceLoader.getString(filter.items[i].label);
}
}
}
if (filter.attribute === "Within") {
var weatherFormatting = WeatherAppJS.Utilities.Formatting;
var displayUnit = WeatherAppJS.SettingsManager.getDisplayUnit();
filter.selectionMode = "Single";
filter.useWideTile = true;
filter.name = resourceLoader.getString(filter.name);
if (filter.items) {
for (var ii = 0, ll = filter.items.length; ii < ll; ii++) {
filter.items[ii].label = weatherFormatting.getDistanceUnit(filter.items[ii].label, displayUnit);
}
}
}
},
forceRefresh: function forceRefresh(bypassCache) {
if (bypassCache) {
this._collection.Provider.forceRefresh();
}
WeatherAppJS.SettingsManager.removeEventListener('settingchanged', this._bindableSettingChanged, false);
CommonJS.forceRefresh();
},
GetAppBarItems: function GetAppBarItems(controller) {
/*must return an array of CommonJS.Button options objects controller is passed in so consumer can grab handles to entities, sort, fitler, etc in their callbacks*/
// Creating buttons for skiResorts
var that = this;
var buttons = [];
var onHelpButtonClicked = function () {
WeatherAppJS.Utilities.UI.openInAppHelp();
};
/// <summary>
/// method to call when change-to-Fahrenheit button on bottom edgy is clicked
/// </summary>
var onChangeUnitToFClick = function () {
CommonJS.dismissAllEdgies();
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NumBottomEdgyUnitChanged");
WeatherAppJS.SettingsManager.setDisplayUnitAsync('F', Platform.Telemetry.ActionContext.bottomEdgy).then();
};
/// <summary>
/// method to call when change-to-Celsius button on bottom edgy is clicked
/// </summary>
var onChangeUnitToCClick = function () {
CommonJS.dismissAllEdgies();
WeatherAppJS.Utilities.Instrumentation.incrementInt32("NumBottomEdgyUnitChanged");
WeatherAppJS.SettingsManager.setDisplayUnitAsync('C', Platform.Telemetry.ActionContext.bottomEdgy).then();
};
var onRefreshButtonClicked = function () {
that.forceRefresh(true);
CommonJS.dismissAllEdgies();
};
var appBar = document.querySelector(".entityappbar");
var multiSelect;
if (!appBar.querySelector("#changeUnitToCButton") ){
multiSelect = {
id: "changeUnitToCButton",
extraClass: 'appexSymbol',
icon: '\uE150',
onclick: onChangeUnitToCClick,
hidden: true,
};
multiSelect.label = PlatformJS.Services.resourceLoader.getString("ChangeUnitToCButtonTitle");
buttons.push(multiSelect);
}
if(!appBar.querySelector("#changeUnitToFButton") ){
multiSelect = {
id: "changeUnitToFButton",
extraClass: 'appexSymbol',
icon: '\uE151',
onclick: onChangeUnitToFClick,
hidden: true,
};
multiSelect.label = PlatformJS.Services.resourceLoader.getString("ChangeUnitToFButtonTitle");
buttons.push(multiSelect);
}
if(!appBar.querySelector("#refreshButton")){
multiSelect = {
id: "refreshButton",
extraClass: 'appexSymbol',
icon: '\uE067',
onclick: onRefreshButtonClicked,
};
multiSelect.label = PlatformJS.Services.resourceLoader.getString("RefreshButtonTitle");
// refresh button added
buttons.push(multiSelect);
}
if (!appBar.querySelector("#helpButton")) {
multiSelect = {
id: "helpButton",
extraClass: 'appexSymbol',
icon: "\uFF60",
onclick: onHelpButtonClicked
};
multiSelect.label = CommonJS.resourceLoader.getString("/platform/HelpLabel");
buttons.push(multiSelect);
}
appBar.addEventListener('beforeshow', function (event) {
if (WeatherAppJS.SettingsManager.getDisplayUnit() === 'F') {
appBar.querySelector("#changeUnitToCButton").winControl.hidden = false;
appBar.querySelector("#changeUnitToFButton").winControl.hidden = true;
}
else {
appBar.querySelector("#changeUnitToFButton").winControl.hidden = false;
appBar.querySelector("#changeUnitToCButton").winControl.hidden = true;
}
});
return buttons;
},
OnClearFiltersClick: function OnClearFiltersClick() {
if (this._collection && this._collection.Provider) {
try {
this._collection.Provider.clearFilters();
}
catch (ex) {}
}
},
onSettingsChanged: function onSettingsChanged(event) {
this.forceRefresh(false);
},
GetTableDefinition: function GetTableDefinition(entities) {
var that = this;
var td = new CommonJS.Table.TableDefinition();
var resourceLoader = PlatformJS.Services.resourceLoader;
var ecvFields = WeatherAppJS.SkiResorts.ECVColumns;
for (var i = 0, len = ecvFields.length; i < len; i++) {
var colDefinition = new CommonJS.Table.ColumnDefinition();
var currentField = ecvFields[i];
colDefinition.ID = currentField.id;
colDefinition.priority = i;
colDefinition.headerText = resourceLoader.getString(currentField.id + "Header");
colDefinition.boundField = currentField.boundField;
colDefinition.sortIdentifier = currentField.sortId;
colDefinition.fieldTemplate = document.getElementById(currentField.fieldTemplateId);
td.columns.push(colDefinition);
}
td.cssWhiteList = ["skiResortsECV.css"];
td.itemSpacing.min = 0;
var weatherCommon = WeatherAppJS.Utilities.Common;
var formatInt = WeatherAppJS.Utilities.Formatting.formatInt;
var displayUnit = WeatherAppJS.SettingsManager.getDisplayUnit();
var tempString = resourceLoader.getString("TemperatureWithDegreeUnit");
var noDataString = weatherCommon.getNoDataString();
var snowUnitString, speedUnitString;
if (displayUnit === "F") {
snowUnitString = resourceLoader.getString('UnitInInches');
speedUnitString = resourceLoader.getString('MilesPerHour');
} else {
snowUnitString = resourceLoader.getString('UnitInCentimetres');
speedUnitString = resourceLoader.getString('KilometresPerHour');
}
var getResortClass = function (st) {
if (st === "Open" || st === "WeekendsOnly") {
return "resortStatus open";
}
return "resortStatus closed";
};
// Direct fill DOM instead of Template binding
td.bindRow = function (rowElem, backingData) {
var index = 0;
var element = null;
var value = null;
// Helper Functions
var getData = function (i) {
return backingData['bf' + i];
};
var getIntOrNoData = function (i) {
var temp = backingData['bf' + i];
if (temp && temp !== '--') {
return formatInt(temp);
}
return noDataString;
};
var columns = rowElem.getElementsByClassName("table-column");
var curColumn = 0;
var rowAriaLabel;
// Name Column
var nameCol = rowElem.getElementsByClassName("nameColContainer")[0];
element = nameCol.getElementsByClassName("skiImage")[0];
element.src = getData(index++);
element = nameCol.getElementsByClassName("nameContainer")[0].getElementsByTagName("div");
// resortName
element[0].innerText = getData(index++);
// resortCity
element[1].innerText = getData(index++);
// resortStatus
value = getData(index++);
element[2].innerText = resourceLoader.getString('ResortStatus' + value);
WinJS.Utilities.addClass(element[2], getResortClass(value));
var ariaText = resourceLoader.getString("NameHeader") + ", " + element[0].innerText + "," + element[1].innerText + "," + element[2].innerText;
that.setAriaLabel(columns[curColumn++], ariaText);
rowAriaLabel = ariaText;
// Forecast Column
var forecastCol = rowElem.getElementsByClassName("forecastColContainer")[0];
element = forecastCol.getElementsByClassName("skyCodeImage")[0];
value = getData(index++);
element.src = "/images/skycodes/48x48/" + weatherCommon.getSafeIconCode(value) + '.png';
value = getIntOrNoData(index++);
var highTemp = (value !== "--") ? tempString.format(value) : value;
value = getIntOrNoData(index++);
var lowTemp = (value !== "--") ? tempString.format(value) : value;
element = forecastCol.getElementsByTagName("span");
element[0].innerText = highTemp;
element[2].innerText = lowTemp;
element = forecastCol.getElementsByClassName("caption")[0];
value = getData(index++);
element.innerText = (value && value !== "") ? resourceLoader.getString(value) : "";
ariaText = resourceLoader.getString("ForecastHeader") + ", " + resourceLoader.getString("HighMapSubType") + ", " + highTemp + ", " + resourceLoader.getString("LowMapSubType") + ", " + lowTemp + ", " + element.innerText;
that.setAriaLabel(columns[curColumn++], ariaText);
rowAriaLabel += ", " + ariaText;
// New Snow Column
element = rowElem.getElementsByClassName("newSnowCol")[0];
value = getIntOrNoData(index++);
element.innerText = (value !== "--") ? snowUnitString.format(value) : value;
ariaText = resourceLoader.getString("NewSnowHeader") + ", " + element.innerText;
that.setAriaLabel(columns[curColumn++], ariaText);
rowAriaLabel += ", " + ariaText;
// Snow Depth Column
element = rowElem.getElementsByClassName("snowDepthCol")[0].getElementsByTagName("span");
var baseDepthLow = getIntOrNoData(index++);
var baseDepthHigh = getIntOrNoData(index++);
if (!(baseDepthLow === WeatherAppJS.Utilities.Common.getNoDataString() && baseDepthHigh === WeatherAppJS.Utilities.Common.getNoDataString())) {
element[0].innerText = baseDepthLow;
element[2].innerText = baseDepthHigh;
element[3].innerText = snowUnitString.format('');
} else {
element[1].innerText = WeatherAppJS.Utilities.Common.getNoDataString();
}
ariaText = resourceLoader.getString("SnowDepthHeader") + ", " + element[0].innerText + ", " + element[2].innerText + ", " + element[3].innerText;
that.setAriaLabel(columns[curColumn++], ariaText);
rowAriaLabel += ", " + ariaText;
// Wind Column
element = rowElem.getElementsByClassName("windCol")[0];
value = getIntOrNoData(index++);
element.innerText = value !== "--" ? speedUnitString.format(value) : value;
ariaText = resourceLoader.getString("WindHeader") + ", " + element.innerText;
that.setAriaLabel(columns[curColumn++], ariaText);
rowAriaLabel += ", " + ariaText;
// Open Trails Column
element = rowElem.getElementsByClassName("openTrailsCol")[0];
element.innerText = getIntOrNoData(index++);
ariaText = resourceLoader.getString("OpenTrailsHeader") + ", " + element.innerText;
that.setAriaLabel(columns[curColumn++], ariaText);
rowAriaLabel += ", " + ariaText;
// Open Lifts Column
element = rowElem.getElementsByClassName("openLiftsCol")[0];
element.innerText = getIntOrNoData(index++);
ariaText = resourceLoader.getString("OpenLiftsHeader") + ", " + element.innerText;
that.setAriaLabel(columns[curColumn++], ariaText);
rowAriaLabel += ", " + ariaText;
that.setAriaLabel(rowElem, rowAriaLabel);
};
return td;
},
getSleepDuration: function getSleepDuration() {
return PlatformJS.Services.appConfig.getString("PageRefreshTime");
},
setAriaLabel: function setAriaLabel(element, ariaText) {
element.setAttribute("role", "listitem");
// Remove quote(s) if any
ariaText = ariaText.replace(/"/g, '');
element.setAttribute("aria-label", ariaText);
},
onNavigateAway: function onNavigateAway() {
WeatherAppJS.SettingsManager.removeEventListener('settingchanged', this._bindableSettingChanged, false);
},
})
});
})();
(function () {
WinJS.Namespace.define("WeatherAppJS.SkiResorts", {
ECVColumns: [
{
id: "Name",
boundField: ["ThumbUrl", "Name", "City", "Status"],
fieldTemplateId: "skiECVNameTemplate",
sortId: "Name"
},
{
id: "Forecast",
boundField: ["SkyCode", "HighTemp", "LowTemp", "Caption"],
fieldTemplateId: "skiECVForecastTemplate",
},
{
id: "NewSnow",
boundField: "NewSnowString",
fieldTemplateId: "skiECVNewSnow",
sortId: "NewSnow"
},
{
id: "SnowDepth",
boundField: ["BaseDepth", "TopDepth"],
fieldTemplateId: "skiECVSnowDepth",
},
{
id: "Wind",
boundField: "WindSpeed",
fieldTemplateId: "skiECVWind",
},
{
id: "OpenTrails",
boundField: "OpenTrailsString",
fieldTemplateId: "skiECVOpenTrails",
sortId: "OpenTrails"
},
{
id: "OpenLifts",
boundField: "OpenLiftsString",
fieldTemplateId: "skiECVOpenLifts",
sortId: "OpenLifts"
}
]
});
})();