﻿(function () {
"use strict";
WinJS.Namespace.define("WeatherAppJS.Pages", {
IFrameReader: WinJS.Class.define(
/// <summary>
/// Constructor for a simple IPage class that controls the page
/// </summary>
/// <param name="state">
/// The state object passed to WinJS.Navigation.navigate or the state object
/// preserved when the page has been navigated away from
/// </param>
function (state) {
PlatformJS.Navigation.mainNavigator.disableSearchOnType();
this._state = state ? state : {};
// Quickly show and hide the progress indicator to get the secondary splash screen to go away when resuming to the story page
CommonJS.Progress.showProgress(CommonJS.Progress.headerProgressType);
CommonJS.Progress.hideProgress(CommonJS.Progress.headerProgressType);
this._loadPageData();
},
{
_storyFrame: null,
_parentContainer: null,
_onLoadHandler: null,
getPageImpressionContext: function getPageImpressionContext() {
var context = WeatherAppJS.Instrumentation.PageContext.IFramePage;
if (this._state && this._state.page) {
context = this._state.page;
}
return context;
},
_loadPageData: function _loadPageData() {
var that = this;
if (this._state && this._state.articleUrl) {
var url = this._state.articleUrl;
if (PlatformJS.Utilities.hasInternetConnection()) {
this._loadiframe(url);
}
else {
CommonJS.Error.showError(CommonJS.CommonJS.Error.NO_INTERNET, function () { that._loadPageData(); });
}
}
},
/// <summary>
/// This function is invoked by the UX Framework when the page is being navigated away and when the app is suspending.
/// Return any data that you need to resurrect your UX when the page is recreated
/// Keep this state object as small as possible
/// </summary>
/// <returns>
/// This function must return a JSON object that can be serializable. Do not pass a Managed object
/// </returns>
getPageState: function getPageState() {
return this._state;
},
/// <summary>
/// This function is invoked by the UX Framework when the page is being created and when
/// it is necessary to bind data to the UX.
/// </summary>
/// <returns>
/// Returns a promise that contains data to bind the UX to
/// </returns>
getPageData: function getPageData() {
var that = this;
return WinJS.Promise.wrap({
title: "",
subTitle: ""
});
},
getSharedContractData: function getSharedContractData() {
},
onLayoutChange: function onLayoutChange(layoutState) {
if (layoutState === Windows.UI.ViewManagement.ApplicationViewState.snapped) {
this.onNavigateAway(null);
}
},
onBindingComplete: function onBindingComplete() {
var that = this;
var resourceLoader = PlatformJS.Services.resourceLoader;
var url = (that._state && that._state.articleUrl) ? that._state.articleUrl : null;
var openInBrowserButton = document.getElementById("openInBrowserButton");
var helpButton = document.getElementById("helpButton");
var refreshButton = document.getElementById("refreshButton");
if (url && openInBrowserButton && openInBrowserButton.winControl) {
openInBrowserButton.winControl.label = resourceLoader.getString("/Common/OpenInBrowser");
openInBrowserButton.addEventListener('click',
function (e) {
that._openInBrowser(url);
});
}
if (helpButton && helpButton.winControl) {
helpButton.winControl.label = CommonJS.resourceLoader.getString("/platform/HelpLabel");
helpButton.winControl.onclick = WeatherAppJS.Utilities.UI.openInAppHelp;
}
if (refreshButton && refreshButton.winControl) {
refreshButton.winControl.label = resourceLoader.getString("RefreshButtonTitle");
refreshButton.addEventListener('click',
function (e) {
CommonJS.dismissAllEdgies();
that._loadPageData();
});
}
},
/// <summary>
/// This function is invoked by the UX Framework when the page is being navigated away
/// </summary>
/// <param name="reason">
/// The reason why the page is being navigated away: "backNavigation" or "newNavigation"
/// </param>
/// <returns>
/// Nothing
/// </returns>
onNavigateAway: function onNavigateAway(reason) {
if (this._storyFrame && this._onLoadHandler) {
this._storyFrame.removeEventListener("load", this._onLoadHandler);
}
// Destroy the iframe
CommonJS.Progress.resetAll();
if (this._parentContainer) {
this._parentContainer.innerHTML = "";
}
},
_loadiframe: function _loadiframe(url) {
if (!this._storyFrame) {
this._initializeiFrame();
}
// Need to first try to hide the progress indicator in case this is a refresh and the previous load
// didn't finish, which means that it didn't hit the onload callback to hide the progress indicator
CommonJS.Progress.hideProgress(CommonJS.Progress.headerProgressType);
CommonJS.Progress.showProgress(CommonJS.Progress.headerProgressType);
this._storyFrame.src = url;
},
_initializeiFrame: function _initializeiFrame() {
var storyFrame = document.createElement("iframe");
storyFrame.className = "storyFrame";
storyFrame.contentEditable = true;
this._onLoadHandler = function _onLoadHandler() {
CommonJS.Progress.hideProgress(CommonJS.Progress.headerProgressType);
};
storyFrame.addEventListener("load", this._onLoadHandler);
this._storyFrame = storyFrame;
this._parentContainer = document.querySelector(".storyFrameContainer");
this._parentContainer.appendChild(storyFrame);
},
_openInBrowser: function _openInBrowser(url) {
try {
var uri = new Windows.Foundation.Uri(url);
Windows.System.Launcher.launchUriAsync(uri);
} catch (e) {
}
}
})
});
})();
