﻿/********************************************************
*                                                        *
*   © Microsoft. All rights reserved.                    *
*                                                        *
*********************************************************/
(function () {
"use strict";
WinJS.Namespace.define("WeatherAppJS", {
PhotosynthViewer: WinJS.Class.derive(WeatherAppJS.WeatherBasePage, 
function (state) {
WeatherAppJS.WeatherBasePage.call(this, state);
var that = this;
that._state = state || {};
if (state && state.panoId) {
that._state.panoId = state.panoId;
}
var loggedWarnAboutTiles = false;
function createViewerFromCidOrJsonUri(panoId) {
if (!PlatformJS.Utilities.hasInternetConnection()) {
CommonJS.Error.showError(CommonJS.Error.NO_INTERNET, function photosynthViewer_showNoInternet() {
WinJS.Navigation.back();
});
} else {
if (panoId === "") {
console.warn("panoId is not specified");
}
else if (panoId.indexOf("http") === 0) {
//pano is specified by the uri of the json.
CommonJS.Progress.showProgress(CommonJS.Progress.centerProgressType);
PhotosynthRml.createFromJsonUri(panoId, rmlCallback);
}
else {
//pano is specified by the photosynth collection id (cid).
CommonJS.Progress.showProgress(CommonJS.Progress.centerProgressType);
PhotosynthRml.createFromCid(panoId, rmlCallback);
}
}
}
function rmlCallback(rml, error) {
if (!rml) {
CommonJS.Progress.hideProgress(CommonJS.Progress.centerProgressType);
if (error instanceof JsonDownloadFailedError) {
//Failed to download
console.warn("json failed to download");
}
else if (error instanceof JsonMalformedError) {
//Failed to parse the response of the json request
console.warn("json was malformed");
}
else {
//Some other unknown error.
console.warn("unknown error when attempting to download json");
}
var errorType = Platform.Networking.NetworkManager.instance.isNetworkAvailable ?
CommonJS.Error.STANDARD_ERROR : CommonJS.Error.NO_INTERNET;
//Show "This page failed to load" error and allow to retry
CommonJS.Error.showError(errorType, function photosynthViewer_showRmlError(evt) {
createViewerFromCidOrJsonUri(that._state.panoId);
});
}
else {
createViewer(rml);
}
}
function createViewer(rml) {
if (that._viewer) {
that._viewer.dispose();
}
var div = document.getElementById("photosynth_viewerDiv");
if (div) {
that._viewer = new RwwViewer(div, {
rml: rml,
tileDownloadFailed: function tileDownloadFailed(failCount, succeedCount) {
if (!Platform.Networking.NetworkManager.instance.isNetworkAvailable) {
CommonJS.Progress.hideProgress(CommonJS.Progress.centerProgressType);
CommonJS.Error.showError(CommonJS.Error.NO_INTERNET, function photosynthViewer_failDownloadNoInternet() {
WinJS.Navigation.back();
});
} else {
var total = failCount + succeedCount;
if (total > 4 && failCount > succeedCount && !loggedWarnAboutTiles) {
//Tile download failures are high
console.warn("tile download failures are high");
loggedWarnAboutTiles = true;
CommonJS.Progress.hideProgress(CommonJS.Progress.centerProgressType);
CommonJS.Error.showError(CommonJS.Error.STANDARD_ERROR, function photosynthViewer_failDownloadHighCount() {
WinJS.Navigation.back();
});
}
if (total > 2) {
CommonJS.Progress.hideProgress(CommonJS.Progress.centerProgressType);
}
}
},
tileDownloadSucceeded: function tileDownloadSucceeded(failCount, succeedCount) {
var total = failCount + succeedCount;
if (total > 2) {
CommonJS.Progress.hideProgress(CommonJS.Progress.centerProgressType);
}
},
renderer: "css",  //OPTIONAL param.  Defaults to 'webgl' if available on the current browser, else 'css'.  At the moment, it needs to be 'css', because the imagery won't show in 'webgl' until we make some changes to the HTTP response headers.
// ToDo@bhargavm : Handle gyration sensor integration
//orientationSensor: AppExJS.Travel.orientationSensor,
});
var panoAttribElement = div.querySelector("#authorText");
if (panoAttribElement) {
panoAttribElement.addEventListener('click', function (event) {
WeatherAppJS.Utilities.Instrumentation.onClickAttribution(this.innerText, 'SkiResort360Pano');
});
}
panoAttribElement = div.querySelector("#publisherText");
if (panoAttribElement) {
panoAttribElement.addEventListener('click', function (event) {
WeatherAppJS.Utilities.Instrumentation.onClickAttribution(this.innerText, 'SkiResort360Pano');
});
}
}
}
var viewerDiv = document.getElementById("photosynth_viewerDiv");
if (viewerDiv) {
viewerDiv.onresize = function () {
if (that._viewer) {
that._viewer.setViewportSize(viewerDiv.offsetWidth, viewerDiv.offsetHeight);
}
};
// Delay setting the key handler to prevent carried key presses from the previous page from prematurely toggling visibility of controls.
setTimeout(function photosynthViewer_setKeyHandler() {
viewerDiv.onkeyup = function (event) {
if (event.keyCode === WinJS.Utilities.Key.enter) {
//ToDo@bhargavm : Toggle UI Controls Visiblity on User Tap
console.log("Toggle UI Controls Visibility here");
}
};
}, 1000);
viewerDiv.onmspointerdown = function (event) {
that._pointerMoveData.x = -1;
that._pointerMoveData.y = -1;
that._pointerMoveData.state = 1;
};
viewerDiv.onmspointermove = function (event) {
if (that._pointerMoveData.state === 1) {
if (that._pointerMoveData.x < 0) {
that._pointerMoveData.x = event.x;
that._pointerMoveData.y = event.y;
} else {
var dx = event.x - that._pointerMoveData.x;
var dy = event.y - that._pointerMoveData.y;
that._pointerMoveData.state = (dx || dy) ? 2 : 1;
}
}
};
viewerDiv.onmspointerup = function (event) {
if (event.button === 0 && that._pointerMoveData.state === 1) {
//ToDo@bhargavm : Toggle UI Controls Visiblity on User Tap
console.log("Toggle UI Controls Visibility here");
}
that._pointerMoveData.x = -1;
that._pointerMoveData.y = -1;
that._pointerMoveData.state = 0;
};
}
if (that._state.panoId) {
createViewerFromCidOrJsonUri(that._state.panoId);
} 
},
{
getPageImpressionContext: function getPageImpressionContext() {
var context = WeatherAppJS.Instrumentation.PageContext.PhotoSynth;
if (this._state.page) {
context = this._state.page;
}
return context;
},
_pointerMoveData: {
x: 0,
y: 0,
state: 0
},
_viewer: null,
getPageData: function getPageData() {
return WinJS.Promise.wrap({
title: "",
});
},
getPageState: function getPageState() {
return this._state;
},
dispose: function dispose() {
if (this._viewer) {
this._viewer.dispose();
this._viewer = null;
}
},
onBindingComplete: function onBindingComplete() {
},
})
});
})();