﻿/********************************************************
*                                                        *
*   © Microsoft. All rights reserved.                    *
*                                                        *
*********************************************************/
(function () {
'use strict';
WinJS.Namespace.define("WeatherAppJS", {
SnapHome: WinJS.Class.define(
/// <summary>
/// Constructor for your IPage class
/// </summary>
function (state) {
msWriteProfilerMark("WeatherApp:SnapHomeLoad:s");
CommonJS.Progress.resetAll();
var that = this;
this._loadingSampleDataFlag = false;
this._presetDefaultLocation = null;
var locationManager = WeatherAppJS.LocationsManager;
var locID = null;
// If state info. has valid locID, use it. This happens when pinned tile is invoked and app is in snapped mode.
if (state && state.locID) {
locID = state.locID;
if (!WeatherAppJS.GeocodeCache.getLocation(state.locID)) {
if (state.geocodeLocation && state.locID === state.geocodeLocation.getId()) {
var geocodeLocation = WeatherAppJS.GeocodeLocation.createGeocodeLocation(state.geocodeLocation);
WeatherAppJS.GeocodeCache.addLocation(geocodeLocation);
} else {
// LatLong info unavailable, show default location.
locID = null;
}
}
}
if (state && state.originPageInfo) {
that._originPageInfo = state.originPageInfo;
}
if (state && state.panoramaState) {
that._originPanoState = state.panoramaState;
}
if (!locID) {
locID = locationManager.getDefaultLocation();
if (!locID) {
var presetDefaultLocation = WeatherAppJS.Utilities.Common.getPresetDefaultLocation();
if (WeatherAppJS.Networking.internetAvailable === false) {
// User snapped the app in FRE Offline scenario, show him the sample data in snapped mode.
that._loadingSampleDataFlag = true;
}
else {
// Default location couldn't be set yet. User should have snapped quickly before picking a default location. Defaulting to a preset location here.
locID = presetDefaultLocation.getId();
WeatherAppJS.SettingsManager.setIsFREAsync(false).then();
WeatherAppJS.SettingsManager.setDefaultLocationAsync(presetDefaultLocation, true);
}
}
}
if (that._loadingSampleDataFlag) {
that._locID = presetDefaultLocation.getId();
that._locationDisplayName = PlatformJS.Services.resourceLoader.getString("SampleDataHeader").format(presetDefaultLocation.getShortDisplayName());
}
else {
that._locID = locID;
that._locationDisplayName = WeatherAppJS.GeocodeCache.getShortDisplayName(locID);
}
/// <summary>
/// This function is invoked when the app comes back online and a refresh event is pending.
/// </summary>
this.onInternetChange = function () {
if (WeatherAppJS.Networking.internetAvailable === true) {
//msWriteProfilerMark("WeatherApp:HomePageRefresh:s");
that._loadPageData(false, WeatherAppJS.PageLoadTypes.loadPageByInternetChange);
//msWriteProfilerMark("WeatherApp:HomePageRefresh:e");
WeatherAppJS.Networking.networkChangeCallback = null;
that._restartPageRefreshTimer();
}
else {
WeatherAppJS.Networking.networkChangeCallback = that.onInternetChange;
}
};
/// <summary>
/// This function is invoked to refresh the page
/// </summary>
this.pageRefresh = function () {
if (that._loadingSampleDataFlag) {
// nothing to be refreshed
return;
}
// Refresh the page
if (WeatherAppJS.Networking.internetAvailable === true) {
//msWriteProfilerMark("WeatherApp:HomePageRefresh:s");
that._loadPageData(false, WeatherAppJS.PageLoadTypes.loadPageByRefresh);
//msWriteProfilerMark("WeatherApp:HomePageRefresh:e");
WeatherAppJS.Networking.networkChangeCallback = null;
}
else {
WeatherAppJS.Networking.networkChangeCallback = that.onInternetChange;
}
};
this.updateLastUpdateTimeStamp = function (parentNode) {
if (parentNode && that._lastUpdatedTimeStamp) {
var timeStampDiv = document.createElement("div");
WinJS.Utilities.addClass(timeStampDiv, "lastUpdatedTimeStamp");
var ts = new Date(that._lastUpdatedTimeStamp);
var formatTimeStamp = PlatformJS.Services.resourceLoader.getString("LastUpdatedDataTime").format(WeatherAppJS.Utilities.Formatting.getFormattedLastUpdateTimeShort(
ts.getFullYear(), ts.getMonth(), ts.getDate(), ts.getHours(), ts.getMinutes(), ts.getSeconds(),
PlatformJS.Services.resourceLoader.getString('ShortDateTimeFormatString')));
timeStampDiv.innerText = formatTimeStamp;
parentNode.appendChild(timeStampDiv);
}
};
/// <summary>
/// Handles the current conditions data on complete/progress handlers
/// <param name="locationID">Location Identifier</param>
/// <param name="bypassCache">Whether to bypass the cache</param>
/// <param name="locObj">Location weather data object</param>
/// <param name="isComplete">Whether the data fetch is complete</param>
/// </summary>
this._handleCCData = function (locationID, bypassCache, locObj, isComplete) {
var resourceLoader = PlatformJS.Services.resourceLoader;
// Load snap template based on market. One for TianQi and rest of world have the same other template
var useTianQi = Platform.Utilities.Globalization.isFeatureEnabled("UseTianQi");
//GetCurrentConidtion.
var currentConditions = locObj.getCurrentConditions();
//Check if snap is done from Ski Page.
var isSki = false;
var skiForeCastData = null;
var skiForecastCurrentIndex = 0;
var skiForecastMaxCount = 0;
if (currentConditions.heroData) {
skiForeCastData = currentConditions.heroData.dailyForeCast;
skiForecastMaxCount = Object.keys(skiForeCastData).length;
isSki = true;
}
var container = document.getElementById('snapHomeContainer');
var locCC = currentConditions;
var locDaily = locObj.getDailyConditions();
// Apply the theme and the background image
if (container) {
if (isSki) {
if (locCC.heroImageData) {
container.style.backgroundImage = "url('" + locCC.heroImageData.highResolutionUrl + "')";
WinJS.Utilities.addClass(container, "snapSkiImg " + locCC.heroImageData.anchorPointPortrait);
}
} else {
WeatherAppJS.Utilities.ThemeManager.applyTheme(container, locCC.iconcode, locCC.dayNightIndicator);
container.style.backgroundImage = "url('" + WeatherAppJS.Utilities.ThemeManager.getHeroImageForSkyCode(locCC.iconcode, locCC.dayNightIndicator) + "')";
}
}
if (useTianQi && WeatherAppJS.GeocodeCache.isTianQiSupportedLocation(locationID)) {
locCC.feelslike = locCC.wind;
}
// Bind CC Data
CommonJS.loadModule({ fragmentPath: "/html/secondaryTemplates.html", templateId: "snapHomeCCTemplate" }, locCC, null, null, null).then(function (snapCCTemplate) {
var snapCCDiv = document.querySelector('.snapCC');
if (snapCCDiv) {
snapCCDiv.innerHTML = "";
// bind provider link
var providerName = "", providerUrl = "";
var ccProviderOverride = PlatformJS.Services.appConfig.getString("CCProviderOverride");
var showLimitedExperience = WeatherAppJS.Utilities.UI.showLimitedExperience(locationID);
if (ccProviderOverride && showLimitedExperience) {
providerName = resourceLoader.getString(ccProviderOverride);
providerUrl = resourceLoader.getString(ccProviderOverride + "Url");
}
else if (locDaily && locDaily[0] && locDaily[0].provider) {
providerName = resourceLoader.getString(locDaily[0].provider);
providerUrl = locDaily[0].url;
}
var providerDiv = snapCCTemplate.querySelector('.snapProvider');
// If skiSnap, Add snow Data Provider.
if (isSki) {
var skiProviderText = "{0} / ";
providerDiv.innerHTML = skiProviderText.format(locCC.heroData.snowProviderData);
WinJS.Utilities.addClass(providerDiv, "snapSkiProvider");
}
if (providerName && providerDiv) {
if (providerUrl) {
providerDiv.innerHTML += '<a class="dailyConditionsProviderLink" href="' + providerUrl + '">' + providerName + '</a>';
} else {
providerDiv.innerHTML = providerName;
}
}
var ariaText = resourceLoader.getString("currentConditionsAriaText").format(locCC.tempCC + " °", locCC.caption,
locCC.feelslikeTemp, locCC.provider);
//For Non China Market Update the Aria Text.
if (!useTianQi) {
// set aria label on current conditions
var unitAriaText = (locCC.tempunit === "F") ? resourceLoader.getString("AriaTextF") : resourceLoader.getString("AriaTextC");
unitAriaText = unitAriaText + ". ";
var toolTipAriaText = (locCC.tempunit === "F") ? resourceLoader.getString("ToolTipTextF") : resourceLoader.getString("ToolTipTextC");
ariaText = resourceLoader.getString("currentConditionsAriaText").format(locCC.tempCC + " °" + unitAriaText + toolTipAriaText, locCC.caption,
locCC.feelslikeTemp, locCC.provider);
}
else {
var ccElemBigTempUnit = snapCCTemplate.querySelector(".snapbigTempUnit");
ccElemBigTempUnit.style.display = "none";
}
snapCCTemplate.querySelector(".snapCCWrapper").setAttribute("aria-label", ariaText);
snapCCDiv.appendChild(snapCCTemplate);
}
});
// Bind Daily Data
if (locDaily) {
var promises = [];
for (var forecastIndex in locDaily) {
var totalConditions = locDaily[forecastIndex].forecasts.length;
var dailyDiv = document.createElement('div');
for (var i = 0; i < totalConditions; i++) {
(function (i) {
promises.push(CommonJS.loadModule({ fragmentPath: "/html/secondaryTemplates.html", templateId: "snapHomeDailyTemplate" }, locDaily[forecastIndex].forecasts[i], null, null, null).then(
function (dailyTemplate) {
var providerData = locDaily[forecastIndex].forecasts[i];
var ariaText = resourceLoader.getString("dailyForecastSnapMode").format(providerData.fullDate, providerData.temphighWithoutUnit + " °",
providerData.templowWithoutUnit + " °", providerData.caption);
dailyTemplate.querySelector(".snapEachDay").setAttribute("aria-label", ariaText);
var precip = locDaily[forecastIndex].forecasts[i].precipitation;
if (precip === "--") {
dailyTemplate.querySelector(".precip").style.display = 'none';
dailyTemplate.querySelector(".raindrop").style.display = 'none';
}
if (isSki) {
var snapSnowDailyNode = dailyTemplate.querySelector('.snowFreshText');
if (skiForeCastData[skiForecastCurrentIndex] && skiForecastCurrentIndex < skiForecastMaxCount && skiForeCastData[skiForecastCurrentIndex].date === providerData.date) {
providerData.snowData = skiForeCastData[skiForecastCurrentIndex].freshSnow;
snapSnowDailyNode.innerHTML = skiForeCastData[skiForecastCurrentIndex].freshSnow;
skiForecastCurrentIndex++;
}
else {
snapSnowDailyNode.style.display = 'none';
}
}
else {
dailyTemplate.querySelector('.snowFreshText').style.display = 'none';
}
if (useTianQi && WeatherAppJS.GeocodeCache.isTianQiSupportedLocation(locationID)) {
dailyTemplate.querySelector('.precipWrapper').innerText = providerData.windspeed;
}
dailyDiv.appendChild(dailyTemplate);
}));
})(i);
}
break;
}
WinJS.Promise.join(promises).then(function () {
var snapDailyDiv = document.querySelector('.snapDaily');
if (snapDailyDiv) {
while (snapDailyDiv.hasChildNodes()) {
snapDailyDiv.removeChild(snapDailyDiv.lastChild);
}
// add daily forecast provider, if required.
var showDailyForecastAttribution = PlatformJS.Services.appConfig.getBool("ShowDailyForecastAttribution");
var showLimitedExperience = WeatherAppJS.Utilities.UI.showLimitedExperience(locationID);
if (showDailyForecastAttribution && showLimitedExperience) {
var forecastDiv = document.createElement("div");
WinJS.Utilities.addClass(forecastDiv, "dailyForecastAttribution");
if (forecastDiv
&& locDaily
&& locDaily[0]
&& locDaily[0].forecasts
&& locDaily[0].forecasts.length > 0) {
var provider = locDaily[0].provider;
if (provider === "Weather Service Ltd") {
provider = resourceLoader.getString("JWSAttributionValue");
}
else {
provider = resourceLoader.getString(provider);
}
forecastDiv.style.display = "block";
forecastDiv.innerText = provider;
dailyDiv.appendChild(forecastDiv);
}
}
// add last updated time
that.updateLastUpdateTimeStamp(dailyDiv);
snapDailyDiv.appendChild(dailyDiv);
}
});
}
// Update alerts data
if (!that._alertDataRequested) {
that._alertDataRequested = true;
if (!that._loadingSampleDataFlag) {
CommonJS.Progress.showProgress(CommonJS.Progress.headerProgressType);
var alertsDataPromise = WeatherAppJS.LocationsManager.getAlertsDataForLocationAsync(locationID, bypassCache);
that._promisesStack.push(alertsDataPromise);
alertsDataPromise.then(
function () {
CommonJS.Progress.hideProgress(CommonJS.Progress.headerProgressType);
},
function (e) {
if (PlatformJS.Utilities.isPromiseCanceled(e)) {
return;
}
CommonJS.Progress.hideProgress(CommonJS.Progress.headerProgressType);
},
function () { });
}
}
CommonJS.Progress.destroySplash(true);
if (isComplete) {
CommonJS.Progress.hideProgress(CommonJS.Progress.headerProgressType);
}
if (isSki) {
document.querySelector(".snapFeels").style.display = "none";
var alertContainer = document.querySelector(".alertIndicator");
if (alertContainer) {
WinJS.Utilities.addClass(alertContainer, "skiAlert");
}
}
};
/// <summary>
/// Current conditions data progress handler
/// <param name="locationID">Location Identifier</param>
/// <param name="bypassCache">Whether to bypass the cache</param>
/// <param name="locObj">Location weather data object</param>
/// </summary>
this._onCCDataProgress = function (locationID, bypassCache, locObj) {
that._lastUpdatedTimeStamp = locObj.getCurrentConditions().lastUpdatedTime;
that._handleCCData(locationID, bypassCache, locObj, false);
};
/// <summary>
/// Current conditions data complete handler
/// <param name="locationID">Location Identifier</param>
/// <param name="bypassCache">Whether to bypass the cache</param>
/// <param name="locObj">Location weather data object</param>
/// </summary>
this._onCCDataComplete = function (locationID, bypassCache, locObj) {
that._lastUpdatedTimeStamp = locObj.getCurrentConditions().lastUpdatedTime;
that._handleCCData(locationID, bypassCache, locObj, true);
};
/// <summary>
/// Current conditions data error handler
/// <param name="pageLoadType">Page load type</param>
/// <param name="error">Error code</param>
/// </summary>
this._onCCDataError = function (pageLoadType, error, bypassCache) {
if (PlatformJS.Utilities.isPromiseCanceled(error)) {
return;
}
CommonJS.Progress.hideProgress(CommonJS.Progress.headerProgressType);
if (pageLoadType === WeatherAppJS.PageLoadTypes.loadPageByDefault) {
// show full screen error...
var errorType = (WeatherAppJS.Networking.internetAvailable === true) ? CommonJS.Error.STANDARD_ERROR : CommonJS.Error.NO_INTERNET;
CommonJS.Error.showError(errorType, function () {
that._loadPageData(bypassCache, pageLoadType);
});
}
};
/// <summary> 
/// Loads the current page
/// <param name="bypassCache">Whether to bypass the cache</param>
/// <param name="pageLoadType">Page load type</param>
/// </summary>
this._loadPageData = function (bypassCache, pageLoadType) {
CommonJS.Progress.showProgress(CommonJS.Progress.headerProgressType);
var locationID = that._locID;
that._alertDataRequested = false; // Reset the data requested flags
CommonJS.setTheme(null, true);
var weatherDataAsyncFunction = WeatherAppJS.LocationsManager.getCCDataForLocationAsync;
var locationObj = WeatherAppJS.GeocodeCache.getLocation(locationID);
if (locationObj.isSkiLocation) {
weatherDataAsyncFunction = WeatherAppJS.LocationsManager.getSkiDetailsAsync;
}
var useTianQi = Platform.Utilities.Globalization.isFeatureEnabled("UseTianQi") && WeatherAppJS.GeocodeCache.isTianQiSupportedLocation(locationID);
if (useTianQi) {
weatherDataAsyncFunction = WeatherAppJS.LocationsManager.getTianQiDataForLocationAsync;
}
var ccDataPromise = weatherDataAsyncFunction(locationID, bypassCache);
that._promisesStack.push(ccDataPromise);
ccDataPromise.then(
function (locObj) { that._onCCDataComplete(locationID, bypassCache, locObj); },
function (error) { that._onCCDataError(pageLoadType, error, bypassCache); },
function (locObj) { that._onCCDataProgress(locationID, bypassCache, locObj); });
};
/// <summary> 
/// Loads the sample data page
/// </summary>
this._loadSamplePageData = function () {
CommonJS.Progress.showProgress(CommonJS.Progress.headerProgressType);
var locationID = that._locID;
that._alertDataRequested = false; // Reset the data requested flags
CommonJS.setTheme(null, true);
var ccDataPromise = WeatherAppJS.Utilities.FRE.getOverviewData();
that._promisesStack.push(ccDataPromise);
ccDataPromise.then(
function (locObj) {
that._onCCDataComplete(locationID, false, locObj);
});
};
msWriteProfilerMark("WeatherApp:SnapHomeLoad:e");
},
{
_alertDataRequested: false,
_lastUpdatedTimeStamp: null,
_promisesStack: [],
_originPageInfo: null, // store where we navigated from so we can go back from snap
_originPanoState: null, // store the origin pano's state if required
getPageImpressionContext: function getPageImpressionContext() {
return WeatherAppJS.Instrumentation.PageContext.Snap;
},
_restartPageRefreshTimer: function _restartPageRefreshTimer() {
if (this._dataRefreshTimerId) {
clearInterval(this._dataRefreshTimerId);
}
this._dataRefreshTimerId = setInterval(this.pageRefresh, PlatformJS.Services.appConfig.getString("PageRefreshTime"));
},
getPageState: function getPageState() {
// No need to preserve state for snap mode, as navigating is not supported in this mode.
// Also, if the state gets persisted once, it is being always read and passed to snap mode for the lifetime of the app. 
return null;
},
getPageData: function getPageData() {
var that = this;
return WinJS.Promise.wrap({
title: that._locationDisplayName,
subtitle: CommonJS.resourceLoader.getString("SnappedAppTitle"),
quickSearchEnable: false,
});
},
onBindingComplete: function onBindingComplete() {
// Load the data
if (this._loadingSampleDataFlag) {
this._loadSamplePageData();
}
else {
this._loadPageData(false, WeatherAppJS.PageLoadTypes.loadPageByDefault);
}
if (this._dataRefreshTimerId) {
clearInterval(this._dataRefreshTimerId);
}
this._dataRefreshTimerId = setInterval(this.pageRefresh, PlatformJS.Services.appConfig.getString("PageRefreshTime"));
WeatherAppJS.Networking.networkChangeCallback = null;
},
onLayoutChange: function onLayoutChange(layoutState) {
if (layoutState !== Windows.UI.ViewManagement.ApplicationViewState.snapped) {
this._destroyPage();
var pageState = null;
var pageInfo = null;
var defaultLocation = WeatherAppJS.LocationsManager.getDefaultLocation();
if (defaultLocation !== this._locID) {
// if originPageInfo is present, navigate to that
// else, just go home
if (this._originPageInfo && this._originPageInfo.page && this._originPageInfo.fragment) {
pageInfo = this._originPageInfo;
} else {
pageInfo = {
page: "WeatherAppJS.Pages.Home",
fragment: "/panoramas/home/home.html"
};
}
pageState = {
pageInfo: pageInfo,
state: {
locID: this._locID,
geocodeLocation: WeatherAppJS.GeocodeCache.getLocation(this._locID),
panoramaState: this._originPanoState
}
};
}
return pageState;
}
},
onOrientationChange: function onOrientationChange(event) {
},
onNavigateAway: function onNavigateAway() {
this._destroyPage();
},
_destroyPage: function _destroyPage() {
clearInterval(this._dataRefreshTimerId);
WeatherAppJS.Networking.networkChangeCallback = null;
WeatherAppJS.Utilities.Common.cleanupPromiseStack(this._promisesStack);
},
})
});
})();