﻿/********************************************************
*                                                        *
*   © Microsoft. All rights reserved.                    *
*                                                        *
*********************************************************/
(function () {
"use strict";
WinJS.Namespace.define("WeatherAppJS.Pages", {
WeatherMaps: WinJS.Class.derive(WeatherAppJS.Core.WeatherBasePage,
/// <summary>
/// Constructor for a simple IPage class
/// </summary>
/// <param name="state">
/// The state object passed to WinJS.Navigation.navigate or the state object
/// preserved when the page has been navigated away from
/// potential properties: region, map type
/// </param>
function (state) {
msWriteProfilerMark("WeatherApp:WeatherMapsPageLoad:s");
WeatherAppJS.Core.WeatherBasePage.call(this, state);
var that = this;
that._pageState = Object.create(state) || {}; // used to preserve state that was passed in
that._localState = Object.create(state) || {};
// create the base map controller
this.baseMapCtlr = new WeatherAppJS.Maps.BaseMapController();
this.bindableUserGestureHandler = this.onUserGesture.bind(this);
this.bindablePageRefreshFn = this.onPageRefresh.bind(this);
this.bindableNetworkOnlineFn = this.onNetworkOnline.bind(this);
this.bindableBaseMapViewChangeFn = this.onBaseMapViewChange.bind(this);
this.bindableProcessMove = this.processMouseMoveEvent.bind(this);
this.mapLayers = [];
// disable search charm on keyboard input since we want to pass along events to map control for accessibility
var searchPane = Windows.ApplicationModel.Search.SearchPane.getForCurrentView();
this._showSearchCharmOnKeyBoardInput = searchPane.showOnKeyboardInput; // save earlier value
searchPane.showOnKeyboardInput = false;
WeatherAppJS.Networking.addEventListener('networkonline', this.bindableNetworkOnlineFn, false);
msWriteProfilerMark("WeatherApp:WeatherMapsPageLoad:e");
},
{
baseMapCtlr: null,
mapLayers: null,
supportedMaps: [],
_currentMapType: null,
_seekBar: null,
_zoomControl: null,
_legend: null,
_isBaseMapCtrlInitialized: false,
_baseMapInitializeCheckTimer: null,
_baseMapInitializeCheckTimeout: {
get: function get() {
return PlatformJS.Services.appConfig.getInt32("BaseMapInitTimeoutInMilliseconds", 15000);
}
},
_baseMapViewChangeEventHandler: null,
_autoHideChromeTimer: null,
_prevZoomLevel: null,
_autoHideChromeTimeOut: {
get: function get() {
return PlatformJS.Services.appConfig.getInt32("ChromeDisplayTimeoutInMilliseconds", 2500);
}
},
_showSearchCharmOnKeyBoardInput: null,
_isMapsConfigInitDone: false,
_isEdgyActionHandlersInitialized: false,
// store mouse move positions to calculate mouse move distances
_prevMousePos: {
x: 0,
y: 0
},
getPageImpressionContext: function getPageImpressionContext() {
var regionName;
if (this._state.isTopEdgyL1Action) {
var regionISOCode = WeatherAppJS.SettingsManager.getUserDefaultRegionISOCode();
var ui = WeatherAppJS.Utilities.UI;
if (ui._isoCodeToRegionMap && ui._isoCodeToRegionMap[regionISOCode]) {
regionName = ui._isoCodeToRegionMap[regionISOCode].region;
}
else {
// If we dont have the region mapping to the corresponding country code, defining the context as "Weathermaps/ISOCode"
regionName = "WeatherMaps/" + regionISOCode;
}
} else {
regionName = this._state.region;
}
return "/" + regionName;
},
/// <summary>
/// This function is invoked by the UX Framework when the page is being created and when
/// it is necessary to bind data to the UX.
/// </summary>
/// <returns>
/// Returns a promise that contains data to bind the UX to
/// </returns>
getPageData: function getPageData() {
var that = this;
var resourceLoader = PlatformJS.Services.resourceLoader;
var pageData = {
HelpButtonTitle: resourceLoader.getString("/platform/HelpLabel"),
RefreshButtonTitle: resourceLoader.getString("RefreshButtonTitle"),
ChangeUnitToFButtonTitle: resourceLoader.getString("ChangeUnitToFButtonTitle"),
ChangeUnitToCButtonTitle: resourceLoader.getString("ChangeUnitToCButtonTitle")
};
if (!this._isMapsConfigInitDone) {
return this._initMapsConfigAsync().then(function () {
that._isMapsConfigInitDone = true;
that._initCurrentMap();
MapConfigs.initMapScenarioConfig(that._currentMapType.id, that._currentMapType.subTypeId);
that._init();
msWriteProfilerMark("WeatherApp:WeatherMapsConfigInit:e");
return WinJS.Promise.wrap(pageData);
}, function (error) {
// could not load maps config
msWriteProfilerMark("WeatherApp:WeatherMapsConfigInit:e");
return WinJS.Promise.wrap(pageData);
});
} else {
return WinJS.Promise.wrap(pageData);
}
},
getPageState: function getPageState() {
this._pageState.map = this._currentMapType; // preserve the current map type
return this._pageState;
},
/// <summary>
/// Caculates the region/continent from tha pagestate
/// </summary>
/// <returns>
/// Returns the region name
/// </returns>
getRegionAsync: function getRegionAsync(pageState) {
if (pageState && pageState.region) {
return WinJS.Promise.wrap(pageState.region);
}
// Identify region based on lat/long supplied
if (pageState && "latitude" in pageState && "longitude" in pageState) {
var matchedRegion = MapUtils.getMapRegionForLatLong(pageState.latitude, pageState.longitude);
if (matchedRegion.length === 1) { // if only one region matched
return WinJS.Promise.wrap(matchedRegion[0]);
}
}
var ui = WeatherAppJS.Utilities.UI;
// Identify region using location isoCode
if (pageState && pageState.isoCode) {
return ui.getMapRegionDataForIsoCodeAsync(pageState.isoCode)
.then(function (data) {
var regionName = data["region"];
var region = (regionName) ? regionName : PlatformJS.Services.appConfig.getString("BaseMapDefaultContinent");
return region;
});
}
// The region information not provided. Top Edgy L1 button used.
var regionCode = WeatherAppJS.SettingsManager.getUserDefaultRegionISOCode();
return ui.getMapRegionDataForIsoCodeAsync(regionCode)
.then(function (data) {
var regionName = data["region"];
var region = (regionName) ? regionName : PlatformJS.Services.appConfig.getString("BaseMapDefaultContinent");
return region;
});
},
getRegionDataAsync: function getRegionDataAsync(pageState) {
if (pageState && pageState.region) {
if (!pageState.isTopEdgyL1Action) {
return WinJS.Promise.wrap({ region: pageState.region });
}
// TopEdgyL1 Button action requires bounds; if present we use them else continue
// ToDo@arunrab : Change the logic for modifying page state
if (pageState.bounds) {
return WinJS.Promise.wrap({ region: pageState.region, bounds: pageState.bounds });
}
}
var ui = WeatherAppJS.Utilities.UI;
// Identify region bounds using location isoCode
if (pageState && pageState.isoCode) {
return ui.getMapRegionDataForIsoCodeAsync(pageState.isoCode)
.then(function (data) {
return data;
});
} else {
// The region information not provided. Top Edgy L1 button used.
var regionCode = WeatherAppJS.SettingsManager.getUserDefaultRegionISOCode();
this._localState.isoCode = regionCode;
return ui.getMapRegionDataForIsoCodeAsync(regionCode)
.then(function (data) {
var regionName = data["region"];
var region = (regionName) ? regionName : PlatformJS.Services.appConfig.getString("BaseMapDefaultContinent");
var bounds = data["bounds"];
return { region: region, bounds: bounds };
});
}
},
getMapOptionsFromPageState: function getMapOptionsFromPageState() {
var mapOptions = {};
if (this._localState) {
mapOptions.region = this._localState.region ? this._localState.region : null;
if (this._localState.isNationalMap || this._localState.isTopEdgyL1Action) {
mapOptions.bounds = this._localState.bounds;
} else {
// if latitude & longitude are passed, set the map center
var center = null;
if (MapUtils.isNumeric(this._localState.latitude) && MapUtils.isNumeric(this._localState.longitude)) {
mapOptions.center = new MicrosoftJS.Maps.Location(this._localState.latitude, this._localState.longitude);
mapOptions.isoCode = this._localState.isoCode ? this._localState.isoCode : null;
}
if (MapUtils.isNumeric(this._localState.zoomLevel)) {
mapOptions.zoom = this._localState.zoomLevel;
}
}
}
return mapOptions;
},
/// <summary>
/// This function is invoked by the UX Framework when the data binding is complete
/// </summary>
/// <returns>
/// Nothing
/// </returns>
onBindingComplete: function onBindingComplete() {
WinJS.Resources.processAll();
var that = this;
var canvasLayer = new WeatherAppJS.Maps.CanvasLayer(this.baseMapCtlr, that._seekBar,
{
map: that._currentMapType,
tsControl: that._tsControl,
legend: that._legend,
baseMapLayerOptions: JSON.parse(MapConfigs.getMapScenarioConfigItem("BaseMapLayerOptions").value)
});
canvasLayer.addEventListener('firstframeloaded', function (event) {
that.logMapLoadTime('firstframe', event.startTime);
});
canvasLayer.addEventListener('allframesloaded', function (event) {
that.logMapLoadTime('allframes', event.startTime);
});
this.mapLayers.push(canvasLayer);
if (!that._isBaseMapCtrlInitialized) {
that._initializeBaseMapCtlr();
} else {
that.loadMapLayers();
}
clearTimeout(that._baseMapInitializeCheckTimer);
that._baseMapInitializeCheckTimer = setTimeout(that.checkBaseMapInit.bind(that), that._baseMapInitializeCheckTimeout);
that._setMapTheme();
that.initializeEdgyActionHandlers();
// Start timer for page data refresh
that.cleanPeriodicTimersStack();
var dataRefreshTimerId = setInterval(this.bindablePageRefreshFn, PlatformJS.Services.appConfig.getString("PageRefreshTime"));
that._timersStack.push(dataRefreshTimerId);
},
_initializeBaseMapCtlr: function _initializeBaseMapCtlr() {
var that = this;
if (!this.baseMapCtlr) {
this.baseMapCtlr = new WeatherAppJS.Maps.BaseMapController();
}
if (!this.baseMapCtlrInitHandler) {
this.baseMapCtlrInitHandler = function () {
that._isBaseMapCtrlInitialized = true;
that._registerEventHandlers();
// Create tile layers after initializing the map
that.loadMapLayers();
};
}
this.baseMapCtlr.addEventListener('initialized', this.baseMapCtlrInitHandler);
var mapDiv = document.getElementById("mapdiv");
var mapOptions = this.getMapOptionsFromPageState();
mapOptions.baseMapOptions = JSON.parse(MapConfigs.getMapScenarioConfigItem("BaseMapLayerOptions").value); //Set base map options (road/aerial etc.) for current map type
this.baseMapCtlr.init(mapDiv, mapOptions);
},
_disposeBaseMapCtlr: function _disposeBaseMapCtlr() {
this._removeEventHandlers();
if (this.baseMapCtlr) {
this.baseMapCtlr.dispose();
this.baseMapCtrl = null;
}
},
logMapLoadTime: function logMapLoadTime(event, startTime) {
var perfContext = "";
if (WinJS.Navigation.location) {
perfContext = WinJS.Navigation.location.page + "*" + WinJS.Navigation.location.channelId;
}
var markerText = (event === 'firstframe') ? "WeatherApp:WeatherMapsPage:CanvasFirstFrameLoad" : "WeatherApp:WeatherMapsPage:CanvasAllFramesLoad";
var eventName = (event === 'firstframe') ? "First Frame Load Time" : "Animation Load Time";
var mapLoadTime = (new Date().getTime() / 1000) - startTime;
var eventAttr = { region: this._localState.region, mapType: this._currentMapType.id, mapSubType: this._currentMapType.subTypeId ? this._currentMapType.subTypeId : "" };
// this event represents time elapsed to load the (first / all) frames of the  weather map.
Platform.Telemetry.FlightRecorder.logPerfWithJsonAttributes(Platform.Telemetry.LogLevel.normal, markerText, eventName, perfContext, mapLoadTime, JSON.stringify(eventAttr));
},
initializeEdgyActionHandlers: function initializeEdgyActionHandlers() {
if (this._isEdgyActionHandlersInitialized) {
return;
}
this._isEdgyActionHandlersInitialized = true;
var bEdgyElement = document.getElementById("bottomEdgy");
this._bEdgy = bEdgyElement.winControl;
this._helpButton = bEdgyElement.querySelector("#helpButton");
this._refreshButton = bEdgyElement.querySelector("#refreshButton");
this._changeUnitToFButton = bEdgyElement.querySelector("#changeUnitToFButton");
this._changeUnitToCButton = bEdgyElement.querySelector("#changeUnitToCButton");
var that = this;
if (this._bEdgy) {
this._bEdgy.addEventListener("beforeshow",
function (e) {
that.onBeforeShowBottomEdgy(e);
});
this._refreshButton.addEventListener("click",
function (e) {
that.onRefreshClick(e);
});
this._helpButton.addEventListener("click",
function (e) {
WeatherAppJS.Utilities.UI.openInAppHelp();
});
this._changeUnitToFButton.addEventListener("click",
function (e) {
that.onChangeUnitToFClick(e);
});
this._changeUnitToCButton.addEventListener("click",
function (e) {
that.onChangeUnitToCClick(e);
});
}
},
onBeforeShowBottomEdgy: function onBeforeShowBottomEdgy(e) {
var useTianQi = Platform.Utilities.Globalization.isFeatureEnabled("UseTianQi");
var toggleShowButton = WeatherAppJS.Utilities.UI.toggleShowButton;
if (!useTianQi) {
var isCurrentUnitF = (WeatherAppJS.SettingsManager.getDisplayUnit() === "F");
toggleShowButton(this._changeUnitToFButton, !isCurrentUnitF);
toggleShowButton(this._changeUnitToCButton, isCurrentUnitF);
}
},
/// <summary>
/// This function is invoked by the device has been rotated
/// </summary>
/// <param name="event">
/// orientation change event
/// </param>
/// <returns>
/// Nothing
/// </returns>
onOrientationChange: function onOrientationChange(event) {
this._updateView();
},
_updateView: function _updateView() {
var that = this;
// update map layers
for (var i = 0; i < that.mapLayers.length; i++) {
that.mapLayers[i].updateLayer();
}
// update seekbar
that._seekBar.update();
if (that.baseMapCtlr && that._isBaseMapCtrlInitialized) {
that.baseMapCtlr.setRestrictedViewDelayed(true);
}
},
onSettingsChange: function onSettingsChange(e) {
this.onPageRefresh(true);
},
onNetworkOnline: function onNetworkOnline() {
this.onPageRefresh(true);
},
checkBaseMapInit: function checkBaseMapInit() {
var that = this;
CommonJS.Progress.hideProgress(CommonJS.Progress.centerProgressType);
CommonJS.Progress.hideProgress(CommonJS.Progress.headerProgressType);
CommonJS.Error.removeError();
if (!this._isBaseMapCtrlInitialized) {
var errorCallback = null;
var errorType = null;
if (WeatherAppJS.Networking.internetAvailable === true) {
errorType = CommonJS.Error.STANDARD_ERROR;
errorCallback = function () {
that.onPageRefresh(true);
};
} else {
errorType = CommonJS.Error.NO_INTERNET;
}
CommonJS.Error.showError(errorType, errorCallback);
// dispose off existing uninitialized basemapctlr and create a new one
this._disposeBaseMapCtlr();
return;
} else {
clearTimeout(this._baseMapInitializeCheckTimer);
}
},
/// <summary>
/// Handles window resize event. Occurs when:
/// - changing resolution in control panel
/// - WINDOWS+ARROW when using multiple monitors
/// </summary>
/// <param name="event">
/// resize event
/// </param>
/// <returns>
/// Nothing
/// </returns>
onWindowResize: function onWindowResize(event) {
this._updateView();
},
/// <summary>
/// Page refresh handler
/// </summary>
/// <param name="bypassCache">Whether to bypass the cache</param>
onPageRefresh: function onPageRefresh(bypassCache) {
var that = this;
while (this.mapLayers.length !== 0) {
var layer = this.mapLayers.pop();
layer.removeLayer();
layer.disposeLayer();
}
that._seekBar.resetAnimation();
CommonJS.Progress.hideProgress(CommonJS.Progress.centerProgressType);
CommonJS.Progress.hideProgress(CommonJS.Progress.headerProgressType);
CommonJS.Error.removeError();
clearTimeout(that._baseMapInitializeCheckTimer);
CommonJS.forceRefresh();
},
onLayoutChange: function onLayoutChange() {
this._updateView();
},
loadMapLayers: function loadMapLayers() {
var that = this;
// Create tile layers after initializing the map
var layerConfig = PlatformJS.Services.appConfig.getDictionary("LabelsLayerUrls");
var layerConfigForEnv = layerConfig[WeatherAppJS.SettingsManager.getMapsEnvironment()];
var labelsLayerStyle = MapConfigs.getMapScenarioConfigItem("LabelsTileLayerStyle").value;
var labelLayerOptions = {
tileSourceOptions: {
uriConstructor: (layerConfigForEnv[labelsLayerStyle]) ? layerConfigForEnv[labelsLayerStyle].value : MapConfigs.getBaseMapGlobalOption(labelsLayerStyle)
}
};
var labelsLayer = new WeatherAppJS.Maps.LabelsTileLayer(that.baseMapCtlr, labelLayerOptions);
that.mapLayers.push(labelsLayer);
var loadLayersPromises = [];
// Asynchronously load the layers
that.mapLayers.forEach(function (layer) {
loadLayersPromises.push(layer.loadLayerAsync());
});
// Synchronously add the layers
var loadLayersMergedPromise = WinJS.Promise.join(loadLayersPromises);
that._promisesStack.push(loadLayersMergedPromise);
loadLayersMergedPromise.then(
function () {
for (var i = 0; i < that.mapLayers.length; i++) {
that.mapLayers[i].addLayer();
that.mapLayers[i].showLayer();
}
},
function (e) {
if (WeatherAppJS.Controls.Utilities.isCancellationError(e)) {
return;
}
// Map layer failed to load.
CommonJS.Error.removeError();
if (WeatherAppJS.Networking.internetAvailable === true) {
WeatherAppJS.Error.showMessageBarWithButton(PlatformJS.Services.resourceLoader.getString("/platform/standardErrorDescription"), false, PlatformJS.Services.resourceLoader.getString("RetryButtonLabel"), function () {
that.onPageRefresh(true);
});
} else {
CommonJS.Error.showError(CommonJS.Error.NO_INTERNET);
}
});
},
mapSwitchedEventHandler: function mapSwitchedEventHandler(event) {
var that = this;
that._currentMapType = event.detail;
MapConfigs.initMapScenarioConfig(that._currentMapType.id, that._currentMapType.subTypeId); // update map scenario configs
that.onPageRefresh(false);
//Radar type includes two subtypes. So, they are logged separately.
var eventAttr = { mapType: event.detail.id, mapSubType : event.detail.subTypeId ? event.detail.subTypeId : "" };
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, WeatherAppJS.Instrumentation.PageContext.WeatherMapsChromeTopBar, "Map Type", Platform.Telemetry.UserActionOperation.select, Platform.Telemetry.UserActionMethod.unknown, 0, JSON.stringify(eventAttr));
},
onUserGesture: function onUserGesture(event) {
var that = this;
if (that._topBar && that._seekBar) {
if (event.type === 'MSPointerDown') {
that.gr.processDownEvent(event.currentPoint); // feed the gesture recognizer
} else if (event.type === 'MSPointerUp' || event.type === 'MSPointerCancel' ||
(event.type === "MSPointerOut" && !event.currentPoint.isInContact) ||
(event.type === "MSLostPointerCapture" && !event.currentPoint.isInContact)) { // Check if pointer is not in contact, otherwise raises error "Failed to process up, because the pointer is in contact"
that.gr.processUpEvent(event.currentPoint); // feed the gesture recognizer
} else if (event.type === 'tapped') {
if (that._topBar.isShown && !that._topBar.isMouseOver && that._seekBar.isEnabled && !that._seekBar.isMouseOver && !that._zoomControl.isMouseOver) {
that._displayChrome(false);
that._zoomControl.hide();
} else {
that._displayChrome(true);
}
} else if (event.type === 'mousewheel') {
if (that._topBar.isShown && that._seekBar.isEnabled) {
that._displayChrome(false);
that._zoomControl.hide();
}
} else if (event.type === 'MSPointerMove') {
// if touch, hide zoom control
if (event.pointerType === event.MSPOINTER_TYPE_TOUCH) {
that._zoomControl.hide();
clearTimeout(that._autoHideChromeTimer);
that._autoHideChromeTimer = setTimeout(that._displayChrome.bind(that), that._autoHideChromeTimeOut);
return;
}
// preserve status quo on mouse hold and drag or mouse move within a threshold
if (event.currentPoint.isInContact || that._isMouseMoveWithinThreshold(event)) {
return;
}
if (!that._topBar.isShown) {
that._displayChrome(true);
} else {
clearTimeout(that._autoHideChromeTimer);
that._autoHideChromeTimer = setTimeout(that._displayChrome.bind(that), that._autoHideChromeTimeOut);
}
that._zoomControl.show();
} else if (event.type === "keydown") {
// for keyboard events
if (event.keyCode === WinJS.Utilities.Key.enter) {
if (!that._topBar.isShown) {
that._isEnterPressed = true;
that._displayChrome(true);
} else {
that._isEnterPressed = false;
that._displayChrome(false);
}
that._seekBar.isPlaying = !that._seekBar.isPlaying; // toggle play on enter button for accessibility
} else if (event.keyCode === WinJS.Utilities.Key.escape) {
that._isEnterPressed = false;
that._displayChrome(false);
}
} else if (event.type === "manipulationstarted") {
// for swipe, pinch zoom gestures, hide chrome
that._displayChrome(false);
that._zoomControl.hide();
} else if (event.type === "manipulationcompleted") {
that.gr.completeGesture();
}
}
},
// on base map viewchangeend, based on zoom level, update the zoom buttons
onBaseMapViewChange: function onBaseMapViewChange() {
var currentZoomLevel = this.baseMapCtlr.baseMapZoomLevel;
if (this._prevZoomLevel) {
var eventAttr = { mapType: this._currentMapType.id, mapSubType: this._currentMapType.subTypeId ? this._currentMapType.subTypeId : "" };
if (currentZoomLevel > this._prevZoomLevel) {
eventAttr['action'] = "Zoom In";
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, WeatherAppJS.Instrumentation.PageContext.WeatherMapsChrome, "Zoom In/Out", Platform.Telemetry.UserActionOperation.click, Platform.Telemetry.UserActionMethod.unknown, 0, JSON.stringify(eventAttr));
} else if (currentZoomLevel < this._prevZoomLevel) {
eventAttr['action'] = "Zoom Out";
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, WeatherAppJS.Instrumentation.PageContext.WeatherMapsChrome, "Zoom In/Out", Platform.Telemetry.UserActionOperation.click, Platform.Telemetry.UserActionMethod.unknown, 0, JSON.stringify(eventAttr));
}
}
this._prevZoomLevel = currentZoomLevel;
this._zoomControl.update();
},
onRegionChange: function onRegionChange(e) {
if (e && e.detail && e.detail.region) {
var changedRegion = e.detail.region;
PlatformJS.Navigation.navigateToChannel("WeatherMaps", { region: changedRegion });
var eventAttr = { region: e.detail.region };
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, WeatherAppJS.Instrumentation.PageContext.WeatherMapsChromeTopBar, "Region", Platform.Telemetry.UserActionOperation.select, Platform.Telemetry.UserActionMethod.unknown, 0, JSON.stringify(eventAttr));
}
},
_registerEventHandlers: function _registerEventHandlers() {
var that = this;
var baseMapContainer = document.querySelector('.baseMapDiv');
// If required, for better fps, don't listen to user gesture events and avoid hiding the chrome.
if (!WeatherAppJS.Utilities.UI.isPerfOptimizedExperience()) {
if (baseMapContainer) {
baseMapContainer.addEventListener("MSPointerMove", that.bindableUserGestureHandler, true);
baseMapContainer.addEventListener('MSPointerMove', that.bindableProcessMove, true);
baseMapContainer.addEventListener("MSPointerDown", that.bindableUserGestureHandler, true);
baseMapContainer.addEventListener("MSPointerUp", that.bindableUserGestureHandler, true);
baseMapContainer.addEventListener("MSPointerCancel", that.bindableUserGestureHandler, true);
baseMapContainer.addEventListener("MSLostPointerCapture", that.bindableUserGestureHandler, true);
baseMapContainer.addEventListener("MSPointerOut", that.bindableUserGestureHandler, false);
baseMapContainer.addEventListener("mousewheel", that.bindableUserGestureHandler, true);
window.addEventListener("keydown", that.bindableUserGestureHandler, true);
that._initGestureRecognition();
}
}
//based on any changes to base map, make changes to the zoom buttons
if (this.baseMapCtlr) {
this._baseMapViewChangeEventHandler = this.baseMapCtlr.addBaseMapThrottledEventHandler('viewchangeend', that.bindableBaseMapViewChangeFn);
}
},
_removeEventHandlers: function _removeEventHandlers() {
var that = this;
var baseMapContainer = document.querySelector('.baseMapDiv');
if (baseMapContainer) {
baseMapContainer.removeEventListener('MSPointerMove', that.bindableUserGestureHandler);
baseMapContainer.removeEventListener('MSPointerMove', that.bindableProcessMove);
baseMapContainer.removeEventListener('MSPointerDown', that.bindableUserGestureHandler);
baseMapContainer.removeEventListener("MSPointerUp", that.bindableUserGestureHandler);
baseMapContainer.removeEventListener("MSPointerCancel", that.bindableUserGestureHandler);
baseMapContainer.removeEventListener("MSLostPointerCapture", that.bindableUserGestureHandler);
baseMapContainer.removeEventListener("MSPointerOut", that.bindableUserGestureHandler);
baseMapContainer.removeEventListener('mousewheel', that.bindableUserGestureHandler);
window.removeEventListener('keydown', that.bindableUserGestureHandler);
}
if (that.gr) {
that.gr.removeEventListener('manipulationstarted', that.bindableUserGestureHandler);
that.gr.removeEventListener('manipulationupdated', that.bindableUserGestureHandler);
that.gr.removeEventListener('manipulationcompleted', that.bindableUserGestureHandler);
that.gr.removeEventListener('tapped', that.bindableUserGestureHandler);
}
if (that.baseMapCtlr) {
that.baseMapCtlr.removeBaseMapEventHandler(that._baseMapViewChangeEventHandler);
that.baseMapCtlr.removeEventListener('initialized', that.baseMapCtlrInitHandler);
}
},
processMouseMoveEvent: function processMouseMoveEvent(event) {
// this sometimes causes a runtime error: event cannot be handled in a non-chronological order
try {
this.gr.processMoveEvents(event.intermediatePoints); // feed the gesture recognizer
} catch (ex) {
// move on
}
},
_isMouseMoveWithinThreshold: function _isMouseMoveWithinThreshold(event) {
var isMouseMoveWithinThreshold = false;
if (event && event.currentPoint && event.currentPoint.position) {
var dx = Math.abs(event.currentPoint.position.x - this._prevMousePos.x);
var dy = Math.abs(event.currentPoint.position.y - this._prevMousePos.y);
this._prevMousePos = {
x: event.currentPoint.position.x,
y: event.currentPoint.position.y
};
if (dx < 15 && dy < 15) {
isMouseMoveWithinThreshold = true;
}
}
return isMouseMoveWithinThreshold;
},
_init: function _init() {
var that = this;
var mapControlsDiv = document.getElementById("mapcontrols");
var topBarDiv = document.createElement("div");
var topBar = new WeatherAppJS.Maps.MapsTopBar(topBarDiv, { maps: that.supportedMaps, currentMap: that._currentMapType, region: that._localState.region });
that._topBar = topBar;
mapControlsDiv.appendChild(topBarDiv);
topBarDiv.addEventListener("mapswitched", that.mapSwitchedEventHandler.bind(that));
topBarDiv.addEventListener("regionchanged", that.onRegionChange.bind(that));
var seekBarDiv = document.createElement("div");
var seekBar = new WeatherAppJS.Maps.SeekBar(seekBarDiv);
that._seekBar = seekBar;
that._seekBar._playButton.addEventListener('click', function (event) {
var eventAttr = { mapType: that._currentMapType.id, mapSubType: that._currentMapType.subTypeId ? that._currentMapType.subTypeId : "", action : !that._seekBar.isPlaying ? "Pause" : "Play" };
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(Platform.Telemetry.LogLevel.normal, WeatherAppJS.Instrumentation.PageContext.WeatherMapsChrome, "Play/Pause", Platform.Telemetry.UserActionOperation.click, Platform.Telemetry.UserActionMethod.unknown, 0, JSON.stringify(eventAttr));
});
mapControlsDiv.appendChild(seekBarDiv);
var zoomButtons = document.createElement("div");
var zoomControl = new WeatherAppJS.Maps.ZoomControl(zoomButtons, { baseMapCtlr: this.baseMapCtlr });
that._zoomControl = zoomControl;
mapControlsDiv.appendChild(zoomButtons);
that._displayChrome(true);
that._zoomControl.hide();
var tsControlDiv = document.createElement("div");
var tsControl = new WeatherAppJS.Maps.TimeStamp(tsControlDiv);
that._tsControl = tsControl;
mapControlsDiv.appendChild(tsControlDiv);
var legendDiv = document.createElement("div");
var legendControl = new WeatherAppJS.Maps.Legend(legendDiv, {});
that._legend = legendControl;
mapControlsDiv.appendChild(legendDiv);
},
_setMapTheme: function _setMapTheme() {
var mapControlsDiv = document.getElementById("mapcontrols");
if (mapControlsDiv) {
if (MapConfigs.getMapScenarioConfigItem("DarkMapThemeActive").value) {
WinJS.Utilities.addClass(mapControlsDiv, "darkMapBase");
}
else {
WinJS.Utilities.removeClass(mapControlsDiv, "darkMapBase");
}
}
},
// toggle displaying the chrome around the map
_displayChrome: function _displayChrome(show) {
var that = this;
if (show) {
that._topBar.show();
that._seekBar.show();
clearTimeout(that._autoHideChromeTimer);
that._autoHideChromeTimer = (!WeatherAppJS.Utilities.UI.isPerfOptimizedExperience()) ? setTimeout(that._displayChrome.bind(that), that._autoHideChromeTimeOut) : null;
} else {
// hide only if we are not interacting with the seekbar
if (that._seekBar.isEnabled && !that._seekBar.isMouseOver && !that._topBar.isMouseOver && !that._zoomControl.isMouseOver && !that._isEnterPressed) {
that._topBar.hide();
that._seekBar.hide();
that._zoomControl.hide();
clearTimeout(that._autoHideChromeTimer);
Platform.Telemetry.FlightRecorder.logAppAction(Platform.Telemetry.LogLevel.normal, WeatherAppJS.Instrumentation.PageContext.WeatherMapsChrome, "", Platform.Telemetry.AppActionOperation.open, 0);
} else { // user is interacting, so reset the timer.
clearTimeout(that._autoHideChromeTimer);
that._autoHideChromeTimer = setTimeout(that._displayChrome.bind(that), that._autoHideChromeTimeOut);
}
}
},
_initMapsConfigAsync: function _initMapsConfigAsync() {
var that = this;
var ui = WeatherAppJS.Utilities.UI;
msWriteProfilerMark("WeatherApp:WeatherMapsConfigInit:s");
return this.getRegionDataAsync(this._localState)
.then(function getRegionData_complete(data) {
that.supportedMaps = [];
if (data) {
var isNationalMap = that._localState.isNationalMap;
var isTopEdgyL1Action = that._localState.isTopEdgyL1Action;
if (isNationalMap || isTopEdgyL1Action) {
that._localState.bounds = data.bounds;
}
that._localState.region = data.region;
var mapTypes = ui.getMapTypesForRegion(that._localState.region);
for (var mapTypeId in mapTypes) {
var mapSubTypes = mapTypes[mapTypeId];
if (mapSubTypes) {
for (var i in mapSubTypes) {
that.supportedMaps.push(ui.getPageStateForMapType(mapTypeId, mapSubTypes[i]));
}
}
}
}
return that.supportedMaps;
}, function (error) {
// could not get region. handle error.
return null;
});
},
_initCurrentMap: function _initCurrentMap() {
var that = this;
if (that._localState.map) {
that._currentMapType = { mapType: that._localState.map.mapType, id: that._localState.map.id, mapSubType: that._localState.map.mapSubType, subTypeId: that._localState.map.subTypeId };
} else {
// Default map type:
// For top edgy L1 button action, use user's current region isocode to find the default map type. 
// Otherwise, use the app wide default map type
var isoCode = (this._localState.isTopEdgyL1Action) ? this._localState.isoCode : null;
var mapType = MapConfigs.getDefaultMapTypeForIsoCode(isoCode);
that._currentMapType = WeatherAppJS.Utilities.UI.getPageStateForMapType(mapType);
}
},
showMessage: function showMessage(msgKey) {
if (CommonJS.messageBar) { // remove any old message bar.
CommonJS.messageBar.hide();
}
if (msgKey) {
WeatherAppJS.Error.showMessageBar(PlatformJS.Services.resourceLoader.getString(msgKey), false);
}
CommonJS.dismissAllEdgies();
},
_initGestureRecognition: function _initGestureRecognition() {
this.gr = new Windows.UI.Input.GestureRecognizer();
// Configuring GestureRecognizer to detect manipulation translation and tap
this.gr.gestureSettings = Windows.UI.Input.GestureSettings.manipulationTranslateX |
Windows.UI.Input.GestureSettings.manipulationTranslateY |
Windows.UI.Input.GestureSettings.manipulationTranslateInertia |
Windows.UI.Input.GestureSettings.tap;
// Turn off UI feedback for gestures
this.gr.showGestureFeedback = false;
// Register event listeners for the gestures
this.gr.addEventListener('manipulationstarted', this.bindableUserGestureHandler);
this.gr.addEventListener('manipulationupdated', this.bindableUserGestureHandler);
this.gr.addEventListener('manipulationcompleted', this.bindableUserGestureHandler);
this.gr.addEventListener('tapped', this.bindableUserGestureHandler);
},
/// <summary>
/// Clean up function for the page.
/// </summary>
_disposePage: function _disposePage() {
var layer = null;
while (this.mapLayers.length !== 0) {
layer = this.mapLayers.pop();
layer.disposeLayer();
}
if (this._seekBar) {
this._seekBar.isPlaying = false;
}
if (this._topBar) {
this._topBar.cleanUp();
}
this._disposeBaseMapCtlr();
WeatherAppJS.Networking.removeEventListener('networkonline', this.bindableNetworkOnlineFn, false);
Windows.ApplicationModel.Search.SearchPane.getForCurrentView().showOnKeyboardInput = this._showSearchCharmOnKeyBoardInput; // restore value on disposing
clearTimeout(this._autoHideChromeTimer);
clearTimeout(this._baseMapInitializeCheckTimer);
},
})
});
})();